<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	2checkout
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansApp2checkout extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
	public function collectAppParams(array $data)
		{
			$app = PayplansApp::getInstance($data['id'], $data['type']);
			$appParam = $app->getAppParam('activation','');
			if($app->app_id == 0 || $data['app_params']['activation'] == $appParam){
				return parent::collectAppParams($data);				
			}
			 XiFactory::getApplication()->redirect('index.php?option=com_payplans&view=app&task=edit&id='.$data['id'],XiText::_('COM_PAYPLANS_PAYMENT_APP_2CHECKOUT_CHANGE_MESSAGE'));
			return false;
		}


	function onPayplansControllerCreation($view, $controller, $task, $format)
	{
		// vendor_order_id should be present in data posted
		// explode it by , 
		// first will be order key 
		// second will be payment key
		if($view == 'payment' && $task == 'notify'){
			$keys = JRequest::getVar('vendor_order_id', false);
			if($keys == false){
				return true;
			}
			
			$keys = explode(',', $keys);
			array_shift($keys);
			JRequest::setVar('payment_key', array_shift($keys), 'POST');
			return true;
		}
		
		return true;
	}
	
    function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		if(is_object($data)){
			$data = (array)$data;
		}
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		if ($this->getAppParam('sandbox', false)) {
			$this->assign('post_url','https://sandbox.2checkout.com/checkout/purchase');
		}else {
			if ($this->getAppParam('alternate_url', false)) {
				$this->assign('post_url', 'https://www2.2checkout.com/checkout/spurchase');
			} else {
				$this->assign('post_url', 'https://www.2checkout.com/checkout/spurchase');
			}
		}

		$this->assign('sid', $this->getAppParam('sid',''));
		$this->assign('invoice_number',$payment->getKey());
		$this->assign('payment_key',$payment->getKey());
		
		// store order key and payment key in merchant order id
		// so that order and payment can be found out at the time of notify
		// this will come as vendor_order_id
		$this->assign('merchant_order_id',$invoice->getKey().','.$payment->getKey());
		
		$this->assign('x_invoice_num',$invoice->getKey());
		// user should not change the quantity
		$this->assign('fixed','Y');
		$this->assign('total',$invoice->getTotal());
		$this->assign('currency', $invoice->getCurrency('isocode'));
	
		$root = JURI::root();
    	if(XiFactory::getConfig()->https == true){
    		$root = JString::str_ireplace("http:", "https:", $root);
    	}
		$this->assign('return_url', $root.'index.php?option=com_payplans&view=payment&task=complete&gateway=2checkout&action=cancel');
		$this->assign('cart_order_id', $invoice->getTitle());
		
		// user details, will be used only for non recurring  payments
		$user = $invoice->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		$this->assign('cust_id',$user->getId());
		$this->assign('username', $user->getUsername());
		$this->assign('name',$user->getRealname());
		
		$this->assign('cart_brand_name', PayplansFactory::getConfig()->sitename);
		$this->assign('cart_version_name', PAYPLANS_REVISION);
		
		if($invoice->isRecurring()){
			$counter = $invoice->getCounter();
			
			// for recurring subscrition we are using Pass-Through-Products Parameters from 2checkout.com
			$this->assign('mode', '2CO'); // always should be 2CO
			$this->assign('li_0_type', 'product'); // 0 is for sequence number of product(starting from 0), product,shipping,tax or coupon
			$this->assign('li_0_name', $invoice->getTitle());
			$this->assign('li_0_quantity', 1);  //  quantity should be 1 for now
			$this->assign('li_0_price', $invoice->getTotal());  // regular payment amount
			
			// Specifies if the corresponding li_#_type is a tangible or intangible. ‘Y’ or ‘N’, if li_#_type is
			// ‘shipping’ forced to ‘Y’.
			$this->assign('li_0_tangible', 'N');  //  No for now in future we may five option for this
			$time = $this->getRecurrenceTime($invoice->getExpiration());
			$this->assign('li_0_recurrence', $time['period'].' '.$time['unit']);
			$this->assign('li_0_duration', $this->_getRecurrenceCount($invoice, $time));	// Forever or # Week | Month | Year – always singular, defaults to Forever.
			
			if(PAYPLANS_RECURRING_TRIAL_1 == $invoice->isRecurring()) {
				$regular_price = $invoice->getTotal($counter+1);
				$this->assign('li_0_price', $regular_price);  // regular  payment amount
				// for start up fee 
				$first_price = $invoice->getTotal();
		        $this->assign('li_0_startup_fee', floatval( floatval($first_price) - floatval($regular_price)));
			}
			
			return $this->_render('form_subscription');
		}
		
		return $this->_render('form');
	}

	/**
	 * validate the key comes in the notification
	 * @param PayplansPayment $payment
	 * @param array $post
	 */
	function validateNotification($post)
	{
		// if notification came from INS then need to do following
		// else, in normal case it 
		if(isset($post['message_type'])){
			$string_to_hash = $post['sale_id'].$this->getAppParam('sid', '').$post['invoice_id'].$this->getAppParam('secret_word', '');
			$postKey = $post['md5_hash'];
		}
		else{
			if($this->getAppParam('sandbox', 0)) {
				$string_to_hash	= $this->getAppParam('secret_word', '').$this->getAppParam('sid', '')."1".$post['total'];
			} else {
				$string_to_hash	= $this->getAppParam('secret_word', '').$this->getAppParam('sid', '').$post['order_number'].$post['total'];
			}
			$postKey = $post['key'];
		}
		
		$check_key = strtoupper(md5($string_to_hash));

		return (strcmp($check_key, $postKey) == 0);
	}
	
	protected function _getRecurrenceCount($invoice, $time)
	{
		$count = $invoice->getRecurrenceCount();
		if(intval($count) === 0){
			return 'FOREVER';
		}
		
		$duration = $count * $time['period'];
		return $duration.' '.$time['unit'];
	} 
	
	protected function _order_created($payment, $data, $transaction)
	{
		$transaction->set('message', 'COM_PAYPLANS_APP_2CHECKOUT_TRANSACTION_ORDER_CREATED_'.JString::strtoupper($data['invoice_status']));
		
		// after refund is processed 2checkout again send a notification having paid as well as refund variables.
		// so if such notification comes, just make a 0 amount transaction.
		if(isset($data['item_type_2']) && !empty($data['item_type_2'])){
			return array();
		}

		if($this->getAppParam('activation','') == 'OrderCreation'){	
			if(in_array($data['invoice_status'], array('approved', 'deposited','pending'))){
				// pending, declined, approved, deposited
				if($data['recurring'] == 0){
					$transaction->set('amount', $data['item_list_amount_1']);
				}
			
			// change status if its recurring order creation 
			// otherwise do nothing
			if(isset($data['recurring']) && $data['recurring']){
				$transaction->set('amount', $data['item_rec_list_amount_1']);
			}
		  }
		}
		
		return array();
	}
	
	protected function _fraud_status_changed($payment, $data, $transaction)
	{
		if($this->getAppParam('activation','') == 'FraudStatus')
		{
			// fail, wait, pass, empty
			if($data['fraud_status'] == 'pass'){
				
				if($data['recurring'] == 0){
						$transaction->set('amount', $data['invoice_list_amount']);
				}
				
				if(isset($data['recurring']) && $data['recurring']){
					$transaction->set('amount', $data['item_rec_list_amount_1']);
				}
			}
		}
		$status = isset($data['fraud_status']) ? $data['fraud_status'] : 'NO_STATUS'; 
		$transaction->set('message', 'COM_PAYPLANS_APP_2CHECKOUT_TRANSACTION_FRAUD_STATUS_CHANGED_'.JString::strtoupper($status));

	}
	
	protected function _ship_status_changed($payment, $data, $transaction)
	{
		//not_shipped, shipped, or empty (if intangible / does not need shipped)		
		if(isset($data['ship_status'])){
			$transaction->set('message', 'COM_PAYPLANS_APP_2CHECKOUT_TRANSACTION_SHIP_STATUS_CHANGED_'.JString::strtoupper($data['ship_status']));
		}
		
		return array();
	}
	
	protected function _invoice_status_changed($payment, $data, $transaction)
	{
		if($this->getAppParam('activation','') == 'OrderCreation')
		{
			// at the time or order creation, if invoice is in pending status, we activate subscription
			// if that actiavted subscription get declined, make a negative transaction.
			 if(strtolower($data['invoice_status']) == 'declined'){
			 	if($data['recurring'] == 0){
							$transaction->set('amount', -$data['invoice_list_amount']);
					}
				
				if(isset($data['recurring']) && $data['recurring']){
					$transaction->set('amount', -$data['item_rec_list_amount_1']);
				}
			}
 		}
		$transaction->set('message', 'COM_PAYPLANS_APP_2CHECKOUT_TRANSACTION_STATUS_INVOICE_STATUS_CHANGED_'.JString::strtoupper($data['invoice_status']));
		return array();
	}
	
	protected function _refund_issued($payment, $data, $transaction)
	{
		$transaction->set('amount', -$data['item_list_amount_1']);
		$transaction->set('message', 'COM_PAYPLANS_APP_2CHECKOUT_TRANSACTION_STATUS_REFUND_ISSUED');
		return array();
	}
	
	protected function _recurring_installment_success($payment, $data, $transaction)
	{
		$transaction->set('amount', $data['item_rec_list_amount_1'])
						->set('message', 'COM_PAYPLANS_APP_2CHECKOUT_TRANSACTION_STATUS_RECURRING_INSTALLMENT_SUCCESS');
		return array();
	}
	
	protected function _recurring_installment_failed($payment, $data, $transaction)
	{
		$transaction->set('message', 'COM_PAYPLANS_APP_2CHECKOUT_TRANSACTION_STATUS_RECURRING_INSTALLMENT_FAILED');
		return array();
	}
	
	protected function _recurring_stopped($payment, $data, $transaction)
	{
		$transaction->set('message', 'COM_PAYPLANS_APP_2CHECKOUT_TRANSACTION_STATUS_RECURRING_STOPPED');
		return array();
	}

	protected function _recurring_complete($payment, $data, $transaction)
	{
		$transaction->set('message', 'COM_PAYPLANS_APP_2CHECKOUT_TRANSACTION_STATUS_RECURRING_COMPLETE');
		return array();
	}
	
	protected function _recurring_restarted($payment, $data, $transaction)
	{
		// XITODO : need to work
		$transaction->set('message', 'COM_PAYPLANS_APP_2CHECKOUT_TRANSACTION_STATUS_RECURRING_RESTARTED');
		return array();
	}
	
	function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		$invoice 		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $payment->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_subscr_id', isset($data['invoice_id']) ? $data['invoice_id'] : 0)
					->set('gateway_parent_txn', isset($data['sale_id']) ? $data['sale_id'] : 0)
					->set('params', PayplansHelperParam::arrayToIni($data));
					
		if(!$this->validateNotification($data)){
			$errors[] = XiText::_('COM_PAYPLANS_APP_2CHECKOUT_INVALID_HASH_KEY');
			$transaction->set('message', 'COM_PAYPLANS_APP_2CHECKOUT_INVALID_HASH_KEY')
								->save();
			return $errors;
		}
		
		
		if($invoice->getStatus() == PayplansStatus::INVOICE_REFUNDED)
		{
			$errors[] = XiText::_('COM_PAYPLANS_PAYMENT_APP_2CHECKOUT_NOTIFICATION_AFTER_REFUND');
			$transaction->set('message', XiText::_('COM_PAYPLANS_PAYMENT_APP_2CHECKOUT_NOTIFICATION_AFTER_REFUND'))
								->save();
			return $errors;
		}

			// if same notification came more than one time
	    	// check if transaction already exists
	    	// if yes then do nothing and return
	    	$txn_id 	=  0;
	    	$subscr_id  = isset($data['invoice_id']) ? $data['invoice_id'] : 0;
	    	$parent_txn = isset($data['sale_id']) ? $data['sale_id'] : 0;
	    	
	    	$transactions = $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_txn);
	    	if($transactions !== false){
	    		foreach($transactions as $record){
	    			$record = PayplansTransaction::getInstance($record->transaction_id, null, $record);
	    			if($record->getParam('message_id','') === $data['message_id'] && !isset($data['item_type_2'])){
	    				return true;
	    			}
	    		}
	    	}
		
		// if the Notification is from INS of 2checkout
		// then message type will be se	
		$func_name = 'invalidFunction';
		if(isset($data['message_type'])){
			$func_name = '_'.JString::strtolower($data['message_type']);
		}

		//XITODO : Log error
		if(method_exists($this, $func_name)){
			$errors = $this->$func_name($payment, $data, $transaction);
		}else{
			$errors = array(XiText::_('COM_PAYPLANS_APP_2CHECKOUT_INVALID_MESSAGE_TYPE'));
		}

		$transaction->save();
		return $errors;
	}
	
    public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		$logs	= PayplansHelperLogger::getLog($payment, XiLogger::LEVEL_ERROR);
		$record = array_pop($logs);				
		if($record && !empty($record)){
			$action = 'error';
		}
		
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	public function getRecurrenceTime($expTime)
	{
		$expTime['year'] 	= isset($expTime['year']) 	? intval($expTime['year'], 10) : 0;
		$expTime['month'] 	= isset($expTime['month']) ? intval($expTime['month'], 10) : 0;
		$expTime['day'] 	= isset($expTime['day']) 	? intval($expTime['day'], 10)  : 0;
		
		// days, if days are not zero then, convert whole time into days and convert it into weeks 
		if(!empty($expTime['day'])){
			$days  = $expTime['year'] * 365;
			$days += $expTime['month'] * 30;
			$days += $expTime['day'];
			
			$weeks = intval($days/7, 10);
			
			return array('period' => $weeks, 'unit' => 'Week', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_2CHECKOUT_RECURRING_MESSAGE'));
		}
		
		// if months are not empty 
		if(!empty($expTime['month'])){
			$months  = $expTime['year'] * 12;
			$months += $expTime['month'];
			
			return array('period' => $months, 'unit' => 'Month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_2CHECKOUT_RECURRING_MESSAGE'));
		}
		
		// if years are not empty 
		if(!empty($expTime['year'])){			
			return array('period' => $expTime['year'], 'unit' => 'Year', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_2CHECKOUT_RECURRING_MESSAGE'));
		}
	}
}
