<?php
/**
 * @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
 * @license		GNU/GPL, see LICENSE.php
 * @package		PayPlans
 * @subpackage	AcyMailing
 * @contact 	support+payplans@readybytes.in
 *
 */
if(defined('_JEXEC')===false) die();

class PayplansAppAcymailing extends PayplansApp
{
	protected $_location	= __FILE__;
	protected $_resource	= 'com_acymailing.list';

	public function _isApplicable(PayplansIfaceApptriggerable $refObject, $eventname='')
	{
		return JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_acymailing');
	}

	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}
		
		if(!require_once(rtrim(JPATH_ADMINISTRATOR,DS).DS.'components'.DS.'com_acymailing'.DS.'helpers'.DS.'helper.php')){
			$message = XiText::_('COM_PAYPLANS_LOGGER_ACYMAILING_LOG_FILE_NOT_FOUND');
			$content = array('previous' => array( 'added_user'=>$userid , 'message' => $message));
			PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $this, $content, 'PayplansAppAcymailingFormatter');
			return false;
		}

		$subid  = $new->getId();
		$userid = $new->getBuyer();
		$remove = array();
		
		//when subscription status is active
		if($new->getStatus() == PayplansStatus::SUBSCRIPTION_ACTIVE){
			$remove[]		 = $this->getAppParam('addToListonHold');
			$remove[]		 = $this->getAppParam('addToListonExpire');
			$addToList 	     = $this->getAppParam('addToListonActive');
			$this->_addOrRemoveFromAcymailingList($userid, $addToList, $remove, $subid);
			//forcefully removes the user from provided mailing list, irrespective of plan subscription.
			$removeFromListActive = $this->getAppParam('removeFromListActive');
			if(!empty($removeFromListActive)) {
				$this->removeForcefully($userid,$removeFromListActive);
			}
			return true;
		}

		//when subscription status is hold
		if($new->getStatus() == PayplansStatus::SUBSCRIPTION_HOLD){
			$remove[] 		= $this->getAppParam('addToListonActive');
			$remove[]       = $this->getAppParam('addToListonExpire');
			$addToList 		= $this->getAppParam('addToListonHold');
			return $this->_addOrRemoveFromAcymailingList($userid, $addToList, $remove, $subid);
		}
		
		//when subscription status is expire
		if($new->getStatus() == PayplansStatus::SUBSCRIPTION_EXPIRED){
			$remove[]		 = $this->getAppParam('addToListonActive');
			$remove[] 		 = $this->getAppParam('addToListonHold');
			$addToList 		 = $this->getAppParam('addToListonExpire');
			return $this->_addOrRemoveFromAcymailingList($userid, $addToList, $remove, $subid);
		}

	}
	
	function _addOrRemoveFromAcymailingList($userid, $addToList, $removeFromList, $subscriptionId)
	{
		$user = XiFactory::getUser($userid);

		$myUser 	   =  new stdClass();
		$myUser->name  =  $user->name;
		$myUser->email =  $user->email;

		$subscriberClass = acymailing_get('class.subscriber');

		$subid = $subscriberClass->save($myUser);

		//Id of the lists you want the user to be subscribed to (can be empty)
		$subscribe = is_array($addToList)? $addToList : array($addToList); 
		
		//Id of the lists you want the user to be removed from (can be empty)
		$remove = is_array($removeFromList)? $removeFromList : array($removeFromList);

		$userClass = acymailing_get('class.subscriber');

		$newSubscription = array();
		
		if(!empty($remove)){
			foreach($remove as $listId){
				if(is_array($listId)){
					$this->_addOrRemoveFromAcymailingList($userid, array(), $listId, $subscriptionId);
				}
				else {
					if($this->_removeFromResource($subscriptionId, $userid, $listId, $this->_resource)){
						$newList = null;
						$newList['status'] = 0;
						$newSubscription[$listId] = $newList;
					}
				}
			}
		}
		
		if(!empty($subscribe)){
			foreach($subscribe as $listId){
				$newList = null;
				$newList['status'] = 1;
				$newSubscription[$listId] = $newList;
				$this->_addToResource($subscriptionId, $userid, $listId, $this->_resource);
			}
		}
		
		

		if(empty($newSubscription)){
			return;	
		}

		//this function returns the ID of the user stored in the 
		//AcyMailing table from a Joomla User ID or an e-mail address
		$subid = $userClass->subid($userid);
		 
		//we didn't find the user in the AcyMailing tables
		if(empty($subid)){
			return false;
		}

		return $userClass->saveSubscription($subid,$newSubscription);
	}
	
	public function renderWidgetHtml()
	{   
		//get user id
		$userid     = XiFactory::getUser()->id;

		//get user's groups
	    $acymailing_lists = $this->_getUserAcymailingLists($userid);
        if(empty($acymailing_lists)){
        	return '';
        }
	    $this->assign('acymailing_lists',$acymailing_lists);
        $data = $this->_render('widgethtml');
        return $data;
	}	
   
	protected function _getUserAcymailingLists($userid)
	{   
		// XITODO : raise error if file not found or do nothing
        require_once(rtrim(JPATH_ADMINISTRATOR,DS).DS.'components'.DS.'com_acymailing'.DS.'helpers'.DS.'helper.php');

		$db = XiFactory::getDBO();
		$db->setQuery('SELECT `subid`'
		              .' FROM #__acymailing_subscriber '
		               .' WHERE `userid` = "'.$userid.'"');

		$subId 		= $db->loadResult();
		$acy_lists 	= array(); 
		$listClass 	= acymailing_get('class.listsub');
		$lists 		= $listClass->getSubscription($subId);

		foreach ($lists as $list){
			$acy_lists[$list->listid] = $list->name;
		}

		return $acy_lists;
	}

	function getNameFromResourceValue($resource, $value)
	{
		// if its a different resource
		if($resource != $this->_resource){
			return false;
		}
		
		$db         = XiFactory::getDBO();		
		$query      = 'SELECT `name`'
			          . ' FROM #__acymailing_list'
			          . ' WHERE `listid` = "'. $value.'"';
		$db->setQuery( $query );
		return $db->loadResult();	
	
	}
	
	// get lists
	static function getAcymailingList()
	{
		$query = new XiQuery();
		return $query->select(' `listid` as list_id, `name` ')
		             ->from('`#__acymailing_list`')
		             ->dbLoadQuery()
		             ->loadObjectList('list_id');
	}

	private function removeForcefully($userId,$removalList) {
		$removeFromListActive = is_array($removalList)? $removalList : array($removalList);
		$userClass = acymailing_get('class.subscriber');
		$newSubscription = array();
		foreach ($removeFromListActive as  $remListId) {
			$newList = null;
			$newList['status'] = 0;
			$newSubscription[$remListId] = $newList;
		}
		//this function returns the ID of the user stored in the
		//AcyMailing table from a Joomla User ID or an e-mail address
		$subid = $userClass->subid($userId);
		return $userClass->saveSubscription($subid,$newSubscription);
	}
	
}

class PayplansAppAcymailingFormatter extends PayplansAppFormatter
{
	public $template	= 'view_log';
	
	//get Rules 
	function getVarFormatter()
	{
		$rules = array('_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       'function' => 'getAppPlans'),
					   'app_params'      => array('formatter'=> 'PayplansAppAcymailingFormatter',
										       'function' => 'getFormattedParams'));
		return $rules;
	}
	
	// formatt params
	function getFormattedParams($key,$value,$data)
	{
		//do nothing if component is not installed
		if(!JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_acymailing')){
				return false;
		}
		
		$value['addToListonActive']	= $this->getListName($value['addToListonActive']);
		$value['addToListonHold']  	= $this->getListName($value['addToListonHold']);
		$value['addToListonExpire']	= $this->getListName($value['addToListonExpire']);
		
		$this->template				= 'view';
	}
	
	// Get Acymailing list names
	public function getListName($values)
	{
		$lists  = PayplansAppAcymailing::getAcymailingList();
		foreach ($values as $value){
			$listName[]   = $lists[$value]->name;
		}
		
		return implode(',', $listName);
	}

}
