<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		shyam@readybytes.in
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppAup extends PayplansApp
{
	protected $_location	= __FILE__;
	protected $_resource	= 'com_altauserpoints.points';
	
	// applicable only if aup exist
	public function _isApplicable(PayplansIfaceApptriggerable $refObject, $eventName='')
	{
		return JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_altauserpoints');
	}
	
	public function onPayplansInvoiceAfterSave($prev,$new)
    {
        // no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}
		$newPlan = array_shift($new->getPlans());
		// check if have to remove aup points on refund and current invoice status is refund 
		// then refund aup points and return
		
		if($this->getAppParam('removePointsOnRefund') && $new->getStatus() == PayplansStatus::INVOICE_REFUNDED)
		{	$sub = $new->getReferenceObject(PAYPLANS_INSTANCE_REQUIRE)->getSubscription();
			$newAupPoints = $this->_getAupPointsForPlan($newPlan);
			$this->_removeAupPoints($newAupPoints,$new->getBuyer(),$sub->getId());
			return true;
		}


     }

	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}
		
		
       	// check the status if not active then return
		if($new->getStatus() != PayplansStatus::SUBSCRIPTION_ACTIVE){
			return true;
		}
	        require_once (JPATH_SITE.DS.'components'.DS.'com_altauserpoints'.DS.'helper.php');
		$order = $new->getOrder(PAYPLANS_INSTANCE_REQUIRE);
		
		$buyer = $new->getBuyer();
		
		// is it upgrading from some plan ?
		$upgradingFrom = $order->getParam('upgrading_from',0);
		
		
		if(!$upgradingFrom){ // not upgrading
		  // then assign aup points and return
                  $assignedPoints  = $this->getAppParam('addPoints', 0);
	 	  $this->_addAupPoints($assignedPoints,$buyer,$new->getId());
		  return true;
		}
		$newPlan = array_shift($new->getPlans());
		//if upgrading from some plan
		$oldSub = PayplansSubscription::getInstance($upgradingFrom);
		$oldPlan = array_shift($oldSub->getPlans());
		// get aup points for that old plan from all apps 		
		$oldAupPoints = $this->_getAupPointsForPlan($oldPlan);
		$newAupPoints = $this->_getAupPointsForPlan($newPlan);
		$assignedPoints=0;
		// if points for new plan is greater than old plan then
		// assign points by subtracting old points from new points
		if($newAupPoints >= $oldAupPoints)
		{
			$assignedPoints = $newAupPoints-$oldAupPoints;
		}
		$this->_addAupPoints($assignedPoints,$buyer,$new->getId());
	}


	protected function _addAupPoints($assignedPoints,$buyer,$subId)
	{
		require_once (JPATH_SITE.DS.'components'.DS.'com_altauserpoints'.DS.'helper.php'); 
        $aupid = AltaUserPointsHelper::getAnyUserReferreID( $buyer);

		//this should use the aup methods for adding points, it should bypass any level restrictions etc...       
		AltaUserPointsHelper::newpoints( 'aup', $aupid, '', '', $assignedPoints, '', 1);

		$this->_addToResource($subId, $buyer,0, $this->_resource,$assignedPoints);

        return;
	}

    protected function _removeAupPoints($aupPoints,$buyer,$subId)
	{
		require_once (JPATH_SITE.DS.'components'.DS.'com_altauserpoints'.DS.'helper.php');
		$aupid = AltaUserPointsHelper::getAnyUserReferreID( $buyer);
		
		//this should use the aup methods for adding points, it should bypass any level restrictions etc...
        AltaUserPointsHelper::newpoints( 'aup', $aupid, '', '', -$aupPoints, '', 1);

		$this->_removeFromResource($subId, $buyer,0, $this->_resource,$aupPoints);
        return;
	}
	
	protected function _getAupPointsForPlan($plan)
	{
		
	    $buyaupApps 		= PayplansHelperApp::getAvailableApps('aup');
  	 	$appResult  		= array();
  	 	$userPointsForPlan 	= 0;

	  	//check which app allow category, which not allow and which app do nothing
		foreach($buyaupApps as $app)
		{
			if($app->getParam('applyAll',false) != false)
			{
				$userPointsForPlan	= $userPointsForPlan + $app->getAppParam('addPoints',0);
				continue;
			}
			
			//if user has app plans
			$appPlans = $app->getPlans();
			if(array_intersect(array($plan), $appPlans) != false){
				$userPointsForPlan	= $userPointsForPlan + $app->getAppParam('addPoints',0);
				continue;
			}
		}
		return $userPointsForPlan;
		
	}
	}
    
