<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	authorize.net
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppAuthorize extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	// over-ride to make it applicable on onPayplansControllerCreation
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
	/**
	 * if app support payment cancel
	 * @since 2.0
	 */
	public function isSupportPaymentCancellation($invoice)
	{	
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}	
	
	function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		if($view == 'payment' && $task == 'notify'){
			
			$paymentKey = JRequest::getVar('invoice_num', null);
			if($paymentKey){
				$prefix = JString::substr($paymentKey, 0,3);
				if($prefix !== 'PK_'){
					//get payment key from order key in payplans 1.4
					$orderId 	= XiHelperUtils::getIdFromKey($paymentKey);
					$order  	= PayplansOrder::getInstance($orderId);
					$paymentId 	= $order->getParam('payment_id');
					$paymentKey = XiHelperUtils::getKeyFromId($paymentId);
				}
				
				else {
					$paymentKey = JString::substr($paymentKey, 3);
				}
				
				JRequest::setVar('payment_key', $paymentKey, 'POST');
				return true;
			}

			//for recurring payments
			$paymentKey = JRequest::getVar('x_invoice_num', null);
			if($paymentKey){
				$prefix = JString::substr($paymentKey, 0,3);
				if($prefix === 'PK_'){
					$paymentKey = JString::substr($paymentKey, 3);
				}
				JRequest::setVar('payment_key', $paymentKey, 'POST');
				return true;
			}
		}
		
		return true;
	}
	
    function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  = $invoice->getTotal();

		$this->assign('post_url', XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&payment_key=".$payment->getKey()));
		$this->assign('payment', $payment);
		$this->assign('invoice', $invoice);
		$this->assign('amount', $amount);

	  	return $this->_render('form');
	}
	
	function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{		
		if($action == 'cancel'){
			return true;
		}
		
		require_once 'AuthorizeNet.php'; 
		(!defined("AUTHORIZENET_API_LOGIN_ID")) 	? define("AUTHORIZENET_API_LOGIN_ID", 	$this->getAppParam('api_login_id', '')) : '';    // Add your API LOGIN ID
		(!defined("AUTHORIZENET_TRANSACTION_KEY")) 	? define("AUTHORIZENET_TRANSACTION_KEY", 	$this->getAppParam('transaction_key', '')): ''; // Add your API transaction key
		(!defined("AUTHORIZENET_SANDBOX")) 			? define("AUTHORIZENET_SANDBOX", 			$this->getAppParam('sandbox', 0)): '';       // Set to false to test against production
		(!defined("TEST_REQUEST")) 					? define("TEST_REQUEST", "FALSE"): '';           // You may want to set to true if testing against production
	
		$errors   = array();
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		if($invoice->isRecurring()){
			$this->_processRecurringRequest($payment, $data);
		}
		else {
			$this->_processNonRecurringRequest($payment, $data);
		}

		$payment->save();
		
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}

	protected function _getRecurrenceCount($invoice)
	{
		$count = $invoice->getRecurrenceCount();
		if(intval($count) === 0){
			// To submit a subscription with no end date (an ongoing subscription), 
			// this field must be submitted with a value of “9999.”			
			return 9999;   
		}
		
		return $count;
	} 

	protected function _processRecurringRequest($payment, $data)
	{
	    // Set up the subscription. Use the developer account for testing..
	    $subscription = new AuthorizeNet_Subscription();
	    $invoice 	  = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
	
	    // XITODO : need to work on authorize.net
		$durationInDays 	= $invoice->getExpiration(PAYPLANS_RECURRING);
		
        $time 				= $this->getRecurrenceTime($invoice->getExpiration());
		$interval 			= $time['period'];
		$unit				= $time['unit'];   // days or months
		$totalOccurrences 	= $this->_getRecurrenceCount($invoice);
		
		//IMP set proper amount
		
		$recurring = $invoice->isRecurring();
		if($recurring == PAYPLANS_RECURRING){
			$amount = $invoice->getTotal();
		}
		
		//for trial
		// XITODO : authorize support multiple trial but with same time and amount
		// handle that as well 
		if($recurring == PAYPLANS_RECURRING_TRIAL_1){
			$amount = $invoice->getTotal($invoice->getCounter() + 1);
			$subscription->trialOccurrences = 1;
        	$subscription->trialAmount = $invoice->getTotal();
		}
		
		//XITODO: free trial case needs to be handled

	    // Set subscription information
	    $subscription->name = $invoice->getTitle();
	    $subscription->amount =  $amount;
	    $subscription->creditCardCardNumber = trim( $data['x_card_num'] );
	    $subscription->creditCardExpirationDate = trim( $data['exp_year'].'-'.str_pad($data['exp_month'], 2, '0', STR_PAD_LEFT));
	    $subscription->creditCardCardCode = trim($data['x_card_code']);
	    $subscription->billToFirstName = trim( $data['x_first_name'] );
	    $subscription->billToLastName = trim( $data['x_last_name'] );
	    $subscription->billToAddress = $data['x_address'];
	    $subscription->billToCity = $data['x_city'];
	    $subscription->billToState = $data['x_state'];
	    $subscription->billToCountry = $data['x_country'];
	    $subscription->billToZip = $data['x_zip'];
	    $subscription->customerEmail = $data['x_email'];
	    $subscription->customerPhoneNumber = $data['x_phone'];
	    $subscription->orderDescription = $invoice->getTitle();
	 
	    // Set the billing cycle for every three months
	    $subscription->intervalLength = $interval;
	    $subscription->intervalUnit = $unit;
	    
	    // start subscription from now
	    $startDate = new XiDate();
	    $subscription->startDate = $startDate->toFormat('%Y-%m-%d');
	    $subscription->totalOccurrences = $totalOccurrences;
	 
		// payment key is invoice key here
		$subscription->orderInvoiceNumber = 'PK_'.$payment->getKey();
		
	    $arbInstance = new AuthorizeNetARB(AUTHORIZENET_API_LOGIN_ID, AUTHORIZENET_TRANSACTION_KEY);
	    $arbInstance->setSandbox(false);
	    if($this->getAppParam('sandbox', 0)){
	    	$arbInstance->setSandbox(true);
	    }
	    
	    $arbInstance->setRefId($payment->getKey());
	    // Create the subscription
	    $response = $arbInstance->createSubscription($subscription);
			
		if (!$response)
		{
			return array("Transaction Failed");
		}
		
		$transactionArray['refId'] 			= $response->getRefID();
		$transactionArray['result_code']	= $response->getResultCode();
		$transactionArray['response_code']	= $response->getMessageCode();
		$transactionArray['text']   		= $response->getMessageText();
	  	
		//create transaction when recurring profile created
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $payment->getBuyer())
					->set('amount', 0)
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_subscr_id', $response->getSubscriptionId());
					
		if($transactionArray['result_code'] == 'Error'){
			$errors['response_reason_code'] = $transactionArray['result_code'];
			$errors['response_code']		= $transactionArray['response_code'];
			$errors['response_reason_text'] = $transactionArray['text'];
			
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_IN_AUTHORIZE_PAYMENT_PROCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $transactionArray, 'PayplansPaymentFormatter', '', true);
			$getApp = XiFactory::getApplication();
			$getApp->enqueueMessage(XiText::_($errors['response_reason_text'].' ('.$errors['response_code'].')'),'error');
		  	$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
		}	  		
		
		$transaction->set('message', 'COM_PAYPLANS_APP_AUTHORIZE_TRANSACTION_RECURRING_PROFILE_CREATED')
					->save();
		
		$message 		= XiText::_('COM_PAYPLANS_LOGGER_AUTHORIZE_RECURRING_PROFILE_CREATED');
		$infomessage 	= XiText::_('COM_PAYPLANS_LOGGER_AUTHORIZE_RECURRING_PROFILE_CREATED_INFO_MESSAGE');
		PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment, array($infomessage), 'PayplansPaymentFormatter');
		
	}
	
	protected function _processNonRecurringRequest(PayplansPayment $payment, $data)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		$transactionData = array(
						        'amount' 		=> number_format($invoice->getTotal(), 2), 
						        'card_num' 		=> trim($data['x_card_num']), 
						        'exp_date' 		=> trim( $data['exp_year'].'-'.str_pad($data['exp_month'], 2, '0', STR_PAD_LEFT)),
						        'first_name' 	=> $data['x_first_name'],
						        'last_name' 	=> $data['x_last_name'],
						        'address' 		=> $data['x_address'],
						        'city' 			=> $data['x_city'],
						        'state' 		=> $data['x_state'],
						        'country' 		=> $data['x_country'],
						        'zip' 			=> $data['x_zip'],
						        'email' 		=> $data['x_email'],
						        'card_code' 	=> $data['x_card_code'],
							    'phone'			=> $data['x_phone'],
								'invoice_num' 	=> 'PK_'.$payment->getKey(),
								'description' 	=> $invoice->getTitle()
	        );

		$transaction = new AuthorizeNetAIM();
	    $transaction->setSandbox($this->getAppParam('sandbox', 0));
	    $transaction->setFields($transactionData);
	    
	    $response 			= $transaction->authorizeAndCapture(); 
	    $transactionArray 	= $response->toArray();
	    // to identify it is testing mode or not
	    $transactionArray['testmode'] = $this->getAppParam('sandbox', 0);
		
	    
		// if same notification came more than one time
		// Check if transaction already exists,if yes then do nothing and return
		$txn_id 		= isset($transactionArray['transaction_id']) 	? $transactionArray['transaction_id'] : 0; 
		$subscr_id		= 0;
		$parent_id		= 0;
		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_id);
		if (!empty($transactions)) {
			foreach ($transactions as $transaction) {
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
				if($transaction->getParam('response_code','') == $transactionArray['response_code']){
					return true;
				}
    		}
		 }
		 
		$txn = PayplansTransaction::getInstance();
		$txn->set('user_id', 			$payment->getBuyer())
			->set('amount', 			0)
			->set('invoice_id', 		$invoice->getId())
			->set('payment_id', 		$payment->getId())
			->set('gateway_txn_id', 	$txn_id)
			->set('gateway_subscr_id', 	$subscr_id)
			->set('gateway_parent_txn', $parent_id)
			->set('params', 			PayplansHelperParam::arrayToIni($transactionArray));
					

	    if($response->approved){
	    	$txn->set('amount', $transactionArray['amount'])
	    		->set('message', 'COM_PAYPLANS_APP_AUTHORIZE_TRANSACTION_COMPLETED');
		}		
		else{
			$errors['response_reason_code'] = $response->response_reason_code;
			$errors['response_code']		= $response->response_code;
			$errors['response_reason_text'] = $response->response_reason_text;
			
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_IN_AUTHORIZE_PAYMENT_PROCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $errors, 'PayplansPaymentFormatter','', true);
			$getApp = XiFactory::getApplication();
			$getApp->enqueueMessage(XiText::_($errors['response_reason_text'].' ('.$errors['response_code'].')'),'error');
		  	$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
		}
		
		$txn->save();
	}
	
	function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		// if its a recurring subscrition
	    if(isset($data['x_subscription_id']) && $data['x_subscription_id'] ){
	    	
	    // if same notification came more than one time
		// Check if transaction already exists,if yes then do nothing and return
		$txn_id 		= isset($data['x_trans_id']) ? $data['x_trans_id'] : 0; 
		$subscr_id		= isset($data['x_subscription_id']) ? $data['x_subscription_id'] : 0;
		$parent_id		= isset($data['parent_txn_id']) ? $data['parent_txn_id'] : 0;
		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_id);
		if (!empty($transactions)) {
			foreach ($transactions as $transaction) {
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			if($transaction->getParam('x_response_code','') == $data['x_response_code']){
					return true;
				}
    		}
		 }

	    	// get the transaction instace of lib
			$transaction = PayplansTransaction::getInstance();
			$transaction->set('user_id', 			$payment->getBuyer())
						->set('invoice_id', 		$invoice->getId())
						->set('payment_id', 		$payment->getId())
						->set('gateway_txn_id', 	$txn_id)
						->set('gateway_subscr_id', 	$subscr_id)
						->set('gateway_parent_txn', $parent_id)
						->set('params', 			PayplansHelperParam::arrayToIni($data));
	    	
			
			$errors = $this->_processNotification($transaction, $data, $payment);
			$transaction->save();
			return count($errors) ? implode("\n", $errors) : ' No Errors';
	    }
	    
	    // if its not a recurring payment
	    // XITODO : we need to do it
	    return ' No Errors ';
	}
	
    function _processNotification(PayplansTransaction &$transaction, array $data, PayplansPayment $payment)
    {
    	$errors = array();

    	// Get the response code. 1 is success, 2 is decline, 3 is error
	    $response_code = (int) $_POST['x_response_code'];
	 
	    // Get the reason code. 8 is expired card.
	    $reason_code = (int) $_POST['x_response_reason_code'];
	 
	    if ($response_code == 1){
	    	//Approved
	    	$transaction->set('message', 'COM_PAYPLANS_APP_AUTHORIZE_TRANSACTION_COMPLETED');
	    }
	    else if ($response_code == 2){
	    	// Declined
	    	$transaction->set('message', 'COM_PAYPLANS_APP_AUTHORIZE_TRANSACTION_DECLINED');
	    }
	    else if ($response_code == 3 && $reason_code == 8){
	        // An expired card
	        $transaction->set('message', 'COM_PAYPLANS_APP_AUTHORIZE_TRANSACTION_EXPIRED_CARD');	 
	    }
	    else 
	    {
	        // Other error
	        $errors[] = XiText::_('COM_PAYPLANS_INVALID_AUTHORIZE_PAYMENT_STATUS');
	        $transaction->set('message', 'COM_PAYPLANS_APP_AUTHORIZE_TRANSACTION_ERROR');
	    }

        // update the payment amount
        $transaction->set('amount', $data['x_amount']);

		//if error present in the transaction then redirect to error page
		if(!empty($errors)){
			$message 	= XiText::_('COM_PAYPLANS_LOGGER_ERROR_IN_AUTHORIZE_PAYMENT_PROCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $errors, 'PayplansPaymentFormatter','', true);

		}
	
        return $errors;
    }
    
    public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{
		require_once 'AuthorizeNet.php'; 
		(!defined("AUTHORIZENET_API_LOGIN_ID")) 	? define("AUTHORIZENET_API_LOGIN_ID", 	$this->getAppParam('api_login_id', '')) : '';    // Add your API LOGIN ID
		(!defined("AUTHORIZENET_TRANSACTION_KEY")) 	? define("AUTHORIZENET_TRANSACTION_KEY", 	$this->getAppParam('transaction_key', '')): ''; // Add your API transaction key
		(!defined("AUTHORIZENET_SANDBOX")) 			? define("AUTHORIZENET_SANDBOX", 			$this->getAppParam('sandbox', 0)): '';       // Set to false to test against production
		(!defined("TEST_REQUEST")) 					? define("TEST_REQUEST", "FALSE"): '';           // You may want to set to true if testing against production 
		 
	 	$arbInstance = new AuthorizeNetARB(AUTHORIZENET_API_LOGIN_ID, AUTHORIZENET_TRANSACTION_KEY);
		
	 	$arbInstance->setSandbox(false);
	    if($this->getAppParam('sandbox', 0)){
	    	$arbInstance->setSandbox(true);
	    }
	    
	    $transactions = $payment->getTransactions();
	    foreach($transactions as $transaction){
	    	$subscriptionId = $transaction->get('gateway_subscr_id', 0);
	    	if(!empty($subscriptionId)){
	    		break;
	    	}
	    }
	    
	    
	   	$arbInstance->setRefId($payment->getKey());
	   	$response = $arbInstance->cancelSubscription($subscriptionId);
	 
	   	if (!$response)
		{
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_LOGGER_ERROR_IN_RECURRING_RESPONSE');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array(), 'PayplansPaymentFormatter', '', true);
			$errors['response_reason_text'] = $message;
			$this->assign('errors', $errors);
			return $this->_render('cancel_error');
		}
		
		$transactionArray['refId'] 					= $response->getRefID();
		$transactionArray['result_code']			= $response->getResultCode();
		$transactionArray['response_code']			= $response->getMessageCode();
		$transactionArray['text']  					= $response->getMessageText();
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$txn = PayplansTransaction::getInstance();
		$txn->set('user_id', 			$payment->getBuyer())
			->set('invoice_id', 		$invoice->getId())
			->set('payment_id', 		$payment->getId())
			->set('gateway_txn_id', 	isset($data['x_trans_id']) ? $data['x_trans_id'] : 0)
			->set('gateway_subscr_id', 	$subscriptionId)
			->set('gateway_parent_txn', isset($data['parent_txn_id']) ? $data['parent_txn_id'] : 0);

	  	$errors = array();	
		if($transactionArray['result_code'] == 'Error'){
			$errors['response_reason_code'] = $transactionArray['result_code'];
			$errors['response_code']		= $transactionArray['response_code'];
			$errors['response_reason_text'] = $transactionArray['text'];
		}	
		  		
		if(count($errors)){
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_LOGGER_ERROR_IN_RECURRING_CANCEL_PROCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $errors, 'PayplansPaymentFormatter','', true);
			$this->assign('errors', $errors);
			return $this->_render('cancel_error');
		}
	
		$txn->set('message', 'COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_TRANSACTION_FOR_CANCEL_ORDER')
			->save();
			
		$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_LOGGER_SUCCESS_IN_RECURRING_CANCEL_PROCESS');
		PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment, array($message), 'PayplansPaymentFormatter');
		parent::onPayplansPaymentTerminate($payment, $controller);
		return $this->_render('cancel_success');
	}
	
	public function getRecurrenceTime($expTime)
	{
		$expTime['year'] = isset($expTime['year']) ? intval($expTime['year']) : 0;
		$expTime['month'] = isset($expTime['month']) ? intval($expTime['month']) : 0;
		$expTime['day'] = isset($expTime['day']) ? intval($expTime['day']) : 0;;
		
		// years
		if(!empty($expTime['year'])){		
			return array('period' => 12, 'unit' => 'months', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'));
		}
		
		// if months are set
		if(!empty($expTime['month'])){
			// if days are not empty
			if(!empty($expTime['day']) && ( ($expTime['month'] * 30 + $expTime['day']) <= 365) ){
				return array('period' => $expTime['month'] * 30 + $expTime['day'], 'unit' => 'days', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
										'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_RECURRING_MESSAGE'));
			}
			
			return array('period' => $expTime['month'], 'unit' => 'months', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
										'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_RECURRING_MESSAGE'));
		}
		
		// if only days are set then return days as it is
		if(!empty($expTime['day'])){
			return array('period' => intval($expTime['day'], 10), 'unit' => 'days', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
										'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_RECURRING_MESSAGE'));
		}
		
		// XITODO : what to do if not able to convert it
		return false;
	}
}



