<?php

/**
 * @copyright	Copyright (C) 2009 - 2016 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
 * @license		GNU/GPL, see LICENSE.php
 * @package		Payplans
 * @subpackage		Basic Tax
 * @contact		support+payplans@readybytes.in
 */
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Payplans Discount Plugin
 *
 * @author shyam
 */
class plgPayplansBasictax extends XiPlugin
{
	public function onPayplansSystemStart()
	{
		$dir = dirname(__FILE__).DS.'basictax'.DS.'app';
		PayplansHelperApp::addAppsPath($dir);
		
		return true;
	}
	
	
	// on render of order, display output
	public function onPayplansViewBeforeRender(XiView $view, $task)
	{
		//if no app exist then don't do anything
		$app_purpose   = PayplansHelperApp::getAvailableApps('basictax');
		if(!$app_purpose){
			return array();
		}

		// display plan price included with tax on subscribe page
		if(($view instanceof PayplanssiteViewPlan) && $task == 'subscribe')
		{
			$userId	  =  XiFactory::getUser()->id;
			$user 	  =  PayplansUser::getInstance($userId);
			$country  =  $user->getCountry();
			$plans 	  = $view->get('plans');
			foreach ($plans as $id => $plan) {
				//is tax should is display on subscribe page
				$tax = $this->_isTaxAppliedOnPlan($id,0);
				if(!empty($tax)){
					$plan       = PayplansPlan::getInstance($id,null,$plan);
					$price      = $plan->getPrice();
					$taxAmount  = ($price * $tax)/100;
					$price      = $price + $taxAmount;
					$plan->getDetails()->set('price', $price);

				}
			}
			$session = JFactory::getSession();
			$session->clear('country_id');

		}

		// if user is not logged in then display modified price on login page also
		if(($view instanceof PayplanssiteViewPlan) && $task == 'login')
		{
			$plan      = $view->get('plan');
			$tax       = $this->_isTaxAppliedOnPlan($plan->getId(),0);
			$price     = $plan->getPrice();
			$taxAmount = ($price * $tax)/100;
			$price     = $price + $taxAmount;
			$plan->getDetails()->set('price', $price);
		}

		if(($view instanceof PayplanssiteViewInvoice && $task == 'confirm') 
				|| ($view instanceof PayplansadminViewInvoice && $task == 'edit'))
		{

			$country 	= 0;
			$invoiceId  = $view->getModel()->getId();
			$invoice 	= PayplansInvoice::getInstance($invoiceId);
			$invoiceKey = $invoice->getKey();
			$userId 	= $invoice->getBuyer();

			//only apply the tax when invoice is in none or confirmed status. 
			if(!in_array($invoice->getStatus(), array(PayplansStatus::NONE, PayplansStatus::INVOICE_CONFIRMED))){
				return true;
			}
				
			$appsInstance = PayplansHelperApp::getApplicableApps('basictax', $invoice);
			if(empty($appsInstance)){
				return array();
			}
			if(!empty($userId)){
				$user 	  =  PayplansUser::getInstance($userId);
				$country  =  $user->getCountry();
				self::_onPayplansTaxRequest($invoiceKey, $country);
			}

			$this->_assign('country', $country);
			$this->_assign('invoiceId', $invoiceId);
			$this->_assign('invoice_key', $invoiceKey);
			if($view instanceof PayplanssiteViewInvoice){
				return array('payplans_order_confirm_payment' => $this->_render('invoiceconfirm'));
					
			}
			if($view instanceof PayplansadminViewInvoice){
				return array('payplans_invoice_edit_modifier' => $this->_render('invoiceedit'));
			}
		}
		return true;
	}

	//apply tax on the order supplied according to mentioned country
	//returns updated order and error
	public function _onPayplansTaxRequest($invoiceKey, $country)
	{
		//trigger the discount
		$invoiceId = XiHelperUtils::getIdFromKey($invoiceKey);
		$invoice   = PayplansInvoice::getInstance($invoiceId);
		$args      = array($invoice, $country, 0, false);
		$results = PayplansHelperEvent::trigger('onPayplansApplyTax', $args, '', $invoice);
		$error = '';
		$tax = array();
		foreach($results as $result=>$result_val){
			if(!is_null($result_val))
			{
				$tax[]= $result_val;
			}
		}

		foreach ($results as $result)
		{
			$modifierRecords = XiFactory::getInstance('modifier', 'model')
			->loadRecords(array('invoice_id' => $invoiceId,
							    'type' => 'basictax'));
			if(!empty($modifierRecords)){
				foreach($modifierRecords as $modifier ) {
					$id[] = $modifier->modifier_id;
				}
				$modifier_id =implode(',', $id);
				$db = JFactory::getDbo();
				$query = $db->getQuery(true);
				$query->delete($db->quoteName('#__payplans_modifier'));
				$query->where($db->quoteName('modifier_id').'IN('.$modifier_id.')');
				$db->setQuery($query);
				$result = $db->execute();
			}
			foreach($tax as $tax_key=>$val) {
				$modifierParams = new stdClass();
				$modifierParams->type 		= 'basictax';
				$modifierParams->percentage	= true;
				$modifierParams->serial 	=  PayplansModifier::PERCENT_TAX;
				$modifierParams->amount 	= $val['0'];
				$modifierParams->message 	= $val['1'];
				$modifierParams->reference	= $country;
				$modifierParams->frequency  = PayplansModifier::FREQUENCY_EACH_TIME;
				$invoice->addModifier($modifierParams);
			}
			$user = $invoice->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
			$invoice->refresh()->save();
			$user->setCountry($country)
			->save();
		}
		return array($invoice, $error);
	}


	// on render of order, display output
	function onPayplansTaxRequest($invoiceKey, $country)
	{
		list($invoice, $error) = self::_onPayplansTaxRequest($invoiceKey, $country);
		// order have been updated
		$response = XiFactory::getAjaxResponse();
		$response->addScriptCall('payplans.apps.basictax.displayError',$error);
		$modifiers= $invoice->getModifiers();
		$modifiers  = PayplansHelperModifier::_rearrange($modifiers);
		if(!empty($modifiers)){
			$modified_html = $this->_loadTemplate('default_partial_invoice_modifier', compact('invoice'));
			$response->addScriptCall('xi.jQuery(\'#payplans\').find(\'.pp-modifiers\').remove');
			$response->addScriptCall('xi.jQuery(\'#payplans\').find(\'.pp-modifier-amount\').after', $modified_html);
		}
		else {
			$response->addScriptCall('xi.jQuery(\'#payplans\').find(\'.pp-modifiers\').remove');
		}

		if(XiFactory::getApplication()->isAdmin()){
			$currency = $invoice->getCurrency();
			$modifiers = $invoice->getModifiers();
			$response->addScriptCall('payplans.jQuery(\'div[name="discount"]\').html', self::getFormattedAmount($currency, $invoice->getDiscount()));
			$response->addScriptCall('payplans.jQuery(\'div[name="total"]\').html', self::getFormattedAmount($currency, $invoice->getTotal()));
			$response->addScriptCall('payplans.jQuery(\'div[name="ppmodifiers"]\').html',PayplansHelperTemplate::partial('default_partial_modifier_table', compact('invoice', 'modifiers')));
			$response->sendResponse();
		}

		$response->addScriptCall('payplans.apps.basictax.validateCountry');
		$response->addScriptCall('xi.jQuery(\'#payplans\').find(\'.regular-amount .pp-amount\').html',PayplansHelperFormat::displayAmount($invoice->getRegularAmount()));
		$response->addScriptCall('xi.jQuery(\'#payplans\').find(\'.first-amount .pp-amount\').html',PayplansHelperFormat::displayAmount($invoice->getTotal()));
		$response->sendResponse();
	}

	public static function getFormattedAmount($currency, $amount)
	{
		return PayplansHelperTemplate::partial('default_partial_amount', compact('currency', 'amount'));
	}

	protected function _isTaxAppliedOnPlan($planId,$country)
	{
		$taxApps = PayplansHelperApp::getAvailableApps('basictax');
		$tax     = 0;
		 
		foreach ($taxApps as $app)
		{
			$applyToCountry = $app->getAppParam('tax_country', array());
			$applyToCountry = (is_array($applyToCountry)) ? $applyToCountry : array($applyToCountry);

			// check whether app is applied on plans or on that plan
			if($app->getParam('applyAll') || in_array($planId, $app->getPlans()))
			{
				//if applicable to all country or no country then donot check for country
				if(!in_array(PAYPLANS_CONST_ALL, $applyToCountry) && !empty($country)){
					//When only one item is selected
					if(is_array($applyToCountry )==false){
						$applyToCountry = array($applyToCountry );
					}
				}
				if($app->getAppParam('displayTaxIncludedPrice',0)){
					$session =JFactory::getSession();
					$country_id = $session->get( 'country_id', '0' );
					if(!$country_id) {
						$ip = $this->getUserIp();
						if ($ip == '127.0.0.1')
						{
							return $tax;
						}
						$url = "http://freegeoip.net/json/" . $ip;
						$ch = curl_init();
						$timeout = 20;
						
						curl_setopt($ch, CURLOPT_URL, $url);
						curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
						curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);

						$result		= curl_exec($ch);
						$curl_error 	= curl_errno($ch);
						curl_close($ch);
						$details 	= json_decode($result);
						$items 		= PayplansFactory::getInstance('country', 'model')->loadRecords(array('isocode2' => $details->country_code));
						$country 	= array_shift($items);
						$country_id 	= $country->country_id;
						$session->set('country_id', $country_id);
						if ($curl_error != 0) {
							return tax;
						}
					}
					if(in_array($country_id,$applyToCountry) ==true || in_array(-1,$applyToCountry) ==true){
						$tax= $tax + floatval($app->getAppParam('tax_rate', 0));
					}
				}
			}
		}
		return $tax;
	}


	public function getUserIp()
	{
		$server = JFactory::getApplication()->input->server;
		if (!empty($server) ) {
			// get IP from HTTP Proxy or load-balancer
			$xfrwd = $server->get('HTTP_X_FORWARDED_FOR');
			$clientIp = $server->get('HTTP_CLIENT_IP');
			if (isset($xfrwd)) {
				$ip = $xfrwd;
			} elseif (isset($clientIp)) {
				$ip = $clientIp;
			} else {
                $ip = $server->get('REMOTE_ADDR');
            }
        } 
        return $ip; 
}

}
