<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	BitPay
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

/**
 * Payplans Bitpay
 * Garima Agal
 */

class PayplansAppBitpay extends PayplansAppPayment
{
	protected $_location	= __FILE__;

   public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
   {
		$invoice   = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);		 
		$recurring = $invoice->isRecurring();
		
	   if($recurring){
	    		$getApp = PayplansFactory::getApplication();
	    		$juri   = new JURI();
	    		$getApp->enqueueMessage(XiText::_("COM_PAYPLANS_PAYMENT_APP_BITPAY_DOES_NOT_SUPPORT_RECURRING"),'error');
    	}
		
		$merchantId 	= trim($this->getAppParam('api_key',false));
		$options 		=  array();
        
		$options['price']  			= $invoice->getTotal();
		$options['currency'] 		= $invoice->getCurrency('isocode');
		$invoiceKey = $invoice->getKey();
		$user = $invoice->getBuyer(true);
		$root = JURI::root();
		$root = JString::str_ireplace("http:", "https:", $root);
		
		$options['notificationURL'] = $root.'index.php?option=com_payplans&view=payment&task=notify&gateway=bitpay&payment_key='.$payment->getKey();
		
		$options['redirectURL']   =   $root.'index.php?option=com_payplans&gateway=bitpay&view=payment&task=complete&action=success&payment_key='.$payment->getKey();
		
		$options['transactionSpeed'] ='high';
		
		$options['fullNotifications'] = true;
		$options['notificationEmail'] = $user->getEmail();
		
		$options['orderID']          = $payment->getKey();
		
		$postData = array('invoice_key' => $invoiceKey, 'hash' => crypt($invoiceKey,$merchantId) );
		
		$options['posData']  		= json_encode($postData);
		
		$options['physical'] = false;
		
		$post = json_encode($options);
		
		$this->createInvoice($post,$payment,$invoice);
		
  	}

  	
  	
  	public function getBitPayUrl(){
  		$sandbox = $this->getAppParam('sandbox',false);
  		$url = 'https://bitpay.com/api/invoice/';
  		if($sandbox){
  			$url = 'https://test.bitpay.com/api/invoice/';
  		}
  		
  		return $url;
  	}

  	
  	public function createInvoice($post, $payment, $invoice)
  	{
  		$url 	  = $this->getBitPayUrl();
  		$response = $this->sendCurlRequest($url, $post);
		if (isset($response['url']))
		{
			$transaction = PayplansTransaction::getInstance();
			$transaction->set('user_id', $payment->getBuyer())
						->set('invoice_id', $invoice->getId())
						->set('payment_id', $payment->getId())
						->set('params', PayplansHelperParam::arrayToIni($response))
						->set('amount', 0)
						->set('message', XiText::_("COM_PAYPLANS_APP_BITPAY_REDIRECTED_TO_BITPAY_SUCCESSFULLY"))
						->save();
				XiFactory::getApplication()->redirect($response['url']);
				return true;
		}
		else{
    		$message = XiText::_('COM_PAYPLANS_LOGGER_PAYMENT_INVALID_DATA');
    		PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
    		XiFactory::getApplication()->enqueueMessage($response['error']['type'].":-".$response['error']['message'],'error');
    		return false;
		}
  	}
  	
  	
  	public function sendCurlRequest($url, $post = null){
  		$curl = curl_init($url);
		$length = 0;
		if ($post)
		{	
			curl_setopt($curl, CURLOPT_POST, 1);
			curl_setopt($curl, CURLOPT_POSTFIELDS, $post);
			$length = strlen($post);
		}
		
		$username = base64_encode(trim($this->getAppParam('api_key')));
		$header = array(
		'Content-Type: application/json',
		"Content-Length: $length",
		"Authorization: Basic $username",
		);
		
		curl_setopt($curl, CURLOPT_PORT, 443);
		curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
		curl_setopt($curl, CURLOPT_TIMEOUT, 10);
		curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC ) ;
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 1); // verify certificate
		curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2); // check existence of CN and verify that it matches hostname
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_FORBID_REUSE, 1);
		curl_setopt($curl, CURLOPT_FRESH_CONNECT, 1);
		
		$responseString = curl_exec($curl);
		$response = json_decode($responseString, true);
		if($responseString == false || isset($responseString['error'])) {
			$response = curl_error($curl);
		} else {
			
			if(JDEBUG){
			// When debug mode is on dump in file
			file_put_contents(JPATH_SITE.DS.'tmp'.DS.time(), var_export($response,true), FILE_APPEND);
		}
		
		}
		curl_close($curl);
		
		return $response;
  	}
  	
	function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		$invoice 		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		// get the transaction instace of lib
		$dataDirect = file_get_contents("php://input");
	
		if(JDEBUG){
			// When debug mode is on dump in file
			file_put_contents(JPATH_SITE.DS.'tmp'.DS.time().'_bitpay', var_export($dataDirect,true), FILE_APPEND);
		}
		
		$postArray = $this->validateNotification($dataDirect);
		if(!$postArray){
			$errors[] = XiText::_('COM_PAYPLANS_APP_BITPAY_AUTHENTICATION_FAILED');
			return $errors;
		}
		
		
		$url = $this->getBitPayUrl();
		
		$url = $url.$postArray['id'];
		
		$response = $this->sendCurlRequest($url);
		
		// if same notification came more than one time
		// Check if transaction already exists,if yes then do nothing and return
	    $txn_id         = isset($response['id']) ? $response['id'] : 0;
		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, 0, 0);
    	if (!empty($transactions)) {
    		foreach ($transactions as $transaction) {
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			if(strtolower($transaction->getParam('status','')) == strtolower($response['status'])){
    					return true;
				}
    		}
    	}
		
		//XITODO:: gateway_subscr_id chaged with gateway_txn_id
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $payment->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', isset($response['id']) ? $response['id'] : 0)
					->set('gateway_parent_txn', 0)
					->set('params', PayplansHelperParam::arrayToIni($response));
					
					
		if($response['currentTime'] > $response['expirationTime'])
		{
			$error = XiText::_('COM_PAYPLANS_APP_BITPAY_EXPIRATION_TIME_EXCEED');
    		PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $error, $payment, $response, 'PayplansPaymentFormatter', '', true);
    		$transaction->set('message', $error)->save();
    		return array($error);
		}

		if(isset($response['status'])){
			$func_name = '_'.JString::strtolower($response['status']);
		}

		//XITODO : Log error
		if(method_exists($this, $func_name)){
			$errors = $this->$func_name($payment, $response, $transaction);
		}else{
			$errors = array(XiText::_('COM_PAYPLANS_APP_BITPAY_INVALID_MESSAGE_TYPE'));
		}

		$transaction->save();
		return count($errors) ? implode("\n", $errors) : ' No Errors';
	}
   
	public function validateNotification($data)
	{
	      $apiKey = trim($this->getAppParam('api_key',''));		
	      $json = json_decode($data, true);
		
		    if (is_string($json) || (is_array($json) && array_key_exists('error', $json)))
		      return false; 
	
		    $posData = json_decode($json['posData'], true);
		   
		    if($posData['hash'] != crypt($posData['invoice_key'],$apiKey)){
		      return false;
		    }
		    $json['posData'] = $posData;
		
		    return $json;
		 
	}
	
	protected function _new($payment, $data, $transaction)
		{
			$transaction->set('message', 'COM_PAYPLANS_APP_BITPAY_NEW');
			return array();
		}
		
	protected function _paid($payment, $data, $transaction)
		{
			$transaction->set('message', 'COM_PAYPLANS_APP_BITPAY_PAID')
						->set('amount', $data['price']);
			return array();
		}
		
	protected function _confirmed($payment, $data, $transaction)
		{
			$transaction->set('message', 'COM_PAYPLANS_APP_BITPAY_CONFIRMED')
						->set('amount', $data['price']);
			return array();
		}
		
	protected function _complete($payment, $data, $transaction)
		{
			$transaction->set('message', 'COM_PAYPLANS_APP_BITPAY_COMPLETE');
			return array();
		}
		
	protected function _expired($payment, $data, $transaction)
		{
			$transaction->set('message', 'COM_PAYPLANS_APP_BITPAY_EXPIRED');
			return array();
		}
		
	protected function _incomplete($payment, $data, $transaction)
		{
			$transaction->set('message', 'COM_PAYPLANS_APP_BITPAY_INCOMPLETE');
			return array();
		}
}
