<?php
/**
* @copyright	Copyright (C) 2009 - 2016 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Braintree
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppBraintree extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	protected function _loadConfig()
	{
		$env 	 = $this->getAppParam('sandbox',false) ? 'sandbox' : 'production';
		require_once __DIR__.'/lib/Braintree.php';			
		Braintree_Configuration::environment($env);
		Braintree_Configuration::merchantId($this->getAppParam('merchant_id', ''));
		Braintree_Configuration::publicKey($this->getAppParam('public_key', ''));
		Braintree_Configuration::privateKey($this->getAppParam('private_key', ''));
	}

	function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		if($view != 'payment' || ($task != 'notify') ){
			return true;
		}
		$input	= JFactory::getApplication()->input;
		$gateway = $input->get('gateway', null);
		if($gateway != 'braintree'){
			return true;
		}

		$bt_challenge = $input->get('bt_challenge', null);
		if(!empty($bt_challenge)){
			$this->_loadConfig();
			echo Braintree_WebhookNotification::verify($bt_challenge);
			exit();
		}

		$bt_signature 	= $input->get('bt_signature', null);
		$bt_payload 	= $input->get('bt_payload', null);
		if(!empty($bt_signature) && !empty($bt_payload)){
			$this->_loadConfig();
			$webhookNotification = Braintree_WebhookNotification::parse($bt_signature, $bt_payload);
			if(!empty($webhookNotification->subscription->id)){
				$transactions = PayplansFactory::getInstance('transaction', 'model')
											->loadRecords(array('gateway_subscr_id' => $webhookNotification->subscription->id));

				if(!empty($transactions)){
					$transaction = array_shift($transactions);
					$paymentKey = XiHelperUtils::getKeyFromId($transaction->payment_id);					
					if(!empty($paymentKey)){
						$input->set('payment_key', $paymentKey, 'POST');
						$this->webhookNotification = $webhookNotification;
						return true;
					}
				}
			}
		}

		return true;
	}


	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}	

	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		if(is_object($data)){
			$data = (array)$data;
		}

		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  = $invoice->getTotal();
		$paymentKey = $payment->getKey();
		
		$this->assign('amount', $amount);
		$this->assign('invoice', $invoice);
		$this->assign('post_url', XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&action=success&payment_key=".$paymentKey));
		$this->assign('cancel_url', XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&action=cancel&payment_key=".$paymentKey));
		$this->assign('payment_key', $paymentKey);

		$this->_loadConfig();

		$token   = Braintree_ClientToken::generate();
		$this->assign('token', $token);
        return $this->_render('form_buynow');
	}
	
	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{	
		$app = XiFactory::getApplication();
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$invoiceAmount = $invoice->getTotal();

		$this->_loadConfig();

		$isRecurring = $invoice->isRecurring();
		if($isRecurring){
			$this->_createCustomer($invoice, $payment, $data);
			$this->_createSubscription($invoice, $payment, $data);
			return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		}

		$result = Braintree_Transaction::sale([
		  'amount' => $invoiceAmount,
		  'paymentMethodNonce' => $data["payment_method_nonce"],
		  'options' => [
		    'submitForSettlement' => True
		  ]
		]);
		
		$app = XiFactory::getApplication();
		if(!($result instanceof Braintree_Result_Successful)){
			$message = XiText::_('COM_PAYPLANS_LOGGER_BRAINTREE_ERROR_IN_TRANSACTION');
			$log_id = PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $result->__toString(), 'PayplansPaymentFormatter', '', true);

			$app->enqueueMessage($result->message, 'error');
			$app->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));	
			return true;
		}
		
		$amount = $result->transaction->amount;
		
		// do this for normal payment
		// save transaction for normal payment
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('amount', $amount)
					->set('gateway_txn_id', $result->transaction->id)
					->set('gateway_subscr_id', 0)
					->set('gateway_parent_txn', 0)
					->set('message', 'COM_PAYPLANS_PAYMENT_APP_BRAINTREE_PAYMENT_COMPLETED_SUCCESSFULLY')
					->set('params', PayplansHelperParam::arrayToIni((array)$result))
					->save();
		
		// save payment record
		$payment->save();			

		return parent::onPayplansPaymentAfter($payment, $action, $result->__toString(), $controller);
	}

	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		if(!isset($this->webhookNotification)){
			return true;
		}

		$invoice         = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);		
		$bt_transactions = $this->webhookNotification->subscription->transactions;
		$bt_transaction  = array_shift($bt_transactions);
		$amount          = $bt_transaction->amount;


		// if same notification came more than one time
    	// check if transaction already exists
    	// if yes then do nothing and return
		$txn_id     = $bt_transaction->id;
		$subscr_id  = $this->webhookNotification->subscription->id;
		$parent_txn = 0;

    	$transactions = $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_txn);
    	if($transactions !== false){
    		foreach($transactions as $transaction){
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
				if(!empty($transaction)){
    				return true;
    			}    
    		}
    	}
    	
    	// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $payment->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $txn_id)
					->set('gateway_subscr_id', $subscr_id)
					->set('gateway_parent_txn', $parent_txn)
					->set('params', PayplansHelperParam::arrayToIni((array)$this->webhookNotification));

		switch($this->webhookNotification->kind){
			case Braintree_WebhookNotification::SUBSCRIPTION_CANCELED :
					// 'subscription_canceled';
					$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_SUBSCR_CANCEL');
					//terminate the order
					$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
					$invoice->terminate();
					break;
		    case Braintree_WebhookNotification::SUBSCRIPTION_CHARGED_SUCCESSFULLY :
		    		// 'subscription_charged_successfully';
					$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_SUBSCRIPTION_CHARGED_SUCCESSFULLY');
					$transaction->set('amount', $amount);
		    		break;
		    case Braintree_WebhookNotification::SUBSCRIPTION_CHARGED_UNSUCCESSFULLY :
		    		// 'subscription_charged_unsuccessfully';		    	
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_SUBSCRIPTION_CHARGED_UNSUCCESSFULLY');
		    		break;
		    case Braintree_WebhookNotification::SUBSCRIPTION_EXPIRED :
		    		// 'subscription_expired';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_SUBSCR_EXPIRED');
		    		break;
		    case Braintree_WebhookNotification::SUBSCRIPTION_TRIAL_ENDED :
		    		// 'subscription_trial_ended';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_SUBSCR_TRIAL_ENDED');
		    		break;
		    case Braintree_WebhookNotification::SUBSCRIPTION_WENT_ACTIVE :
		    		// 'subscription_went_active';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_SUBSCR_WENT_ACTIVE');
		    		$transaction->set('amount', $amount);
		    		break;
		    case Braintree_WebhookNotification::SUBSCRIPTION_WENT_PAST_DUE :
		    		// 'subscription_went_past_due';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_SUBSCR_WENT_PAST_DUE');
		    		break;
		    case Braintree_WebhookNotification::SUB_MERCHANT_ACCOUNT_APPROVED :
		    		// 'sub_merchant_account_approved';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_SUBSCR_MERCHANT_ACCOUNT_APPROVED');
		    		break;
		    case Braintree_WebhookNotification::SUB_MERCHANT_ACCOUNT_DECLINED :
		    		// 'sub_merchant_account_declined';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_SUBSCR_MERCHANT_ACCOUNT_DECLINED');
		    		break;
		    case Braintree_WebhookNotification::TRANSACTION_DISBURSED :
		    		// 'transaction_disbursed';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_TRANSACTION_DISBURSED');
		    		break;
		    case Braintree_WebhookNotification::DISBURSEMENT_EXCEPTION :
		    		// 'disbursement_exception';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_DISBURSEMENT_EXCEPTION');
		    		break;
		    case Braintree_WebhookNotification::DISBURSEMENT :
		    		// 'disbursement';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_DISBURSEMENT');
		    		break;
		    case Braintree_WebhookNotification::DISPUTE_OPENED :
		    		// 'dispute_opened';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_DISPUTE_OPENED');
		    		break;
		    case Braintree_WebhookNotification::DISPUTE_LOST :
		    		// 'dispute_lost';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_DISPUTE_LOST');
		    		break;
		    case Braintree_WebhookNotification::DISPUTE_WON :
		    		// 'dispute_won';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_DISPUTE_WON');
		    		break;
		    case Braintree_WebhookNotification::PARTNER_MERCHANT_CONNECTED :
		    		// 'partner_merchant_connected';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_PARTNER_MERCHANT_CONNECTED');
		    		break;
		    case Braintree_WebhookNotification::PARTNER_MERCHANT_DISCONNECTED :
		     		// 'partner_merchant_disconnected';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_PARTNER_MERCHANT_DISCONNECTED');
		    		break;
		    case Braintree_WebhookNotification::PARTNER_MERCHANT_DECLINED :
		    		// 'partner_merchant_declined';
		    		$transaction->set('message', 'COM_PAYPLANS_APP_BRAINTREE_PARTNER_MERCHANT_DECLINED');
		    		break;

		}
		
		//store the response in the payment AND save the payment
		if(!$transaction->save()){
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
		}
		
		$payment->save();
		
		return ' No Errors';
	}
	
	
	protected function _getRecurrenceCount($invoice)
	{
		return $invoice->getRecurrenceCount();
	}

	protected function _createCustomer($invoice, $payment, $data)
	{
		$user = $invoice->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		$name = $user->getRealname();
		$result = Braintree_Customer::create([
			    'firstName' => $name,
			    //'lastName' => 'Jones',
			    'email' => $user->getEmail(),
			    'paymentMethodNonce' => $data["payment_method_nonce"]
		]);
		if ($result->success) {
			$customerId 	= $result->customer->id;
			$paymentToken 	= $result->customer->paymentMethods()[0]->token;
		    // get the transaction instace of lib
			$transaction = PayplansTransaction::getInstance();
			$transaction->set('user_id', $invoice->getBuyer())
						->set('invoice_id', $invoice->getId())
						->set('payment_id', $payment->getId())
						->set('gateway_subscr_id', $result->customer->id)
						->set('message', 'COM_PAYPLANS_LOGGER_BRAINTREE_CUSTOMER_CREATED')
						->set('params', PayplansHelperParam::arrayToIni((array)$result))
						->save();				
						
			$message 				= XiText::_('COM_PAYPLANS_LOGGER_BRAINTREE_CUSTOMER_PROFILE_CREATED');
			PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment, $result->__toString(), 'PayplansPaymentFormatter');
	
		} else {
			$message = '';
		    foreach($result->errors->deepAll() AS $error) {
		        $message .=  $error->code . ": " . $error->message . "\n";
		    }
		    $logmessage 		= XiText::_('COM_PAYPLANS_LOGGER_BRAINTREE_ERROR_IN_CUSTOMER_CREATION');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $result->__toString(), 'PayplansPaymentFormatter', '', true);
			$app = JFactory::getApplication();
			$app->enqueueMessage($message,'error');
		  	$app->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
		}		

		$payment->getGatewayParams()->set('customer_id',$customerId);
		$payment->getGatewayParams()->set('payment_token',$paymentToken);
		$payment->save();
	}

	protected function _createSubscription($invoice, $payment, $data)
	{
		$customerId 	= $payment->getGatewayParams()->get('customer_id');
		$paymentToken 	= $payment->getGatewayParams()->get('payment_token');

		$counter 		= $invoice->getCounter();
		
       		
       	$subArgs = array(
				  'paymentMethodToken' => $paymentToken,
				  'planId' => $this->getAppParam('plan_id', ''),
				  'price'  => $invoice->getTotal($counter + 1)
				  //'merchantAccountId' => $this->getAppParam('merchant_id', '')
				);

		$recurring = $invoice->isRecurring();
		if($recurring == PAYPLANS_RECURRING_TRIAL_1){
			$expTime = $invoice->getExpiration(PAYPLANS_RECURRING_TRIAL_1);
	       	$expTime = $this->getRecurrenceTime($expTime);
	       		
			$subArgs['trialPeriod'] = true; 
  			$subArgs['trialDuration'] = $expTime['period'];
  			$subArgs['trialDurationUnit'] = $expTime['unit'];
		} 
		else{			
			$subArgs['trialPeriod'] = false; 
  			$subArgs['trialDuration'] = 0;
		}

		$recCount = $invoice->getRecurrenceCount();
		if($recCount){
			$subArgs['neverExpires'] = false;
			$subArgs['numberOfBillingCycles'] = $recCount;
		}
		else{
			$subArgs['neverExpires'] = true;
		}		

		$result = Braintree_Subscription::create($subArgs);
		if ($result instanceof Braintree_Result_Error){
			$message = '';
		    foreach($result->errors->deepAll() AS $error) {
		        $message .=  $error->code . ": " . $error->message . "\n";
		    }
		    $logmessage 		= XiText::_('COM_PAYPLANS_LOGGER_BRAINTREE_ERROR_IN_SUBSCRIPTION_CREATION');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $result->__toString(), 'PayplansPaymentFormatter', '', true);
			$app = JFactory::getApplication();
			$app->enqueueMessage($message,'error');
		  	$app->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
		}
		else {
			// get the transaction instace of lib
			$transaction = PayplansTransaction::getInstance();
			$transaction->set('user_id', $invoice->getBuyer())
						->set('invoice_id', $invoice->getId())
						->set('payment_id', $payment->getId())
						->set('gateway_subscr_id', $result->subscription->id)
						->set('message', 'COM_PAYPLANS_LOGGER_BRAINTREE_SUBSCRIPTION_SIGN_UP')
						->set('params', PayplansHelperParam::arrayToIni((array)$result))
						->save();	

			// Case of Free Trial 
			// Do not create Transaction Profile of user
			// (Just create transaction  and mark invoice as paid)
			if($invoice->getTotal() == 0 && $invoice->isRecurring()){
				$transaction = PayplansTransaction::getInstance();
				$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('amount', 0)
					->set('gateway_txn_id', 0)
					->set('gateway_subscr_id', $result->subscription->id)
					->set('gateway_parent_txn', 0)
					->set('message', 'COM_PAYPLANS_PAYMENT_APP_BRAINTREE_PAYMENT_COMPLETED_SUCCESSFULLY')
					->set('params', PayplansHelperParam::arrayToIni((array)$result->subscription))
					->save();
			}			

			$payment->getGatewayParams()->set('subscription_id', $result->subscription->id);
			$payment->save();
						
			$message 				= XiText::_('COM_PAYPLANS_LOGGER_BRAINTREE_SUBSCRIPTION_CREATED');
			PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment, $result->__toString(), 'PayplansPaymentFormatter');
		} 
	}

	public function getRecurrenceTime($expTime)
	{
		$expTime['year'] = isset($expTime['year']) ? intval($expTime['year']) : 0;
		$expTime['month'] = isset($expTime['month']) ? intval($expTime['month']) : 0;
		$expTime['day'] = isset($expTime['day']) ? intval($expTime['day']) : 0;;
		
		// years
		if(!empty($expTime['year'])){
			$months =  $expTime['year'] * 12;
			if(isset($expTime['month']) && $expTime['month']){
				$months	 += $expTime['month'];
			}	

			return array('period' => $months, 'unit' => 'month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
										'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_BRAINTREE_RECURRING_MESSAGE'));
		}
		
		// if months are set
		if(!empty($expTime['month'])){
			// if days are empty
			if(empty($expTime['day'])){
				return array('period' => $expTime['month'], 'unit' => 'month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
										'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_BRAINTREE_RECURRING_MESSAGE'));
			}
			
			// if total days are less or equlas to 90, then return days
			//  IMP : ASSUMPTION : 1 month = 30 days
			$days = $expTime['month'] * 30;
			if(($days + $expTime['day']) <= 999){
				return array('period' => $days + $expTime['day'], 'unit' => 'day', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
										'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_BRAINTREE_RECURRING_MESSAGE'));
			}
			
			return array('period' => $expTime['month'], 'unit' => 'month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
										'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_BRAINTREE_RECURRING_MESSAGE'));
		}
		
		// if only days are set then return days as it is
		if(!empty($expTime['day'])){
			return array('period' => intval($expTime['day'], 10), 'unit' => 'day', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
										'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_BRAINTREE_RECURRING_MESSAGE'));
		}
		
		// XITODO : what to do if not able to convert it
		return false;
	}

	//function after payment is done
	public function refundRequest(PayplansTransaction $transaction, $refund_amount)
	{
 		$this->_loadConfig();
		
		$txnId 		= $transaction->getGatewayTxnId();
		$amount  		= ($refund_amount*100);

		$bt_transaction = Braintree_Transaction::find($txnId);
		$status = strtolower($bt_transaction->status);
		if(in_array($status, array('settled', 'settling'))){
			$result = Braintree_Transaction::refund($txnId);	
		}
		else{
			$result = Braintree_Transaction::void($txnId);
		}

		if($result->success == true){			
			$paymentId = $transaction->getPayment();
			$newtransaction = PayplansTransaction::getInstance();
			$newtransaction->set('user_id', 			$transaction->getBuyer())
							->set('invoice_id', 		$transaction->getInvoice())
							->set('payment_id',		 	$paymentId)
							->set('gateway_subscr_id', 	0)
							->set('params', 			PayplansHelperParam::arrayToIni((array)$result->transaction));

			if($result->transaction->id == $txnId){
				$newtransaction->set('gateway_txn_id',	'refund_'.$result->transaction->id)
								->set('gateway_parent_txn', $result->transaction->id);
			}
			else{
				$newtransaction->set('gateway_txn_id',	$result->transaction->id)
								->set('gateway_parent_txn', $result->transaction->refundedTransactionId);
			}

			$negativeAmt = -($refund_amount);
		    $newtransaction->set('amount',$negativeAmt)
		    				->set('message', 'COM_PAYPLANS_APP_BRAINTREE_TRANSACTION_REFUNDED')
		    				->save();
			return true;
		}
		else{
			$user		= $transaction->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
    		$username	= $user->getUsername();
    		$userId		= $user->getId();
    		$invoice 	= $transaction->getInvoice(PAYPLANS_INSTANCE_REQUIRE);

    		$message = '';
		    foreach($result->errors->deepAll() AS $error) {
		        $message .=  $error->code . ": " . $error->message . "\n";
		    }
			//It is needed to create a log for wrong response
			$errors['error_code']	  = '404';
			$errors['error_message']  = sprintf(XiText::_('COM_PAYPLANS_APP_BRAINTREE_LOGGER_ERROR_IN_PAYMENT_PROCESS_DETAILS'),$message,$username,$userId,$invoice->getKey());
			
			$message = XiText::_('COM_PAYPLANS_APP_BRAINTREE_LOGGER_ERROR_IN_STRIPE_RESPONSE_INVALID');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $transaction, $errors, 'PayplansPaymentFormatter', '', true);
			
			return false;
		}		
		
		return false;
	}
	
	public function supportForRefund()
	{
		return true;
	}

	public function isSupportPaymentCancellation($invoice)
	{
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}

	public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$customer_id = $payment->getGatewayParam('customer_id');
		$subscription_id = $payment->getGatewayParam('subscription_id');
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $customer_id)
					->set('gateway_subscr_id', $subscription_id);
		
		$this->_loadConfig();

		$result = Braintree_Subscription::cancel($subscription_id);		
		
		if($result->success === true)
		{
			$response = Braintree_Customer::delete($customer_id);
			
			if($response->success === true)
			{
				$params = new XiParameter();
				$params->set('customer_id',$customer_id);				
				$transaction->set('message', 'COM_PAYPLANS_PAYMENT_APP_BRAINTREE_CANCEL_SUCCESS')
									->save();
				$payment->set('gateway_params', $params)
								->save();					
				parent::onPayplansPaymentTerminate($payment, $controller);
				return $this->_render('cancel_success');		
			}

			$transaction->set('message', 'COM_PAYPLANS_PAYMENT_APP_BRAINTREE_CANCEL_ERROR')
								->save();
			return $this->_render('cancel_error');
		}

		$transaction->set('message', 'COM_PAYPLANS_PAYMENT_APP_BRAINTREE_CANCEL_ERROR')
							->save();
		return $this->_render('cancel_error');
	}
}


