<?php
/**
* @copyright	Copyright (C) 2009 - 2014 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppCashenvoy extends PayplansAppPayment
{
	protected $_location	= __FILE__;

   public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
   {
		if(is_object($data)){
			$data = (array)$data;
		}

		if(!isset($data["initiate"])){
			return $this->initiatePaymentRequest($payment);
		}

		$root = JURI::root();
		if(XiFactory::getConfig()->https == true){
		    $root         = JString::str_ireplace("http:", "https:", $root);
		}

		$invoice 		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$merchantId 	= $this->getAppParam('merchant_id',false);
		$merchantKey 	= $this->getAppParam('merchant_key',false);
		$cecustomerid 	= $data['customer_id'];
		$cetxref		= $payment->getKey();
		$ceamt 			= $invoice->getTotal();
		$cememo			= $invoice->getTitle();
		$cenurl 		= $root.'index.php?option=com_payplans&gateway=cashenvoy&view=payment&task=complete&action=success&payment_key='.$cetxref; 

		// generate request signature
		$data 			= $merchantKey.$cetxref.$ceamt;
		$signature 		= hash_hmac('sha256', $data, $merchantKey, false);


		if($this->getAppParam('sandbox',0)){
			$post_url  		= 'https://www.cashenvoy.com/sandbox/?cmd=cepay';
		}else {
			$post_url  		= 'https://www.cashenvoy.com/webservice/?cmd=cepay';
		}
    	
    	$this->assign('ce_merchantid', 	$merchantId);
    	$this->assign('ce_transref', 	$cetxref);
    	$this->assign('ce_amount', 		$ceamt);
    	$this->assign('ce_customerid', 	$cecustomerid);
    	$this->assign('ce_memo', 		$cememo);
    	$this->assign('ce_notifyurl', 	$cenurl);
    	$this->assign('ce_signature', 	$signature);
    	$this->assign('url',         	$post_url); 
	    	
		return $this->_render('postform');
  	}

    public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		if($action == 'cancel'){
				return true;
		}

		$invoice 	= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount 	= number_format($invoice->getTotal(),2);
		
		$this->_processPayment($invoice, $payment, $_POST);
	
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	public function initiatePaymentRequest($payment)
	{	
		$root = JURI::root();
		if(XiFactory::getConfig()->https == true){
		    $root         = JString::str_ireplace("http:", "https:", $root);
		}
		
		$invoice   		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);		 
		$paymentKey		= $payment->getKey();
		$amount 		= $invoice->getTotal();

		$postURL     = $root.'index.php?option=com_payplans&gateway=cashenvoy&view=payment&task=pay&payment_key='.$paymentKey;
		$this->assign('currency', 	$invoice->getCurrency('isocode'));
		$this->assign('payment', 	$payment);
		$this->assign('amount',		$amount);
		$this->assign('post_url',   $postURL); 
	    	
		return $this->_render('form');

	}
	
	public function _processPayment($invoice, $payment, $response)
	{
	  	$key 		= $this->getAppParam('merchant_key',false);
		$transref 	= $response['ce_transref'];
		$mertid 	= $this->getAppParam('merchant_id',false);
		$type 		= ''; //Data return format. Options are xml or json. leave blank if you want data returned in string format.
		$cdata 		= $key.$transref.$mertid;
		$signature 	= hash_hmac('sha256', $cdata, $key, false);
		$resData 	= $this->getStatus($transref,$mertid,$type,$signature);

		$cnt   = 0;
		foreach ($resData as $subdata) 
		{ 
			$cnt++; 
		}
		
		$error = '';
		if($cnt == 3){ 
			$response['returned_transref'] 	= $resData[0];
			$response['returned_status'] 	= $resData[1];
			$response['returned_amount'] 	= $resData[2];
		} else {
			$error = $resData[0];
		}
		
		// if same notification came more than one time, check if transaction already exists if yes then do nothing and return
		$transactions = $this->_getExistingTransaction($invoice->getId(), $response['returned_transref'], 0, 0);
    	    if($transactions !== false){
    		  foreach($transactions as $transaction){
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			if($transaction->getParam('returned_status', '') === $response['returned_status'])
    				return true;
    		 }
    	  }

		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', 			$payment->getBuyer())
					->set('invoice_id', 		$invoice->getId())
					->set('payment_id', 		$payment->getId())
					->set('gateway_txn_id', 	isset($response['returned_transref']) 		? $response['returned_transref'] 			: 0)
					->set('gateway_subscr_id', 	0)
					->set('gateway_parent_txn', 0)
					->set('params', 			PayplansHelperParam::arrayToIni($response));

		if(empty($error))
		{
			$transaction->set('amount', 	$response['returned_amount'])
						->set('message', 	XiText::_('COM_PAYPLANS_PAYMENT_APP_CASHENVOY_TRANSACTION_COMPLETED'));
		}else {
			$transaction->set('message', 	XiText::_('COM_PAYPLANS_PAYMENT_APP_CASHENVOY_TRANSACTION_FAILED'));
		}
		
		if($error){
				$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_IN_CASHENVOY_PAYMENT_PROCESS');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		}
        
        //store the response in the payment AND save the payment
		if(!$transaction->save()){
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		}
		
		$payment->save();
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}

	public function getStatus($transref, $mertid, $type='', $sign)
	{
		$request 	= 'mertid='.$mertid.'&transref='.$transref.'&respformat='.$type.'&signature='.$sign; //initialize the request variables
		
		if($this->getAppParam('sandbox',0)){
			$url 		= 'https://www.cashenvoy.com/sandbox/?cmd=requery';
		}else {
			$url 		= 'https://www.cashenvoy.com/	/?cmd=requery';
		}
		
		$ch = curl_init(); //initialize curl handle
		curl_setopt($ch, CURLOPT_URL, $url); //set the url
		curl_setopt($ch, CURLOPT_RETURNTRANSFER,true); //return as a variable
		curl_setopt($ch, CURLOPT_POST, 1); //set POST method
		curl_setopt($ch, CURLOPT_POSTFIELDS, $request); //set the POST variables
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		$response = curl_exec($ch); // grab URL and pass it to the browser. Run the whole process and return the response
		$response = strip_tags(preg_replace('#(<title.*?>).*?(</title>)#', '$1$2', $response)); 
       	
       	$data = explode('-',$response);
		curl_close($ch); //close the curl handle
		return $data;	

	}
}
