<?php
/**
* @copyright	Copyright (C) 2009 - 2014 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppCcavenue extends PayplansAppPayment
{
	protected $_location	= __FILE__;

   public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
   {
		if(is_object($data)){
			$data = (array)$data;
		}

		$invoice   		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);		 
		$recurring 		= $invoice->isRecurring();
		
		$merchantId 	= $this->getAppParam('merchant_id',false);
		$accessCode 	= $this->getAppParam('access_code',false);
		$paymentKey		= $payment->getKey();
		$amount 		= $invoice->getTotal();
    	
    	$root = JURI::root();
		if(XiFactory::getConfig()->https == true){
		    $root         = JString::str_ireplace("http:", "https:", $root);
		}
		
		$redirectURL     = $root.'index.php?option=com_payplans&gateway=ccavenue&view=payment&task=complete&action=success&payment_key='.$paymentKey;
		$cancelURL       = $root.'index.php?option=com_payplans&gateway=ccavenue&view=payment&task=complete&action=cancel&payment_key='.$paymentKey;
    	
    	$this->assign('merchant_id', 		$merchantId);
		$this->assign('access_code', 		$accessCode);
		$this->assign('order_id', 			$paymentKey);
    	$this->assign('amount',				$amount);
    	$this->assign('payment', 			$payment);
    	$this->assign('currency', 			$invoice->getCurrency('isocode'));
    	$this->assign('invoice_key', 		$invoice->getKey());
    	$this->assign('redirect_url', 		$redirectURL);
    	$this->assign('cancel_url', 		$cancelURL);
    	$this->assign('post_url',         	XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&payment_key=".$paymentKey)); 
    	$this->assign('merchant_param1', 	$paymentKey);
    	
		return $this->_render('form');
  	}

    public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		if($action == 'cancel'){
				return true;
		}
		
		if(!isset($data["encResp"])){
			return $this->initiatePaymentRequest($data);
		}else {
			$invoice 	= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
			$amount 	= number_format($invoice->getTotal(),2);
			
			// Decrypt response sent by CcAvenue server
			list($orderStatus, $response)	= $this->validateData($data);
			
			$this->_processPayment($invoice, $payment, $orderStatus, $response);
		
			return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		}

	}
	
	public function initiatePaymentRequest($data)
	{
		require_once 'Crypto.php';
		//error_reporting(0);

		$merchant_data						= '';
		$working_key						= $this->getAppParam('encyption_key',false);		//Shared by CCAVENUES
		$access_code						= $this->getAppParam('access_code',false);			//Shared by CCAVENUES
		
		$gatewaydata						=  array();
		$gatewaydata['merchant_id']  		= $data['merchant_id'];
		$gatewaydata['amount']  	  		= $data['amount'];
		$gatewaydata['order_id']  			= $data['order_id'];
		$gatewaydata['redirect_url'] 		= $data['redirect_url'];
		$gatewaydata['cancel_url']  		= $data['cancel_url'];
		$gatewaydata['billing_name']  		= $data['billing_name'];
		$gatewaydata['billing_address']  	= $data['billing_address'];
		$gatewaydata['billing_city']  		= $data['billing_city'];
		$gatewaydata['billing_state']  		= $data['billing_state'];
		$gatewaydata['billing_zip']  		= $data['billing_zip'];
		$gatewaydata['billing_country']  	= $data['billing_country'];
		$gatewaydata['billing_tel']  		= $data['billing_tel'];
		$gatewaydata['billing_email']  		= $data['billing_email'];
		$gatewaydata['language']  			= $data['language'];
		$gatewaydata['currency']  			= $data['currency'];
		$gatewaydata['merchant_param1']		= $data['merchant_param1'];
				
		foreach ($gatewaydata as $key => $value){
 				$merchant_data .= $key.'='.urlencode($value).'&';
		}
		
		$encrypted_data		= encrypt($merchant_data,$working_key);
		$post_url  			= 'https://secure.ccavenue.com/transaction/transaction.do?command=initiateTransaction&encRequest='.$encrypted_data.'&access_code='.$access_code;			
		
		return XiFactory::getApplication()->redirect($post_url);
	}
	
	public function _processPayment($invoice, $payment, $orderStatus, $response)
	{
		// if same notification came more than one time, check if transaction already exists if yes then do nothing and return
		$transactions = $this->_getExistingTransaction($invoice->getId(), $response['tracking_id'], 0, 0);
    	    if($transactions !== false){
    		  foreach($transactions as $transaction){
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			if($transaction->getParam('order_status', '') === $orderStatus)
    				return true;
    		 }
    	  }

		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', 			$payment->getBuyer())
					->set('invoice_id', 		$invoice->getId())
					->set('payment_id', 		$payment->getId())
					->set('gateway_txn_id', 	isset($response['tracking_id']) 		? $response['tracking_id'] 			: 0)
					->set('gateway_subscr_id', 	isset($response['Customer_identifier']) ? $response['Customer_identifier'] 	: 0)
					->set('gateway_parent_txn', 0)
					->set('params', 			PayplansHelperParam::arrayToIni($response));

		$error	= '';
		if($orderStatus === "Success")
		{
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_CCAVENUE_TRANSACTION_COMPLETED');
			$transaction->set('amount', 	$response['amount'])
						->set('message', 	$message);
		}
		else if($orderStatus === "Aborted"){
			$error = XiText::_('COM_PAYPLANS_PAYMENT_APP_CCAVENUE_TRANSACTION_ABORTED');
			$transaction->set('message', 	$error);
		}
		else if($orderStatus === "Failure"){
			$error = XiText::_('COM_PAYPLANS_PAYMENT_APP_CCAVENUE_TRANSACTION_FAILED');
			$transaction->set('message', 	$error);
		}
		else{
			$error = XiText::_('COM_PAYPLANS_PAYMENT_APP_CCAVENUE_TRANSACTION_ILLEGAL');
			$transaction->set('message', 	$error);
		}
		
		if($error){
				$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_IN_CCAVENUE_PAYMENT_PROCESS');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		}
        
        //store the response in the payment AND save the payment
		if(!$transaction->save()){
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		}
		
		$payment->save();
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	public function validateData($data)
	{
		require_once 'Crypto.php';
		//error_reporting(0);
		
		$workingKey			= $this->getAppParam('encyption_key',false);		//Working Key should be provided here.
		$encResponse		= $data["encResp"];								//This is the response sent by the CCAvenue Server
		$rcvdString			= decrypt($encResponse,$workingKey);				//Crypto Decryption used as per the specified working key.
		$order_status		= "";
		$decryptValues		= explode('&', $rcvdString);
		$dataSize			= sizeof($decryptValues);
	
		for($i = 0; $i < $dataSize; $i++) 
		{
			$information		= explode('=',$decryptValues[$i]);
			if($i==3){
				$order_status 	= $information[1];
			}
		}
		
		for($i = 0; $i < $dataSize; $i++) 
		{
			$information					= explode('=',$decryptValues[$i]);
			$response[$information[0]]	 	= urldecode($information[1]);
		}

		return array($order_status, $response);
	}

}
