<?php
/**
* @copyright	Copyright (C) 2009 - 2012 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppCcbill extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	// over-ride to make it applicable on onPayplansControllerCreation
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
	function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		if($view != 'payment'){
			return true;
		}

		if($task == 'notify'){
	        $paymentKey = JRequest::getVar('X-payment_key', null);
			if(!empty($paymentKey)){
				JRequest::setVar('payment_key', $paymentKey, 'POST');
				return true;
			}
        }
		return true;
	}
	
	 function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	 {	
		$invoice 		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$formPrice  	= $invoice->getTotal();
		$ccbill_url 	= 'https://bill.ccbill.com/jpost/signup.cgi';
		//by default currency code of USD is set
		$currencyCode 	= $this->getAppParam('currencyValue','840');
		$saltKey 		= $this->getAppParam('saltKey','');
		
		$expTime 		= $invoice->getExpiration();
		$expTime 		= $this->getRecurrenceTime($expTime);
		$formPeriod 	= empty($expTime['period'])? 2 : $expTime['period'];

	 	$recurring = 0;
		$this->assign('payment', 		$payment);
		$this->assign('saltKey', 		$saltKey);
		$this->assign('currency', 		$currencyCode	);
		$this->assign('post_url', 		$ccbill_url);
		$this->assign('clientAccNo',  	$this->getAppParam('clientAccountNumber',''));
		$this->assign('subAccount',   	$this->getAppParam('subAccount',''));
		$this->assign('formName',     	$this->getAppParam('formName',''));
		$this->assign('formPeriod', 	$formPeriod);
		$this->assign('formPrice', 		$formPrice);
	 	
		if($invoice->isRecurring())
		{
			$counter   = $invoice->getCounter();
			$recurring = 1;
			$recurrenceCount = $invoice->getRecurrenceCount(); 
			//to get infinite payment, recurrence count is either set to 0 or 99. In ccbill it is required to send 99 for indifinite payments
			if($recurrenceCount == 0 || $recurrenceCount == 99 ){
			 	$formRebills = 99;
			}
			else {
				$formRebills = ($recurrenceCount-1);
			}
			
			$formRecurringPrice 	= $formPrice;
       		$formRecurringPeriod 	= $formPeriod;

			$this->assign('formRebills',			$formRebills);
            $this->assign('formRecurringPrice', 	$formRecurringPrice);
	       	$this->assign('formRecurringPeriod',	$formRecurringPeriod);
	       	
	       	if($invoice->isRecurring() == PAYPLANS_RECURRING_TRIAL_1){
	       		
	       		$expTime 			= $invoice->getExpiration(PAYPLANS_RECURRING_TRIAL_1);
				$expTime 			= $this->getRecurrenceTime($expTime);
				$formPeriod 		= empty($expTime['period'])? 2 : $expTime['period'];				
				$formRebills 		= $recurrenceCount;
	       		$formRecurringPrice = $invoice->getTotal($counter + 1);

	       		$this->assign('formRebills', $formRebills);
				$this->assign('formPeriod', $formPeriod);	       		
	       		$this->assign('formRecurringPrice',$formRecurringPrice);
	       	}
	       	$parameter = array($formPrice,$formPeriod,$formRecurringPrice,$formRecurringPeriod,$formRebills,$currencyCode,$saltKey);
		}
		else {
			$parameter = array($formPrice,$formPeriod,$currencyCode,$saltKey);
		}		

		$formDigest = $this->calculateMD5($parameter);
		
		$this->assign('formDigest', $formDigest);
	    $this->assign('recurring', $recurring);

	  return $this->_render('form');
	 }
	 
	 public function getRecurrenceTime($expTime)
	 {
		$expTime['year']  = isset($expTime['year']) ? intval($expTime['year']) : 0;
		$expTime['month'] = isset($expTime['month']) ? intval($expTime['month']) : 0;
		$expTime['day']   = isset($expTime['day']) ? intval($expTime['day']) : 0;
		$time = 0;

		if( $expTime['year']){
			$time = ($expTime['year'] * 365);
		}
		
		if($expTime['month']){
			$time = $time + ($expTime['month']*30);
		}

		if ($expTime['day']){
			$time = $time + ($expTime['day']);
		}
		
		return array('period' => $time, 'unit' => 'Days', 'frequency' => XiText::_('COM_PAYPLANS_APP_PAYMENT_CCBILL_EXPIRATION_TIME_IN_DAYS') );
		
	 }
	 
	 
	function calculateMD5($parameters)
	{
	 	$finalString = '';
	 	foreach ($parameters as $parameter)
	 	{
	 		$finalString .= $parameter;
	 	}
	 	return md5($finalString);
	}
	 
	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{		
		$logs	= PayplansHelperLogger::getLog($payment, XiLogger::LEVEL_ERROR);	
		$record = array_pop($logs);			
		if($record && !empty($record)){
			$action = 'error';
		}
		
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	 
	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
	 	if($data['eventType'] == 'NewSaleSuccess'){
	 	    $saltKey 	=  $this->getAppParam('saltKey','');
		 	$parameter  = array($data['subscriptionId'],'1',$saltKey);	
		 	
		 	//calculate the md5. It is calculated from 3 values(subscriptionId,1,Saltkey)
		    $formDigest = $this->calculateMD5($parameter);
				
			if(($data['dynamicPricingValidationDigest'] != $formDigest)){
				$message = XiText::_('COM_PAYPLANS_APP_PAYMENT_CCBILL_NOTIFICATION_NOT_RECIEVED_PROPERLY');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
				return true;
			}
	 	}
		
	 	$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
	 		
		$txn_id 		= isset($data['transactionId'])	? $data['transactionId'] 	: 0;
		$subscr_id 		= isset($data['subscriptionId'])? $data['subscriptionId'] 	: 0;
		$parent_txn 	= 0;

		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_txn);
		if(!empty($transactions)){
			return true;
	 	}
		
		
	 	  // get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $payment->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $txn_id)
					->set('gateway_subscr_id', $subscr_id)
					->set('gateway_parent_txn', 0)
					->set('params', PayplansHelperParam::arrayToIni($data));
			
	   	 $func_name	= isset($data['eventType']) ? '_on_payment_'.JString::strtolower($data['eventType']) : 'EMPTY';
		 if(method_exists($this, $func_name)){
				$errors = $this->$func_name($payment, $data, $transaction);
		 }
		 else{
				$errors[] = XiText::_('COM_PAYPLANS_APP_PAYMENT_CCBILL_INVALID_EVENT_TYPE');
			}
		    
		 if(!empty($errors)){
				$message 	= XiText::_('COM_PAYPLANS_APP_PAYMENT_CCBILL_PAYMENT_PROCESS');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
			}
		
		 //store the response in the payment AND save the payment
		if(!$transaction->save()){
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
		}
		
		$payment->save();
		
		return count($errors) ? implode("\n", $errors) : ' No Errors';
	 }
	 
	 
	 function _on_payment_NewSaleSuccess($payment, $data, $transaction)
	 {
	 		$transaction->set('amount', $data['billedInitialPrice'])
						->set('message', 'COM_PAYPLANS_APP_PAYMENT_CCBILL_TRANSACTION_COMPLETED');
		    return array();
	 }
	 
	 function _on_payment_RenewalSuccess($payment, $data, $transaction)
	  {
	 	$transaction->set('amount', $data['billedAmount'])
						->set('message', 'COM_PAYPLANS_APP_PAYMENT_CCBILL_RECUR_TRANSACTION_COMPLETED');
		return array();
	 }
	 
	 function _on_payment_Cancellation($payment, $data, $transaction) {
	   	$transaction->set('message', $data['reason']);
		//terminate the order
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$invoice->terminate();
		return array();
	 }
	 
	 function _on_payment_RenewalFailure() {
	 	$errors = array();
	 	$transaction->set('message', $data['reason'].' Decline code:-'.$data['failureCode']);
	 	$errors = $data['reason'];
		return $errors;
	 }
	 
     function _on_payment_NewSaleFailure($payment, $data, $transaction)
	 {
		$errors = array();
		$transaction->set('message', $data['failureReason'].' Decline code:-'. $data['failureCode']);
		$errors = $data['failureReason'];
		return $errors;
	 }
	 
	 function _on_payment_refund($payment, $data, $transaction){
	    $transaction->set('amount', -$data['amount'])
						->set('message', $data['reason']); 	
	    return array();
	}
}