<?php

/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	Contentacl
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppContentacl extends PayplansApp
{
	protected $_location	= __FILE__;
	
	public function isApplicable($refObject = null, $eventName='')
	{
		$option =JRequest::getVar('option');
		if($eventName === 'onContentPrepare' && $option == 'com_content'){
			return true;
		}
		return parent::isApplicable($refObject, $eventName);
	}
	
	function onContentPrepare($context, &$row, &$params, $page = 0)
	{
		$block	  = $this->getAppParam('block_j17', 'none');
		$url   	  = XiRoute::_('index.php?option=com_payplans&view=plan');
		$view	  = JRequest::getVar('view');
		
		if($view != 'article')
			return true;
			
		if($block == 'none')
			return true;
		
		if($this->_isUserAllowed() === true)
			return true;

		if($block == 'joomla_category'){
			$allowedCat = $this->getAppParam('joomla_category', 0);
			$allowedCat = is_array($allowedCat) ? $allowedCat : array($allowedCat);
			$catId 		= (isset($row->catid)) ? $row->catid : null;
			
			//get all parent category of current cat
			$allCat		= $this->_getParentCategories($catId);
			
			$tempArray = array_intersect($allCat, $allowedCat);
			if(empty($tempArray)){
				return true;
			}

			if($this->getParam('applyAll',false)){
				$row->text = $row->introtext.'<a id="pp_contentacl_joomla_category" href="'.$url.'">'.XiText::_('COM_PAYPLANS_CONTENTACL_SUBSCRIBE_PLAN').'</a>';
			}else {
				$links      = $this->getPlanlinks();
				$row->text  = $row->introtext.$links;
			}
		}
		
		if($block == 'joomla_article'){
			$allowedArticle = $this->getAppParam('joomla_article', 0);
			$allowedArticle = is_array($allowedArticle) ? $allowedArticle : array($allowedArticle);
			$articleId 		= (isset($row->id)) ? $row->id : null;
			
			if(!in_array($articleId, $allowedArticle)){
				return true;
			}	

			if($this->getParam('applyAll',false)){
				$row->text = $row->introtext.'<a id="pp_contentacl_joomla_article" href="'.$url.'">'.XiText::_('COM_PAYPLANS_CONTENTACL_SUBSCRIBE_PLAN').'</a>';
			}else {
				$links      = $this->getPlanlinks();
				$row->text  = $row->introtext.$links;
			}
		}

		return true;
	}
	
	function _isUserAllowed()
	{
		$userid 	 = XiFactory::getUser()->id;
		if(!$userid){
			return false;
		}
		
		$user = PayplansUser::getInstance($userid);
		
		$userSubs  = $user->getPlans();
		$plans 	   = $this->getPlans();
		
		// return false when user is non-subscriber
		if(empty($userSubs)){
			return false;
		}
		
		// return true when app is core app,
		// no need to check whether plan is attached with this app or not
		if($this->getParam('applyAll',false) != false){
			return true;
		}

		// if user have an active subscription of the plan attached with the app then return true
		foreach($userSubs as $sub){
			if(in_array($sub, $plans)){
				return true;
			}
		}

		return false;
	}

	protected function _getParentCategories($catId)
	{
		$allCat   = array();
		while($catId)
		{
			$allCat[] = $catId;
			
			$db 	= PayplansFactory::getDBO();
			$query = 'SELECT `parent_id`'
				 	. ' FROM #__categories'
				 	. ' WHERE `published` = 1 AND `id` = '.$catId;
				 	;
		 	$db->setQuery( $query );
		 	$catId = $db->loadResult();
		}
		return $allCat;
	}
	
	public function getPlanlinks()
	{
		$plans 	   	= $this->getPlans();
		$links     	= '<strong>'.XiText::_('COM_PAYPLANS_CONTENTACL_SUBSCRIBE_PLAN').'</strong> ( ';
		$plans  	= $this->_isValidPlan($plans);
		
		//If plans are not vaild then show subscribe page link
		if($plans == false){
			$url   	= XiRoute::_('index.php?option=com_payplans&view=plan');
			return '<a href="'.$url.'">'.XiText::_('COM_PAYPLANS_CONTENTACL_SUBSCRIBE_PLAN').'</a>';
		}

		foreach ($plans as $plan){
			$planName		= $plan->title;
			$planLinks[]   	= '<a href="'.XiRoute::_('index.php?option=com_payplans&view=plan&task=subscribe&plan_id='.$plan->plan_id).'">'.$planName.'</a>';
		}
		$planLinks    = implode(" , ", $planLinks);
		$links   	 .= $planLinks. ' )';
		return $links;
	}
	
	public function _isValidPlan($plans)
	{
		$records = XiFactory::getInstance('plan', 'model')
							->loadRecords(array('plan_id' => array(array('IN', '('.implode(",", $plans).')')), 'published'=>1, 'visible'=>1));
							
		if(!empty($records)){			
			return $records;
		}
		
		return false;
	}
}

class PayplansAppContentaclFormatter extends PayplansAppFormatter
{
	// get rules
	function getVarFormatter()
	{
		$rules = array('_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       'function' => 'getAppPlans'),
					   'app_params'      => array('formatter'=> 'PayplansAppContentaclFormatter',
										       'function' => 'getFormattedParams'));
		return $rules;
	}
	
	// format app param
	function getFormattedParams($key, $value, $data)
	{
		$categories = XiHelperJoomla::getJoomlaCategories();
		$sections   = XiHelperJoomla::getJoomlaSections();
		$articles   = XiHelperJoomla::getJoomlaArticles();
		foreach ($value as $param=>$val)
		{
			if($param == 'joomla_category'){
				if(is_array($val)){
					foreach($val as $v){
						$cat[] = $categories[$v]->title;
					}
					$value[$param] = $cat;
				}
				else{
					$value[$param] = $categories[$val]->title;
				}
			}
			if($param == 'joomla_article'){
				if(is_array($val)){
					foreach($val as $v){
						$allArticles[] = $articles[$v]->title;
					}
					$value[$param] = $allArticles;
				}
				else{
					$value[$param] = $articles[$val]->title;
				}
			}
		}
	}
}
