<?php
/**
* @copyright	Copyright (C) 2009 - 2014 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license	GNU/GPL, see LICENSE.php
* @contact	support+payplans@readybytes.in
* @author	Puneet Singhal
*/
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );


/**
 * Payplans Content Seller on Subscription basis
 */

class plgPayplansContentseller extends XiPlugin
{
	protected $_resource	= 'contentseller.article';
	protected $_cat_resource= 'contentseller.category';
	protected $_tag_values	= array('article_id','rightholder_id','plan_id','expiration_type',
									'expiration_time', 'recurrence_count', 'trial_price_1',
									'trial_time_1', 'trial_price_2', 'trial_time_2', 
									'price', 'currency');
	
	public function onContentPrepare($context, &$article, &$params, $page = 0)
	{
		// Check whether the plugin should process or not
		if ( JString::strpos( $article->text, 'pp-contentseller' ) === false ){
			return true;
		}
		
		return $this->processCallback($article);
	}

	
	public function onPayplansGetPlugin()
	{
	        $xml = dirname(__FILE__).DS.'contentseller.xml';
	        if (file_exists($xml)) {
	                    $xmlContent = simplexml_load_file($xml);
	                }
	                else {
	                    $xmlContent = null;
	                }
	         return array('name'   => 'contentseller', 
	                	  'value' =>$xmlContent);
	 }
	
	public function replaceTagCallback($match)
	{
		$user_id = (!isset($this->user_id)) ? XiFactory::getUser()->id : $this->user_id;
		if($this->_accessCheck($user_id)){
			return '';
		}
		
		$attributes = array();
		preg_match_all( '/(?P<key>\w+)\=(?P<val>[^\s]+)/', $match[1], $attributes);
		$attributes = array_filter($attributes);
		
		if(empty($attributes)){
			return '';
		}
		
		$values = array_combine( $attributes['key'], $attributes['val'] );
		$additional = array();
		$additional['article_id'] = isset($values['article_id']) ? $values['article_id'] : $this->article_id;
		$additional['rightholder_id'] = $this->rightholder_id;
		$values	= array_merge($values, $additional);
		
		if(!isset($values['purpose'])){
			$this->doErrorLogForPurpose($this->article_id);
			return '';
		}
		
		if(!isset($values['plan_id'])){
			$this->doErrorLogForPlanid($this->article_id);
			return '';
		}
		
		$purpose = $values['purpose'];
		$data	 = array();
		foreach ($this->_tag_values as $tag){
			$key = $purpose.'_'.$tag;
			if(isset($values[$tag])){
				$data[$key] = $values[$tag];
			}
		}
		
		$key	 = md5(serialize($data));
		$session = PayplansFactory::getSession();
		$session->clear($key);
		$session->set($key, $data);
		
		$url 	 = 'index.php?option=com_payplans&view=plan&task=subscribe&plan_id='.$values['plan_id'];
		$html  	 = '<form method="post" action="'.XiRoute::_($url).'">';
		$html	.= '<input type="hidden" name="purpose" value="'.$purpose.'">';
		$html	.= '<input type="hidden" name="contentSellerKey" value="'.$key.'">';
		$html 	.= '<button type="submit" style="padding:5px 10px; border-radius:5px; font-weight:bold;">';
		$html 	.= (empty($match[2]) ? 'Subscribe' : $match[2]);
		$html 	.= '</button>';
		$html 	.= '</form>';
		
		return $html;
	}
	
	/**
	 * 
	 * Step 1 :- First check whether user has access of category or not
	 * Step 2 :- Secondly check whether user has access of article or not
	 * @param int $user_id
	 * @return boolean
	 */
	protected function _accessCheck($user_id)
	{
		if(!$user_id){
			return false;
		}
		
		$access  = PayplansHelperApp::getResource($user_id, $this->cat_id, $this->_cat_resource);
		if($access && $access->value == $this->cat_id){
			return true;
		}
		
		$access  = PayplansHelperApp::getResource($user_id, $this->article_id, $this->_resource);
		if($access && $access->value == $this->article_id){
			return true;
		}
		
		return false;
	}
	
	public function onPayplansSubscriptionBeforeSave($prev, $new)
	{
		// do nothing if prev instance is not null
		if($prev != null){
			return true;
		}
		
		$session 	= PayplansFactory::getSession();
		$purpose 	= XiFactory::getApplication()->input->get('purpose', false);
		$sellerkey 	= XiFactory::getApplication()->input->get('contentSellerKey', false);
		$data		= $session->get($sellerkey, array());
		
		if (empty($data)) {
			return true;
		}

		foreach ($this->_tag_values as $tag){
			$key = $purpose.'_'.$tag;
			
			if(!isset($data[$key])){
				continue;
			}
			
			switch ($tag)
			{
				case "rightholder_id":
					$new->setParam('rightholder_id', $data[$key]);
					break;
					
				case "article_id":
					$new->setParam('article_id', $data[$key]);
					break;
					
				case "price":
					$new->setPrice($data[$key]);
					break;
				
				case "expiration_type":
					$new->setParam('expirationtype',$data[$key]);
					break;
					
				case "expiration_time":
					$new->setExpiration($data[$key]);
					break;
					
				case "recurrence_count":
					$new->setParam('recurrence_count',$data[$key]);
					break;
					
				case "trial_price_1":
					$new->setParam('trial_price_1',$data[$key]);
					break;
					
				case "trial_time_1":
					$new->setExpiration($data[$key], PAYPLANS_RECURRING_TRIAL_1);
					break;

				case "trial_price_2":
					$new->setParam('trial_price_2',$data[$key]);
					break;
					
				case "trial_time_2":
					$new->setExpiration($data[$key], PAYPLANS_RECURRING_TRIAL_2);
					break;
					
				case "currency":
					$new->setParam('currency',$data[$key]);
					break;
			}
		}
		$new->setParam('purpose', $purpose);
		$session->clear($sellerkey);
		
		return true;
	}
	
	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}
		
		$status 	= $new->getStatus();
		$sub_id 	= $new->getId();
		$user_id 	= $new->getBuyer();
		$article_id = $new->getParams()->get('article_id', false);
		
		if(!$article_id){
			return true;
		}
		
		if($status == PayplansStatus::SUBSCRIPTION_ACTIVE){
			PayplansHelperApp::addToResource($sub_id, $user_id, $article_id, $this->_resource);
		}
		
		if($status == PayplansStatus::SUBSCRIPTION_EXPIRED){
			PayplansHelperApp::removeFromResource($sub_id, $user_id, $article_id, $this->_resource);
		}
		
		return true;
	}
	//payplans-dashboard-right
	
	public function onPayplansViewBeforeRender(XiView $view, $task)
	{
		$user_id = PayplansFactory::getUser()->id;
		if(($view instanceof PayplanssiteViewDashboard) == false || !($user_id) || ($task != 'frontview')){
			return true;
		}
		
		$query = new XiQuery();
		$query->select('`value`')
				->from('`#__payplans_resource`')
				->where("`user_id` = $user_id")
				->where("`title` LIKE '$this->_resource'");
				
		$article_ids = $query->dbLoadQuery()->loadColumn();
		
		$article_ids = array_filter($article_ids);
		if(empty($article_ids)){
			return true;
		}
		
		$article_ids = implode(',', $article_ids);
		
		$query = new XiQuery();
		$query->select('`id`, `title`')
				->from('`#__content`')
				->where("`id` IN ($article_ids)");
				
		$articles = $query->dbLoadQuery()->loadObjectList('id');
		
		$params = $this->params;
		$title = $params->get('widget_title');
		$this->_assign('title', $title);
		$this->_assign('articles', $articles);
		return array('payplans-dashboard-right' => $this->_render('default_widget'));
	}
	
	public function processCallback(&$article)
	{
	 	$this->article_id 		= $article->id;
	 	$this->rightholder_id 	= $article->created_by;
	 	$this->cat_id			= $article->catid;
	 	
		// Search for this tag in the content
		$regex = "#{pp-contentseller-subscribe(.*?)}(.*?){/pp-contentseller-subscribe}#s";
		$article->text = preg_replace_callback( $regex, array($this, 'replaceTagCallback'), $article->text );
		
		$regex = "#{pp-contentseller-restrict(.*?)}(.*?){/pp-contentseller-restrict}#s";
		$article->text = preg_replace_callback( $regex, array($this, 'restrictContentCallback'), $article->text );
		
		return true;
	}
	
	/**
	 * preg_match callback to process each match
	 */
	public function restrictContentCallback($match)
	{
		$ret = '';
		
		if(!isset($match[2])){
			return $ret;
		}
		
		$user_id = (!isset($this->user_id)) ? XiFactory::getUser()->id : $this->user_id;
		
		//For admin, Nothing will be hidden
		if(XiFactory::getApplication()->isAdmin()){
			return $match[2];
		}
		
		//For handling case of {payplans}{/payplans}
		if(empty($match[1])){
			if($this->_accessCheck($user_id)){
				return $match[2];
			}
			return $ret;
		}
	}
	
	protected function doErrorLogForPlanid($article_id)
	{
		$msz = XiText::_('COM_PAYPLANS_APP_CONTENT_SELLER_PLAN_NOT_SET_LOG_TITLE');
		$content = array('Message' => XiText::_('COM_PAYPLANS_APP_CONTENT_SELLER_PLAN_NOT_SET_LOG_DESC'), 'article_id'=>$article_id);
		PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $msz, null, $content, 'PayplansFormatter','PayplansContentSeller');
		
		return true;
	}
	
	protected function doErrorLogForPurpose($article_id)
	{
		$msz = XiText::_('COM_PAYPLANS_APP_CONTENT_SELLER_PURPOSE_NOT_SET_LOG_TITLE');
		$content = array('Message' => XiText::_('COM_PAYPLANS_APP_CONTENT_SELLER_PURPOSE_NOT_SET_LOG_DESC'), 'article_id'=>$article_id);
		PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $msz, null, $content, 'PayplansFormatter','PayplansContentSeller');
		
		return true;
	}
}