<?php
/**
 * ebanx
 * This is payment gateway of Ebanx.
 * @copyright Copyright (C) 2009 - 2016 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
 * @license GNU/GPL, see LICENSE.php
 * @package payplans
 * @version 1.0.0
 * @author Team Readybytes <support+payplans@readybytes.in>
 */

if(defined( '_JEXEC' )==false) {
	die( 'Restricted access' );
}

class PayplansAppebanx extends PayplansAppPayment
{

	//inherited properties
	protected $_location = __FILE__;

	public function isApplicable($refObject = null, $eventName = '')
	{
		if ($eventName == 'onPayplansControllerCreation') {
			return true;
		}
		return parent::isApplicable($refObject, $eventName);
	}

	function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		if ($view == 'payment' && $task == 'notify') {
			$input = JFactory::getApplication()->input;
			$hash_codes = $input->get('hash_codes');
			$integration_key = $this->getAppParams()->get('integration_key', '');
			$sandbox = $this->getAppParams()->get('sandbox', false);
			if($sandbox){
				$requestUrl = 'https://sandbox.ebanx.com/ws/query';
			}
			else{
				$requestUrl = 'https://api.ebanx.com/ws/query';
			}

			$requestData = "integration_key=$integration_key&hash=$hash_codes";
			$link = new JURI($requestUrl);
			$curl = new JHttpTransportCurl(new JRegistry());
			$curl_result = $curl->request('POST', $link, $requestData);
			$response = json_decode($curl_result->body);
			if ($response->status == 'SUCCESS') {
				$paymentKey = $response->payment->merchant_payment_code;
				if (!empty($paymentKey)) {
					$input->set('payment_key', $paymentKey);
					return true;
				}
			}
			return true;
		}
		if ($view == 'payment' && $task == 'complete') {
			$input = JFactory::getApplication()->input;
			$paymentKey = $input->get('merchant_payment_code');
			if (!empty($paymentKey)) {
				$input->set('payment_key', $paymentKey);
				return true;
			}
			return true;
		} else {
			return true;
		}
	}

	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$integration_key = $this->getAppParams()->get('integration_key', '');
		$sandbox = $this->getAppParams()->get('sandbox', false);
		$currency_code = $invoice->getCurrency('isocode');
		$merchant_payment_code = $payment->getKey();
		$payment_type_code = '_all';
		$instalments = '1-12';
		$amount = $invoice->getTotal();
		$user = $payment->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		$email = $user->getEmail();
		$name = $user->getusername();
		$requestData = "integration_key=$integration_key&payment_type_code=$payment_type_code&instalments=$instalments&currency_code=$currency_code&amount=$amount&merchant_payment_code=$merchant_payment_code&name=$name&email=$email";
		if($sandbox){
			$requestUrl = 'https://sandbox.ebanx.com/ws/request';
		}
		else{
			$requestUrl = 'https://api.ebanx.com/ws/request';
		}

		$link = new JURI($requestUrl);
		$curl = new JHttpTransportCurl(new JRegistry());
		$curl_result = $curl->request('POST', $link, $requestData);
		$response = json_decode($curl_result->body);
		if ($response->status == 'SUCCESS') {
			$this->assign('post_url', $response->redirect_url);
			return $this->_render('form_standard');
		} else {
			$this->assign('code', $response->status_code);
			$this->assign('errormessage', $response->status_message);
			$message = XiText::_('COM_PAYPLANS_LOGGER_EBANX_ERROR_OCCURED_IN_PAYMENT_CREATION');
			$error = $response;
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($error), 'PayplansPaymentFormatter', '', true);
			return $this->_render('error');
		}

	}

	function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		if ($action == 'cancel') {
			return true;
		}

		if ($action == 'success') {
			$input = JFactory::getApplication()->input;
			$payment_key = $input->get('merchant_payment_code');

			XiRoute::_('index.php?option=com_payplans&view=payment&task=complete&payment_key=' . $payment_key);

		}
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}

	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		$input = JFactory::getApplication()->input;
		$hash_codes = $input->get('hash_codes');
		$integration_key = $this->getAppParams()->get('integration_key', '');
		$sandbox = $this->getAppParams()->get('sandbox', false);
		if($sandbox){
			$requestUrl = 'https://sandbox.ebanx.com/ws/query';
		}
		else{
			$requestUrl = 'https://api.ebanx.com/ws/query';
		}
		$requestData = "integration_key=$integration_key&hash=$hash_codes";
		$link = new JURI($requestUrl);
		$curl = new JHttpTransportCurl(new JRegistry());
		$curl_result = $curl->request('POST', $link, $requestData);
		$result = json_decode($curl_result->body);
		if ($result->status == 'SUCCESS') {
			$response = (array)$result->payment;

		}
		$validation = $this->_validateResponse($response, $payment);
		if ($validation) {
			$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
			// if same notification came more than one time
			// Check if transaction already exists,if yes then do nothing and return
			$txn_id = isset($response['hash']) ? $response['hash'] : 0;
			$subscr_id = isset($response['subscriptionid']) ? $response['subscriptionid'] : 0;
			$parent_id = isset($response['subscriptionid']) ? $response['subscriptionid'] : 0;
			$transactions = $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_id);
			if (!empty($transactions)) {
				foreach ($transactions as $transaction) {
					$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
					if ($transaction->getParam('status', '') == $response['status']) {
						return true;
					}
				}
			}
			// get the transaction instace of lib
			$transaction = PayplansTransaction::getInstance();
			$transaction->set('user_id', $payment->getBuyer())
				->set('invoice_id', $invoice->getId())
				->set('payment_id', $payment->getId())
				->set('gateway_txn_id', $txn_id)
				->set('gateway_subscr_id', $subscr_id)
				->set('gateway_parent_txn', $parent_id)
				->set('params', PayplansHelperParam::arrayToIni($response));

			//set the transaction message as per status

			if ($response['status'] == 'CO') {
				$transaction->set('amount', $response['amount_ext'])
					->set('message', 'COM_PAYPLANS_APP_EBANX_TRANSACTION_COMPLETED');

				return $transaction->save();

			}
		}
		return false;
	}

	protected function _validateResponse($response, $payment)
	{
		if ($response['status'] == 'CO') {
			return true;
		} else {
			$message = XiText::_('COM_PAYPLANS_LOGGER_EBANX_ERROR_OCCURED_IN_PAYMENT_CREATION');
			$error = $response;
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($error), 'PayplansPaymentFormatter', '', true);
			return false;
		}
	}

	public function supportForRefund()
	{
		return true;
	}

	public function refundRequest(PayplansTransaction $transaction, $refund_amount)
	{
		$hash = $transaction->getGatewayTxnId();
		$integration_key = $this->getAppParams()->get('integration_key', '');
		$sandbox = $this->getAppParams()->get('sandbox', false);

		$operation = 'request';
		$description = 'refund request';
		$requestData = "integration_key=$integration_key&operation=$operation&hash=$hash&amount=$refund_amount&description=$description";

		if($sandbox){
			$requestUrl = 'https://sandbox.ebanx.com/ws/refund';
		}
		else{
			$requestUrl = 'https://api.ebanx.com/ws/refund';
		}

		$link = new JURI($requestUrl);
		$curl = new JHttpTransportCurl(new JRegistry());
		$curl_result = $curl->request('POST', $link, $requestData);
		$result = json_decode($curl_result->body);
		if ($result->status == 'SUCCESS') {
			$response = (array)$result->payment;

		}
		$invoice = $transaction->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$refund_details = (array)$response['refunds']['0'];
		// if same notification came more than one time
		// Check if transaction already exists,if yes then do nothing and return
		$txn_id = isset($refund_details['id']) ? $refund_details['id'] : 0;
		$subscr_id = isset($response['payer']['id']) ? $response['payer']['id'] : 0;
		$parent_id = isset($response['parent_txn_id']) ? $response['parent_txn_id'] : 0;
		$transactions = $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_id);
		if (!empty($transactions)) {
			foreach ($transactions as $transaction) {
				$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
				if ($transaction->getParam('status', '') == $response['status']) {
					return true;
				}
			}
		}
		// get the transaction instace of lib
		$paymentId = PayplansHelperUtils::getIdFromKey($response['merchant_payment_code']);
		$newtransaction = PayplansTransaction::getInstance();
		$newtransaction->set('user_id', $transaction->getBuyer())
			->set('invoice_id', $invoice->getId())
			->set('payment_id', $paymentId)
			->set('gateway_txn_id', $txn_id)
			->set('gateway_subscr_id', $subscr_id)
			->set('gateway_parent_txn', $parent_id)
			->set('params', PayplansHelperParam::arrayToIni($response));


		if ($response['status'] == "CO" && $refund_details['status'] == 'RE') {
			$newtransaction->set('amount', -$refund_details['amount_ext'])
				->set('message', 'COM_PAYPLANS_APP_EBANX_TRANSACTION__REFUND_COMPLETED')
				->save();
			return $this->_render('refund_success');

		}
		else {
			$message = XiText::_('COM_PAYPLANS_LOGGER_EBANX_ERROR_OCCURED_IN_PAYMENT_REFUND');
			$error = $response;
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, (array)$error, 'PayplansPaymentFormatter', '', true);
			return false;
		}

	}


}
 
