<?php
/**
 * @copyright 	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
 * @license 	GNU/GPL, see LICENSE.php
 * @package 	payplans
 * @version 	1.0.0
 * @author 		support+customwork <support+customwork@readybytes.in>
 */

if(defined( '_JEXEC' )==false) {
	die( 'Restricted access' );
}

class PayplansAppEpay extends PayplansAppPayment
{
	//inherited properties
	protected $_location	= __FILE__;

	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansSubscriptionBeforeSave
		if($eventName == 'onPayplansSubscriptionBeforeSave'){
			return true;
		}

		return parent::isApplicable($refObject, $eventName);
	}

	public function isSupportPaymentCancellation($invoice)
	{
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}


	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		$invoice 			= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$post_url 			= ' https://ssl.ditonlinebetalingssystem.dk/integration/ewindow/Default.aspx';
		//this need to be done because epay accept payment only in cents
		$amount  = (($invoice->getTotal())*100);
		$this->assign('post_url', 		$post_url);
		$this->assign('merchantNumber', $this->getAppParam('merchant_id', ''));
		$this->assign('orderId', 		$payment->getKey());
		$this->assign('description', 	$invoice->getTitle());
		$this->assign('amount', 	    $amount);
		$this->assign('currency', 		$invoice->getCurrency('isocode','DKK'));
		$this->assign('subscription', 	$invoice->isRecurring()? 1 : 0);
		$this->assign('success_url',	JURI::root().'index.php?payment_key='.$payment->getKey().'&option=com_payplans&gateway=epay&view=payment&task=complete&action=success');
		$this->assign('cancel_url', 	JURI::root().'index.php?payment_key='.$payment->getKey().'&option=com_payplans&gateway=epay&view=payment&task=complete&action=cancel');
		//$this->assign('notify_url', 	JURI::root().'index.php?payment_key='.$payment->getKey().'&option=com_payplans&gateway=epay&view=payment&task=notify');


		return $this->_render('form_buynow');
	}


	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
			if($action == 'cancel'){
				return true;
			}

			//get transaction data from payment gateway.
			$txnId = $data['txnid'];

			if(!isset($txnId))
			{
				$message 					= XiText::_('COM_PAYPLANS_APP_EPAY_ERROR');
				$action						= 'error';
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter');
				return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);

			}

			$invoice 						= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);

			if(JDEBUG){
				// When debug mode is on dump in file
				file_put_contents(JPATH_SITE.DS.'tmp'.DS.time().$invoice->getKey(), var_export($data,true), FILE_APPEND);
			}
			//Duplicate Transactions Checking
			$transactions 					= $this->_getExistingTransaction($invoice->getId(), $txnId, 0,0);
			if($transactions !== false){
				foreach($transactions as $transaction){
					$transaction 			= PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
					$txnRefNum 				= $transaction->getParam('transactionid', '');
					if($txnRefNum == $data['transactionid']){
						return true;
					}
				}
			}

			$transaction 					= PayplansTransaction::getInstance();
			$transaction->set('user_id', 			$invoice->getBuyer())
						->set('invoice_id', 		$invoice->getId())
						->set('payment_id', 		$payment->getId())
						->set('amount',				$data['amount']/100)
						->set('message',            XiText::_('COM_PAYPLANS_APP_EPAY_PAYMENT_COMPLETED_SUCCESSFULLY'))
						->set('gateway_txn_id', 	$txnId)
						->set('gateway_subscr_id', 	isset($data['subscriptionid']) 	? $data['subscriptionid'] : 0)
						->set('gateway_parent_txn', 0)
						->set('params', 			PayplansHelperParam::arrayToIni($data))
						->save();

			if ($invoice->isRecurring()) {
				$recurrence_count			= $this->_getRecurrenceCount($invoice);
				$payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
				$payment->getGatewayParams()->set('subscriptionid',$data['subscriptionid']);
				$payment->save();
			}
			return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}

	public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$invoice_count = $invoiceCount + 1;

		if($invoice->isRecurring())
		{
			$recurrence_count 	= $payment->getGatewayParam('pending_recur_count');

			if($recurrence_count > 0)
			{
				$subscriptionId = $payment->getGatewayParam('subscriptionid');

				$amount = $invoice->getTotal($invoiceCount)*100;

				$epay_params = array();
				$epay_params['merchantnumber'] 	= $this->getAppParam('merchant_id', '');
				$epay_params['subscriptionid'] 	= $subscriptionId;
				$epay_params['orderid'] 		= time();
				$epay_params['amount'] 			= $amount;
				$epay_params['fraud'] 			= "0";
				$epay_params['currency'] 		= "208"/*$invoice->getCurrency('isocode','DKK')*/;
				$epay_params['instantcapture'] 	= "1";
				$epay_params['transactionid'] 	= "-1";
				$epay_params['pbsresponse'] 	= "-1";
				$epay_params['epayresponse'] 	= "-1";

				$client = new SoapClient('https://ssl.ditonlinebetalingssystem.dk/remote/subscription.asmx?WSDL');

				$result = $client->authorize($epay_params);
				if($result->authorizeResult == true) {
					$transaction 		= PayplansTransaction::getInstance();
					$transaction->set('user_id', 			$invoice->getBuyer())
								->set('invoice_id', 		$invoice->getId())
								->set('payment_id', 		$payment->getId())
								->set('amount',				$amount/100)
								->set('message',            XiText::_('COM_PAYPLANS_APP_EPAY_PAYMENT_COMPLETED_SUCCESSFULLY'))
								->set('gateway_txn_id', 	$result->transactionid)
								->set('gateway_subscr_id', 	$subscriptionId)
								->set('gateway_parent_txn', 0)
								->set('params', 			PayplansHelperParam::arrayToIni($result))
								->save();

					$recurrence_count 	= $recurrence_count -1;
					$payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
					$payment->save();
				} else {
					$message 		= XiText::_('COM_PAYPLANS_APP_EPAY_ERROR_RECURRING_PAYMENT');
					PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $result, 'PayplansPaymentFormatter');
					return false;
				}
			}
		}
	}

	public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$subscriptionId = $payment->getGatewayParam('subscriptionid');
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $subscriptionId);

		$epay_params = array();
		$epay_params['merchantnumber'] 	= $this->getAppParam('merchant_id', '');
		$epay_params['subscriptionid'] 	= $subscriptionId;
		$epay_params['epayresponse'] 	= "-1";
		$epay_params['pbsresponse'] 	= "-1";
		$client = new SoapClient('https://ssl.ditonlinebetalingssystem.dk/remote/subscription.asmx?WSDL');

		$result = $client->deletesubscription($epay_params);

		if($result->deletesubscriptionResult == true )
		{
				$params = new XiParameter();
				$params->set('$subscriptionId',$subscriptionId);
				$params->set('pending_recur_count', 0);
				$transaction->set('message', 'COM_PAYPLANS_PAYMENT_APP_EPAY_CANCEL_SUCCESS')
					->save();
				$payment->set('gateway_params', $params)
					->save();
				parent::onPayplansPaymentTerminate($payment, $controller);
				return $this->_render('cancel_success');
		}
		else {
			$transaction->set('message', 'COM_PAYPLANS_PAYMENT_APP_EPAY_CANCEL_ERROR')
				->save();
			return $this->_render('cancel_error');
		}
	}

	private function _getRecurrenceCount(PayplansInvoice $invoice)
	{
		$count = $invoice->getRecurrenceCount();
		if(intval($count) === 0){
			return 9999;
		}

		$recurring = $invoice->isRecurring();
		if($recurring){

			// Recurrence Count For Regular Recurring Plan
			if($recurring == PAYPLANS_RECURRING){
				$recurrence_count = $invoice->getRecurrenceCount();
			}
			// Recurrence Count For Recurring + Trial 1 Plan
			if($recurring == PAYPLANS_RECURRING_TRIAL_1){
				$recurrence_count = $invoice->getRecurrenceCount() + 1;
			}
			// Recurrence Count For Recurring + Trial 2 Plan
			if($recurring == PAYPLANS_RECURRING_TRIAL_2){
				$recurrence_count = $invoice->getRecurrenceCount() + 2;
			}
		}
		return $recurrence_count;
	}


	public function supportForRefund()
	{
		return true;
	}

	//function after payment is done
	public function refundRequest(PayplansTransaction $transaction, $refund_amount)
	{

		$txnId 		= $transaction->getGatewayTxnId();
		$amount  		= ($refund_amount*100);

		$epay_params = array();
		$epay_params['merchantnumber'] 	= $this->getAppParam('merchant_id', '');
		$epay_params['transactionid'] 	= $txnId;
		$epay_params['amount'] 			= $amount;
		$epay_params['pbsresponse'] 	= "-1";
		$epay_params['epayresponse'] 	= "-1";

		$client = new SoapClient('https://ssl.ditonlinebetalingssystem.dk/remote/payment.asmx?WSDL');

		$result = $client->credit($epay_params);


		if($result->creditResult == true){
			$paymentId = $transaction->getPayment();
			$newtransaction = PayplansTransaction::getInstance();
			$newtransaction->set('user_id', 			$transaction->getBuyer())
							->set('invoice_id', 		$transaction->getInvoice())
							->set('payment_id',		 	$paymentId)
							->set('gateway_subscr_id', 	0)
							->set('params', 			PayplansHelperParam::arrayToIni($result));
			$negativeAmt = -($refund_amount);
			$newtransaction->set('amount',$negativeAmt)
							->set('message', 'COM_PAYPLANS_PAYMENT_APP_EPAY_TRANSACTION_REFUNDED')
							->save();
			return true;
		}
		else{
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_EPAY_TRANSACTION_REFUND_ERROR');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $transaction, $result, 'PayplansPaymentFormatter', '', true);
			return false;
		}

	}

}
