<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	EuVat
* @contact		support+payplans@readybytes.in
*/
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

define('PLG_EUVAT_PURPOSE_PERSONAL', 1);
define('PLG_EUVAT_PURPOSE_BUSINESS', 2);

class plgPayplansEuvat extends XiPlugin
{
	public function onPayplansSystemStart()
	{
		$dir = dirname(__FILE__).DS.'euvat'.DS.'app';
		PayplansHelperApp::addAppsPath($dir);
		
		return true;
	}
	
	
	// on render of order, display output
	public function onPayplansViewBeforeRender(XiView $view, $task)
	{
		
		if(!(($view instanceof PayplanssiteViewInvoice) && $task == 'confirm') 
				&& !(($view instanceof PayplansadminViewInvoice) && $task == 'edit')){
			return true;
		}
		
		$invoiceId  	= $view->getModel()->getId();
		$invoice		= PayplansInvoice::getInstance($invoiceId);
		
		//if no app exist then don't do anything
		$apps = PayplansHelperApp::getApplicableApps('euvat', $invoice);
		if(empty($apps)){
			return true;
		}

		$country 		= 0;
		$purpose		= 0;
		$businessName	= '';
		$businessVatno	= '';
		$invoiceKey 	= $invoice->getKey();
		$userId 		= $invoice->getBuyer();
		
		//only apply the tax when invoice is in none or confirmed status. 
		if(!in_array($invoice->getStatus(), array(PayplansStatus::NONE, PayplansStatus::INVOICE_CONFIRMED))){
			return true;
		}

		if(!empty($userId)){
			$user 	  =  PayplansUser::getInstance($userId);
			$country  =  $user->getCountry();
			$businessVatno  =  $user->getPreference('tin');
			$businessName 	=  $user->getPreference('business_name');
			$purpose 		= $user->getPreference('business_purpose');
			
			// Apply tax as per user country
			self::_onPayplansTaxRequest($invoiceKey, $country, $purpose, $businessVatno, $businessName);
		}

		$this->_assign('country', $country);
		$this->_assign('invoiceId', $invoiceId);
		$this->_assign('invoice_key', $invoiceKey);
		$this->_assign('purpose', $purpose);
		$this->_assign('business_name', $businessName);
		$this->_assign('business_vatno', $businessVatno);
		
		if($view instanceof PayplanssiteViewInvoice){
			return array('payplans_order_confirm_payment' => $this->_render('invoiceconfirm'));
		}
		
		if($view instanceof PayplansadminViewInvoice){
			return array('payplans_invoice_edit_modifier' => $this->_render('invoiceedit'));
		}
		
		return true;
	}

	// 	on render of order, display output
    function onPayplansTaxRequest($invoiceKey, $country,$purpose=0, $businessNo='', $businessName='')
     {
     	list($invoice, $error, $proceed) = self::_onPayplansTaxRequest($invoiceKey, $country, $purpose, $businessNo, $businessName);
     	
		// order have been updated
		$response = XiFactory::getAjaxResponse();
		
		// if error = '', then no error will be shown
		foreach($error as $loc => $msg)
        {
			//if country selected is not in app then disable purpose fields
			if($msg == PAYPLANS_EUVAT_NOT_APPLICABLE){
				$response->addScriptCall('payplans.apps.euvat.disablefields');
			}
			else{
				$response->addScriptCall('payplans.apps.euvat.displayError',$msg,$loc);
			}
		}
        // enable fields again if country selected is in app
		if(in_array(true,$proceed))
		{
			$response->addScriptCall('payplans.apps.euvat.enablefields');
		}
		
  	    // this is for when appliying discount through admin panel
		if(XiFactory::getApplication()->isAdmin()){
			$response->addScriptCall('payplans.jQuery(\'input[name="discount"]\').val', $invoice->getDiscount());
			$response->addScriptCall('payplans.jQuery(\'input[name="taxamount"]\').val', $invoice->getTaxAmount());
			$response->addScriptCall('payplans.jQuery(\'input[name="total"]\').val', $invoice->getTotal());				
			$response->sendResponse();
		}
		
		$modifiers= $invoice->getModifiers();
		$modifiers  = PayplansHelperModifier::_rearrange($modifiers);
		if(!empty($modifiers)){
				$modified_html = $this->_loadTemplate('default_partial_invoice_modifier', compact('invoice'));
				$response->addScriptCall('xi.jQuery(\'#payplans\').find(\'.pp-modifiers\').remove');
		     	$response->addScriptCall('xi.jQuery(\'#payplans\').find(\'.pp-modifier-amount\').after', $modified_html);   
		  	}      
     	else {	
     			$response->addScriptCall('xi.jQuery(\'#payplans\').find(\'.pp-modifiers\').remove');
			 }
			 
		$response->addScriptCall('payplans.apps.euvat.validateCountryAndPurpose');
		$response->addScriptCall('payplans.jQuery(\'#payplans\').find(\'.regular-amount .pp-amount\').html',PayplansHelperFormat::displayAmount($invoice->getRegularAmount()));
		$response->addScriptCall('payplans.jQuery(\'#payplans\').find(\'.first-amount .pp-amount\').html',PayplansHelperFormat::displayAmount($invoice->getTotal()));
		
		$response->sendResponse();
	}
	
	// on render of order, display output
	public function _onPayplansTaxRequest($invoiceKey, $country, $purpose, $businessVat, $businessName)
	{
		//should allow user to proceed
		$proceed = array();
		
		$error = array();
		$error['#app-euvat-country-id-error']='';
		
		//trigger the discount
		$invoiceId = XiHelperUtils::getIdFromKey($invoiceKey);
		$invoice = PayplansInvoice::getInstance($invoiceId);
		
		
		$args  = array($invoice, $country, &$purpose, $businessVat);
		$results = PayplansHelperEvent::trigger('onPayplansApplyTax', $args, '', $invoice);

		
		$tax =0; // default tax is Zero
		foreach($results as $result){
			// 	check if app returned error string, convert to location aware error
			if(is_string($result)){
				$result = array('#app-euvat-country-id-error' => $result);
				// break ; // IMP : do not break, we need to process 
			}
			
			// check if app returned error string for particular location or tax rate
			if(is_array($result)){
				foreach($result as $key => $value){
					
					if(is_numeric($value)){
						$tax = $value;
						$proceed[] = true;
					}
					else {
						$error[$key] = (isset($error[$key]) &&  !empty($error[$key]) )? $error[$key]  : $value ;
						$proceed[] = false;
					}
				}
			}

			// tax rate was provided
			if(is_float($result)){
				$tax= $result;
				$proceed[] = true;
			}
		}
		$modifierRecord =  XiFactory::getInstance('modifier', 'model')
										->loadRecords(array('invoice_id' => $invoiceId,
															'type' => 'eu-vat'));
		// if modifier already exixts then load it
		$modifier = array_shift($modifierRecord);
										
		$id = 0;
		if($modifier && $modifier->modifier_id){
			$id = $modifier->modifier_id;
		}
		
		$modifier = PayplansModifier::getInstance($id);
		$modifier->set('message', XiText::_('COM_PAYPLANS_APP_EU_VAT_MODIFIER_MESSAGE'))
				 ->set('invoice_id', $invoiceId)
				 ->set('user_id', $invoice->getBuyer())
				 ->set('type', 'eu-vat')
				 ->set('amount', $tax) // Discount should be negative
				 ->set('reference', $country.'-'.$businessVat)
				 ->set('percentage', true)
				 ->set('frequency', PayplansModifier::FREQUENCY_EACH_TIME)
				 ->set('serial', PayplansModifier::PERCENT_TAX)
				 ->save();
		
		//save user country
		$user =  $invoice->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		$invoice->refresh()->save();
		$user->setPreference('business_purpose', $purpose)
			->setPreference('business_name', $businessName)
			->setPreference('tin', $businessVat)
			->setCountry($country)
			->save();

        return array($invoice, $error, $proceed);
	}
}
