<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	Discount
* @contact 		shyam@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppEuvat extends PayplansApp
{
	//inherited properties
	protected $_location	= __FILE__;
	
	// 	Entry Function 
	public function onPayplansApplyTax(PayplansInvoice $invoice, $countryCode, $purpose = 0, $vatno= false)
	{
		$applyTo = $this->getAppParam('tax_country', array());
		
		if(!in_array(PAYPLANS_CONST_ALL,$applyTo)){
			//When only one item is selected 
			if(is_array($applyTo )==false){
				$applyTo = array($applyTo );
			}
		
			// not applicable
			if(in_array($countryCode,$applyTo) ==false){
				return PAYPLANS_EUVAT_NOT_APPLICABLE;
			}
		}

		$response = XiFactory::getAjaxResponse();
		// return tax rate to be used
		if($purpose == PLG_EUVAT_PURPOSE_BUSINESS){
			
			// no need to verify VAT
			if($this->getAppParam('tax_check_vatno',1) == 0){
				return floatval($this->getAppParam('tax_rate_business', 0));
			}
			
			//verification of VAT required
			$result = $this->_validateVAT($countryCode, $vatno);
			
			// verification failed : success and apply personal tax if vat no. is not valid
        	//return ($result !==true) ? $result : floatval($this->getAppParam('tax_rate_business', 0));
            if($result !==true)
            {   
				if(is_array($result)){
	                foreach($result as $loc => $msg){
	                   $response->addScriptCall('payplans.apps.euvat.displayError',$msg,$loc);
	                   $response->addScriptCall('payplans.jQuery(\'#app_euvat_business_vatno\').addClass','invalid');
	                }
				}

                $purpose = PLG_EUVAT_PURPOSE_PERSONAL;
                return floatval($this->getAppParam('tax_rate_personal', 0));	
            }
            else{
            	$response->addScriptCall('payplans.jQuery(\'#app_euvat_business_vatno\').removeClass','invalid');
                return floatval($this->getAppParam('tax_rate_business', 0));
            }
		}
		
		return floatval($this->getAppParam('tax_rate_personal', 0));	
	}	
	
	protected function _validateVAT($country, $vatNumber)
	{
		// convert country id to country code
		$countries = XiFactory::getInstance('country', 'model')->loadRecords();
		$country = $countries[$country];
		$countryCode = $country->isocode2;
		
		// Construct the REST query
		if($countryCode == 'GR'){
			$countryCode = 'EL'; // Greece is a special case; the ISO code is GR, the VAT country code is EL. Duh!
		}
		try{
			//validate vatnumber through official way
			if(!$result = $this->_soapValidation($countryCode, $vatNumber)) {
				//validate vatnumber through some third party solution
				$result = $this->_curlValidation($countryCode, $vatNumber);		
			}
			
			//Set error if both extensions was not loaded 
			if(!$result ){
				// Invalid VAT or VAT validation service is down
				return array('#app-euvat-business-vatno-error' => XiText::_('PLG_EUVAT_VAT_VALIDATION_NO_RESPONSE')); 
			}
			
			// check property 'valid' which will available in response
			if(!isset($result->valid) || !$result->valid){
				// Invalid VAT or VAT validation service is down
				return array('#app-euvat-business-vatno-error' => XiText::_('PLG_EUVAT_VAT_VALIDATION_FAILED'));
			}
		}
		catch(Exception $e){
			return array('#app-euvat-business-vatno-error' => XiText::_('PLG_EUVAT_VAT_VALIDATION_FAILED'));
		}
		
		return true;
	}
	
	/**
	 * Soap method to validate vat number
	 * It return an object of stdclass have properties :
	 *  class stdClass (6) {
	 *			  public $countryCode => string(2) "COUNTRY_CODE"
	 *			  public $vatNumber   => string(9) "VAT_NUMBER"
	 *			  public $requestDate => string(16) "DATE REQUESTED"
	 *			  public $valid       => bool(true/false)
	 *			  public $name        => string(3) "---"
	 *			  public $address     => string(3) "---"
	 *			} 
	 * @param string $countryCode
	 * @param string $vatNumber
	 */
	protected function _soapValidation($countryCode,$vatNumber)
	{
		if(extension_loaded('soap')){
			$client   = new SoapClient("http://ec.europa.eu/taxation_customs/vies/checkVatService.wsdl");
			return $client->checkVat(array(
											  'countryCode' => $countryCode,
											  'vatNumber'   => $vatNumber
											 ));
		}
		return false;
	}
	
	/**
	 * Curl method to validate vat number 
	 * It returns json like :
	 *      {
	 *		  "response": {
	 *		    "country_code": "COUNTRY_CODE",
	 *		    "vat_number": "VAT_NUMBER",
	 *		    "valid": "false/true",
	 *		    "name": "---",
	 *		    "address": "---"
	 *		  }
	 *	   }
	 * @param string $countryCode
	 * @param string $vatNumber
	 */
	protected function _curlValidation($countryCode,$vatNumber)
	{
		if(extension_loaded('curl')){
			$curl   = new JHttpTransportCurl(new JRegistry());
			$result = $curl->request('GET', new JURI('http://vatid.eu/check/'.$countryCode.'/'.$vatNumber), null, array('Accept'=>'application/json'),30);
			$output = json_decode($result->body)->response;
			
			//true and false is in string, so convert it to boolean
			$output->valid = ($output->valid === 'true')? true: false;
			
			return $output;
		}
		return false;
	}
	
}

class PayplansAppEuvatFormatter extends PayplansAppFormatter
{
	// get rules to apply
	function getVarFormatter()
	{
		$rules = array( '_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       'function' => 'getAppPlans'),
						'app_params'      => array('formatter'=> 'PayplansAppEuvatFormatter',
										       	   'function' => 'getFormattedParams')
						);
		return $rules;
	}	
	
	function getFormattedParams($key, $value, $data)
	{
		$items = XiFactory::getInstance('country', 'model')->loadRecords();
		if(is_array($value['tax_country']))
		{
			foreach ($value['tax_country'] as $param){
			$country[] = $items[$param]->title;
			}
			$value['tax_country'] = $country;
		}
		else{
			$value['tax_country'] = $items[$value['tax_country']]->title;
		}
	}
}	
