<?php
/**
* @copyright	Copyright (C) 2009 - 2016 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	E-way Payment App
* @contact		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

/**
 * Eway payment gateway
 */
class PayplansAppEway extends PayplansAppPayment
{
	//inherited properties
	protected $_location	= __FILE__;
    private $_client;
    private $_responsecodes;
    private $_paymentMessage = false;

	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		//if($eventName == 'onPayplansSubscriptionBeforeSave'){
		if($eventName == 'onPayplansControllerCreation' || $eventName == 'updateCustomer'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}

    function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
    {
        if($view != 'payment' || ($task != 'notify') ){
            return true;
        }

        $paymentKey = JRequest::getVar('invoice', null);
        if(!empty($paymentKey)){
            JRequest::setVar('payment_key', $paymentKey, 'POST');
            return true;
        }

        return true;
    }

	// Website eway support different time period for trial subscription, so return true
	function isFirstExpirationtimeApplicable()
	{
		return true;
	}

	public function isSupportPaymentCancellation($invoice)
	{
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}

    //@todo add a check to see if SoapClient class exists, if not then use nusoap
    //This connects to the soap API so calls can be executed to send and receive data
    private function _ewaySoapGateway(PayplansInvoice $invoice)
    {

        $live 	  		= $this->getAppParam('sandboxTesting', false);
        $url 			= $live ? EWAY_PAYMENT_TOKEN_PAYMENTS_LIVE_MODE : EWAY_PAYMENT_TOKEN_PAYMENTS_TESTING_MODE;
        $ewayCustomerId = $this->getAppParam('ewayCustomrerID','87654321');
        $ewayUsername 	= $this->getAppParam('ewayUsername', "test@eway.com.au");
        $ewayPassword 	= $this->getAppParam('ewayPassword', "test123");

        try {
            $this->_client = @new SoapClient($url,array("exceptions" => 1));
        } catch (SoapFault $E) {
            $message 					= XiText::_('COM_PAYPLANS_APP_EWAY_SOAP_CONNECTION_ERROR');
			$error						= array();            
			$error['error_code'] 		= $E->faultcode;
            $error['error_message']		= $E->faultstring;
        	PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $error, 'PayplansPaymentFormatter', '', true);
            $getApp 					= XiFactory::getApplication();
            $getApp->enqueueMessage(XiText::_($error['error_message']),'error');
            $getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
        }

        $args = array(
            'eWAYCustomerID'	=> $ewayCustomerId,
            'Username'			=> $ewayUsername,
            'Password'			=> $ewayPassword);
        $header = new SoapHeader( "https://www.eway.com.au/gateway/managedpayment",'eWAYHeader',$args);
        $this->_client->__setSoapHeaders(array($header));
    }


	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
        $invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  = $invoice->getTotal();
		
		$paymentKey = $payment->getKey();
		
		$this->assign('amount', $amount);
		$this->assign('currency', $invoice->getCurrency());
		//$this->assign('method', $this->getAppParam('ewayMethod', 'REAL_TIME_CVN'));
        $user = $invoice->getBuyer(true);
        $realname = $user->getRealname();
        $name = explode(' ', $realname);
        if(is_array($name)){
            $firstname = $name[0];
          	$lastname =  isset($name[1])?$name[1]:'';
        }else{
            $firstname = $realname;
            $lastname = '';
        }

		$this->assign('firstname', $firstname);
		$this->assign('lastname', $lastname);
		$this->assign('email', $user->getEmail());
		$this->assign('post_url', XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&action=success&payment_key=".$paymentKey));
		$this->assign('cancel_url', XiRoute::_("index.php?option=com_payplans&gateway=eway&view=payment&task=complete&action=cancel&payment_key=".$paymentKey));
		$this->assign('payment_key', $paymentKey);
	  	return $this->_render('form');
	}

    public function get_url_contents($url)
    {
        $crl = curl_init();
        $timeout = 5;
        curl_setopt ($crl, CURLOPT_URL,$url);
        curl_setopt ($crl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt ($crl, CURLOPT_CONNECTTIMEOUT, $timeout);
        $ret = curl_exec($crl);
        curl_close($crl);
        return $ret;
    }

    function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
        $action = JRequest::getVar('action', false);
        if($action == 'cancel'){
            return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
        }

        XiError::assert(is_array($data) , XiText::_('COM_PAYPLANS_ERROR_INVALID_DATA_ARRAY'));

        if(isset($data['payment_key'])==false)
            return false;

        $invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
        //
        // Step 1 :- Create Customer Profile and Customer Payment Profile
        //
        $this->_ewaySoapGateway($invoice);
        $managedCustomerID = ($payment->getGatewayParam('profile_id'))?$payment->getGatewayParam('profile_id'):$this->_createCustomer($payment, $invoice, $data);

        // Case:- Customer profile created successfully now do your process
        $amount = number_format($invoice->getTotal(),2);

//        $error = XiText::_('COM_PAYPLANS_APP_EWAY_AMOUNT_PROCESSED');
//        $message = $amount;
//        PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $error, $payment, $message);

        // Case of Free Trial
        if($amount == '0.00' && $invoice->isRecurring()){
            $recurrence_count = $this->_getRecurrenceCount($invoice);
            $this->_processRecurringPayment($payment, $invoice,  $recurrence_count, $managedCustomerID, array());
            //we return here so no transactions get recorded and no payment gets processed.
            return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
        }
        else
        {
            // Step 2 :- After successful profile creation create Transaction Profile
            $record = (array)$this->_processPayment($invoice, $managedCustomerID);
			
            // if same notification came more than one time
	    	// Check if transaction already exists,if yes then do nothing and return
	    	$txn_id         = isset($record['ewayTrxnNumber']) ? $record['ewayTrxnNumber'] : 0;
	    	$subscr_id		= $managedCustomerID;
	    	$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, 0);
	    	if (!empty($transactions)) {
	    		foreach ($transactions as $transaction) {
	    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
	    			if(strtolower($transaction->getParam('ewayTrxnStatus','')) == strtolower($record['ewayTrxnStatus'])){
	    					return true;
	    			}
	    		}
	    	}
            
            // Case:- Error in Transaction Profile creation
	   		if(strtolower($record) == 'false' || $record === false){
                //we return here to because it means nothing was processed at eway so we should not continue on to record a transaction
                return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
            }

            // Case:- Connection Failed then show proper error message
            if(strtolower($record['ewayTrxnStatus']) == 'false'){
                $message 	= XiText::_('COM_PAYPLANS_APP_EWAY_TRANSACTION_ERROR');
                $error 		= $record['ewayTrxnError'].'('.$record['ewayTrxnNumber'].')';
                PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $record, 'PayplansPaymentFormatter', '', true);
                $getApp 	= XiFactory::getApplication();
                $getApp->enqueueMessage(XiText::_($error),'error');
                $getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
            }
        }
        //this records the transaction information into payplans so it can activate stuff or not
        if($invoice->isRecurring()){
            $recurrence_count = $this->_getRecurrenceCount($invoice);
			
            $this->_processRecurringPayment($payment, $invoice, $recurrence_count, $managedCustomerID, $record);
        }else{
            $this->_processNonRecurringPayment($payment, $invoice, $record,$managedCustomerID);
        }

        // save payment record
        $payment->save();

        return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
  }


    protected function _createCustomer(PayplansPayment $payment, $invoice, $data)
    {
        $params = array(
            'Title'=>$data['name_title'],
            'FirstName'=>$data['first_name'],
            'LastName'=>$data['last_name'],
            'Address'=>$data['address'],
            'Suburb'=>$data['city'],
            'State'=>$data['state'],
            'Company'=>$data['company'],
            'PostCode'=>$data['zip'],
            'Country'=>strtolower($data['country']),
            'Email'=>$data['email'],
            'Fax'=>$data['fax'],
            'Phone'=>$data['phone'],
            'Mobile'=>$data['mobile'],
            'CustomerRef'=>$invoice->getBuyer(),
            'JobDesc'=>"",
            'Comments'=>"",
            'URL'=>"",
            'CCNumber'=>trim($data['card_num']),
            'CCNameOnCard'=>$data['card_name'],
            'CCExpiryMonth'=>$data['exp_month'],
            'CCExpiryYear'=>$data['exp_year'],
        );
       /* $error = XiText::_('Customer Post Info');
        $message = PayplansHelperParam::arrayToIni($params);
        PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $error, $payment, $message);*/

        try {
            $result = $this->_client->CreateCustomer($params);
        } catch (SoapFault $E) {
            $message 				= XiText::_('COM_PAYPLANS_APP_EWAY_CREATE_CUSTOMER_ERROR');
			$error					= array();            
			$error['error_code']	= $E->faultcode;
            $error['error_message']	= $E->faultstring;
            PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $error, 'PayplansPaymentFormatter', '', true);
            $getApp 				= XiFactory::getApplication();
            $getApp->enqueueMessage(XiText::_($error['error_message']),'error');
            $getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
        }


        //managedcustomerID
        return $result->CreateCustomerResult;
    }

    //eWay Requires an integer amount so payplans amount is multiplied by 100 to get ti
    private function _processPayment(PayplansInvoice $invoice, $managedCustomerID)
    {
        $args = array($invoice);
        $message = PayplansHelperEvent::trigger('calculateMeteredPricing', $args, '', $invoice);
        $this->_paymentMessage = $message[0];

        $params = array(
            'managedCustomerID'=>(double)$managedCustomerID,
            'amount'=>(number_format($invoice->getTotal(),2,'.','')*100),
            'invoiceReference'=>$invoice->getId(),
            'invoiceDescription'=>$invoice->getTitle()
        );

        try {
            $result = $this->_client->ProcessPayment($params);
        } catch (SoapFault $E) {
            $message 				= XiText::_('COM_PAYPLANS_APP_EWAY_TRANSACTION_ERROR');
			$error					= array();              
			$error['error_code'] 	= $E->faultcode;
            $error['error_message'] = $E->faultstring;
            $log_id 	= PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $invoice, $error, 'PayplansPaymentFormatter', '', true);
            $getApp 	= XiFactory::getApplication();
            $getApp->enqueueMessage(XiText::_($error['error_message']),'error');
            $getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$invoice->getKey()));
        }

        //ewayTrxnError,ewayTrxnStatus,ewayTrxnNumber,ewayReturnAmount,ewayAuthCode
        if($result){
//            $error 	= XiText::_('COM_PAYPLANS_APP_EWAY_PROCESS_PAYMENT');
//            $message 		= PayplansHelperParam::arrayToIni($result);
//            $log_id 	= PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $error, $invoice, $message);
        return $result->ewayResponse;
        }else{
            return false;
        }
    }

    //eWay Requires an integer amount so payplans amount is multiplied by 100 to get it
    private function _processPaymentWithBeagle(PayplansInvoice $invoice){
        $this->_ewaySoapGateway($invoice);
        $params = array(
            'managedCustomerID'=>"9876543211000",
            'amount'=>(number_format($invoice->getTotal(),2,'.','')*100),
            'invoiceReference'=>"9876543211000",
            'invoiceDescription'=>"9876543211000",
            'cvn'=>"9876543211000",
            'ipAddress'=>"9876543211000",
            'billingCountry'=>"9876543211000"
        );

        try {
            $result = $this->_client->ProcessPaymentWithBeagle($params);
        } catch (SoapFault $E) {
            $message 	= XiText::_('COM_PAYPLANS_APP_EWAY_PROCESS_WITH_BEAGLE_ERROR');
            $error 		= $E->faultstring;
            $log_id 	= PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $error, $invoice, $message);
            $getApp 	= XiFactory::getApplication();
            $getApp->enqueueMessage(XiText::_($error),'error');
            $getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$invoice->getKey()));
        }


        //ewayTrxnError,ewayTrxnStatus,ewayTrxnNumber,ewayReturnAmount,ewayAuthCode
        $result->ewayResponse;
    }

    private function _processPaymentWithCVN(PayplansInvoice $invoice){
        $this->_ewaySoapGateway($invoice);
        $params = array(
            'managedCustomerID'=>"9876543211000",
            'amount'=>(number_format($invoice->getTotal(),2,'.','')*100),
            'invoiceReference'=>"9876543211000",
            'invoiceDescription'=>"9876543211000",
            'cvn'=>"9876543211000"
        );

        try {
            $result = $this->_client->ProcessPaymentWithCVN($params);
        } catch (SoapFault $E) {
            $message 	= XiText::_('COM_PAYPLANS_APP_EWAY_PROCESS_WITH_CVN_ERROR');
            $error 		= $E->faultstring;
            $log_id 	= PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $error, $invoice, $message);
            $getApp 	= XiFactory::getApplication();
            $getApp->enqueueMessage(XiText::_($error));
            $getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$invoice->getKey()));
        }

        //ewayTrxnError,ewayTrxnStatus,ewayTrxnNumber,ewayReturnAmount,ewayAuthCode
        $result->ewayResponse;
    }

    public function queryCustomer(PayplansInvoice $invoice)
    {
        $payment = $invoice->getPayment();
        if(!is_object($payment)){
            //print_r($invoice);exit;
            return false;
        }
        $managedCustomerID = $payment->getGatewayParam('profile_id');
       if(!$managedCustomerID){
           return false;
       }

        $this->_ewaySoapGateway($invoice);
        $params = array(
            'managedCustomerID'=>$managedCustomerID
        );


        try {
            $result = $this->_client->QueryCustomer($params);
        } catch (SoapFault $E) {
            $message 	= XiText::_('COM_PAYPLANS_APP_EWAY_QUERY_CUSTOMER_ERROR');
            $error 		= $E->faultstring;
            $log_id 	= PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $error, $invoice, $message);
            $getApp 	= XiFactory::getApplication();
            $getApp->enqueueMessage(XiText::_($error),'error');
            $getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$invoice->getKey()));
        }

        //CCName,CCNumber,CCExpiryMonth,CCExpiryYear
        //print_r($result);exit;
        return $result->QueryCustomerResult;
    }

    private function _queryCustomerByReference(PayplansInvoice $invoice){
        $this->_ewaySoapGateway($invoice);
        $params = array(
            'CustomerReference'=>"9876543211000"
        );

        try {
            $result = $this->_client->QueryCustomerByReference($params);
        } catch (SoapFault $E) {
            $message 	= XiText::_('COM_PAYPLANS_APP_EWAY_QUERY_BY_REFERENCE_ERROR');
            $error 		= $E->faultstring;
            $log_id 	= PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $error, $invoice, $message);
            $getApp 	= XiFactory::getApplication();
            $getApp->enqueueMessage(XiText::_($error),'error');
            $getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$invoice->getKey()));
        }

        //CCName,CCNumber,CCExpiryMonth,CCExpiryYear
        $result->QueryCustomerByReferenceResult;
    }

    private function _queryPayment(PayplansInvoice $invoice)
    {
        $this->_ewaySoapGateway($invoice);
        $params = array(
            'managedCustomerID'=>"9876543211000"
        );

        try {
            $result = $this->_client->QueryPayment($params);
        } catch (SoapFault $E) {
            $message 	= XiText::_('COM_PAYPLANS_APP_EWAY_QUERY_PAYMENT_ERROR');
            $error 		= $E->faultstring;
            $log_id 	= PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $error, $invoice, $message);
            $getApp 	= XiFactory::getApplication();
            $getApp->enqueueMessage(XiText::_($error),'error');
            $getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$invoice->getKey()));
        }

        //multiple
        //ManagedTransaction-> TotalAmount,Result,ResponseText,TransactionDate,ewayTrxnNumber
        //ManagedTransaction-> TotalAmount,Result,ResponseText,TransactionDate,ewayTrxnNumber
        $result->QueryPaymentResult;
    }

    public function updateCustomer(PayplansInvoice $invoice, $data)
    {
        $this->_ewaySoapGateway($invoice);
        $payment = $invoice->getPayment();
        $managedCustomerID = $payment->getGatewayParam('profile_id');
        if($managedCustomerID){
        $params = array(
            'managedCustomerID'=>$managedCustomerID,
            'Title'=>$data['name_title'],
            'FirstName'=>$data['first_name'],
            'LastName'=>$data['last_name'],
            'Address'=>$data['address'],
            'Suburb'=>$data['city'],
            'State'=>$data['state'],
            'Company'=>$data['company'],
            'PostCode'=>$data['zip'],
            'Country'=>strtolower($data['country']),
            'Email'=>$data['email'],
            'Fax'=>$data['fax'],
            'Phone'=>$data['phone'],
            'Mobile'=>$data['mobile'],
            'CustomerRef'=>$invoice->getBuyer(),
            'JobDesc'=>"",
            'Comments'=>"",
            'URL'=>"",
            'CCNumber'=>trim($data['card_num']),
            'CCNameOnCard'=>$data['card_name'],
            'CCExpiryMonth'=>$data['exp_month'],
            'CCExpiryYear'=>$data['exp_year'],
        );

        try {
            $result = $this->_client->UpdateCustomer($params);
        } catch (SoapFault $E) {
            $message 					= XiText::_('COM_PAYPLANS_APP_EWAY_UPDATE_CUSTOMER_ERROR');
			$error						= array();
			$error['error_code'] 		= $E->faultcode;
            $error['error_message'] 	= $E->faultstring;
            PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $invoice, $error, 'PayplansPaymentFormatter', '', true);
            $getApp 					= XiFactory::getApplication();
            $getApp->enqueueMessage(XiText::_($error['error_message']),'error');
            return false;
        }

        if ($result->UpdateCustomerResult) {
        	$message 	= XiText::_('COM_PAYPLANS_APP_EWAY_CUSTOMER_UPDATE_SUCCESS');
        	PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment);
			return true;
        }
        }else{
            return false;
        }
    }

    private function _getEwayUserPassword(PayplansInvoice $invoice)
    {
        // Get a database object
        $db		= JFactory::getDbo();
        $query	= $db->getQuery(true);
        $buyer = $invoice->getBuyer(true);
        $query->select('password');
        $query->from('#__users');
        $query->where('username=' . $db->Quote($buyer->getUsername()));

        $db->setQuery((string)$query);
        $password = $db->loadResult();

        if ($password) {
            $parts	= explode(':', $password);
            $crypt	= $parts[0];
            $salt	= @$parts[1];
            //this re-encrypts the enrypted password by the same salt previously used, in essence creating a new password based on the old password
            //note that if the user changes their password this will no longer work their profile will need to be recreated.
            //@todo add routine to update eway password??? if user changes it on main site.
            return JUserHelper::getCryptedPassword($crypt, $salt);
        }
        return false;
    }

    //this is triggered when a subscription is about to expire.
    public function processPayment(PayplansPayment $payment)
    {
        //this is the previous payment object?
        $invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);

        if($invoice->isRecurring())
        {
            $recurrence_count 	= $payment->getGatewayParam('pending_recur_count');

            if($recurrence_count > 0)
            {
                //$this->_defineCredentials();
                $this->_ewaySoapGateway($invoice);
                $profileId = $payment->getGatewayParam('profile_id');

                //possibly used to check if card has expired
                //$customerProfile = $this->_queryCustomer($profileId);

                $transaction = (array)$this->_processPayment($invoice, $profileId);

                //ewayTrxnError,ewayTrxnStatus,ewayTrxnNumber,ewayReturnAmount,ewayAuthCode
                if(strtolower($transaction['ewayTrxnStatus']) == 'false'){
                    $message 	= XiText::_('COM_PAYPLANS_APP_EWAY_INVALID_TRANSACTION_TYPE_OR_PAYMENT_STATUS');
                    $error 		= $transaction['ewayTrxnError'].'('.$transaction['ewayTrxnNumber'].')';
                    PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $transaction, 'PayplansPaymentFormatter', '', true);
                }

                if($transaction != false){
                    //$record = $this->_getTransactionParams($transaction);
//                    $message 	= XiText::_('COM_PAYPLANS_APP_EWAY_RECURRING_TRANSACTION_COMPLETE');
//                    $error 		= $transaction['ewayTrxnError'].'('.$transaction['ewayTrxnNumber'].')';
//                    $log_id 	= PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment, $error);
                    $this->_processRecurringPayment($payment, $invoice, $recurrence_count, $profileId, $transaction);
                }
            }
        }
    }

    //@todo should we clear out the saved profile??
    public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
    {
/*        $invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
        $profileId = $payment->getGatewayParam('profile_id');

        //$customerProfile = $this->getCustomerProfile($profileId);
        $customerProfile = $this->_queryCustomer($profileId);

        if($customerProfile !== false)
        {
            $deleteProfile = $this->deleteCustomerPaymentProfile($payment, $profileId, $customerProfile['paymentProfileId']);

            if($deleteProfile != false)
            {
                return $this->_render('cancel_success');
            }

            return $this->_render('cancel_error');
        }*/
        //$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
       // $invoice->terminate();
       // return array();
        //$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
        //$this->assign('expiration', PayplansHelperParam::arrayToIni($invoice->getCurrentExpiration()));
		parent::onPayplansPaymentTerminate($payment, $controller);
    	return $this->_render('cancel_success');
    }


    private function _getRespnseCode($response)
    {
		//sets the error codes and message
        $this->_responsecodes = new stdClass();
        $this->_responsecodes->deny = array("01"=>"Refer to Issuer","02"=>"Refer to Issuer, special","03"=>"No Merchant","04"=>"Pick Up Card","05"=>"Do Not Honour","06"=>"Error",
            "07"=>"Pick Up Card, Special","08"=>"Honour With Identification","09"=>"Request In Progress","10"=>"Approved For Partial Amount","12"=>"Invalid Transaction",
            "13"=>"Invalid Amount","14"=>"Invalid Card Number","15"=>"No Issuer","19"=>"Re-enter Last Transaction","21"=>"No Action Taken",
            "22"=>"Suspected Malfunction","23"=>"Unacceptable Transaction Fee","25"=>"Unable to Locate Record On File","30"=>"Format Error","31"=>"Bank Not Supported By Switch",
            "33"=>"Expired Card, Capture","34"=>"Suspected Fraud, Retain Card","35"=>"Card Acceptor, Contact Acquirer, Retain Card","36"=>"Restricted Card, Retain Card",
            "37"=>"Contact Acquirer Security Department, Retain Card","38"=>"PIN Tries Exceeded, Capture","39"=>"No Credit Account","40"=>"Function Not Supported",
            "41"=>"Lost Card","42"=>"No Universal Account","43"=>"Stolen Card","44"=>"No Investment Account","51"=>"Insufficient Funds","52"=>"No Cheque Account",
            "53"=>"No Savings Account","54"=>"Expired Card","55"=>"Incorrect PIN","56"=>"No Card Record","57"=>"Function Not Permitted to Cardholder","58"=>"Function Not Permitted to Terminal",
            "59"=>"Suspected Fraud","60"=>"Acceptor Contact Acquirer","61"=>"Exceeds Withdrawal Limit","62"=>"Restricted Card","63"=>"Security Violation",
            "64"=>"Original Amount Incorrect","66"=>"Acceptor Contact Acquirer, Security","67"=>"Capture Card","75"=>"PIN Tries Exceeded","82"=>"CVV Validation Error",
            "90"=>"Cutoff In Progress","91"=>"Card Issuer Unavailable","92"=>"Unable To Route Transaction","93"=>"Cannot Complete, Violation Of The Law",
            "94"=>"Duplicate Transaction","96"=>"System Error");

        $this->_responsecodes->accept = array("00"=>"Transaction Approved","11"=>"Approved, VIP","16"=>"Approved, Update Track 3");

        if(strpos($response["EWAYTRXNERROR"],',')){
            $pieces = explode(',',$response["EWAYTRXNERROR"]);
            $response["EWAYTRXNERROR"] = $pieces[0];
        }
        $return = array();
        //if($data["EWAYTRXNERROR"] == 00 || $data["EWAYTRXNERROR"] == 11 || $data["EWAYTRXNERROR"] == 16){
        if(array_key_exists($response["EWAYTRXNERROR"],$this->_responsecodes->accept)){
            $return['accept']   = true;
            $return['deny']     = false;
            $return['message']  = $this->_responsecodes->accept[$response["EWAYTRXNERROR"]];
            return $return;
        }else if(array_key_exists($response["EWAYTRXNERROR"],$this->_responsecodes->deny)){
            $return['deny']     = true;
            $return['accept']   = false;
            $return['message']  = $this->_responsecodes->deny[$response["EWAYTRXNERROR"]];
            return $return;
        }else{
            $return['deny']     = false;
            $return['accept']   = false;
            $return['message']  = '';
            return $return;
        }

    }

    private function _getRecurrenceCount(PayplansInvoice $invoice)
    {
        $count = $invoice->getRecurrenceCount();
        if(intval($count) === 0){
            return 9999;
        }

        $recurring = $invoice->isRecurring();
        if($recurring){

            // Recurrence Count For Regular Recurring Plan
            if($recurring == PAYPLANS_RECURRING){
                $recurrence_count = $invoice->getRecurrenceCount();
            }
            // Recurrence Count For Recurring + Trial 1 Plan
            if($recurring == PAYPLANS_RECURRING_TRIAL_1){
                $recurrence_count = $invoice->getRecurrenceCount() + 1;
            }
            // Recurrence Count For Recurring + Trial 2 Plan
            if($recurring == PAYPLANS_RECURRING_TRIAL_2){
                $recurrence_count = $invoice->getRecurrenceCount() + 2;
            }
        }
        return $recurrence_count;
    }

    private function _processRecurringPayment(PayplansPayment $payment, PayplansInvoice $invoice, $recurrence_count, $customerProfileId, $record = array())
    {
        $recurrence_count 	    = $recurrence_count - 1;
        $amount					= number_format($invoice->getTotal(), 2);
        $transaction            = PayplansTransaction::getInstance();
        //ewayTrxnError,ewayTrxnStatus,ewayTrxnNumber,ewayReturnAmount,ewayAuthCode
        $transaction->set('user_id', $invoice->getBuyer())
            ->set('invoice_id', $invoice->getId())
            ->set('payment_id', $payment->getId())
            ->set('amount', number_format(($record['ewayReturnAmount'] / 100), 2,'.',''))
            ->set('gateway_parent_txn', 0)
            ->set('gateway_subscr_id', $customerProfileId)
            ->set('gateway_txn_id', isset($record['ewayTrxnNumber']) ? $record['ewayTrxnNumber'] : 0)
            ->set('message', $this->_paymentMessage?$this->_paymentMessage:'COM_PAYPLANS_PAYMENT_EWAY_PAYMENT_COMPLETED_SUCCESSFULLY')
            ->set('params', PayplansHelperParam::arrayToIni($record))
            ->save();

        $params = array('profile_id' => $customerProfileId,
            'pending_recur_count' => $recurrence_count);

        $payment->set('gateway_params', PayplansHelperParam::arrayToIni($params))
            ->save();


    }

    private function _processNonRecurringPayment(PayplansPayment $payment, PayplansInvoice $invoice, $record = array(),$managedCustomerID)
    {
        $amount					= number_format($invoice->getTotal(), 2);
        //ewayTrxnError,ewayTrxnStatus,ewayTrxnNumber,ewayReturnAmount,ewayAuthCode
        // do this for normal payment
        // save transaction for normal payment
        $transaction = PayplansTransaction::getInstance();
        $transaction->set('user_id', $invoice->getBuyer())
            ->set('invoice_id', $invoice->getId())
            ->set('payment_id', $payment->getId())
            ->set('amount', number_format(($record['ewayReturnAmount'] / 100), 2,'.',''))
            ->set('gateway_txn_id', isset($record['ewayTrxnNumber']) ? $record['ewayTrxnNumber'] : 0)
            ->set('gateway_subscr_id', isset($managedCustomerID) ? $managedCustomerID : 0)
            ->set('gateway_parent_txn', 0)
            ->set('message', $this->_paymentMessage?$this->_paymentMessage:'COM_PAYPLANS_PAYMENT_EWAY_PAYMENT_COMPLETED_SUCCESSFULLY')
            ->set('params', PayplansHelperParam::arrayToIni($record))
            ->save();

    }



    public function getRecurrenceTime($expTime)
    {
        $expTime['year'] = isset($expTime['year']) ? intval($expTime['year']) : 0;
        $expTime['month'] = isset($expTime['month']) ? intval($expTime['month']) : 0;
        $expTime['day'] = isset($expTime['day']) ? intval($expTime['day']) : 0;
        $expTime['hour'] = isset($expTime['hour']) ? intval($expTime['hour']) : 0;

        // if only hours are set then return days as it is
        if(!empty($expTime['hour'])){
            $hours = $expTime['hour'];

            if(!empty($expTime['day'])){
                $hours += $expTime['day'] * 24;

                if(!empty($expTime['month'])){
                    $hours += $expTime['month'] * 30 * 24;

                    if(!empty($expTime['year'])){
                        $hours += $expTime['year'] * 365 * 24;
                    }
                }
            }
            return array('period' => $hours, 'unit' => 'Hour', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'));
        }


        // if only days are set then return days as it is
        if(!empty($expTime['day'])){
            $days = $expTime['day'];

            if(!empty($expTime['month'])){
                $days += $expTime['month'] * 30;

                if(!empty($expTime['year'])){
                    $days += $expTime['year'] * 365;
                }
            }
            return array('period' => $days, 'unit' => 'Day', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'));
        }

        // if months are set
        if(!empty($expTime['month'])){
            $month = $expTime['month'];

            if(!empty($expTime['year'])){
                $month += $expTime['year'] * 12 ;
            }

            return array('period' => $month, 'unit' => 'Month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'));
        }

        // years
        if(!empty($expTime['year'])){
            return array('period' => $expTime['year'], 'unit' => 'Year', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'));
        }

        return false;
    }

    public function supportForRefund()
    {
    	return true;
    }
    
    public function refundRequest(PayplansTransaction $transaction, $refund_amount)
	{
		$invoice = $transaction->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$this->_ewaySoapGateway($invoice);
		
		$profileId 	   		  = $transaction->getGatewaySubscriptionId();
		$refund_amount 		  = (number_format($refund_amount,2,'.','')*100);
		$gatewayTransactionId = $transaction->getGatewayTxnId();
		$password 			  = $this->_getEwayUserPassword($invoice);      
        $params 			  = "<ewaygateway><ewayCustomerID>{$profileId}</ewayCustomerID><ewayOriginalTrxnNumber>{$gatewayTransactionId}</ewayOriginalTrxnNumber>"
								."<ewayTotalAmount>{$refund_amount}</ewayTotalAmount><ewayCardExpiryMonth></ewayCardExpiryMonth>"
								."<ewayCardExpiryYear></ewayCardExpiryYear><ewayOption1></ewayOption1><ewayOption2></ewayOption2><ewayOption3></ewayOption3>"
								."<ewayRefundPassword>{$password}</ewayRefundPassword></ewaygateway>";
        
		$response 			  = $this->_client->__doRequest($params,"https://www.eway.com.au/gateway/xmlpaymentrefund.asp","POST","1.2");
		$response 			  = simplexml_load_string($response);
		
		//refund response transaction status is true then terminate the order
		
		if(strtolower($response['ewayTrxnStatus']) == True){
			
    		$payment 	= $transaction->getPayment(PAYPLANS_INSTANCE_REQUIRE);
    		
    		if($payment != false || !empty($payment)){
    			
    			 $newtransaction = PayplansTransaction::getInstance();
				 $newtransaction->set('user_id', 			$transaction->getBuyer())
								->set('invoice_id', 		$transaction->getInvoice())
								->set('payment_id',		 	$payment->getId())
								->set('gateway_txn_id',	 	isset($response['ewayTrxnNumber']) ? $response['ewayTrxnNumber'] : 0)
								->set('gateway_subscr_id', 	$profileId)
								->set('gateway_parent_txn', 0)
								->set('params', 			PayplansHelperParam::arrayToIni($response));

				$negativeAmt = -($response['ewayReturnAmount']/100);
		    	$newtransaction->set('amount',$negativeAmt)
		    				->set('message', 'COM_PAYPLANS_APP_PAYSEAL_TRANSACTION_COMPLETED')
		    				->save();
    		}
		    		
    		return true; 
		}
		
		return false;
	}
}
