<?php
/**
* @copyright	Copyright (C) 2009 - 2014 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppFastspring extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}

	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{	
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);

		if (isset($_REQUEST['id'])) {
			return XiFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=invoice&task=thanks&invoice_key='.$invoice->getKey()));
		}

		if (is_object($data)) {
		$data = (array)$data;
		}

		$storefrontUrl = $this->getAppParam('storefront_url',false);
		$accessKey = $this->getAppParam('access_key',false);
		$amount = $invoice->getTotal();
		$title = $invoice->getTitle();
		$productId = $this->getPlanMapping($payment);

		$this->assign('storefront_url', $storefrontUrl);
		$this->assign('access_key', 	$accessKey);
		$this->assign('amount', $amount);
		$this->assign('title', $title);
		$this->assign('product_id', $productId);
		$this->assign('payment',    $payment);

		return $this->_render('form');
	}

	//mapping of g2s rebilling products and payplans recurring plans
	public function getPlanMapping($payment)
	{
		$planMapping = $this->getAppParam('plan_product_mapping');
		$plans = $payment->getPlans();
		$planId = array_shift($plans);
		
		$productId = "";			
		foreach ($planMapping as $key => $mapping) {
			if($mapping['0'] == $planId) {   				
			 	$productId	 = $mapping['1'];
				break;
			}
		}

		return $productId;
	}
	
	public function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{		
		if($view != 'payment' || ($task != 'notify') ){
			return true;
		}

		$contents = file_get_contents('php://input');
		$obj = json_decode($contents);
		
		$data = array_pop($obj->events);
		$paymentData = $data->data->tags;

		$paymentKey = $paymentData->payment_key;
		if ($paymentKey) {
			JRequest::setVar('payment_key', $paymentData->payment_key, 'POST');
			JRequest::setVar('ipn_data', $contents, 'POST');
		}

		return true;
	}
	
	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);

		$contents = file_get_contents('php://input');
		$obj = json_decode($contents);
		
		$ipnData = array_pop($obj->events);
		$paymentData = $ipnData->data;
		
		// get the transaction instace of lib
  		$transaction = PayplansTransaction::getInstance();
      	$transaction->set('user_id', $payment->getBuyer())
				    ->set('invoice_id', $invoice->getId())
				    ->set('payment_id', $payment->getId())
				    ->set('gateway_txn_id', isset($paymentData->order) ? $paymentData->order : "" )
				    ->set('gateway_subscr_id', isset($paymentData->reference) ? $paymentData->reference : "" )
				    ->set('gateway_parent_txn', 0)
				    ->set('params', PayplansHelperParam::arrayToIni($ipnData));

	      if ($paymentData->completed) {
  			$transaction->set('amount', $invoice->getTotal())
		              	->set('message', XiText::_('COM_PAYPLANS_PAYMENT_APP_FASTSPRING_TRANSACTION_COMPLETED'));

	      } else {
		  		$transaction->set('amount',0)
		              		->set('message', 'COM_PAYPLANS_APP_FASTSPRING_TRANSACTION_NOT_COMPLETED');
	      }

	      if (!$transaction->save()) {
				$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
	      }

	      //store the response in the payment AND save the payment
	      if (!$transaction->save()) {
				$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
	      }
	      
	      $payment->save();
	}

}
