<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @contact		bhavya@readybytes.in
*/
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Payplans Fixed-Date-Expiration
 */
class plgPayplansFixeddateexpiration extends XiPlugin
{	
	public function onPayplansSystemStart()
	{
		//add discount app path to app loader
		$appPath = dirname(__FILE__).DS.'fixeddateexpiration'.DS.'app';
		PayplansHelperApp::addAppsPath($appPath);

		return true;
	}
	
	public function onPayplansViewBeforeRender(XiView $view, $task)
	{
		if($view instanceof PayplanssiteViewPlan){
			if($task== 'subscribe'){
				$plans = $view->get('plans');
				$this->_newDateMapping($plans, $view);
			}
			
			if($task=='login'){
				$plan = $view->get('plan');
				$plans[$plan->getId()] = $plan;
				$this->_newDateMapping($plans, $view);
			}
		}
		
		if(($view instanceof PayplanssiteViewInvoice) && $task == 'confirm'){
			$invoice = $view->get('invoice');
			if($invoice->isRecurring()){
				return true;
			}
			
            $getplans = $invoice->getPlans(PAYPLANS_INSTANCE_REQUIRE);
			$plan = array_shift($getplans);
			$plans[$plan->getId()] = $plan;
			$newdates = $this->_newDateMapping($plans, $view);
			
			if(array_key_exists($plan->getId(), $newdates)){
				$document = PayplansFactory::getDocument();
				$document->addScriptDeclaration("var fdeInvoiceConfirm = true ;");
			}
		}
	}
	
	function _newDateMapping($plans, $view)
	{
		if(empty($plans)){
			return true;
		}
		
		$latest = true;
		//if payplans version is 2.1 then display date as per shown at other screens  
		$version = explode('.', PAYPLANS_VERSION);
		if($version[0] == 2 && $version[1] == 1){
			$latest = false;
		}
		
		$apps  = PayplansHelperApp::getAvailableApps('fixeddateexpiration');
		if(empty($apps))
		{
			return array();
		}
		$newDates = array();
		
		foreach ($apps as $app){
			$expirationDate = $app->getAppParam('expiration_date', 0);
			if(empty($expirationDate)){
				continue;
			}
			
			$result = $this->_dateChangeApplicable($app);
			if ($result == false){
				continue ;
			}

			$app_plans  = is_array($app->_appplans) ? $app->_appplans : array($app->_appplans);			
			foreach ($plans as $id => $plan){
				if($app->getParam('applyAll', false) || in_array($id, $app_plans)){		 			
					$format 	        = '%Y-%m-%d %H:%M';
					$UTCexpirationDate  = gmdate('d.m.Y H:i', strtotime($expirationDate));
					$date               = new XiDate($expirationDate);
		 			$newDates[$id] = $date->toFormat($format);
		 			
		 			if($latest === true){
		 				$newDates[$id] = PayplansHelperFormat::date($date);
		 			}
				}
			}
		}

		//if not applicable then do nothing
		if(!isset($newDates) || empty($newDates)){
			return array();
		}

		JText::script('COM_PAYPLANS_APP_FIXEDDATEEXPIRATION_JS_SEPARATOR');
		PayplansHtml::script(dirname(__FILE__).DS.'fixeddateexpiration'.DS.'tmpl'.DS.'fixeddateexpiration.js');
		
		// add some javascript variable in document
		// so that, these variables can be used in js file
		$document = PayplansFactory::getDocument();
		$document->addScriptDeclaration("var fdeExpDates = ".json_encode($newDates).";");
		return $newDates;
	}
	
	protected function _dateChangeApplicable($app)
	{
		$currentDate = new XiDate('now');
		
		$from = $app->getAppParam('subscription_from', 0);
		$to   = $app->getAppParam('subscription_to', 0);
		
		$from = !empty($from) ? new XiDate($from) : $from;
		$to   = !empty($to)   ? new XiDate($to)   : $to;
		
		//when range is not set then change date anyway
		if(empty($from) && empty($to)){
			return true;
		}
		
		// when range is set then check subscription date whether lies in that range 
		if(!empty($from) && !empty($to) && ($currentDate->toUnix() >= $from->toUnix()) && ($currentDate->toUnix() <= $to->toUnix())){
			return true;
		}                                             
		
		//when only start date is set
		if(!empty($from) && empty($to) && $currentDate->toUnix() >= $from->toUnix() ){
			return true;
		}
		
		//when only end date is set
		if(empty($from) && !empty($to) && $currentDate->toUnix() <= $to->toUnix() && empty($from)){
			return true;
		}
		
		return false;
	}
}
