<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		bhavya@readybytes.in
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppFixeddateexpiration extends PayplansApp
{
	protected $_location	= __FILE__;
	
	public function isApplicable($refObject = null, $eventName='')
	{
		$option = JRequest::getVar('option');
		if($eventName === 'onPayplansViewBeforeRender'){
			return true;
		}

		return parent::isApplicable($refObject, $eventName);
	}
	
	public function onPayplansSubscriptionBeforeSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}

		//do nothing in case of recurring subscription
		if($new->isRecurring()){
			return true;
		}
		$expirationDate = $this->getAppParam('expiration_date', 0);
		$expireOn  = gmdate('d.m.Y H:i', strtotime($expirationDate));
		
		if(empty($expireOn)){
			return true;
		}

		$newStatus = $new->getStatus();
		//if subscription is active
		if($newStatus == PayplansStatus::SUBSCRIPTION_ACTIVE){
			return $this->_checkActivationDate($new, $expireOn);
		}
		
		return true;
	}
	
	protected function _checkActivationDate($new, $expireOn)
	{
		$expiration       = new XiDate($expireOn);
		$subscriptionDate = $new->getSubscriptionDate();
		$actualExpiration = $new->getExpirationDate();
		$planislifetime	  = $actualExpiration->toUnix();
		
		//when extend subscription is set to No and 
		//check for Life time plan 
		//actual exp date is closer than that of fixed exp date then do nothing
		if(($this->getAppParam('extend_subscription', 0) == 0) && $planislifetime && ($expiration->toUnix() > $planislifetime)){
			return true;
		}
		
		$from = $this->getAppParam('subscription_from', 0);
		$to   = $this->getAppParam('subscription_to', 0);
		
		$from = !empty($from) ? new XiDate($from) : $from;
		$to   = !empty($to)   ? new XiDate($to)   : $to;
		
		//when subscription range is set and current subscription does not lie within that range
		if(!empty($from) && !empty($to) && ($subscriptionDate->toUnix() < $from->toUnix()) && ($subscriptionDate->toUnix() > $to->toUnix())){
			return true;
		}
		
		//when range is not set then change exp date anyway
		if(empty($from) && empty($to)){
			return $this->_changeExpirationDate($new, $expiration);
		}
		
		// when range is set then check subscription date whether lies in that range 
		if(!empty($from) && !empty($to) && ($subscriptionDate->toUnix() >= $from->toUnix()) && ($subscriptionDate->toUnix() <= $to->toUnix())){
			return $this->_changeExpirationDate($new, $expiration);
		}                                             
		
		//when start date is set
		if(!empty($from) && $subscriptionDate->toUnix() >= $from->toUnix()){
			return $this->_changeExpirationDate($new, $expiration);
		}
		
		//when end date is set
		if(!empty($to) && $subscriptionDate->toUnix() <= $to->toUnix()){
			return $this->_changeExpirationDate($new, $expiration);
		}
		
		return false;
	}
	
	protected function _changeExpirationDate($new, $expirationDate)
	{
		$currentDate   = new XiDate('now');
		
		//do nothing when current date is greater than the expiration date 
		if($currentDate->toMySQL() > $expirationDate->toMySQL()){
			return true;
		}
		
		$new->setExpirationDate($expirationDate);
	}
}
