<?php
/**
* @copyright	Copyright (C) 2009 - 2013 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		support+payplans@readybytes.in
* @author		Puneet Singhal
*/
if(defined('_JEXEC')===false) die();


class PayplansAppGate2shop extends PayplansAppPayment
{
	protected $_location = __FILE__;

	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		$this->_getAppParams();
		
    	$paymentKey	= $payment->getKey();
		$invoice 	= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$title 		= $invoice->getTitle();
		$now 		= new XiDate('now');
		
		$amount  	= $invoice->getTotal();
		$title		= str_replace(' ', '_', $title);
    	$currency	= $invoice->getCurrency('isocode');
    	$timestamp	= $now->toMySQL();
    	
    	$checksum 	= $this->secret_key.$this->merchant_id.$currency.$amount.$title.$amount.'1'.$timestamp;
    	$checksum 	= md5($checksum);
    	
    	// Set values of custom fields
    	$customField1 = 'payplans';
    	$customField2 = 'gate2shop';
    	$customField3 = $paymentKey;
    	$customField5 = $this->getAppParam('random_key', 'author');
    	$customField4 = $customField1.$customField2.$customField3.$customField5;
    	$customField4 = md5($customField4);
    	
    	
    	// Set variables for template
    	$this->assign('merchant_site_id', 	$this->site_id);
    	$this->assign('merchant_id', 		$this->merchant_id);
    	$this->assign('version', 			$this->version);
    	
    	$this->assign('total_amount', 		$amount);
		$this->assign('currency', 			$currency);
		
		$this->assign('item_name_1', 		$title);
		$this->assign('item_amount_1', 		$amount);
		$this->assign('item_quantity_1', 	1);
    	
    	$this->assign('checksum', 		$checksum);
    	$this->assign('time_stamp', 	$timestamp);
		
    	$this->assign('customField1', $customField1);
    	$this->assign('customField2', $customField2);
    	$this->assign('customField3', $customField3);
    	$this->assign('customField4', $customField4); // md5 = payplans + gate2shop + payment_key + random_key
		$this->assign('customField5', $customField5);
		
		$this->assign('post_url', 	$this->_getPostUrl());
		
		//Sends Required parameter for Recurring subscription
		$recurring = $invoice->isRecurring();
	   	if($recurring == PAYPLANS_RECURRING){
	   		list($productId, $templateId) = $this->getPlanMapping($payment);
   		
	   		$this->assign('recurring', true);
	   		$this->assign('isRebilling', true);
	   		$this->assign('rebillingProductId', $productId);
	   		$this->assign('rebillingTemplateId', $templateId);
			$this->assign('initialRbillAmt', $invoice->getTotal(1));
			$this->assign('reBillAmt', $invoice->getTotal(2));	   
	   		return $this->_render('form_subscription');
	   	}		

		return $this->_render('form');
	}
	
	function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		//Request is not from Gate2shop then do nothing.
		if(!isset($data['fromGateToShop'])){
			return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		}

		switch( $action )
		{
			//Notification with successful payment
			case 'success':
				$errCode = isset($data['ErrCode']) ? $data['ErrCode'] : $data['rebilling_error_code'];
				//Notification status other then ok must have to be logged.
				if(strtoupper($data['ppp_status']) == 'OK' && $errCode == 0)
				{
					//Save transaction with 0 amount
					$invoice	 = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
					$transaction = PayplansTransaction::getInstance();
					$transaction->set('user_id', $payment->getBuyer())
						->set('invoice_id', $invoice->getId())
						->set('payment_id', $payment->getId())
						->set('message', 'COM_PAYPLANS_APP_PAYMENT_GATE2SHOP_PAYMENT_PROFILE_CREATED')
						->set('params', PayplansHelperParam::arrayToIni($data))
						->save();

					break;
				}
			
			//Cancel notification
			case 'cancel':
				if($data['ppp_status'] == 'CANCEL' || $data['ppp_status'] == 'FAIL'){
					PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $data['message'], $payment, $data, 'PayplansPaymentFormatter', '', true);
				}
		}
	}
	
	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		if(!isset($data['fromGateToShop'])){
			return true;
		}
		
		//For Validating a Secure and Non-Fraud Transaction.
		$validIPN = null;
		$productId = isset($data['productId']) ? $data['productId'] : '';

		if(strtoupper($productId) == 'FIXED'){
			$validIPN = $this->isValidIPN_Fixed($data);
		}elseif(isset($data['rebillingProductName'])){
			$validIPN = $this->isValidIPN_Recurring($data);		
		}else{
			$validIPN = $this->isValidIPN_DMN($data);
		}
		
		if(!$validIPN){
			$message	= XiText::_('COM_PAYPLANS_PAYMENT_APP_GATE2SHOP_FRAUD_PAYMENT_ATTEMPT');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message , $payment, $data, 'PayplansPaymentFormatter', '', true);
			return false;
		}
		
		$invoice	 = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$transaction = PayplansTransaction::getInstance();
		
		//For Fixed plan notification
		$gateway_txn_id 	= 'TransactionID';
		
		//For use only in recurring plans
		$gateway_parent_txn = isset($data['membershipId']) ? $data['membershipId'] : 0;

		$recurring = $invoice->isRecurring();
		if($recurring){
			//Txn_id = rebilling_initial_transaction_id (For First payment). Txn_id = gatewayTransactionId (For Next payment notification through DMN) 		
			$gateway_txn_id 	= isset($data['rebilling_initial_transaction_id']) ? 'rebilling_initial_transaction_id' : 'gatewayTransactionId';
		}
		
		//For Checking Existing Transactions
		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $data[$gateway_txn_id], 0, $gateway_parent_txn);
		if(!empty($transactions)){
			return true;
	 	}

		//Status of Notification. 'gatewayTransactionStatus' in case of DMN.
		$status		= isset($data['ppp_status']) ? strtoupper($data['ppp_status']): strtoupper($data['gatewayTransactionStatus']);
		//Gets Error code sent by G2S
		$errCode	= isset($data['ErrCode']) ? $data['ErrCode'] : $data['rebilling_error_code'];
		
		// Notification status other then ok must have to be logged.
		if(($status != 'OK' || $status != 'APPROVED') && $errCode != 0)
		{
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $data['message'], $payment, $status, 'PayplansPaymentFormatter', '', true);
			return true;
		}
			
	 	//Sets new transaction details
		$transaction->set('user_id', $payment->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('message', 'COM_PAYPLANS_APP_PAYMENT_GATE2SHOP_PAYMENT_RECEIVED')
					->set('params', PayplansHelperParam::arrayToIni($data));
					
		
	    // Adds Respactive transaction on invoce.
    	if(!($recurring) && ($status == 'OK'))
    	{    			   			
    		$transaction->set('amount', $data['totalAmount'])
						->set('gateway_txn_id', $data[$gateway_txn_id])
    					->save();
    		return true;
    	}
		
		//For recuuring transaction
    	//Notification Status OK for first payment, 'APPROVED' for next payment'.
    	if(($recurring) && ($status == 'OK' || $status =='APPROVED'))
    	{
    		//amount = rebilling_initial_amount (For First Payment). amount = amount (For Next payments through DMN)
    		$amount			= isset($data['rebilling_initial_amount']) ? $data['rebilling_initial_amount'] : $data['amount'];
    		$transaction->set('gateway_txn_id', $data[$gateway_txn_id])
    					->set('amount', $amount)
    					->set('gateway_subscr_id',$data['membershipId'])
	    	//Save Transaction record.
						->save();
	    	return true;
    	}
    }

	private function isValidIPN_Fixed(&$data)
	{
		$secretKey 	= $this->getAppParam('secret_key', false);
		$checkSum	= md5($secretKey.$data['totalAmount'].$data['currency'].$data['responseTimeStamp'].$data['PPP_TransactionID'].$data['Status'].$data['productId']);
		
		if(strtoupper($checkSum) == strtoupper($data['advanceResponseChecksum'])){
			return true;
		}

		return false;		
	}
	
	private function isValidIPN_Recurring($data)
	{
		$secretKey	= $this->getAppParam('secret_key', false);
		$checkSum	= md5($secretKey.$data['membershipId'].$data['status']);

		if(strtoupper($checkSum) == strtoupper($data['responsechecksum'])){
			return true;
		}
		
		return false;
	}
	
	private function isValidIPN_DMN($data)
	{
		$checkSum = md5($data['membershipId'].$data['gatewayTransactionId'].$data['gatewayTransactionStatus'].$data['amount'].$data['currency']);

		if(strtoupper($checkSum) == strtoupper($data['responsechecksum'])){
			return true;
		}
		
		return false;
	}
		
	function _getPostUrl()
	{
		return 'https://secure.gate2shop.com/ppp/purchase.do';
	}
	
	function _getAppParams()
	{
		$this->site_id		= $this->getAppParam('merchant_site_id', null);
    	$this->merchant_id 	= $this->getAppParam('merchant_id', null);
    	$this->secret_key 	= $this->getAppParam('secret_key', null);
    	$this->version		= $this->getAppParam('api_version', '3.0.0');
    	
    	return true;
	}
	
	//formats and serialize data of product and plan mapping.  
	public function collectAppParams(array $data)
	{
		$data['app_params']['plan_product_mapping'] = isset($data['app_params']['plan_product_mapping']) ? $data['app_params']['plan_product_mapping'] : array( 0 => array(0 => '', 1 => '', 2 => '')); 
		return PayplansHelperParam::collectParams($data,'app_params');
	}
	
	//mapping of g2s rebilling products and payplans recurring plans
	public function getPlanMapping($payment)
	{
   		$planMapping = $this->getAppParam('plan_product_mapping');
   		$plans		 = $payment->getPlans();
   		$plan_id	 = array_shift($plans);
   		   		
   		foreach($planMapping as $key => $mapping)
   		{
   			if($mapping['0'] == $plan_id){   				
				$productId	 = $mapping['1'];
   				$templateId = $mapping['2'];
   				break;
   			}
   		}

   		return array($productId, $templateId);
	}
}
