<?php

/**
* @copyright	Copyright (C) 2009 - 2012 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	invitex
* @contact		payplans@readybytes.in
*/
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Payplans Invitex App
 *
 */
class PayplansAppInvitex extends PayplansApp
{
	public $_location       = __FILE__;
	protected $_resource	= 'com_invitex.limit';
	
    public function isApplicable( $refObject= '', $eventName='')
	{
          if($eventName == 'onPayplansInvitexOnInviteSuccess' || $eventName == 'onPayplansInvitexSendInvitation' || $eventName == 'onPayplansApplyDiscount')
          {
          	return true;
          }
          return parent::isApplicable($refObject, $eventName);
	}
	

	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if(($new->getStatus() == PayplansStatus::NONE) || ($prev != null && $prev->getStatus() == $new->getStatus())){
			return true;
	    }

		$newStatus          = $new->getStatus();
		$userid 			= $new->getBuyer();
		$subid  			= $new->getId();
		$buyer_name         = $new->getBuyerName();
		$plan               = $new->getTitle();
		$limit              = $this->getAppParam('limit_count');
	
		// Add limit in invitex for send an invitation when subscription status is Active
	    if($newStatus == PayplansStatus::SUBSCRIPTION_ACTIVE){
	     	$this->_addIntoLimit($userid,$limit);   
	     	$this->_addToResource($subid, $userid, $limit, $this->_resource);
	    } 

          return true;
	}

	// add invitation limit for user
    protected  function _addIntoLimit($userid, $limit)
    {
    	$initial_limit = $this->_getInvitationLimit($userid);
    	$limit        = $initial_limit + $limit;
    	
    	 $query   = new XiQuery();
         return $query->update('`#__invitex_invitation_limit`')
         		      ->set('`limit` = '.$limit)
	    	   	      ->where('`userid` = '.$userid)
					  ->dbLoadQuery()
					  ->query();
	}	
	
	// delete invitation limit after subscription expire,hold or none
	protected  function _deleteFromLimit($userid, $limit)
    {
    	$initial_limit = $this->_getInvitationLimit($userid);
    	if($initial_limit >= $limit){
    		$limit = $initial_limit - $limit;
    	}
    	
    	 $query   = new XiQuery(); 
         return $query->update('`#__invitex_invitation_limit`')
         			  ->set('`limit` = '.$limit)
	    	   	      ->where('`userid` = '.$userid)
					  ->dbLoadQuery()
					  ->query();
	}	
	
	// calculate previous invitation limit of user
	protected  function _getInvitationLimit($userid)
	{
		$db	= PayplansFactory::getDBO();
		$query = new XiQuery();
		return $query->select('`limit`')
					 ->from('`#__invitex_invitation_limit`')
					 ->where('`userid` = '.$userid)
					 ->dbLoadQuery()
					 ->loadResult();
	}
	
	// Trigger for wallet updation after Invitation send	
	public function onPayplansInvitexSendInvitation()
	{
		$userid     = PayplansFactory::getUser()->id;
        if(!$userid){
           return false;
        }

		$user 	     = PayplansUser::getInstance($userid);
		$userPlans   = $user->getPlans();
		if(empty($userPlans)){
			return false;
		}
		
        // Get all the available apps of invitex
		$applicableApps = PayplansHelperApp::getAvailableApps('invitex',$user);
		foreach ($applicableApps as $app)
		{
			if($app->getParam('applyAll') || array_intersect($app->getPlans(), $userPlans))
			{
				$invite_amount = $this->getAppParam('invite_amount');
				if(floatval(0) != floatval($invite_amount)){
					$this->_createDiscountcode($userid, $invite_amount, 'INVITATION_SEND');
				}
			}
		}
		
		return true;
	}

    // Trigger for send discount code after accept & register to the site
    function onPayplansInvitexOnInviteSuccess($inviter_id, $invitee_id)
	{
	    $user      = PayplansUser::getInstance($inviter_id);
   	    $isAdmin   = XiHelperJoomla::isAdmin($inviter_id);
		$userPlans = array();

   	    if(!$isAdmin){
		   $userPlans   = $user->getPlans();
		   if(empty($userPlans)){
			return false;
		   }
   	     }
		
		$applicableApps = PayplansHelperApp::getAvailableApps('invitex',$user);
		foreach ($applicableApps as $app)
		{
			if($isAdmin || $app->getParam('applyAll') || array_intersect($app->getPlans(), $userPlans))
			{
                if(empty($inviter_id) || empty($invitee_id))
                   return true;
                
                // Send discount code to inviter when accept an invitation
	           	$inviter_amount = $this->getAppParam('inviter_amount');
				if(floatval(0) != floatval($inviter_amount)){
		   			$this->_createDiscountcode($inviter_id, $inviter_amount, 'INVITATION_ACCEPT');
	           	}

			  	
                // Send Discount code to invitee wallet for register through invitation
		  	  	$invitee_amount = $this->getAppParam('invitee_amount');
				if(floatval(0) != floatval($invitee_amount)){
		  	   		$this->_createDiscountcode($invitee_id, $invitee_amount, 'REGISTRATION_THROUGH_INVITATION');
				}
			}
		}

		// Set this variable so this event will not trigger multiple times
		$session = JFactory::getSession();
		$session->set('PP_INVITATION_ACCEPT', true);
		
		return true;
	}
	
	public function _createDiscountcode($userId, $amount, $type)
	{
		$code 				= substr(number_format(time() * rand(), 0, '', ''), 0, 6); 
	 	$code 				= 'INVITEX'.$userId.'_'.$code;
					
	 	$param 				= '{"extend_time_discount":"000000000000","coupon_amount_type":"fixed","allowed_quantity":"1","reusable":"no","allow_clubbing":"1"}';
     	$discount 			= "( '$code', '$code', 'invitex', 1, $amount , '' , '0000-00-00 00:00:00', '0000-00-00 00:00:00', 1, '$param')";
	 	
      	$discounts 			= 'INSERT INTO `#__payplans_prodiscount` (`title`, `coupon_code`, `coupon_type`, `core_discount`, `coupon_amount`, `plans`, `start_date`, `end_date`, `published`, `params`) VALUES'.$discount;
	 	
      	$db 				= XiFactory::getDBO();
      	$db->setQuery($discounts);
	 	$db->query(); 
	 	
	 	$user				= PayplansUser::getInstance($userId);
	 	$data 				= $user->getProperties();
	 	$config 			= JFactory::getConfig();
						 
		$data['fromname']   = $config->get('fromname');
        $data['mailfrom']   = $config->get('mailfrom');
        $data['sitename']   = $config->get('sitename');
        $mail               = JFactory::getMailer();
        $mail->IsHTML(true);
             
	 	$subject 			= XiText::sprintf( 'PLG_PAYPLANS_INVITEX_'.$type.'_EMAIL_SUBJECT', $data['sitename']);
	 	$body 				= XiText::sprintf( 'PLG_PAYPLANS_INVITEX_'.$type.'_EMAIL_BODY', $user->getUsername(), $amount, $code);
        $result    			= $mail->sendMail($data['fromname'],$data['fromname'],$user->getEmail(),$subject,$body);
	 	
        if($result === true) {                  
        	PayplansHelperLogger::log(XiLogger::LEVEL_INFO, XiText::_("PLG_PAYPLANS_INVITEX_MESSAGE_SEND_SUCCESSFULLY"), $this,array() ,'PayplansAppInvitexFormatter');
        }
        else {
      		PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, XiText::_("PLG_PAYPLANS_INVITEX_MESSAGE_SEND_UNSUCCESSFULL"), $this, array(), 'PayplansAppInvitexFormatter', '', true);
       }
	
       return $result;
	 	
	}

    
	public function onPayplansApplyDiscount($object, $invitexDiscountCode)
	{

		//do nothing no discount coupon is entered
		if(empty($invitexDiscountCode)){
			return false;
		}

		return $this->_applyDiscountonInvitation($object, $invitexDiscountCode);
	}
	
	public function _applyDiscountonInvitation($object, $invitexDiscountCode)
	{
		$query      	= new XiQuery();
		$discounts		= $query->select('*')
							->from('#__payplans_prodiscount')
							->where('`published` = 1')
							->where('`coupon_code`= '."'".$invitexDiscountCode."'")
							->where('`coupon_type` = "invitex"')
							->dbLoadQuery()
		  					->loadObjectList();
		  				
		$finalResult = false;
		foreach ($discounts as $discount)
		{
			$result = PayplansHelperModifier::getTotalUsage($invitexDiscountCode, 'invitex');
			if($result >= 1){
				return XiText::_('COM_PAYPLANS_APP_INVITEX_ERROR_ALREADY_USED');
			}
			$finalResult = $this->doApplyDiscount($object, $invitexDiscountCode, $discount);
		}
		
		return $finalResult;
	}
	
	public function doApplyDiscount($object, $invitexDiscountCode, $discount)
	{
		$amount		= $discount->coupon_amount;
		
		// if amount of applied discount is greater than total 
		if( $amount > $object->getTotal()){
			return XiText::_('COM_PAYPLANS_INVITEX_ERROR_EXCEED_TOTAL_AMOUNT');
		}
			
		// Provide invitex discount to user
		$params 			= new stdClass();
		$params->message    = XiText::_('COM_PAYPLANS_APP_INVITEX_DISCOUNT_MESSAGE');
		$params->type       = 'invitex';
		$params->reference  = $invitexDiscountCode;
		$params->frequency  = PayplansModifier::FREQUENCY_ONE_TIME;
		$params->amount     = -$amount;
		$params->percentage = 'fixed';
		$params->serial     = PayplansModifier::FIXED_DISCOUNT;
									                     
		$object->addModifier($params);
			
		// Change Recurring Plan to Trial + Recurring, so Invitex discount will not be applicable on next recurrence
		if($object->isRecurring() == PAYPLANS_RECURRING){		
				$objParams = $object->getParams()->toArray();
				$object->setParam('expirationtype', 	'recurring_trial_1');
				$object->setParam('recurrence_count', 	($objParams['recurrence_count']> 0 ) ? $objParams['recurrence_count']-1 : 0);
				$object->setParam('trial_price_1', 		$objParams['price']);
				$object->setParam('trial_time_1', 		$objParams['expiration']);
		}
	
		$object->refresh();
		$object->save();
		return true;
	}
   
}	

class PayplansAppInvitexFormatter extends PayplansAppFormatter
{
	public $template	= 'view_log';
	
	// get Ignore data 
	function getIgnoredata()
	{
		$ignore = array('_trigger', '_tplVars', '_mailer', '_location', '_errors', '_component');
		return $ignore;
	}
	
	// get rules
	function getVarFormatter()
	{
		$rules = array('_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       'function' => 'getAppPlans'),
					   'app_params'      => array('formatter'=> 'PayplansAppInvitexFormatter',
										       'function' => 'getFormattedContent'));
		return $rules;
	}

	// format email app content,status, expiration time 
	function getFormattedContent($key, $value, $data)
	{	
		$this->template = 'view';
	}
}	

