<?php

/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	JReview
* @contact		support+payplans@readybytes.in
*/
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Payplans Jreview Plugin
 *
 */
class plgPayplansJreview extends XiPlugin
{

	const ALLOWED 	 =  1;
	const BLOCKED    =  0;
	const ApplyAll	 =  -1;
	
	public function onPayplansSystemStart()
	{
		if(!JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_jreviews')){
			return false;
		}
			
		//add jreview app path to app loader
		$appPath = dirname(__FILE__).DS.'jreview'.DS.'app';
		PayplansHelperApp::addAppsPath($appPath);

		return true;
	}
	
	public function onPayplansJreviewBeforeSaveList($Listdata)
	{
		$user = PayplansUser::getInstance(PayplansFactory::getUser()->id);
		
		if($user->isAdmin() == true){
			return true;	
		}

		/*<--get all the variables--->*/
		$listingData	  = $Listdata['Listing'];
		$edited 		  = (isset($listingData['id']) && !empty($listingData['id']))? true : false;
				
		$postInCategories = array($listingData['catid']);
		$postInCategories = plgPayplansJreview::getAllParentCategories($listingData['catid'],$postInCategories);

		/*<--check user is allowed to post in category or not--->*/
		$allowedOrNot = $this->isAllowed($postInCategories, $user,false,$edited);
				
		if($allowedOrNot == self::BLOCKED){
			return false;
		}
				
		return true;
	}
	
	public function onPayplansJreviewBeforeSaveMedia($Mediadata)
	{
		/*<--get all the variables--->*/
		 // also contain parent category ids

		$user = PayplansUser::getInstance(PayplansFactory::getUser()->id);
		
		if($user->isAdmin() == true){
			return true;	
		}
		
		$catId = $this->getCategoryFromlist($Mediadata['listing_id']);
		$postInCategories = array($catId);
		$postInCategories = plgPayplansJreview::getAllParentCategories($catId,$postInCategories);
		
		//check what type of media is going to be checked
		$mediaType = $Mediadata['media_type'];
	
		/*<--check user is allowed to post in category or not--->*/
		$allowedOrNot = $this->isAllowed($postInCategories, $user,true,false,$mediaType);
		
			
		if($allowedOrNot == self::BLOCKED){
			return false;
		}
				
		return true;
	}
	
	function isAllowed($postInCategories = array(),$user,$mediaRestriction = false,$edited = false,$mediaType = 'image')
	{
		$postInCategories[] = 0;
		/*<--get all plans on category on which app is created--->*/
		$allAppCategories = $this->getAllAppCategories();
				
		foreach ($postInCategories as $category){
			$resourceCount = $this->getUserResourceOfCategory($category,$user->getId(),$mediaRestriction,$mediaType);
			
			/*<---if user count is empty means no plan subscribed--->*/
			if(empty($resourceCount)){
			
				/*<--if intersect count is zero means no app is created--->*/
				if(!array_key_exists($category,$allAppCategories)){
					/*<---allowed for that category but parent may not--->*/
					continue;
				}
				
				/*<---restrict if any one is not allowed--->*/
				return self::BLOCKED;
			}
			
			/*<---allowed for that category on which app is not created or disabled after allocating resorces--->*/
			if(!array_key_exists($category,$allAppCategories)){
					continue;
			}

			if($mediaRestriction == false){
				/*<---we calculate the past as well as current entry--->*/
				$userCount = $this->getUserEntriesInCategory($category,$user->getId())+1;
				if($edited) $userCount--;
			}
			else{
				/*<---we calculate the past as well as present entry--->*/
				$userCount = $this->getUserMediaEntries($category,$user->getId(),false,$mediaType)+1;
			}
			
			if($userCount > $resourceCount){
				/*<---restrict if he posted more than allowed--->*/
				return self::BLOCKED;
			}
		}	
		
		return self::ALLOWED;
	}
	
	/*<---this function will give array of all categories on which app is created--->*/
	static function getAllAppCategories()
	{
		$reviewApps = PayplansHelperApp::getAvailableApps('jreview');
		$allcategory = array();
		foreach ($reviewApps as $app){
			
			$no_of_submisssion = $app->getAppParam('no_of_submisssion');	
			$no_of_images	   = $app->getAppParam('no_of_images');	
			$no_of_audio	   = $app->getAppParam('no_of_audio');
			$no_of_video	   = $app->getAppParam('no_of_video');
			$no_of_attachment  = $app->getAppParam('no_of_attachment');
			
			if('any_category' == ($addEntryIn = $app->getAppParam('add_entry_in', 'any_category')) ){
				$category = 0;
			}
			else{ 
				$category 	 = $app->getAppParam('add_entry_in_category');
			}
			$category	 = (!is_array($category)) ? array($category) : $category;  
			
			$tempCategory = array();
			
			foreach ($category as $category){
				$allcategory[$category]['listing'] 		= $no_of_submisssion;
				$allcategory[$category]['images']  		= $no_of_images;
				$allcategory[$category]['audio']  		= $no_of_audio;
				$allcategory[$category]['video']  		= $no_of_video;
				$allcategory[$category]['attachment']  	= $no_of_attachment;
			}			
		}	
		
		return $allcategory;
	}
	
	static function getUserResourceOfCategory($category,$userId,$mediaRestriction = false,$mediaType = 'photo')
	{
		
		if($mediaType == 'photo')
		{
			$mediaType = 'image';
		}
		
		$query = new XiQuery();
		$query->select('`count`')
					 ->from('`#__payplans_resource`');
					 if($mediaRestriction == false){
					 	$query->where("`title` = 'com_jreview.category'");
					 }
					 else{
					 	$query->where("`title` = 'com_jreview.{$mediaType}'");
					 }
		return $query->where('`user_id` ='.$userId)
					 ->where('`value` ='.$category)
					 ->dbLoadQuery()
					 ->loadResult();
	}
	
	//it will fetch userentries only for a category all it's childeren's entries also
	static function getUserEntriesInCategory($category = 0,$userId)
	{
		//if all entries are required
		if($category == 0)
		{
			return count(plgPayplansJreview::getCategoryCount(0,$userId));	
		}
		
		
		$tempCount = 0;
		
		$childCategories = array($category);
		$childCategories = plgPayplansJreview::getAllChildCategories($category, $childCategories);
		
		foreach ($childCategories as $childCategory)
		{
			$tempCount += count(plgPayplansJreview::getCategoryCount($childCategory,$userId));		
		}
		
		return $tempCount;
	}
	
	//it will fetch userentries only for a category
	static function getCategoryCount($category = 0,$userId)
	{	
		$query = new XiQuery();
			
		$query->select('id')->from('`#__content`')
							->where('`state` = 1')
							->where('`created_by` ='.$userId);
						 if($category != 0){
							$query->where('`catid` ='.$category);
						 }
		
		return $query->dbLoadQuery()->loadobjectList();
	}
	
	/*---got all the no of images posted by user ----*/
	static function getUserMediaEntries($category,$userId,$all = false,$mediaType='photo')
	{
		$rawlistings 	 = array();
		$childCategories = array($category);
		$childCategories = plgPayplansJreview::getAllChildCategories($category, $childCategories);
		
		foreach ($childCategories as $childCategory)
		{
			$rawlistings = array_merge($rawlistings, plgPayplansJreview::getCategoryCount($childCategory,$userId));		
		}

		foreach ($rawlistings as $list)
		{
			$listing[] = $list->id;
		}
		
		$imageCount = 0;
		$query = new XiQuery();
		$query->select('count(*)')
			  ->from('`#__jreviews_media` as media')
			  ->where('media.user_id = '.$userId)
			  ->where('media.published = 1')
			  ->where('media.approved  = 1')
			  ->where("media.media_type  = '{$mediaType}'");
			  
			  
		 if($all == false && $category!= 0 && !empty($listing)){
		 	$query->where('media.`listing_id` in ('.implode(',',$listing).')');
		 }
		 
		
		return $query->dbLoadQuery()
					 ->loadResult();
	}
		
	function getCategoryFromlist($listingId = 0)
	{
		$query = new XiQuery();
		return $query->select('catid')
			  ->from('`#__content`')
			  ->where('id = '.$listingId)
			  ->dbLoadQuery()
			  ->loadResult();
	}
	
	static function getAllParentCategories($catId = 0,&$parents = array())
	{
			
		$query = new XiQuery();
		
		$parent = $query->select('parent_id')
								->from('#__categories')
								->where('`extension` = "com_content"')
								->where("`id` = $catId")
								->dbLoadQuery()
								->loadResult();	

		if($parent != 0){
			$parents[] = (int)$parent;
			plgPayplansJreview::getAllParentCategories((int)$parent,$parents);
		}

		return $parents;
	}
	
	//it will return children of category
	static function getAllChildCategories($catId = 0, &$allChilds =array())
	{

		if($catId == 0){
			return $allChilds;
		}		

		$query = new XiQuery();
		
		$childs = $query->select('id')
						->from('#__categories')
						->where('`extension` = "com_content"')
						->where("`parent_id` = $catId")
						->dbLoadQuery()
						->loadObjectList();

						
		if(empty($childs))
		return $allChilds;
						
		foreach ($childs as $child)
		{
			$allChilds[] = $child->id;
			plgPayplansJreview::getAllChildCategories($child->id,$allChilds);
		}
		
		return $allChilds;
	}
	
}

/**	Here we need to work for the jreview plugin first user there trigger to detect changes then,
*	then trigger our events.
*/
if(PayplansFactory::getApplication()->isSite() && file_exists(JPATH_ROOT .DS.'components' .DS.'com_jreviews'. DS . 'jreviews' . DS . 'framework.php')){

	$option = JFactory::getApplication()->input->get('option', 'com_content');
	
	if ($option != 'com_jreviews') {
		return;
	}
	
	require_once ( JPATH_ROOT .DS.'components' .DS.'com_jreviews'. DS . 'jreviews' . DS . 'framework.php' );
	defined('MVC_FRAMEWORK') or die( 'Direct Access to this location is not allowed.' );
	
	$App = S2App::getInstance();
       
    $App->jreviewsPaths['Plugin']['payplans.php'] = "components/com_jreviews/jreviews/plugins/cron_functions.php";
        	
	abstract class payplansJreview extends S2Component {
	}
}else
{
	abstract class payplansJreview {
	}
}

class PayplansComponent extends payplansJreview {
    
    var $plugin_order = 101;
    
    var $name = 'payplans';
    
    var $published = true;
    
    var $autoPublishTranslation = true; 
    
    var $controllerActions = array(       
        'media_upload'=>array('create','_save'),
        'listings'=>array('create','_loadForm','_save')
    );
    
    var $responseSend = false;
    
    function startup(&$controller)
    { 
        if(!defined('MVC_FRAMEWORK_ADMIN')){
	        	$this->c = & $controller;        
        }
    }    

  
    function plgBeforeSave(&$model,&$data)
    {
	$res='';        
	switch($model->name){
        	
            case 'Discussion':		break;   
            case 'Favorite'	:		break;             
            case 'Listing'	:		$res = $this->_plgBeforeSaveListing($data);
            						break;  
            case 'Review'	:		break;  
            case 'Vote'		:		break;  
            case 'Media'	:       $res = $this->_plgBeforeSaveMedia($data);
            						break;  
        }  

        if($this->responseSend){
        	return $data;
        }
        
        $this->responseSend = true;
        
		if ( is_array($res) && array_shift($res) == false) {
						
			$message  = ($model->name == "Listing")?	XiText::_("COM_PAYPLANS_APP_JREVIEW_YOU_ARE_NOT_ALLOWED"):XiText::_("COM_PAYPLANS_APP_JREVIEW_YOU_ARE_NOT_ALLOWED_ADD_MEDIA");
			$message .=		"<a href='"
							.XiRoute::_('index.php?option=com_payplans&view=plan&task=subscribe')
							."'>"
							.XiText::_('COM_PAYPLANS_APP_JOOMLA_ARTICLE_SUBSCRIBE_PLAN')
							."</a><br>";
	
			
			$result   = array(
							'success'=> false,
							'str' 	 => $message 
			);
					
			header('Content-Type: application/json');
			echo json_encode($result);
			exit();
		}
		
        return $data;
    }   

    //triggering an event before listing save and send images and data as argument
    function _plgBeforeSaveMedia($data)
    {
    	$args = array($data['Media']);
    	$results = PayplansHelperEvent::trigger('onPayplansJreviewBeforeSaveMedia',$args);
		return $results;	
    }

    function _plgBeforeSaveListing($data)
    {
		$args    = array($data);
		$results = PayplansHelperEvent::trigger('onPayplansJreviewBeforeSaveList',$args);
		return $results;		
    }  
}

