<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class  PayplansAppJusertype extends PayplansApp
{
	protected $_location	= __FILE__;
	protected $_resource	= 'com_user.usergroup';

	
	public function isApplicable($refObject = null, $eventName='')
	{
		if(($eventName == 'onPayplansSubscriptionCleanResource')){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		return $this->_triggerJusertype($prev,$new);
	}


	protected function _triggerJusertype($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}

		$subid 		=  $new->getId();
		
		$usersConfig 		= JComponentHelper::getParams( 'com_users' );
		$defaultUserGroup	= $usersConfig->get('new_usertype');
		
		$active	    		= $this->getAppParam('jusertypeOnActive', array());
		$hold 	    		= $this->getAppParam('jusertypeOnHold', array());
		$expire				= $this->getAppParam('jusertypeOnExpire', array());
		$removeFromDefault	= $this->getAppParam('removeFromDefault', 0);
		
		$newstatus  =  $new->getStatus();
		$userid		=  $new->getBuyer();
			
		$active		= (is_array($active)) ? $active : array($active);
		$hold		= (is_array($hold)) ? $hold : array($hold);
		$expire		= (is_array($expire)) ? $expire : array($expire);
		$groups 	= XiHelperJoomla::getJoomlaGroups();
		
		//if subscription is active
		if($newstatus == PayplansStatus::SUBSCRIPTION_ACTIVE){	

			$holdActiveDiff		= array_diff($hold, $active);
			$expireActiveDiff	= array_diff($expire, $active);
		
			$result 			= $this->_setJusertype($userid, $active, $subid, $groups);
			
			$this->_unsetJusertype($userid, $holdActiveDiff, $subid, $groups);
			$this->_unsetJusertype($userid, $expireActiveDiff, $subid, $groups);
			
			if($removeFromDefault && !in_array($defaultUserGroup, $active)){
				XiHelperJoomla::removeUserFromGroup($userid, $defaultUserGroup);
			}

			//forcefully removes the user from provided user group, irrespective of plan subscription.
			$removeFromListActive = $this->getAppParam('removeFromGroup');
			if(!empty($removeFromListActive)) {
				return $this->removeForcefully($userid,$removeFromListActive,$subid, $groups);
			}
			return $result;
		}
		
		//if subscription is hold			
		if($newstatus == PayplansStatus::SUBSCRIPTION_HOLD){
			
			$activeHoldDiff		= array_diff($active, $hold);
			$expireHoldDiff		= array_diff($expire, $hold);
			
			if($this->_isRequiredDefault($userid) && $hold[0] == null){
				$hold[0] = $defaultUserGroup;
			}
			
			$result 			= $this->_setJusertype($userid, $hold, $subid, $groups);
			
			$this->_unsetJusertype($userid, $activeHoldDiff, $subid, $groups);
			$this->_unsetJusertype($userid, $expireHoldDiff, $subid, $groups);
			
			return $result;
		}
		
		//if subscription is expire			
		if ($newstatus == PayplansStatus::SUBSCRIPTION_EXPIRED){
			
			$activeExpireDiff	= array_diff($active, $expire);
			$holdExpireDiff		= array_diff($hold, $expire);
			
			if($this->_isRequiredDefault($userid) && $expire[0] == null){
				$expire[0] = $defaultUserGroup;
			}
			
			$result 		= $this->_setJusertype($userid, $expire, $subid, $groups);
			
			$this->_unsetJusertype($userid, $activeExpireDiff, $subid, $groups);
			$this->_unsetJusertype($userid, $holdExpireDiff, $subid, $groups);
			
			return $result;
		}

		return true;
	}

	protected function _setJusertype($userid, $group, $subid, $groups)
	{
		if(!is_array($group) || empty($group)){
			return true;
		}
		
		jimport('joomla.user.helper');
		foreach ($group as $groupid){
			XiHelperJoomla::addUserToGroup($userid, $groupid);
			$this->_addToResource($subid, $userid, $groupid, $this->_resource);
			
			$message = XiText::_("COM_PAYPLANS_APP_JUSERTYPE_LOG_ADD_INTO_USERGROUP");
           	$content = array( 'User Name'=>$userid, 'Usergroup' => $groups[$groupid]->name,'Subscription Id'=> $subid );
       		PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $this, $content,'PayplansAppJusertypeFormatter');
		}

		return true;
	}
	
	protected function _unsetJusertype($userid, $jusertype, $subid, $groups)
	{
		if(!is_array($jusertype)){
			return true;
		}
		
		foreach ($jusertype as $group){
			if($this->_removeFromResource($subid, $userid, $group, $this->_resource)){
				XiHelperJoomla::removeUserFromGroup($userid, $group);
				
				$message = XiText::_("COM_PAYPLANS_APP_JUSERTYPE_LOG_REMOVE_FROM_USERGROUP");
           		$content = array( 'User Name'=>$userid, 'Usergroup' => $groups[$group]->name,'Subscription Id'=> $subid );
       			PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $this, $content,'PayplansAppJusertypeFormatter');
			}
		}
		return true;
	}
	
	//render Widget
	public function renderWidgetHtml()
	{
		//get user id
		$userid     = XiFactory::getUser()->id;
		
		// do nothing if user is not logged in
		if(empty($userid)){
			return '' ;
		}
		//get joomla Usertype
		$jusertype  = XiHelperJoomla::getJoomlaUserGroups($userid);
		if(empty($jusertype)){
			return '' ;
		}
		$this->assign('joomla_usertypes',$jusertype);
		$data = $this->_render('widgethtml');
	    return $data;
	}
	
	function getNameFromResourceValue($resource, $value)
	{
		// if its a different resource
		if($resource != $this->_resource){
			return false;
		}
		
		$groups = XiHelperJoomla::getJoomlaGroups();
		return $groups[$value]->name;
	}
	
	//check if need to assign default user type
	protected function _isRequiredDefault($userid)
	{
		$user		= PayplansUser::getInstance($userid);
		$userPlans	= $user->getPlans();
		
		if(empty($userPlans))
			return true;
			
		//get all user type apps
		$userTypeApps = PayplansHelperApp::getAvailableApps('jusertype');
		
		foreach($userTypeApps as $app)
		{
			$appPlans = $app->getPlans();
			
			//if any active plan has attached user type app, we don't need default type
			if(array_intersect($userPlans, $appPlans))
				return false;
		}
		return true;
	}
	
	public function onPayplansSubscriptionCleanResource($sub)
	{
		$userid		=  $sub->getBuyer();
		$sub_id 	=  $sub->getId();
		$groups 	= XiHelperJoomla::getJoomlaGroups();

		$resources	  = XiFactory::getInstance('resource', 'model')
								->loadRecords(array(
													'subscription_ids'=> array(array('LIKE', "'%,".$sub_id.",%'" )), 
													'title' => $this->_resource
													)
											);
		
		$usersConfig 		= JComponentHelper::getParams( 'com_users' );
		$defaultUserGroup	= $usersConfig->get('new_usertype');
		
		//Imp : before unseting usergroup ensure that user must have anyother usergroup attached
		//if no other usergroup is attached apart from the one which is going to be unset then add user to default usergroup
		foreach ($resources as $res){
			$subscription_ids = explode(',', JString::trim($res->subscription_ids, ','));
			$subscription_ids = array_unique($subscription_ids);

			if (count($subscription_ids) != 1){
				$this->_removeFromResource($sub_id, $userid, $res->value, $this->_resource);
				continue;
			}
			
			$assignedGroups = self::getJoomlaUserGroups($userid);
			if (count($assignedGroups) == 1 && $assignedGroups[0] == $defaultUserGroup && $assignedGroups[0] == $res->value){
				$this->_removeFromResource($sub_id, $userid, $res->value, $this->_resource);
				continue;
			}
			
			if (count($assignedGroups) == 1 && $res->value != $defaultUserGroup){
				XiHelperJoomla::setUserGroups($userid, $defaultUserGroup);
			}
			$this->_unsetJusertype($userid, array($res->value), $sub_id, $groups);
		}
		
		return true;
	}
	
	public static function getJoomlaUserGroups($userid)
	{
	  jimport('joomla.user.helper');
	  $usergroups = JUserHelper::getUserGroups($userid);
	  if(PAYPLANS_JVERSION_25 || PAYPLANS_JVERSION_30)
	  {
	  	$db      = XiFactory::getDBO();
	  	$groups  = implode(',', $usergroups);
		$db->setQuery( 'SELECT `id`'
				. ' FROM #__usergroups'
				. ' WHERE `id` IN (' . $groups . ')');
		return $db->loadColumn();	
	  }

	  $joomlagroups = array_keys($usergroups);
	  return $joomlagroups;
	}


	private function removeForcefully($userid,$removalList,$subid, $groups) {
		$removeFromGroups = is_array($removalList)? $removalList : array($removalList);
		foreach ($removeFromGroups as  $remGrpId) {
			XiHelperJoomla::removeUserFromGroup($userid, $remGrpId);
			$message = XiText::_("COM_PAYPLANS_APP_JUSERTYPE_LOG_REMOVE_FROM_USERGROUP");
			$content = array( 'User Name'=>$userid, 'Usergroup' => $groups[$remGrpId]->name,'Subscription Id'=> $subid );
			PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $this, $content,'PayplansAppJusertypeFormatter');
		}
		return true;
	}
}

class PayplansAppJusertypeFormatter extends PayplansAppFormatter
{
	public  $template = 'view_log';
	
	function getVarFormatter()
	{
		$rules = array('_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       	  'function' => 'getAppPlans'),
					   'app_params'      => array('formatter'=> 'PayplansAppJusertypeFormatter',
										          'function' => 'getFormattedContent'));
		return $rules;
	}
	
	function getFormattedContent($key,$value,$data)
	{
		$value['jusertypeOnActive'] 	   	= $this->getGroupNames($value['jusertypeOnActive']);
		$value['jusertypeOnHold']  	 		= $this->getGroupNames($value['jusertypeOnHold']);
		$value['jusertypeOnExpire'] 		= $this->getGroupNames($value['jusertypeOnExpire']);
		$value['removeFromDefault'] 		= $value['removeFromDefault'];

		$this->template		= 'view';
	}
	
	function getGroupNames($values)
	{
		$groups 	= XiHelperJoomla::getJoomlaGroups();
		foreach ($values as $value){
			$group[]  = $groups[$value]->name;
		}
		
        return implode(',', $group);
	}
}	