<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	k2
* @contact 		support+payplans@readybytes.in
* 
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppK2 extends PayplansApp
{
	protected $_location	= __FILE__;
	
	// applicable only if k2 component exist
	public function _isApplicable(PayplansIfaceApptriggerable $refObject, $eventname='')
	{
		if(!JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_k2')){
			return false;
		}
		return  true;
		

	}

	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}

		$newStatus  = $new->getStatus();
		$userid		= $new->getBuyer();
		
		// if subscription is active
		if($newStatus == PayplansStatus::SUBSCRIPTION_ACTIVE){
			$groupid = $this->getAppParam('addToGroupOnActive');
			return $this->_addToGroup($userid, $groupid);
		}
		
		//if subscription is hold then add user to 
		if($newStatus == PayplansStatus::SUBSCRIPTION_HOLD){
			$groupid 	= $this->getAppParam('addToGroupOnHold');
			return $this->_addToGroup($userid, $groupid);
		}
		
		//if subscription is expire
		if ($newStatus == PayplansStatus::SUBSCRIPTION_EXPIRED){
			$groupid = $this->getAppParam('addToGroupOnExpire');
			return $this->_addToGroup($userid, $groupid);
		}

		return true;
	}

	protected function _addToGroup( $userid, $groupid )
	{
		// if there is no group to add into
		if(!$groupid){
			return true;
		}
		$db = XiFactory::getDBO();

		$user = XiFactory::getUser($userid);
		
		$query = 'SELECT id FROM #__k2_users'
			. ' WHERE `userID` = "' . $user->id . '" ';
			
		$db->setQuery( $query );
		$id = $db->loadResult();

		if ( empty( $id ) ) {
			$query = 'INSERT INTO #__k2_users'
				. ' (`userID`, `userName`, `group` )'
				. ' VALUES ( "' . $user->id . '", "' . $user->username . '", "' . $groupid . '" )';
		} else {
			$query = 'UPDATE #__k2_users'
				. ' SET `group` = "' . $groupid . '"'
				. ' WHERE `userID` = "' . $user->id . '"';			
		}

		$db->setQuery( $query );
		$db->query();
		
		//log when user is added to k2 group
		$message = sprintf(XiText::_('COM_PAYPLANS_APP_K2_LOG_ADDED_TO_GROUP'), $userid, $groupid);
		$content =  array('previous' => array( 'user_id'=>$userid , 'k2_user_group' => $groupid));
		PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, null, $content,'PayplansAppK2Formatter');
		return true;
	}
	
	//render Widget
	public function renderWidgetHtml()
	{   
		//get user id
		$userid      = XiFactory::getUser()->id;
		
		if(!$userid){
			return '';
		}
		//get k2 User Group
	    $k2UserGroup = $this->_getK2UserGroup($userid);
	    
		if(empty($k2UserGroup)){
			return '';
	    }
	    $this->assign('k2_usergroup',$k2UserGroup);
        $data        = $this->_render('widgethtml');
        return $data;
	}	
	
	protected function _getK2UserGroup($userid)
	{
		require_once  JPATH_ROOT.DS.'components'.DS.'com_k2'.DS.'helpers'.DS.'permissions.php';  
		
		$k2user 	= K2HelperPermissions::getK2User($userid);
		if(!$k2user){
			return '';
		}
		$k2grp  	= K2HelperPermissions::getK2UserGroup($k2user->group);
		
		return $k2grp->name;
	}
	
   public static function getK2UserGroups()
	{
		$db 	= JFactory::getDBO();
		$query 	= 'SHOW COLUMNS FROM #__k2_user_groups'
					. ' LIKE \'groups_id\'';

		$db->setQuery( $query );
		$result = $db->loadResult();

		$query = 'SELECT ' . ( $result ? 'groups_id' : 'id'  ) . ' as groups_id, name'
			 	. ' FROM #__k2_user_groups'
			 	;
	 	$db->setQuery( $query );
	 	return $db->loadObjectList('groups_id');
	}
}

class PayplansAppK2Formatter extends PayplansAppFormatter
{
	public $template	= 'view_log';
	
	//get Rules 
	function getVarFormatter()
	{
		$rules = array('_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       'function' => 'getAppPlans'),
					   'app_params'      => array('formatter'=> 'PayplansAppK2Formatter',
										       'function' => 'getFormattedParams'));
		return $rules;
	}
	
	// formatt params
	function getFormattedParams($key,$value,$data)
	{
		//do nothing if k2 is not installed
		if(!JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_k2')){
			return false;
		}
		
		$value['addToGroupOnActive']	= $this->getGroupName($value['addToGroupOnActive']);
		$value['addToGroupOnHold']  	= $this->getGroupName($value['addToGroupOnHold']);
		$value['addToGroupOnExpire']	= $this->getGroupName($value['addToGroupOnExpire']);
		
		$this->template = 'view';
	}
	
	function getGroupName($values)
	{
		$groups      = PayplansAppK2::getK2UserGroups();
		foreach ($values as $value){
			$group[] = $groups[$value]->name;
		}
		
		return implode(', ', $group);
	}
}
