<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		support+payplans@readybytes.in
* 
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppK2category extends PayplansApp
{
	protected $_location	= __FILE__;
	protected $_resource	= 'com_k2.submission';
	const DO_NOTHING = -1;
	const ALLOWED 	 =  1;
	const BLOCKED    =  0;
	
	public function isApplicable($refObject=null, $eventName='')
	{
		// applicable only if k2 component exists
		if(!JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_k2')){
			return false;
		}		
		
		if($eventName == 'onPayplansAccessCheck'){
			return true;
		}
		return parent::isApplicable($refObject, $eventName);
	}

	function onK2PrepareContent($item, $params, $limitstart)
	{
		if(! $item instanceof stdClass){
			return true;
		}
		
		$userId = XiFactory::getUser()->id;
		// when no user is logged-in
		if($userId == 0){
			return true;
		}
		$user = PayplansUser::getInstance($userId);
		
		//don't apply restriction on admin
		if($user->isAdmin()){
			return true;
		}
		
		$categoryId = (isset($item->category) && is_object($item->category)) ? $item->category->id : $item->categoryid;
    	
		
		if($this->_isCategoryAllowedbyK2UsrApp($categoryId)== true)
		{
			return  true;
		}
		
		$categories		= $this->_getParentCategories($categoryId);
		
	    $allowedOrNot = plgPayplansK2::isUserAllowed($categories,'k2category', 'category');
	    
	    if(in_array(self::BLOCKED, $allowedOrNot)){
			$url = XiRoute::_('index.php?option=com_payplans&view=plan');
			// if face any issue with content plugin then replcae $item->text = $item->text in below line as K2 fetch things from $item->text rather than $item->introtext
			$item->text = $item->introtext.'<a href="'.$url.'">'.XiText::_('COM_PAYPLANS_APP_K2_CATEGORY_SUBSCRIBE_TO_READ_MORE').'</a>';
			//$item->introtext = '';
			$item->fulltext = '';
			$item->link = $url;
			return true;
       }
	    
        if(in_array(self::ALLOWED, $allowedOrNot)){
       		return true;
        }
	}
	
	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}

		$newStatus  = $new->getStatus();
		
		$category        	= $this->getAppParam('category', array());
		$allowedEntries 	= $this->getAppParam('allowed_submission',0);
		$allowed_submission = explode(',', $allowedEntries);
		$subid              = $new->getId();
		$userid             = $new->getBuyer();
		$category			= (is_array($category))?$category:array($category);
		
		if($newStatus == PayPlansStatus::SUBSCRIPTION_ACTIVE)
		{
			//action is publish
			$action = 1;
			for($i=0;$i<sizeof($category);$i++)
			{   
				if(!isset($allowed_submission[$i]))
				{
					$allowed_submission[$i]= $allowed_submission[sizeof($allowed_submission)-1];
		     	}
				$this->_addToResource($subid, $userid, $category[$i], $this->_resource.$category[$i], $allowed_submission[$i]);
				$this->_publishOrUnpublishItem($userid, $allowed_submission[$i], $action, $category[$i]);
			}
		}
		
		if(($prev != null && $prev->getStatus() == PayPlansStatus::SUBSCRIPTION_ACTIVE) && 
				($newStatus == PayPlansStatus::SUBSCRIPTION_EXPIRED || $newStatus == PayPlansStatus::SUBSCRIPTION_HOLD))
		{
			//action is unpublish
			$action = 0;
			for($i=0;$i<sizeof($category);$i++)
			{   
				if(!isset($allowed_submission[$i]))
				{
					$allowed_submission[$i]= $allowed_submission[sizeof($allowed_submission)-1];
		     	}
				$this->_removeFromResource($subid, $userid, $category[$i], $this->_resource.$category[$i], $allowed_submissions[$i]);
				$this->_publishOrUnpublishItem($userid, $allowed_submission[$i], $action, $category[$i]);
			 }
		}

		return true;
	}
	
	function onPayplansAccessCheck(PayplansUser $user)
	{
		$option = JRequest::getVar('option', false);
		$view 	= JRequest::getVar('view', false);
		$task 	= JRequest::getVar('task', false);
		
		if($option!== 'com_k2'){
			return true;
		}
		
		// Is user Admin/SuperAdmin
		if($user->isAdmin()){
			return true;
		}
		
		if($view == 'item' && $task == 'save'){
			
			$catid 			= JRequest::getVar('catid', false);
			$categories		= $this->_getParentCategories($catid);
		
			foreach($categories as $category){
			    $allowedOrNot = plgPayplansK2::isUserAllowed(array($category),'k2category', 'category');

				if(in_array(self::BLOCKED, $allowedOrNot)){
                	$msg = XiText::_('COM_PAYPLANS_APP_K2_CATEGORY_SELECT_PLAN');
					XiFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=dashboard'), $msg);
				}
			    
				if(in_array(self::ALLOWED, $allowedOrNot)){
					$userItems   = $this->_getUserSubmission($user->getId(), $category);
					$record 	 = $this-> _getK2Resource($user->getId(),$category);
					
					//if no record found, return bcoz there is no data in resource table
					if(empty($record))
						return true;
					
					$userItemsCount = count($userItems);
						
					//when user want to edit it's already published entries then 1 entry should be margined
					//so when all things are done then allow user to post one if it is already published
				
					$editItemId = JRequest::getVar('id',false);
					
					if(!empty($editItemId) && array_key_exists($editItemId, $userItems)){
						continue;
					}
	                if( $userItemsCount >= $record->count)
					{
						$msg = XiText::_('COM_PAYPLANS_APP_K2_CATEGORY_YOU_ARE_NOT_ALLOWED_TO_ADD_MORE_SUBMISSIONS');	
						XiFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=dashboard'), $msg);
					}
				}
				
				if(in_array(self::DO_NOTHING, $allowedOrNot)){
		       		continue;
		        }
			
			}
		}
		return true;
	}
	
    // check if category is allowed for that user by some other App
	function _isCategoryAllowedbyK2UsrApp($categoryId)
	{   
        require_once  JPATH_ROOT.DS.'components'.DS.'com_k2'.DS.'helpers'.DS.'permissions.php';  
		$flag=false;
	 	
		$userid 	= XiFactory::getUser()->id;
		$k2user 	= K2HelperPermissions::getK2User($userid);
		
		//do nothing if user has no k2 user ID
		if($k2user)
		{
			$k2grp  	= K2HelperPermissions::getK2UserGroup($k2user->group);  
			$permissions= new XiParameter($k2grp->permissions);
			$k2Category = $permissions->get('categories');

			if((is_array($k2Category) && in_array($categoryId, $k2Category))
		 		     ||(!is_array($k2Category) && $k2Category==$categoryId))
	     	{
		 	     $flag = true;
	 		}
            //set flag if all categories are allowed
	 		if($k2Category == "all"){
	 			$flag = true;
	 		}
		}
    	return $flag;
	    	 		
	    }
	    
	 //render Widget
	public function renderWidgetHtml()
	{   
		$userid         = XiFactory::getUser()->id;
		if(!$userid){
			return '';
		}

		$k2Categories   = array();

		//get k2 categories
		if($userid){
	       $k2Categories = $this->_getAllowedK2Categories($userid);
		}

        if(empty($k2Categories)){
        	return '';
        }
	    $this->assign('k2_categories',$k2Categories);
        $data = $this->_render('widgethtml');
        return $data;
	}	
	
	function _getAllowedK2Categories($userid)
	{   
		$db         = XiFactory::getDBO();
		$db->setQuery(' SELECT `id`, `name` '
						.' FROM #__k2_categories '
						.' WHERE `trash` = "0" ');
		$categories     = $db->loadObjectList('id');

	  	//check which category is allowed to the user
        foreach($categories as $category){

         	$categoryArray = array($category->id);
	        $allowedOrNot  = plgPayplansK2::isUserAllowed($categoryArray,'k2category', 'category');

			// if some one allows then its allowed
			if(in_array(self::ALLOWED, $allowedOrNot)) {
				 continue;
			}

			// if no one allows but some one blocks then its blocked
			if(in_array(self::BLOCKED, $allowedOrNot)) {
	     	   	 unset($categories[$category->id]);
    	    }
    	     
			//else nothing to do
        }
        return $categories;
	 }
	 
	 function _getUserSubmission($userid, $catId)
	 {
	 		$db 	= JFactory::getDBO();
			$query = 'SELECT *'
				 	. ' FROM #__k2_items'
				 	. ' WHERE `created_by` = '.$userid.' AND `catid` = '. $catId
				 	;
		 	$db->setQuery( $query );
		 	return $db->loadObjectList('id');
	 }
	 
	public function _getParentCategories($catId)
	{
		$allCat   = array();
		while($catId)
		{
			$allCat[] = $catId;
			
			$db 	= XiFactory::getDBO();
			$query = 'SELECT `parent`'
				 	. ' FROM #__k2_categories'
				 	. ' WHERE `published` = 1 AND `id` = '.$catId;
				 	;
		 	$db->setQuery( $query );
		 	$catId = $db->loadResult();
		}
		return $allCat;
	}
	
	protected function _getK2Resource($userId,$category)
    {
		$rmodel = PayplansApp::getResourceModel();
		$record = $rmodel->loadRecords(array('user_id'  => $userId,
											'title' => 'com_k2.submission'.$category));
		
		$record = array_shift($record);
		if(empty($record) || !$record)
		{
			return false;
		}
		return $record;
	}
	
	protected function _publishOrUnpublishItem($userid, $publishCount, $action, $category)
	{
		// publish only those entries which are not currently published
		//and vice versa
		$complement = 1 - $action;
		$db    = XiFactory::getDBO();
		
		
		$query = ' UPDATE `#__k2_items`'
			 .' SET `published`='.$action 
			 .' WHERE `created_by`='.$userid.' AND `published`='.$complement.' AND `catid` ='.$category
			 . ' LIMIT '.$publishCount;
			
		$db->setQuery( $query );
		$db->query();
	}
	
	public static function getK2Categories()
	{
		$query = new XiQuery();
		return $query->select(' `id` as category_id, `name` ')
		             ->from('`#__k2_categories`')
		             ->dbLoadQuery()
		             ->loadObjectList('category_id');
	}
}

class PayplansAppK2categoryFormatter extends PayplansAppFormatter
{
	//get Rules 
	function getVarFormatter()
	{
		$rules = array('_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       'function' => 'getAppPlans'),
					   'app_params'      => array('formatter'=> 'PayplansAppK2categoryFormatter',
										       'function' => 'getFormattedParams'));
		return $rules;
	}
	
	// formatt params
	function getFormattedParams($key,$value,$data)
	{
		//do nothing if k2 is not installed
		if(!JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_k2')){
			return false;
		}
		$categories  = PayplansAppK2category::getK2Categories();
		if(empty($value['category']))
		{
			return false;
		}
		// categories
		if(is_array($value['category'])){
		foreach($value['category'] as $param){
			$category[] = $categories[$param]->name;
		}
		$value['category']    = implode(',', $category);
		}
		else{
			$value['category'] = $categories[$value['category']]->name;
		}
		
	}
}
