<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	k2
* @contact 		support+payplans@readybytes.in
* 
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppK2item extends PayplansApp
{
	protected $_location = __FILE__;
	const DO_NOTHING     = -1;
	const ALLOWED 	     =  1;
	const BLOCKED        =  0;
	
	public function isApplicable($refObject=null, $eventName='')
	{
		// applicable only if k2 component exists
		if(!JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_k2')){
			return false;
		}

		if($eventName == 'onPayplansAccessCheck' || $eventName == 'onK2PrepareContent' ){
			return true;
		}
		return parent::isApplicable($refObject, $eventName);

	}
	
	function onPayplansAccessCheck(PayplansUser $user)
	{
		//return if user is admin
		if($user->isAdmin())
		{
			return false;
		}
		
		$input 	 = PayplansFactory::getApplication()->input;
		$option	 = $input->get('option', '');	
		
		if($option != 'com_k2'){
			return false;
		}
		
	
		$view 		= $input->get('view', '');
		$layout 	= $input->get('layout');
		$task 		= $input->get('task');
		$cat_id		= strtok($input->get('id', 0, 'string'),':');
		
		if($view == 'itemlist' && ($layout == 'category' || $task == 'category')){
			//it should also restricted for parent categories also
			$parentCat = $this->_getParentCategories($cat_id);
			
			$apprestriction = $this->getAppParam('restricted_category');
			if(empty($apprestriction))
			{
				return true;
			}
			$apprestriction = (is_array($apprestriction))? $apprestriction : array($apprestriction);
			if(array_intersect($parentCat, $apprestriction)){
			
				//if app restricted for specific plans then need to check user plans
				if($this->getParam('applyAll',false) == false){
				 	// if object is of interest as per plans selected
					$ret = array_intersect($this->getPlans(), $user->getPlans());
					if(count($ret) > 0 ){
						return true;
					}
					
					$msg = XiText::_('COM_PAYPLANS_APP_K2_YOU_ARE_NOT_ALLOWED_TO_VIEW_CATEGORY');
					PayplansFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=dashboard'), $msg);
				}			
				
				//if appy all is true then user can have any plan
				if(count($user->getPlans()) == 0){
					$msg = XiText::_('COM_PAYPLANS_APP_K2_YOU_ARE_NOT_ALLOWED_TO_VIEW_CATEGORY');
					PayplansFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=dashboard'), $msg);
				}
			}
		}
		return true;
		
	} 


	function onK2PrepareContent($item, $params, $limitstart)
	{
		if(!is_a($item,'stdClass')){
			return true;
		}

		$itemId       = array($item->id);
		$appType	  = $this->getAppParam('restrictionOn', '');
		
		if ($appType == 'item'){
			$allowedOrNot = plgPayplansK2::isUserAllowed($itemId,'k2item','k2item');
		}
		else {
			$all_category		= $this->_getParentCategories($item->catid);
			$allowedOrNot  		= plgPayplansK2::isUserAllowed($all_category, 'k2item','restricted_category');
		}
        if(in_array(self::ALLOWED, $allowedOrNot)){
       		return true;
        }

        if(in_array(self::BLOCKED, $allowedOrNot)){
			$url             = XiRoute::_('index.php?option=com_payplans&view=plan');
		    $item->text      = $item->introtext. '<a href="'.$url.'">'.XiText::_('COM_PAYPLANS_APP_K2ITEM_SUBSCRIBE_PLAN').'</a>';
			$item->fulltext  = '';
			$item->video 	 = '';
			$item->link      = $url;
			//handling of K2 item and content hider 
			$args = array($context='com_k2.item', &$item, &$params, $limitstart);
			return XiHelperPlugin::trigger('onContentPrepare', $args);
			
       }
	}
	
	 //render Widget
	public function renderWidgetHtml()
	{   
		$userid         = XiFactory::getUser()->id;
		if(!$userid){
			return '';
		}

		$k2Items   = array();

		//get k2 categories
		if($userid){
	       $k2Items = $this->_getAllowedK2Items($userid);
		}

        if(empty($k2Items)){
        	return '';
        }
	    $this->assign('k2_items',$k2Items);
        $data = $this->_render('widgethtml');
        return $data;
	}	
	
	function _getAllowedK2Items($userid)
	{   
		
		$query      =  new XiQuery();
		$items      =  $query->select(' `id`, `title` ')
		               ->from('`#__k2_items`')
		               ->where('`published` = 1')
		               ->dbLoadQuery()
		               ->loadObjectList('id');

	  	//check which item is allowed to the user
        foreach($items as $item){
	        $allowedOrNot  = plgPayplansK2::isUserAllowed(array($item->id),'k2item','k2item');

			// if some one allows then its allowed
			if(in_array(self::ALLOWED, $allowedOrNot)) {
				 continue;
			}

			// if no one allows but some one blocks then its blocked
			if(in_array(self::BLOCKED, $allowedOrNot)) {
	     	   	 unset($items[$item->id]);
    	    }
    	     
			//else nothing to do
        }

        return $items;
	 }
	 
 	public static function getK2Items()
	{
		$query = new XiQuery();
		return $query->select(' `id` as item_id, `title` ')
		             ->from('`#__k2_items`')
		             ->where('`trash` = 0')
		             ->dbLoadQuery()
		             ->loadObjectList('item_id');
	}
	
	protected function _getParentCategories($catId)
    {
		$allCat   = array();
		while($catId)
		{
			$allCat[] = $catId;
			
			$db 	 = XiFactory::getDBO();
			
			$query = 'SELECT `parent`'
				 	. ' FROM #__k2_categories'
				 	. ' WHERE `published` = 1 AND `id` = '.$catId;

		 	$db->setQuery( $query );
		 	$catId = $db->loadResult();
		}
		
		return $allCat;
    }
}

class PayplansAppK2itemFormatter extends PayplansAppFormatter
{
	//get Rules 
	function getVarFormatter()
	{
		$rules = array('_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       'function' => 'getAppPlans'),
					   'app_params'      => array('formatter'=> 'PayplansAppK2itemFormatter',
										       'function' => 'getFormattedParams'));
		return $rules;
	}
	
	// formatt params
	function getFormattedParams($key,$value,$data)
	{
		//do nothing if component is not installed
		if(!JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_k2')){
			return false;
		}
		$items  = PayplansAppK2item::getK2Items();
		if(empty($value['k2item']))
		{
			return false;
		}
		// categories
		if(is_array($value['k2item'])){
			foreach($value['k2item'] as $param){
				$item[] = $items[$param]->title;
			}
			$value['k2item']    = implode(', ', $item);
		}
		else{
			$value['k2item'] = $items[$value['k2item']]->title;
		}		
	}
}
