 <?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Kiss-Metrics Tracking
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppKissmetrics extends PayplansApp
{
	protected $_location	= __FILE__;
	
	public function isApplicable($refObject = null, $eventName='')
	{
		// if not with reference to payment then return
		if($eventName == 'onPayplansViewAfterRender' 
			|| $eventName == 'onPayplansSubscriptionAfterSave' 
			|| $eventName == 'onPayplansOrderAfterSave' 
			|| $eventName == 'onPayplansInvoiceAfterSave'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
	protected function _initializeTracking()
	{
		KM::init($this->getAppParam('api_key'));
	}
	
	protected function _track($eventName, $args, $task = '')
	{
		//Checked to Track Login page and Pricing page Visit
		if(($task == 'login' || $task == 'subscribe') && $args['User Id'] == 0){
			
			//If User Id == 0 means page visited by a visitor
			$args['User Id'] = 'Non-Subscribed';
			KM::identify($args['User IP']);
			KM::record($eventName, $args);
			return true;
		}

		$userId 		= array_pop($args);
		$userDetails 	= $this->_getUserDetails($userId);
		$args		 	= array_merge($args, $userDetails);
		//User Logged-in then Tracking done via Email-Id of User
		KM::identify($args['Email']);
		KM::record($eventName, $args);
	}
	
	//For Tracking Login page visit
	public function onPayplansViewAfterRender(XiView $view, $task, $output)
	{
		//If view != plan
		if(!($view instanceof PayplanssiteViewPlan)){
			return;
		}

		//Initialize Tracking of KM
		$this->_initializeTracking();
				
		//If $task is login then Have to track Login page visit Else $task is subscribe and have to track pricing page visit
		$eventName    = XiText::_('COM_PAYPLANS_APP_KISSMETRICS_EVENT_VISITED_PRICING_PAGE');
		
		if($task == 'login'){
			$eventName	  = XiText::_('COM_PAYPLANS_APP_KISSMETRICS_EVENT_VISITED_LOGIN_PAGE');
		}
				
		$user			  = PayplansFactory::getUser();
		$data['User IP']  = $_SERVER['REMOTE_ADDR'];
		$data['User Id']  = $user->id;

		return $this->_track($eventName, $data, $task);
	}
	
	//For tracking Events like "Plan Upgradation", "Subscription Status Change",etc.
	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		//Initialize Tracking of KM
		$this->_initializeTracking();
		
		//For Tracking Add to Cart
		if($prev == null && $new->getStatus() == PayplansStatus::NONE){
			
			//Tracks Event Add To Cart Event			
			$eventName	  = XiText::_('COM_PAYPLANS_APP_KISSMETRICS_EVENT_PLAN_ADDED_TO_CART');
			$agrs 		  = $this->_addToCartDetails($new);
			return $this->_track($eventName, $agrs);		
		}
		
		//For Tracking Upgradation of a plan.		
		if($prev->getStatus() != PayplansStatus::SUBSCRIPTION_ACTIVE 
				&& $new->getStatus() == PayplansStatus::SUBSCRIPTION_ACTIVE)
		{
			//Gets the order from subscription instance $new
			$order = $new->getOrder(PAYPLANS_INSTANCE_REQUIRE);
			$oldSubscriptionId = $order->getParam('upgrading_from',null);

			if(($order)	&& isset($oldSubscriptionId) && !empty($oldSubscriptionId))
			{					
				$eventName = XiText::_('COM_PAYPLANS_APP_KISSMETRICS_EVENT_SUBSCRIPTION_UPGRADED');
				$args = $this->_upgradeDetails($oldSubscriptionId, $new);
				// do not return, we need to track subscription status change
				$this->_track($eventName, $args);
			}
		}
		
		//Tracks Status Change of Subscription
		if($prev->getStatus() != $new->getStatus())
		{		
			$eventName	  = XiText::_('COM_PAYPLANS_APP_KISSMETRICS_EVENT_SUBSCRIPTION_STATUS_CHANGED');
			$args 		  = $this->_subStatusChangeDetails($prev, $new);
			return $this->_track($eventName, $args);
		}
		
		return true;
 	}
 	
 	//For Tracking "Order Cancellation"
	public function onPayplansOrderAfterSave($previous, $current)
	{
		//If Both object of order are set and Status from $current ORDER_CANCEL and Status from $previous isa not ORDER_CANCEL means Order is going to be cancelled
		if(isset($previous) && $previous->getStatus() !== PayplansStatus::ORDER_CANCEL && $current->getStatus() == PayplansStatus::ORDER_CANCEL)
		{
			//Initialize Tracking of KM
			$this->_initializeTracking();
	
			$eventName		= XiText::_('COM_PAYPLANS_APP_KISSMETRICS_EVENT_ORDER_CANCELLED');
			//Gets Details of Order cancellation
			$args			= $this->_orderCancelDetails($current);
			//Tracks Order Cancellation event in case of Recurring Subscription
			$this->_track($eventName, $args);	
		}
	}
	
	//For Tracking "Check-out", "Paid", "refund"
	public function onPayplansInvoiceAfterSave($prev, $new)
	{
		if(!isset($prev) || !isset($new)){
			return true;			
		}
		// Before save and after save, Invoice status should not be same.
		if ($new->getStatus() == $prev->getStatus()){
			return true;
		}
		
		//Initialize Tracking of KM
		$this->_initializeTracking();
		$status = $new->getStatus();

		//Which Event to track, Decision made on status of invoice from $new
		switch ($status)
		{
			// Invoice paid
			case PayplansStatus::INVOICE_PAID : 
				$args		= $this->_invoicePaidDetails($new);
				$eventName	= array_pop($args);
				break ;

			// Invoice Refund
			case PayplansStatus::INVOICE_REFUNDED:
				$eventName	= XiText::_('COM_PAYPLANS_APP_KISSMETRICS_EVENT_INVOICE_REFUNDED');
				$args		= $this->_refundDetails($new);
				break;

			// Invoice Confirm or none
			case PayplansStatus::INVOICE_CONFIRMED:
				$eventName	= XiText::_('COM_PAYPLANS_APP_KISSMETRICS_EVENT_INVOICE_CHECKED_OUT');
				$args		= $this->_invoiceConfirmDeatils($new);
				break;

			case PayplansStatus::NONE:
					return true;
		}
		return	$this->_track($eventName, $args);		 	
	}
	
	//To Find User Details From Buyer ID.
	protected function _getUserDetails($buyer_id)
	{
		$data['User Details'] = '';
		$user				  = PayplansFactory::getUser($buyer_id);
		$data['User Name']	  = $user->username;
		$data['Email']		  = $user->get('email');
		$data['User Id']	  = $buyer_id;
		return $data;
	}
	
	//To Find Upgrading details
	protected function _upgradeDetails($oldSubscriptionId, $newSubscription)
	{
		//Data from old subxcription from which we are going to upgrade.
		$oldSubscription	 			   = PayplansSubscription::getInstance($oldSubscriptionId);
		$data['Old Subscription Details']  = '';
		$data['Subscription Key']		   = $oldSubscription->getKey();
		$data['Total']					   = $oldSubscription->getTotal();
		$data['Activation Date']		   = $oldSubscription->getSubscriptionDate()->toMySQL();
		$data['Expiration Date']		   = $oldSubscription->getExpirationDate()->toMySQL();
		$data['Plan Name']				   = $oldSubscription->getTitle();
							
		//Data From new subscription to which we are going to upgrade.
		$data['New Subscription Details']  = '';
		$data['Subscription-Key']		   = $newSubscription->getKey();
		$data['New Total']		     	   = $newSubscription->getTotal();
		$data['Activation-Date'] 		   = $newSubscription->getSubscriptionDate()->toMySQL();
		$data['Expiration-Date'] 		   = $newSubscription->getExpirationDate()->toMySQL();
		$data['Plan-Name']	  	 	       = $newSubscription->getTitle();

		//User id must be at last because _initializeTracking() pops up User Id from last to find User Details.
		$data['User Id']				   = $oldSubscription->getBuyer();
		
		return $data;
	}
	
	//To Find Details When an Invoice Gets Paid
	protected function _invoicePaidDetails($invoice)
	{
		$data['Paid Invoice Details'] = '';
		//Gets Details of invoice
		$data['Status of Invoice']	  = $invoice->getStatusName();
		$data['Plan Name']			  = $invoice->getTitle();
		$data['Price']				  = $invoice->getPrice();
		$data['User Id']			  = $invoice->getBuyer();		

		//Set Event "Payment Recieved"
		$data['Event Name']			= XiText::_('COM_PAYPLANS_APP_KISSMETRICS_EVENT_PAYMENT_RECIEVED');

		// Case of renewal
		$obj = $invoice->getReferenceObject(PAYPLANS_INSTANCE_REQUIRE);
		if($obj->getLastMasterInvoice() == $invoice->getId() && $invoice->getCounter() > 1){
			$data['Event Name']	= XiText::_('COM_PAYPLANS_APP_KISSMETRICS_EVENT_SUBSCRIPTION_RENEWED');
		}
		return $data;
	}
	
	//To Find Details of Invoice made Check-Out
	protected function _invoiceConfirmDeatils($invoice)
	{
		//Gets Details of invoice
		$data['Invoice Details']	 = '';
		$data['Plan Name']			 = $invoice->getTitle();
		$data['Price']				 = $invoice->getPrice();
		$data['User Id']			 = $invoice->getBuyer();
		
		return $data;
	}
	
	//To Find Details of subscription status changes
	protected function _subStatusChangeDetails($prevSubscription, $newSubscription)
	{
		//Gets Subscription Key
		$subscription_key 					 = $newSubscription->getKey($newSubscription->getId());

		//finds Previous status of subscription
		$data['Subscription Status Details'] = '';
		$prev_status						 = $prevSubscription->getStatusName();
		$prev_status						 = explode("-", $prev_status);
		$data['Previous-Status']	 		 = $prev_status[1];
		$data['Previous-Status']        	 = ($data['Previous-Status'] == 'Status') ? 'No-Status' : $data['Previous-Status'];
		
		//finds New status of subscription
		$new_status 	 					 = $newSubscription->getStatusName();
		$new_status							 = explode("-", $new_status);
		$data['New Status']					 = $new_status[1];
		$data['New Status']					 = ($data['New Status'] == 'Status') ? 'No-Status' : $data['New Status'];
		
		$data['Plan Name']					 = $newSubscription->getTitle();
		$data['Price']						 = $newSubscription->getPriceForInvoice(1);
		$data['Activation Date']			 = $newSubscription->getSubscriptionDate()->toMySQL();
		$data['Expiration Date']			 = $newSubscription->getExpirationDate()->toMySQL();
		$data['User Id']				     = $newSubscription->getBuyer();

		return $data;
	}
	
	//Find Details of Plan Added To Cart
	protected function _addToCartDetails($subscription)
	{
		//Gets subscription Details
		$data['Selected Plan Details']  = '';
		$data['Price']					= $subscription->getPrice();
		$data['Plan Name']				= $subscription->getTitle();
		$data['User Id']				= $subscription->getBuyer();

		return $data;
		
	}
	
	//To Find Details of Order Cancellation
	protected function _orderCancelDetails($order)
	{
		//Gets Details of Order
		$data['Order Cancellation Deatils'] = '';
		$data['Price']						= $order->getTotal();
		$data['Plan Name']					= $order->getTitle();

		$subscription						= $order->getSubscription();
		$data['Subscription Key']			= $subscription->getKey();
		$data['User Id']					= $order->getBuyer();

		return $data;
	}
	
	//To Find Refund Details
	protected function _refundDetails($invoice)
	{
		$data['Refund Details']	= '';
		$data['Invoice Key']	= $invoice->getKey();
		$data['Price']			= $invoice->getPrice();
		$data['User Id']		= $invoice->getBuyer();
		return $data;
	}
}
