<?php

/**
* @copyright	Copyright (C) 2009 - 2012 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	KUNENA
* @contact		shyam@joomlaxi.com
*/
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Payplans Kunena Plugin
 *
 */
class plgPayplansKunena extends XiPlugin
{
	const DO_NOTHING = -1;
	const ALLOWED 	 =  1;
	const BLOCKED    =  0;
	
	public function onPayplansSystemStart()
	{
		//add discount app path to app loader
		$appPath = dirname(__FILE__).DS.'kunena'.DS.'app';
		PayplansHelperApp::addAppsPath($appPath);

		return true;
	}
	
	function onPayplansAccessCheck(PayplansUser $user)
	{
		$option = JRequest::getVar('option', false);
		$func 	= JRequest::getVar('func', false);
		$view 	= JRequest::getVar('view', false);
		
		if($option!== 'com_kunena'){
			return true;
		}
		
		// Is user SuperAdmin/moderator
		$kunenaUser = KunenaFactory::getUser($user->getId());
		if($user->isAdmin() || $kunenaUser->isModerator()){
			return true;
		}

		//get category id
		$catId 	 	= JRequest::getVar('catid', 0);
		
		$columnName = $this->_getCloumnName();
		$allCat   	= $this->_getParentCategories($catId, $columnName);
		
		$funcArray	=	array('showcat', 'view', 'post','listcat');
		$viewArray	=	array('category', 'topic');
		
	   //if user is accessing thread list or any thread
	   if(in_array($func, $funcArray) || in_array($view, $viewArray)){
	   		
	   		//we need to check block access(app param)
	   		//bcoz admin can give access to these
	   		$blockAccess = 0;	
	   }
	   else{
	   		//else we don't need to check block access(app param)
	   		//bcoz user can't edit or post on any thread without subscription
			$blockAccess  = 1;
	   }
	   
	   //check if category is allowed by kunena app
	   $allowedOrNot = $this->_isAllowed($allCat, $user, $blockAccess, $func, $view);
	   
       if(in_array(self::ALLOWED, $allowedOrNot)){
       	return true;
       }

       if(in_array(self::BLOCKED, $allowedOrNot)){
       	     //check if user is not logged-in then redirect him/her to login page
       		if(!XiFactory::getUser()->id){
	       		$join = "<a href='index.php?option=com_payplans&view=plan&task=subscribe' >".XiText::_('COM_PAYPLANS_ELEMENT_POPUP_CLICK_HERE')." </a>";
		   		$msg  = XiText::sprintf('COM_PAYPLANS_APP_KUNENA_EITHER_LOGIN_OR_SUBSCRIBE',$join);
		   		XiFactory::getApplication()->redirect(XiHelperJoomla::getLoginLink(),$msg);
	       	}
	       	$msg = XiText::_('COM_PAYPLANS_APP_KUNENA_UPGRADE_PLAN_DESC');
	   		XiFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=plan&task=subscribe'), $msg);
		}

		// else do nothing
	   return true;
	}
	
	//render Widget
	public function renderWidgetHtml()
	{   
		$kunenaCategories  = array();
		$userid            = XiFactory::getUser()->id;
        $appCategories 	   = array();
		if($userid){
			//get kunena categories
		    $kunenaCategories =    $this->_getAllowedCategories($userid);
		}

		//don't display widget if empty
		if(empty($kunenaCategories)){
	    	return '';
		}

	    $this->assign('kunena_categories',$kunenaCategories);
        $data = $this->_render('widgethtml');
        return $data;
	}	
	
	protected function _getAllowedCategories($userid)
	{
		$columnName    = $this->_getCloumnName(); 
		
		$db         = XiFactory::getDBO();

		$db->setQuery(' SELECT `id`, `name` '
						.' FROM #__kunena_categories '
						.' WHERE `published` = "1" AND ' . $columnName . ' IN (SELECT `id` FROM #__kunena_categories WHERE `published` = "1" ) ');

		$categories = $db->loadObjectList('id');

		$user       = PayplansUser::getInstance($userid);
        $kunenaApps = PayplansHelperApp::getAvailableApps('Kunena');
      
	  	//check which category is allowed to the user
        foreach($categories as $category){

         	$categoryArray = array($category->id);
	        $allowedOrNot  = $this->_isAllowed($categoryArray, $user, 1);
    	    $count 		   = array_count_values($allowedOrNot);

			// if some one allows then its allowed
			if(in_array(self::ALLOWED, $allowedOrNot)) {
				 continue;
			}

			// if no one allows but some one blocks then its blocked
			if(in_array(self::BLOCKED, $allowedOrNot)) {
	     	   	 unset($categories[$category->id]);
    	    }
    	     
			//else nothing to do
        }

        return $categories;
      }

	//check if category is allowed or not    
	protected function _isAllowed($category,$user,$checkBlockAccess, $func = '', $view = '')
    {
     	$kunenaApps = PayplansHelperApp::getAvailableApps('Kunena');
  	 	$appResult  = array();
		
		$kunenaVersion   = $this->_getKunenaVersion();
		
	  	//check which app allow category, which not allow and which app do nothing
		foreach($kunenaApps as $app){

			$allowedCategoryByApp =  $app->getAppParam('kunenaCategories', array());
			
			if(!$checkBlockAccess){
				$blockAccess =  $app->getAppParam('blockaccess', 1);
			}
			else{
				$blockAccess = $checkBlockAccess;
			}

			if(!is_array($allowedCategoryByApp)){
				$allowedCategoryByApp   = array($allowedCategoryByApp);
			}

		 	//if app is not applicable on that category 
		 	if(count(array_intersect($category, $allowedCategoryByApp)) == 0){
		 		$appResult[]    = self::DO_NOTHING;
		 		continue;
		 	}

  	 	 	$subs       = $user->getPlans();
  	 	 	//if user has no plans & visibility access is set to none
         	if(empty($subs) && $blockAccess == 1){
           		$appResult[]  = self::BLOCKED;
           		continue;
          	}
         
			// if user has plans and app is core app 
		 	if(!empty($subs) && $app->getParam('applyAll',false) != false){
		 		$appResult[]  = self::ALLOWED;
           		continue;
		 	}

			//if user has app plans
			$appPlans = $app->getPlans();
			if(array_intersect($subs, $appPlans) != false){
				 $appResult[]  = self::ALLOWED;
        	  	 continue;
			}
			
			if($kunenaVersion === '1'){
				$appResult		= $this->_kunenaLowerVersion($blockAccess, $appResult, $func);	
			}
			else{
				$appResult		= $this->_kunenaHigherVersion($blockAccess, $appResult, $view);
			}
  	   	}
  	  return $appResult;
    }
    
	//kunena 1.xx
    protected function _kunenaLowerVersion($blockAccess, $appResult = array(), $func = '')
    {
		$action 	= JRequest::getVar('action', false);
		$do 		= JRequest::getVar('do', false);
			
		$doArray		=	array('reply', 'edit');
		
		//visibility access is set to readonly, it means user can access thread
		//if user has allowed to access category(only visible not editable)
		if(($blockAccess == 0 || $blockAccess == 3) && ($func == 'view' || $func == 'showcat' || $func == 'listcat')){
			$appResult[]  = self::ALLOWED;
		}
		//visibility access is set to overview, it means user can access thread list
		//but he can't access particular thread
		elseif($blockAccess == 2 && ($func == 'showcat' || $func == 'listcat')){
			$appResult[]  = self::ALLOWED;
		}
		//visibility access is set to ReplyOnly, it means user can reply on any thread
		//but he can't create new thread
  		elseif($blockAccess == 3 && $func == 'post'){
			$id 	  = JRequest::getVar('id');
			$parentId = JRequest::getVar('parentid');
			if(in_array($do, $doArray) || 
             (($action == 'post' ) && !isset($parentid) && !isset($id)))
				$appResult[]  = self::ALLOWED;
			else
				$appResult[]  = self::BLOCKED;
		}
		else {
		   $appResult[]  = self::BLOCKED;
		}
		
  	  return $appResult;
    }
    
    //kunena 2.xx
	protected function _kunenaHigherVersion($blockAccess, $appResult = array(), $view = '')
    {
		$task 		= JRequest::getVar('task', false);
		$layout 	= JRequest::getVar('layout', false);
		
	  	//check which app allow category, which not allow and which app do nothing
			
		$taskArray		=	array('subscribe','post', 'edit');
		$layoutArray 	=	array('default', 'edit', 'reply', 'unread');
		
		if($blockAccess != 1 && $view == 'category' && $task == 'subscribe')
		{
			$appResult[] = self::BLOCKED;
			return $appResult;
		}
		
		//visibility access is not set to none, it means user can access thread list
		if($blockAccess != 1 && $view == 'category'){
			$appResult[]  = self::ALLOWED;
		}
		//visibility access is set to ReadOnly
		elseif($blockAccess == 0 && $view == 'topic' && !in_array($task, $taskArray)){
			$appResult[]  = self::ALLOWED;
		}
		//visibility access is set to ReplyOnly, it means user can reply on any thread
		//but he can't create new thread
  		elseif($blockAccess == 3 && $view == 'topic'){
			if(!$task && !$layout){
				$appResult[]  = self::ALLOWED;
			}
			// in case of new topic creation task is post and parent id is 0
			// so we have to restrict here 
			elseif(( in_array($task, $taskArray) && (int)JRequest::getVar('parentid') != 0) 
                  || in_array($layout, $layoutArray) )
			{
				$appResult[]  = self::ALLOWED;
			}
			else{
				$appResult[]  = self::BLOCKED;
			}
		}
		else {
		   $appResult[]  = self::BLOCKED;
		}
		
  	  return $appResult;
    }
    
    protected function _getKunenaVersion()
    {
    	$db 	 = XiFactory::getDBO();
		
		//check if kunena version is lower than 2.x
		$db->setQuery("Select `version` FROM `#__kunena_version` ORDER BY `version` DESC");
		$version = $db->loadResult();
		
		$kunenaVersion = substr($version,0,1);
		
		return $kunenaVersion;	
    }
    
    protected function _getCloumnName()
    {
    	$kunenaVersion = $this->_getKunenaVersion();
    	$columnName    = 'parent_id';
		
		if($kunenaVersion ===  '1'){
			$columnName = 'parent';
		}
		
		return $columnName;
    }
    
    protected function _getParentCategories($catId, $columnName)
    {
    	//we have to traverse to parent =0
		//XITODO : improve performance
		$allCat   = array();
		while($catId)
		{
			$allCat[] = $catId;
			
			$db 	 = XiFactory::getDBO();
			
			$query = 'SELECT `'.$columnName.'`'
				 	. ' FROM #__kunena_categories'
				 	. ' WHERE `published` = 1 AND `id` = '.$catId;

		 	$db->setQuery( $query );
		 	$catId = $db->loadResult();
		}
		
		return $allCat;
    }
}
