<?php

/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	Limit Plan
* @contact		support+payplans@readybytes.in
*/
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Payplans Limit Plan Plugin
 *
 */
class plgPayplansLimitplan extends XiPlugin
{ 	
	public function onPayplansViewBeforeRender(XiView $view, $task)
	{	
		if(($view instanceof PayplansadminViewPlan) && $task == 'edit'){
			return $this->_renderAdminPlanEditScreen($view);
		} 
	}

	public function onPayplansGetPlugin()
	{
	    $xml = dirname(__FILE__).DS.'limitplan.xml';
	    if (file_exists($xml)) {
	           $xmlContent = simplexml_load_file($xml);
	     }
	     else {
	           $xmlContent = null;
	     }
	     return array('name'   => 'limitplan', 
	             	  'value' =>$xmlContent);
	 }

	
	protected  function _renderAdminPlanEditScreen(PayplansadminViewPlan $view)
	{	
		$planId 			= $view->getModel()->getId();
		$plan				= PayplansPlan::getInstance($planId);
	
		$limit_count		= $plan->getParams()->get('limit_count', 	0);
		$scheduled			= $plan->getParams()->get('scheduled', 		0);
		$startDate			= $plan->getParams()->get('start_date',  	'');
		$endDate			= $plan->getParams()->get('end_date',  	    '');
		$totalCount			= $plan->getParams()->get('total_count',    0);		
		
		$this->_assign('limit_count', 	$limit_count);
		$this->_assign('scheduled', 	$scheduled);
		$this->_assign('start_date', 	$startDate);
		$this->_assign('end_date', 		$endDate);
		$this->_assign('total_count',	$totalCount);
		
		return array('payplans-admin-plan-limit-parameters' => $this->_render('limitplan_planparam'));
	}
	
	
	public function onPayplansPlanBeforeSave($previous, $current)
	{
		$session		= PayplansFactory::getSession();
		if($session->get('is_limitsave_no', false)){
			$session->clear('is_limitsave_no');
			return true;
		}
		
		$application	= PayplansFactory::getApplication();
		$limitCount		= $application->input->get('limit_count',  0);
		$totalCount		= $current->getParams()->get('total_count', 0);
		$planId			= $current->getId();

		// Do nothing when limit count value is less than total count		
		if($limitCount && $limitCount < $totalCount){
			$message = XiText::_('PLG_PAYPLANS_LIMITPLAN_PLAN_LIMITCOUNT_ERROR');
			$application->redirect(XiRoute::_('index.php?option=com_payplans&view=plan&task=edit&id='.$planId),$message, 'error');
			return true;
		}

		$scheduled		= $application->input->get('scheduled', 0);
		$startDate		= $application->input->get('start_date', '');
		$endDate		= $application->input->get('end_date', '');
		
		$current->setParam('limit_count', $limitCount);
		$current->setParam('scheduled', $scheduled);
		if($scheduled){
			$current->setParam('start_date', 	$startDate);
			$current->setParam('end_date',  	$endDate);
		}
		
		if($scheduled && $current->getPublished()){
			$result = $this->_checkplanApplicable($planId);
			if($result == false){
				$current->set('published', 0);
			}
		}		
	}
	
	public function onPayplansOrderAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}	
		
		if($new->getStatus() == PayplansStatus::ORDER_COMPLETE)
		{
			$subscription = $new->getSubscription();
			if(is_a($subscription, 'PayplansSubscription')){
				$subscription->load($subscription->getId());
				$plans 		= $subscription->getPlans(PAYPLANS_INSTANCE_REQUIRE);
				$plan		= array_pop($plans);
				
				$limitCount	= $plan->getParams()->get('limit_count', 0); 
				$totalCount	= $plan->getParams()->get('total_count', 0);
			
				if(!$limitCount){
					return true;
				}
			
				$totalCount	= $totalCount + 1 ;
				$plan->setParam('total_count', $totalCount);
				if($totalCount >= $limitCount){
					$plan->set('published', 0);	
				}
				
				// Set session vairable because no need to work on onPayplansPlanBeforeSave defined in this plugin itself
				$session 	= PayplansFactory::getSession();
				$session->set( 'is_limitsave_no' ,  true);
				$plan->save();	
			}
			
		}
	}
	
	/**
	 * publish or unpublish plan
	 */
	public function _publishOrUnpublishPlan($plan, $published)
	{
		$session 	= PayplansFactory::getSession();
		$session->set( 'is_limitsave_no' ,  true);
		
		$plan->set('published', $published);
		$plan->save();
	}
	
	
	public function onPayplansCron()
	{		
		$pubPlans			= PayplansHelperPlan::getPlans(array('published' => 1));
		$unpubPlans			= PayplansHelperPlan::getPlans(array('published' => 0));
		$plans 				= array_merge($pubPlans, $unpubPlans);
		
		foreach ($plans as $plan) {
			$scheduled	= $plan->getParams()->get('scheduled',  0);
			if($scheduled){
				//If limitcount is also set on this plan then do nothing when totalcount equals to limitcount
				$limitcount			= $plan->getParams()->get('limit_count',  0);
				$totalcount			= $plan->getParams()->get('total_count', 0);
				if($limitcount && ($limitcount == $totalcount))
					continue;

				$result = $this->_checkplanApplicable($plan->getId());
				if($result){
					$this->_publishOrUnpublishPlan($plan, 1);
				}else{
					$this->_publishOrUnpublishPlan($plan, 0);
				}
			}
		}
		
		return true;
	}
	
	/**
	 * This function is used to check that plan lies between particular time duration or not.
	 */
	protected function _checkplanApplicable($planId)
	{
		$plan				= PayplansPlan::getInstance($planId);
		$startDate			= $plan->getParams()->get('start_date',  0);
		$endDate			= $plan->getParams()->get('end_date',  	 0);
		
		$currentDate		= new XiDate('now');
		$startDate 			= !empty($startDate) ? new XiDate($startDate) : $startDate;
		$endDate   			= !empty($endDate)   ? new XiDate($endDate)   : $endDate;
		
		//when range is not set then do nothing
		if(empty($startDate) && empty($endDate)){
			return true;
		}
		
		// when both date set then check current date lies between this range
		if(!empty($startDate) && !empty($endDate) && ( ($currentDate->toUnix() > $endDate->toUnix()) || ($currentDate->toUnix() < $startDate->toUnix()))){
			return false;
		}
		
		// when range is set then check subscription date whether lies in that range 
		if(!empty($startDate) && !empty($endDate) && ($currentDate->toUnix() >= $startDate->toUnix()) && ($currentDate->toUnix() <= $endDate->toUnix())){
			return true;
		}                                             
		
		//when start date is set and end date is not, check plan lies in the range
		if(!empty($startDate) && $currentDate->toUnix() >= $startDate->toUnix()){
			return true;
		}
		
		//when end date is set and start date not , check plan lies in the range
		if(!empty($endDate) && $currentDate->toUnix() <= $endDate->toUnix()){
			return true;
		}
		
		//when end date is set then check current date is higher then end date (to unpublish plan)
		if(!empty($endDate) && $currentDate->toUnix() > $endDate->toUnix()){
			return false;
		}

		return true;
	}
}
