<?php
/**
 * @copyright	Copyright (C) 2009 - 2016 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
 * @license		GNU/GPL, see LICENSE.php
 * @package		PayPlans
 * @subpackage		MailChimp
 * @contact 		support+payplans@readybytes.in
 *
 */
if(defined('_JEXEC')===false) die();


class PayplansAppMailchimp extends PayplansApp
{
	protected $_location	= __FILE__;
	protected $_resource	= 'com_mailchimp.list';
	
	public function collectAppParams(array $data)
	{  
		$appParams = $data['app_params'];
		$apiKey = $appParams['mailchimpApiKey'];
		$DataCenter = substr($apiKey, strrpos($apiKey, '-') + 1);
		$merchantemail = $appParams['mailchimpMerchantEmail'];
		$header = array(
			"Authorization: Basic " . base64_encode($merchantemail.':'. $apiKey),

		);
		$retval = $retval = $this->curlGetrequest($header, 'https://'.$DataCenter.'.api.mailchimp.com/3.0/lists');
		if ($retval == false && (isset($appParams['addToListonActive']) || isset($appParams['addToListonHold'])
								|| isset($appParams['addToListonExpire'])))
		{		
				$appParams['addToListonActive'] ='';
				$appParams['addToListonHold'] ='';
				$appParams['addToListonExpire'] ='';
				$data['app_params'] = $appParams;
		}
		return parent::collectAppParams($data);
	}

	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}

		$newStatus  = $new->getStatus();
		$emailComfirmation=$this->getAppParam('published');
		
		$active = $this->getAppParam('addToListonActive');
		$active	= (is_array($active)) ? $active : array($active);
		
		$hold 	= $this->getAppParam('addToListonHold');
		$hold	= (is_array($hold)) ? $hold : array($hold);
		
		$expire = $this->getAppParam('addToListonExpire');
		$expire	= (is_array($expire)) ? $expire : array($expire);

		$apiKey = $this->getAppParam('mailchimpApiKey');
		$subid  = $new->getId();
		$userid = $new->getBuyer();
		 
		// #1 : remove from groups assigned from other status (if applicable)
		// #2 : add to group and add resource for current status 
		
		// if subscription is active
		if($newStatus == PayplansStatus::SUBSCRIPTION_ACTIVE){
			$this->_removeFromList($userid, $hold, $subid,$apiKey);
			$this->_removeFromList($userid, $expire, $subid,$apiKey);
			$this->_addToList($userid, $active, $subid,$apiKey,$emailComfirmation);
			return true;
		}
		
		if($newStatus == PayplansStatus::SUBSCRIPTION_HOLD){
			$this->_removeFromList($userid, $active, $subid,$apiKey);
			$this->_removeFromList($userid, $expire, $subid,$apiKey);
			$this->_addToList($userid, $hold, $subid,$apiKey,$emailComfirmation);
			return true;
		}
		
		if($newStatus == PayplansStatus::SUBSCRIPTION_EXPIRED){
			$this->_removeFromList($userid, $active, $subid,$apiKey);
			$this->_removeFromList($userid, $hold, $subid,$apiKey);
			$this->_addToList($userid, $expire, $subid,$apiKey,$emailComfirmation);
			return true;
		}
	
		return true;

	}
	
	protected function _addToList( $userid, $listIds, $subId,$apiKey,$emailComfirmation)
	{
		$DataCenter = substr($apiKey, strrpos($apiKey, '-') + 1);
		$user = XiFactory::getUser($userid);
		if(!is_array($listIds)){
			return false;
		}
		
		foreach($listIds as $listId) {
			if (empty($listId)) {
				continue;
			}
			$merchantemail = $emailComfirmation=$this->getAppParam('published');
			$header = array(
				"Authorization: Basic " . base64_encode($merchantemail.':'. $apiKey),

			);
			$users = $this->curlGetrequest($header,'https://'.$DataCenter.'.api.mailchimp.com/3.0/lists/'.$listId.'/members/');

			$userEmails = array();
			foreach ($users['members'] as $data) {
				if($data->status == 'subscribed')
				$userEmails[] = $data->email_address;
			}
			if (in_array($user->email, $userEmails)) {
				$this->_addToResource($subId, $userid, $listId, $this->_resource);
				continue;
			}
			$payplansuser = PayplansUser::getInstance($userid);
			$subscriber_hash = $payplansuser->getParams()->get('subscriber_hash');

			if (isset($subscriber_hash)) {
				$response = $this->curlGetrequest($header,'https://'.$DataCenter.'.api.mailchimp.com/3.0/lists/' . $listId . '/members/' . $subscriber_hash);
				if ($response->status != 'subscribed') {
					$requestData['email_type'] = 'html';
					$requestData['status'] = 'subscribed';
					$requestData['merge_fields'] = array('FNAME' => $user->name);
					$requestData['email_address'] = $user->email;

					$response = $this->curlPutrequest($header, $requestData, 'https://'.$DataCenter.'.api.mailchimp.com/3.0/lists/' . $listId . '/members/' . $subscriber_hash);
					if ($response->id == $subscriber_hash) {
						$this->_addToResource($subId, $userid, $listId, $this->_resource);
						continue;
					} else {
						$payplansuser = PayplansUser::getInstance($userid);
						if (isset($response->id)) {
							$payplansuser->getParams()->set('subscriber_hash', $response->id);
							$user->save();
						}
					$this->_addToResource($subId, $userid, $listId, $this->_resource);
					}
				}

			} else {

				$requestData['email_type'] = 'html';
				$requestData['status'] = 'subscribed';
				$requestData['merge_fields'] = array('FNAME' => $user->name);
				$requestData['email_address'] = $user->email;

				$ch = curl_init();
				curl_setopt($ch, CURLOPT_URL, 'https://'.$DataCenter.'.api.mailchimp.com/3.0/lists/' . $listId . '/members');
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
				curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
				curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($requestData));
				$httpResponse = curl_exec($ch);

				$response = json_decode($httpResponse);
			}


				if ($httpResponse != false) {
					$payplansuser = PayplansUser::getInstance($userid);
					if (isset($response->id)) {
						$payplansuser->setParam('subscriber_hash',$response->id);
						$payplansuser->save();
					}

					$this->_addToResource($subId, $userid, $listId, $this->_resource);
				} else {
					$message = sprintf(XiText::_('COM_PAYPLANS_LOGGER_MAILCHIMP_LOG_ERROR_OCCURED'), $userid, $listId);
					$content = (array)$httpResponse;
					PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $this, $content, 'PayplansAppMailchimpFormatter');
				}

			}

		
		return true;
	}
	
	protected function _removeFromList($userid, $listIds, $subId, $apiKey)
	{
		$DataCenter = substr($apiKey, strrpos($apiKey, '-') + 1);
		$user = XiFactory::getUser($userid);
		if(!is_array($listIds)){
			return false;
		}
		
		foreach($listIds as $listId)
		{
			if(empty($listId)){
				continue;
			}
			$merchantemail = $emailComfirmation=$this->getAppParam('published');
			$header = array(
				"Authorization: Basic " . base64_encode($merchantemail.':'. $apiKey),

			);

			$users = $this->curlGetrequest($header,'https://'.$DataCenter.'.api.mailchimp.com/3.0/lists/'.$listId.'/members/');
			$userEmails = array();
			foreach ($users['members'] as $data) {
				if($data->status == 'subscribed')
					$userEmails[] = $data->email_address;
			}
			if(!in_array($user->email,$userEmails)){
				continue;
			}
			$payplansuser    = PayplansUser::getInstance($userid);
			$subscriber_hash = $payplansuser->getParams()->get('subscriber_hash','');

			$httpresponse = $this->curlDeleterequest($header,'https://'.$DataCenter.'.api.mailchimp.com/3.0/lists/'.$listId.'/members/'.$subscriber_hash);

			if($this->_removeFromResource($subId, $userid, $listId, $this->_resource) 
					){
			}
		}
		
		return true;
	}
	
	public function renderWidgetHtml()
	{   
		//get user
		$user    = XiFactory::getUser();
        
		//get user's mailchimp lists
	    $mailchimp_lists = $this->_getUserMailchimpLists($user);
        if(empty($mailchimp_lists)){
        	return '';
        }
	    $this->assign('mailchimp_lists',$mailchimp_lists);
        $data = $this->_render('widgethtml');
        return $data;
	}	
   
	protected function _getUserMailchimpLists($user)
	{   $payplansUser = PayplansUser::getInstance($user->id);
		$mailchimpApps = PayplansHelperApp::getAvailableApps('mailchimp',$payplansUser);
		$list = array();
		foreach($mailchimpApps as $app) {
			$apiKey = $app->getAppParam('mailchimpApiKey');
			$DataCenter = substr($apiKey, strrpos($apiKey, '-') + 1);
			if (empty($apiKey)) {
				continue;
			}
			$merchantemail =$this->getAppParam('mailchimpMerchantEmail');
			$header = array(
				"Authorization: Basic " . base64_encode($merchantemail.':'. $apiKey),

			);
			$retval = $this->curlGetrequest($header, 'https://'.$DataCenter.'.api.mailchimp.com/3.0/lists');
			if ($retval == false) {
				continue;
			}
			foreach ($retval['lists'] as $data) {
				$users = $this->curlGetrequest($header, 'https://'.$DataCenter.'.api.mailchimp.com/3.0/lists/' . $data->id . '/members/');
				$userEmails = array();
				foreach ($users['members'] as $member) {
					if ($member->status == 'subscribed')
						$userEmails[] = $member->email_address;
					if(in_array( $user->email, $userEmails) ){
						if(!in_array($data->name, $list)){
							$list[] = $data->name;
						}
					}
				}
			}
		}
		
		return $list;
	}

	protected function curlGetrequest($header, $requestedURL){
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $requestedURL);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		$output = curl_exec($ch);

		return (array)json_decode($output);
	}

	protected function curlDeleterequest($header,$requestedURL){
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $requestedURL);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "DELETE");
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);

		 return curl_exec($ch);
	}
	protected function curlPutrequest($header, $requestData, $requestedURL){
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $requestedURL);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($requestData));
		$httpResponse = curl_exec($ch);

		return json_decode($httpResponse);
	}
}

class PayplansAppMailchimpFormatter extends PayplansAppFormatter
{
	public $template	= 'view_log';
	
	// get Ignore data 
	function getIgnoredata()
	{
		$ignore = array('_trigger', '_tplVars', '_mailer', '_location', '_errors', '_component');
		return $ignore;
	}
	
	// get rules
	function getVarFormatter()
	{
		$rules = array('_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       'function' => 'getAppPlans'),
					   'app_params'      => array('formatter'=> 'PayplansAppMailchimpFormatter',
										       'function' => 'getFormattedContent'));
		return $rules;
	}

	// format email app content,status, expiration time 
	function getFormattedContent($key, $value, $data)
	{	
		$this->template = 'view';
	}
}
