<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppMoneybookers extends PayplansAppPayment
{	
	protected $_location	= __FILE__;
	private   $_supportedLanguages   = array('da','de','en','fo','fr','kl','it','no','nl','pl','ru','sv');
	
	private $txnStates = array('MONEYBOOKERS_TRANSACTION_STATE_PROCESSED'=>2, 
						      'MONEYBOOKERS_TRANSACTION_STATE_PENDING'=>0,
							  'MONEYBOOKERS_TRANSACTION_STATE_CANCELLED'=>-1,
							  'MONEYBOOKERS_TRANSACTION_STATE_FAILED'=>-2,
							  'MONEYBOOKERS_TRANSACTION_STATE_CHARGEBACK'=>-3);

	//moneybookers transaction state
	const MONEYBOOKERS_TRANSACTION_STATE_PROCESSED	 = 2;
	const MONEYBOOKERS_TRANSACTION_STATE_PENDING	 = 0;
	const MONEYBOOKERS_TRANSACTION_STATE_CANCELLED	 = -1;
	const MONEYBOOKERS_TRANSACTION_STATE_FAILED	 	 = -2;
	const MONEYBOOKERS_TRANSACTION_STATE_CHARGEBACK	 = -3;
	
	function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		if($view != 'payment' || ($task != 'notify') ){
			return true;
		}
		
		$paymentKey = JRequest::getVar('transaction_id', null);
		if(!empty($paymentKey)){
			JRequest::setVar('payment_key', $paymentKey, 'POST');
			return true;
		}
		
		return true;
	}
	
	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		if(is_object($data)){
			$data = (array)$data;
		}
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$lang    = XiHelperJoomla::getLanguageCode();
		$userid  = $invoice->getBuyer();
		$amount  = number_format($invoice->getTotal(),2) ;
		
		// basic setup
		$this->assign('post_url', "https://www.moneybookers.com/app/payment.pl");
		$this->assign('language', 'en');
		
		if(in_array($lang['language'], $this->_supportedLanguages)){
			$this->assign('language', $lang['language']);
		}
		
		// setup urls
		$root = JURI::root();
    	if(XiFactory::getConfig()->https == true){
    		$root = JString::str_ireplace("http:", "https:", $root);
    	}

		$this->assign('merchant', $this->getAppParam('merchantemail', ''));
		$this->assign('currency', $invoice->getCurrency('isocode'));
		$this->assign('invoicekey', $invoice->getKey());
		$this->assign('paymentkey', $payment->getKey());
		$this->assign('userid', $userid);
		$this->assign('title', $invoice->getTitle());

		$this->assign('return_url', 	$root.'index.php?option=com_payplans&gateway=moneybookers&view=payment&task=complete&action=success&payment_key='.$payment->getKey());
    	$this->assign('cancel_url', 	$root.'index.php?option=com_payplans&gateway=moneybookers&view=payment&task=complete&action=cancel&payment_key='.$payment->getKey());
    	$this->assign('notify_url', 	$root.'index.php?option=com_payplans&gateway=moneybookers&view=payment&task=notify');

		if($invoice->isRecurring()) {
			$expiration 	   = $invoice->getExpiration(PAYPLANS_RECURRING, true);
			$time 			   = $this->getRecurrenceTime($invoice->getExpiration());

			$endDate = $this->_calculateEndDate($invoice, $expiration);
			
			$this->assign('rec_end_date', $endDate);
			$this->assign('rec_amount', number_format($amount,2));
			$this->assign('rec_cycle', $time['unit']);
			$this->assign('rec_period', $time['period']);

			// recurring payment has been started
			return $this->_render('form_subscription');
		}

		$this->assign('amount', $amount);
		
        return $this->_render('form_standard');
	}

	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		$logs	= PayplansHelperLogger::getLog($payment, XiLogger::LEVEL_ERROR);
		$record = array_pop($logs);			
		if($record && !empty($record)){
			$action = 'error';
		}
	
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	
	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		// If same notification came more than one time
    	// Check if transaction already exist then do nothing and return
		$txn_id			= isset($data['mb_transaction_id']) ? $data['mb_transaction_id'] : 0;
		$transactions	= $this->_getExistingTransaction($invoice->getId(), $txn_id, 0, 0);
    	if($transactions !== false){
    		foreach($transactions as $transaction){
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			if($transaction->getParam('status','') == $data['status']){
    				return true;
    			}
    		}
    	}
		
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $payment->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('amount', 0)
					->set('gateway_txn_id', isset($data['mb_transaction_id']) ? $data['mb_transaction_id'] : 0)
					->set('params', PayplansHelperParam::arrayToIni($data));
		
		$secretWord = $this->getAppParam('merchantsecretword','');
		
		$md5Content = $data['merchant_id'].$data['transaction_id'].strtoupper(md5($secretWord)).$data['mb_amount'].$data['mb_currency'].$data['status'];
		$md5string	= JString::strtoupper( md5($md5Content) );
		
		$errors = false;
		if(in_array($data['status'], array(self::MONEYBOOKERS_TRANSACTION_STATE_PROCESSED, self::MONEYBOOKERS_TRANSACTION_STATE_CHARGEBACK))){
			$errors = $this->_processNotification($transaction, $data, $md5string, $payment);
		}
		
		if(is_array($errors) && !empty($errors) ){
			$transaction->set('message', array_shift($errors));
		}
		//set the transaction message as per status
		else {
			if($data['status']){
				$transaction->set('message', XiText::_("COM_PAYPLANS_APP_MONEYBOOKERS_TRANSACTION_".array_search($data['status'], $this->txnStates)));
			}
			
			else {
				$transaction->set('message', XiText::_('COM_PAYPLANS_APP_MONEYBOOKERS_TRANSACTION_NO_STATUS'));
			}
		}
		
		return $transaction->save();

	}
	
	protected function _processNotification(&$transaction, $data, $md5string, $payment)
	{
		$errors = array();
		if($this->getAppParam('merchantemail') !== JString::strtolower($data['pay_to_email'])){
				$errors[] = XiText::_($data['merchantemail']);
		}
		
		$transaction->set('amount', $data['amount']);
        
		if( (!in_array($data['status'], array(self::MONEYBOOKERS_TRANSACTION_STATE_PROCESSED, self::MONEYBOOKERS_TRANSACTION_STATE_CHARGEBACK)) ) || $data['md5sig'] != $md5string){
			$error = XiText::_('COM_PAYPLANS_APP_MONEYBOOKERS_PAYMENT_FAIL');
		}
		
		if( self::MONEYBOOKERS_TRANSACTION_STATE_FAILED == $data['status']){
			if(isset($data['failed_reason_code'])){
				$error = XiText::_("COM_PAYPLANS_APP_MONEYBOOKERS_PAYMENT_FAILED_REASON_CODE_".$data[failed_reason_code]);
			}
		}
	
		if($error){
				$transaction->set('amount', 0);
				$message 	      			= XiText::_('COM_PAYPLANS_APP_MONEYBOOKERS_LOGGER_ERROR_IN_PAYMENT');
				$response	                = array();
				$response['error_message']  = $error;
				$response['data']			= $data;
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
				return $errors;
		}
		return true;
	}

	public function getRecurrenceTime($expTime)
	{
		$expTime['year'] = isset($expTime['year']) ? intval($expTime['year']) : 0;
		$expTime['month'] = isset($expTime['month']) ? intval($expTime['month']) : 0;
		$expTime['day'] = isset($expTime['day']) ? intval($expTime['day']) : 0;;
		
		// if only days are set then return days as it is
		if(!empty($expTime['day'])){
			$days = $expTime['day'];
			
			if(!empty($expTime['month'])){
				$days += $expTime['month'] * 30;

				if(!empty($expTime['year'])){
					$days += $expTime['year'] * 365;
				}
			}
			return array('period' => $days, 'unit' => 'day', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_MONEYBOOKERS_RECURRING_MESSAGE'));
		}
		
		// if months are set
		if(!empty($expTime['month'])){
			$month = $expTime['month'];
			
			if(!empty($expTime['year'])){
				$month += $expTime['year'] * 12 ;
			}
			
			return array('period' => $month, 'unit' => 'month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_MONEYBOOKERS_RECURRING_MESSAGE'));
		}
		
		// years
		if(!empty($expTime['year'])){		
			return array('period' => $expTime['year'], 'unit' => 'year', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_MONEYBOOKERS_RECURRING_MESSAGE'));
		}
		
		
		// XITODO : what to do if not able to convert it
		return false;
	}
	
	protected function _calculateEndDate($invoice, $expiration)
	{
		$startDate = new XiDate('now');
		$endDate   = $startDate;
		
		$recurrence_count  = $invoice->getRecurrenceCount();
		// IMP : add 10 years when recurrence count is 0
		if(intval($recurrence_count) ===0){
			$endDate = $endDate->addExpiration('100000000000');
			return $endDate->toFormat('%d/%m/%Y');
		}
		
		
		for($i=0; $i< $recurrence_count; $i++){
			$endDate   = $endDate->addExpiration($expiration);
		}
		
		return $endDate->toFormat('%d/%m/%Y');
	}
}

