<?php
/**
* @copyright	Copyright (C) 2009 - 2012 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppOgone extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
	function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		if($view != 'payment' || ($task != 'notify') ){
			return true;
		}
		
		$paymentKey = JRequest::getVar('orderID', null);
		if(!empty($paymentKey)){
			JRequest::setVar('payment_key', $paymentKey, 'POST');
			return true;
		}
		
		return true;
	}
	
	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		$root       = JURI::root();
    	if(XiFactory::getConfig()->https == true){
    		$root = JString::str_ireplace("http:", "https:", $root);
    	}
    	
    	$invoice 	  = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
    	$user		  = $invoice->getBuyer(PAYPLANS_INSTANCE_REQUIRE);	
		$paymentKey   = $payment->getKey();
		$email		  = $user->getEmail();
		$amount  	  = round($invoice->getTotal(),2)*100;
    	$pspId 		  = $this->getAppParam('psp_id');
    	$currency     = $invoice->getCurrency('isocode');
    	$acceptUrl    = $root.'index.php?option=com_payplans&view=payment&task=notify&gateway=ogone';
    	$declineUrl   = $root.'index.php?option=com_payplans&view=payment&task=notify&gateway=ogone';
    	
    	$sha_params   = array('ACCEPTURL' 		  => $acceptUrl,
    	                      'AMOUNT'    		  => $amount,
    	                      'CURRENCY'  		  => $currency,
    	                      'DECLINEURL'		  => $declineUrl,
    						  'EMAIL'			  => $email,
    	                      'ORDERID'   		  => $paymentKey,
    	                      'PSPID'     		  => $pspId);
    	
    	if($invoice->isRecurring()){
    		$sha_params = $this->setRecurringParams($sha_params, $invoice,$payment, $paymentKey, $currency);
    		//if plan not supported then redirect to the same page
    		if(!$sha_params){
    			XiFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=invoice&task=confirm&invoice_key='.$invoice->getKey()),XiText::_('COM_PAYPLANS_PAYMENT_APP_OGONE_DOES_NOT_SUPPORT_THE_PLAN'));
    		}
    	}
    	
    	$shaString    = strtoupper($this->_calculateSHA($sha_params)); 
    	
		$this->assign('shaString',$shaString);
		$this->assign('postUrl', $this->_getOgoneUrl());
		$this->assign('invoice', $invoice);
	  	$this->assign('paymentKey', $paymentKey);
	  	$this->assign('acceptUrl', $acceptUrl);
	  	$this->assign('declineUrl', $declineUrl);
	  	
	  	foreach ($sha_params as $key=>$value){
	  		$this->assign(strtolower($key),$value);
	  	}
	  	
	  	return $this->_render('form');
	}
	
	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{		
		$txn_id 	  = isset($data['PAYID'])?$data['PAYID']:0;
		$subscr_id    =  0;
    	$parent_txn   =  0;
    	$invoice      = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
    	$transactions = $invoice->getTransactions();
    	
    	//if already have transaction for same PAYID then don't create another transaction 
    	if(!empty($transactions)){
	    	foreach ($transactions as $transaction){
	    		if( $transaction->getAmount() != 0 && $transaction->getParam('PAYID') == $txn_id){
	    			return true;
	    		}
	    	}
    	}
    	
    	// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $txn_id)
					->set('gateway_parent_txn', $parent_txn)
					->set('gateway_subscr_id',$subscr_id)
					->set('params', PayplansHelperParam::arrayToIni($data));
		
		//filter sha-out parameters
		$source = self::_filterShaOutParameters($data);
		ksort($source);
		$newSHA = strtoupper($this->_calculateSHA($source));
		
		//create log if SHA doesn't match
		if($newSHA != $data['SHASIGN']){
			$message                  	= XiText::_('COM_PAYPLANS_PAYMENT_APP_OGONE_SHA_DOES_NOT_MATCH');
			$error['Expected']	  	  	= $data['SHASIGN'];
			$error['Actual']      	  	= $newSHA;
			$error['response'] 	  		= $data;
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $error,'PayplansPaymentFormatter', '', true);
			
			//save transaction to dump records
			$transaction->save();
			
			XiFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=dashboard'),XiText::_('COM_PAYPLANS_PAYMENT_APP_OGONE_SHA_DOES_NOT_MATCH'));
			return true;
		}
		
		$func_name = isset($data['STATUS']) ? '_process_ogone_status_'.JString::strtoupper($data['STATUS']) : 'EMPTY';
		
		if(method_exists($this, $func_name)){
			$result = $this->$func_name($data, $payment, $transaction, $invoice );
		}
		
		//save transaction everytime
		if(!$transaction->save()){
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
		}
		
		//if any issue occurs in recurring then create error log
		if(!isset($result)){
			$message              = XiText::sprintf('COM_PAYPLANS_PAYMENT_APP_OGONE_OTHER_STATUS',$data['STATUS']);
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
			return true;
		}
		
		//if recurring then set a parameter 
		$recurring 			 = $invoice->isRecurring();
		$pending_recur_count = $payment->getParam('pending_recur_count');
		if($recurring){
			if(!isset($pending_recur_count)){
				$recurrence_count = $this->_getRecurrenceCount($invoice);
				$payment->setParam('pending_recur_count', $recurrence_count);
			}
			else{
				$payment->setParam('pending_recur_count',$pending_recur_count-1);
			}	
		}
		
		$payment->save();
		
		//in case of refund, don't redirect
		if($data['STATUS'] == '7'){
			return true;
		}
		
		//redirect to invoice-thank you page after first payment request
		//XiTODO : improve it
		if(!isset($pending_recur_count)){
			XiFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=invoice&task=thanks&invoice_key='.$invoice->getKey()));
			return false;
		}
	}
	
	public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{
		$userId			= $payment->getBuyer();
		$name 			= PayplansUser::getInstance($userId)->getRealname();
		$sub			= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE)->getReferenceObject(PAYPLANS_INSTANCE_REQUIRE)->getSubscription(PAYPLANS_INSTANCE_REQUIRE);
		$subKey     	= $sub->getKey();
		$mainframe  	= XiFactory::getApplication();
		
		$mailfrom 		= $mainframe->getCfg( 'mailfrom' );
		$fromname 		= $mainframe->getCfg( 'fromname' );
		$subject        = XiText::sprintf("COM_PAYPLANS_PAYMENT_APP_OGONE_RECURRING_CANCEL_REQUEST_FOR_SUBSCRIPTION_ID",$subKey);
		$message		= XiText::sprintf("COM_PAYPLANS_PAYMENT_APP_OGONE_RECURRING_CANCEL_REQUEST_BODY",$name,$userId,$subKey);
		$mail 			= JFactory::getMailer();
		
		//send an email to super-administrator for cancellation of recurring payment
		$rows = XiHelperJoomla::getUsersToSendSystemEmail();
		foreach ( $rows as $row ){
			$mail->sendMail($mailfrom, $fromname, $row->email, $subject, $message);
		}
		
		return $this->_render('cancel_request');
	}
	
	/**
	 * Set other params needed to setup recurring at ogone end
	 */
	function setRecurringParams($sha_params, $invoice, $payment, $paymentKey, $currency)
	{   
		$type 		   = $invoice->isRecurring();
		$recurCount    = $invoice->getRecurrenceCount();
		$rawTime 	   = 'expiration';
		$amount	       = $invoice->getTotal();
		
		if($type == PAYPLANS_RECURRING_TRIAL_1){
			$amount			  = $invoice->getTotal($invoice->getCounter()+1);	
			$expirationRaw	  = $invoice->getExpiration(PAYPLANS_RECURRING_TRIAL_1, true);
			$rawTime 		  = 'trial_time_1';
		}
		elseif($type == PAYPLANS_RECURRING && intval($recurCount) != intval(0)){
			//first payment is direct payment at ogone end and create subscription at ogone end for next payments only
			$recurCount--;
			$expirationRaw	  = $invoice->getExpiration(PAYPLANS_RECURRING, true);
		}
		elseif($type == PAYPLANS_RECURRING_TRIAL_2){
			return false;
		}
		
		//calculate period unit and number 
		$expiration    = PayplansHelperPlan::convertIntoTimeArray($invoice->getParam($rawTime, '000000000000'));
		$period	       = $this->_createPeriodUnitAndNumber($expiration, $payment);
		
		//calculate start date
		$now		   = new Xidate();
		$moment        = $now->toFormat('%d');
    	$now	   	   = $now->addExpiration($invoice->getCurrentExpiration(true));
    	$startDate	   = $now->toFormat('%Y-%m-%d');
    		
    	//calculate end date
    	$endDate	   = new XiDate();
    	if($recurCount != 0){
	    	for($recurCount; $recurCount > 0; $recurCount--){
	    			$endDate = $endDate->addExpiration($expirationRaw);				
	    	}
			//end date must be set one day after the actual enddate, otherwise ogone 
			//doesn't consider last date
	    	$endDate = $endDate->addExpiration('000001000000');
    	}
    	else{
    		//set date of after 25 years for lifetime recurring
    		$endDate = $endDate->addExpiration('250000000000');
    	}
    	
    	//if expiration time not supported then don't set params and return 
    	if(!isset($period['subPeriodNumber']) || !isset($period['subPeriodUnit'])){
    		return false;
    	}
  	
		return array_merge($sha_params, array( 'SUBSCRIPTION_ID'   => $invoice->getId(),
    						  				   'SUB_AMOUNT' 	   => round($amount,2)*100,
    						 	 		       'SUB_CUR'		   => $currency,
    						  			       'SUB_ENDDATE'	   => $endDate->toFormat('%Y-%m-%d'),
    						  			       'SUB_ORDERID' 	   => $paymentKey,
    						  			       'SUB_PERIOD_MOMENT' => $moment,
    						  			       'SUB_PERIOD_NUMBER' => $period['subPeriodNumber'],
    						  			       'SUB_PERIOD_UNIT'   => $period['subPeriodUnit'],
    						  			       'SUB_STARTDATE'     => $startDate,  
    						  			       'SUB_STATUS'		   => '1') );
		
	}
	
	/**
	 * create period unit and period number parmas required for recurring setup 
	 */
	private function _createPeriodUnitAndNumber($expiration, $payment)
	{
		//create period unit and number as supported by ogone
		$count   = 0;
		$period  = array();
		
		foreach ($expiration as $key=>$value){
			if($value == '00'){
				$count++;
				continue;
			}
			//if year is set then do nothing as ogone doesn't support this type of recurring
			if($key == 'year'){
				$support = false;
				break;
			}
			
			if($key == 'month'){
				$period['subPeriodUnit']   = 'm';
				$period['subPeriodNumber'] = (int)$value;
				continue;
			}
			
			if($key == 'day'){
				$period['subPeriodUnit']   = 'd';
				$period['subPeriodNumber'] = (int)$value;
				continue;
			}
		}
		
		//if not a single data in expiration date then generate error log 
		if($count != 5 || (isset($support) &&  $support == false)){
			$message                  = XiText::_('COM_PAYPLANS_PAYMENT_APP_OGONE_EXPIRATION_NOT_SUPPORTED');
			$error['Payment ID'] 	  = $payment->getId();
			$error['Expiration']	  = $expiration;
			$plans					  = $payment->getPlans();
			$planid					  = array_pop($plans);
			$error['Plan Name']		  = PayplansPlan::getInstance($planid)->getTitle();
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $error, 'PayplansPaymentFormatter', '', true);
			return false;
			
		}
		return $period;
	}
	
	/**
	 * get url on which to redirect on ogone end either test or production account link
	 */
 	private function _getOgoneUrl()
    {
       	return $this->getAppParam('sandbox') ? 'https://secure.ogone.com/ncol/test/orderstandard.asp' : 'https://secure.ogone.com/ncol/prod/orderstandard.asp';
    }
    
    /**
     * Calculate SHA-1 for given parameters
     */
    private function _calculateSHA($parameters)
    {
    	$additional  = $this->getAppParam('sha_pass_phrase');
    	$finalString = ''; 
    	foreach ($parameters as $key=>$parameter){
    		$finalString .= $key.'='.$parameter.$additional;
    	}
    	return sha1($finalString);
    }
	
    /**
     * Get recurrence count for given invoice
     */
	private function _getRecurrenceCount(PayplansInvoice $invoice)
	{
		$count = $invoice->getRecurrenceCount();
		if(intval($count) === 0){
			return 9999;
		}
		
		$recurring = $invoice->isRecurring();
		if($recurring){
			
			// Recurrence Count For Regular Recurring Plan
       		if($recurring == PAYPLANS_RECURRING){
	       		$recurrence_count = $invoice->getRecurrenceCount();
       		}
       		// Recurrence Count For Recurring + Trial 1 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_1){
				$recurrence_count = $invoice->getRecurrenceCount() + 1;
       		}
       		// Recurrence Count For Recurring + Trial 2 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_2){
	       		$recurrence_count = $invoice->getRecurrenceCount() + 2;
       		}
		}
		return $recurrence_count;
	}
	
	/**
	 * set amount when refund notification from ogone 
	 */
	private function _process_ogone_status_7($data, $payment, $transaction, $invoice)
	{
		$transaction->set('amount', -$data['amount'])
					->set('message', 'COM_PAYPLANS_PAYMENT_APP_OGONE_TRANSACTION_REFUNDED');
		return true;
	}
	
	/**
	 * Decide when payment processing status is SUCCESS
	 */
	private function _process_ogone_status_9($data, $payment, $transaction, $invoice)
	{		
		if($invoice->isRecurring()){
			return $this->_processRecurringPayment($data, $payment, $transaction);
		}
		else{
			return  $this->_processPayment($transaction, $data);
		}
		
		return false;
	}
	
	/**
	 * process payment for fixed subscription
	 */
	private function _processPayment($transaction, $data)
	{
		$transaction->set('amount', $data['amount'])
					->set('message', 'COM_PAYPLANS_PAYMENT_APP_OGONE_TRANSACTION_COMPLETED');
		return true;
	}
	
	/**
	 * process payment for recurring subscription
	 */
	private function _processRecurringPayment($data, $payment, $transaction)
	{		
		if($this->_checkForRecurringParameters($data, $payment)){
			$transaction->set('amount', $data['amount'])
						->set('message', 'COM_PAYPLANS_PAYMENT_APP_OGONE_TRANSACTION_COMPLETED');
			return true;
		}
		return false;
	}
	
	/**
	 * Check whether its first recurring payment or not
	 */
	private function _checkForRecurringParameters($data, $payment)
	{
		if(isset($data['creation_status']) &&  isset($data['subscription_id']) ){
			if($data['creation_status'] == 'OK' ){
				return true;
			}
		} 
		elseif( $payment->getKey() == $data['orderID']){
			return true;
		}
		
		return false;
	}
	
	private function _filterShaOutParameters($data)
	{   
        //list of SHA-OUT calculation parameters
		$accept = array ('AAVADDRESS','AAVCHECK','AAVZIP','ACCEPTANCE','ALIAS','AMOUNT','BIN','BRAND','CARDNO','CCCTY',
						 'CN','COMPLUS','CREATION_STATUS','CURRENCY','CVC','CVCCHECK','DCC_COMMPERCENTAGE','DCC_CONVAMOUNT','DCC_CONVCCY',
						 'DCC_EXCHRATE','DCC_EXCHRATESOURCE','DCC_EXCHRATETS','DCC_INDICATOR','DCC_MARGINPERCENTAGE','DCC_VALIDHOURS','DIGESTCARDNO',
		                 'ECI','ED','ENCCARDNO','IP','IPCTY','NBREMAILUSAGE','NBRIPUSAGE','NBRIPUSAGE_ALLTX','NBRUSAGE','NCERROR','NCERRORCARDNO',
		                 'NCERRORCN','NCERRORCVC','NCERRORED','ORDERID','PAYID','PM','SCO_CATEGORY','SCORING','STATUS','SUBBRAND','SUBSCRIPTION_ID',
		                 'TRXDATE','VC');
		
		$source		  = array();
		
		foreach ($data as $key=>$value){
			$key = strtoupper($key);
			if(!in_array($key, $accept) || (!isset($value) || trim($value) == '') ){
				continue;
			}
			$source[$key] = $value;
		}
		
		return $source;
	}
}