<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license	GNU/GPL, see LICENSE.php
* @package	PayPlans
* @subpackage	Frontend
* @contact 	payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppPagseguro extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
	function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		$gateway = JRequest::getVar('gateway', '');
		if($view != 'payment' || $task != 'notify' || $gateway != $this->getType()){
			return true;
		}
		
		$code = JRequest::getVar('notificationCode', null);
    	$type = JRequest::getVar('notificationType', null);

    	if(!$code && !$type){
    		return true;
    	}
    	
    	// load the library file
		require_once dirname(__FILE__).DS.'libraries'.DS.'PagSeguroLibrary.php';
		
    	if ( $code && $type ) {
			
    		$notificationType = new NotificationType($type);
    		$strType = $notificationType->getTypeFromValue();
			
			switch($strType) {
				
				case 'TRANSACTION':
					/*
					* #### Crendencials ##### 
					* Substitute the parameters below with your credentials (e-mail and token)
					*/
			    	$credentials = new AccountCredentials($this->getappParam('merchant', ''), $this->getappParam('token', ''));
			    	try {
			    		$transaction = NotificationService::checkTransaction($credentials, $code);
			    	} catch (PagSeguroServiceException $e) {
			    		//the token is not valid
						PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $e->getMessage(), $this, array($e->getMessage()), '', '', true);
			    		return true;
			    	}
					break;
					
				default:
					$message = XiText::_("COM_PAYPLANS_APP_PAGSEGURO_UNKNOWN_NOTIFICATION_TYPE"). ' : '.$notificationType->getValue();
					PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message(), $this, array($message),'', '', true);
					return true;		
			}
			
		}
		
		$reference = explode('-',$transaction->getReference());
	
		// V V IMP : assign this so that it can be used later
		$this->_transactionXML = $transaction;
		
//		 file_put_contents(JPATH_SITE.DS.'tmp'.DS.time(), var_export($transaction,true), FILE_APPEND);
		$paymentKey = $reference['1'];
		
		if($paymentKey == false){
			return true;
		}
			
		JRequest::setVar('payment_key', $paymentKey, 'POST');
		
		return true;
	}
	
	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		// load the library file
		require_once dirname(__FILE__).DS.'libraries'.DS.'PagSeguroLibrary.php';
		
		// Instantiate a new payment request
		$paymentRequest = new PaymentRequest();
		
		// Sets the currency
		$paymentRequest->setCurrency($invoice->getcurrency('isocode'));
		
		// Add an item for this payment request
		$paymentRequest->addItem($invoice->getKey(), $invoice->getTitle(), 1, $invoice->getTotal());
		
		// Sets a reference code for this payment request, it is useful to identify this payment in future notifications.
		$paymentRequest->setReference($invoice->getKey().'-'.$payment->getKey());
		
		$root = JURI::root();
    	if(XiFactory::getConfig()->https == true){
    		$root = JString::str_ireplace("http:", "https:", $root);
    	}
    	
    	$url = $root.'index.php?option=com_payplans&view=payment&task=complete'
    			.'&action=success&payment_key='.$payment->getKey();
		
		$paymentRequest->setRedirectUrl($url);
		
		try {
			
			/*
			* #### Crendentials ##### 
			* Substitute the parameters below with your credentials (e-mail and token)
			* You can also get your credentails from a config file. See an example:
			* $credentials = PagSeguroConfig::getAccountCredentials();
			*/			
			$credentials = new AccountCredentials($this->getappParam('merchant', ''), $this->getappParam('token', ''));
			
			// Register this payment request in PagSeguro, to obtain the payment URL for redirect your customer.
			$url = $paymentRequest->register($credentials);
			
			$this->assign('post_url', $url);
			return $this->_render('form');
			
		} catch (PagSeguroServiceException $e) {
			$message = $e->getMessage();
			$this->assign('errors', array($message));
        	return $this->_render('post_error');
		}
	}

	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		$logs	= PayplansHelperLogger::getLog($payment, XiLogger::LEVEL_ERROR);
		$record = array_pop($logs);			
		if($record && !empty($record)){
			$action = 'error';
		}
		
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		// load the library file
		require_once dirname(__FILE__).DS.'libraries'.DS.'PagSeguroLibrary.php';
		
		$errors = array();
		
		$amount = $this->_transactionXML->getGrossAmount();
		
		switch(intval($this->_transactionXML->getStatus()->getValue())){
			// XITODO : use constant instead of number
			case 1:
				$message = 'COM_PAYPLANS_APP_PAGSEGURO_TRANSACTION_PAYMENT_INITIATED';
				$amount  = 0;
				break;
		
			case 2:
				$message = 'COM_PAYPLANS_APP_PAGSEGURO_TRANSACTION_PAYMENT_IN_ANALYSIS';
				$amount  = 0;
				break;
						
			case 5:
				$message = 'COM_PAYPLANS_APP_PAGSEGURO_TRANSACTION_PAYMENT_IN_DISPUTE';
				$amount  = 0;
				break;
				
			case 6:
				$message = 'COM_PAYPLANS_APP_PAGSEGURO_TRANSACTION_PAYMENT_RETURNED';
				$amount  = -$amount;
				break;
		
			case 7:
				$message = 'COM_PAYPLANS_APP_PAGSEGURO_TRANSACTION_PAYMENT_CANCELLED';
				$amount  = 0;
				break;
		
			case 3:
				$message = 'COM_PAYPLANS_APP_PAGSEGURO_TRANSACTION_PAYMENT_COMPLETE';
				break;
				
			default:
				$message = 'COM_PAYPLANS_APP_PAGSEGURO_TRANSACTION_PAYMENT_UNKNOWN_STATUS';
				$amount  = 0;
				break;
		}	
		
		// if same notification came more than one time
		// Check if transaction already exists,if yes then do nothing and return
		$txn_id 		= $this->_transactionXML->getCode()!=null ? $this->_transactionXML->getCode() : 0; 
		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, 0, 0);
		if (!empty($transactions)) {
			foreach ($transactions as $transaction) {
	    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
	    			if(strtolower($transaction->getParam('status','')) == strtolower($this->_transactionXML->getStatus()->getValue())){
	    					return true;
    				}
	    		}
		 }
		
		 // get the transaction instance of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $payment->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_subscr_id', 0)
					->set('gateway_parent_txn', 0)
					->set('gateway_txn_id', $this->_transactionXML->getCode())
					->set('amount', $amount)
					->set('message', $message)
					->set('params', PayplansHelperParam::arrayToIni($this->_getTransactionArray($this->_transactionXML)))
					->save();		
		return true;
	}
	
	protected function _getTransactionArray($transactionXML)
	{
		
		$transaction['code']  		= $transactionXML->getCode();
    	$transaction['email'] 		= $transactionXML->getSender() ? $transactionXML->getSender()->getEmail() : "null";
    	$transaction['date']  		= $transactionXML->getDate();
    	$transaction['reference']   = $transactionXML->getReference();
    	$transaction['status']  	= $transactionXML->getStatus() ? $transactionXML->getStatus()->getTypeFromValue().($transactionXML->getStatus()->getValue()) : "null";
    	$transaction['itemsCount']  = is_array($transactionXML->getItems()) ? count($transactionXML->getItems()) : "null";
    	
    	$transaction['xml_string']  = serialize($transactionXML);
    	return $transaction;
	}
}
