<?php
/**
* @copyright	Copyright (C) 2009 - 2013 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Post Affiliate Pro Tracking
* @contact 		payplans@readybytes.in
* @author		Puneet Singhal
*/
if(defined('_JEXEC')===false) die();

class PayplansAppPap extends PayplansApp
{
	protected $_location	= __FILE__;
	
	public function isApplicable($refObject = null, $eventName='')
	{
		// if not with reference to payment then return
		if($eventName === 'onPayplansInvoiceAfterSave' || $eventName === 'onPayplansInvoiceBeforeSave'){
			return true;
		}
		return parent::isApplicable($refObject, $eventName);
	}
	
	public function onPayplansInvoiceBeforeSave($prev, $new)
	{
		if($prev == null){
			$session = PayplansFactory::getSession();
			$affiliateId = $session->get('PAPAffiliateId');
			
			$new->setParam('PAPAffiliateId', $affiliateId);
			$new->setParam('PAPVisitorId', @$_COOKIE['PAPVisitorId']);
		}
		return true;
	}
	
	public function onPayplansInvoiceAfterSave($prev, $new) 
	{
		// Before save and after save, Invoice status should not be same. 
		if ( $prev != null && $new->getStatus() == $prev->getStatus()) {
			return true;
		}
		
		if($new->getStatus() != PayplansStatus::INVOICE_PAID){
			return true;
		}
		
		$user 		= $new->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		$orderKey	= PayplansHelperUtils::getKeyFromId($new->getReferenceObject());
		$invoiceKey = $new->getKey();
		
		$papUrl        = $this->getAppParam('papUrl', '');
		$accountId     = $this->getAppParam('accountId','default1');
		
		require_once('PapApi.class.php');
		
	    $visitorId = $new->getParam('PAPVisitorId', false);
	    $affiliateId = $new->getParam('PAPAffiliateId', $accountId);
	    
	    // Tracking enabled for recurring invoices
	    if($new->isRecurring() && ($new->getCounter() > 1) ){
	    	$object = $new->getReferenceObject(PAYPLANS_INSTANCE_REQUIRE);
	    	if($object instanceof PayplansOrder){
	    		$masterInvoice 	= $object->getLastMasterInvoice(PAYPLANS_INSTANCE_REQUIRE);
		    	$visitorId 		= $masterInvoice->getParam('PAPVisitorId', false);
		    	$affiliateId 	= $masterInvoice->getParam('PAPAffiliateId', false);
	    	}
	    }
		
		$saleTracker = new Pap_Api_SaleTracker($papUrl.'/scripts/sale.php',true);
	    $saleTracker->setAccountId($accountId);
	    $saleTracker->setVisitorId($visitorId);
	    
	    $sale1 = $saleTracker->createSale();
        $sale1->setTotalCost($new->getTotal());
		$sale1->setOrderID($orderKey);
		$sale1->setProductID($invoiceKey);
		$sale1->setAffiliateId($affiliateId);
		
		$sale1->setData1($user->getEmail());
		$sale1->setData2($user->getRealname());
		$saleTracker->register();
		
		return true;
	}
}
