<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Paxum
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppPaxum extends PayplansAppPayment
{
	
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		if($eventName == 'onPayplansControllerCreation' || $eventName == 'onPayplansPaymentTerminate'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
    function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		if($view != 'payment' || ($task != 'notify') ){
			return true;
		}
		
		$paymentKey = JRequest::getVar('item_id', null);
		if(!empty($paymentKey)){
			JRequest::setVar('payment_key', $paymentKey, 'POST');
			return true;
		}
		
		return true;
	}
	
 	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{	
	   if(is_object($data)){
			$data = (array)$data;
		}
		
		$invoice        = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount         = $invoice->getTotal();
		$button_type_id = 1; // In case of fixed payment
		$business_email = $this->getAppParam('business_email', false);
		$currency       = $invoice->getCurrency('isocode'); 
	    $post_url       = "https://paxum.com/payment/phrame.php?action=displayProcessPaymentLogin"; 
    	
	 	$recurring = 0;
	 	$this->assign('post_url', $post_url);
		$this->assign('payment', $payment);
		$this->assign('invoice', $invoice);
		$this->assign('amount', $amount);
		$this->assign('business_email', $business_email);
		$this->assign('item_name', $invoice->getTitle()); 
		$this->assign('item_id' , $payment->getKey()); 
		$this->assign('currency', $currency);
		$this->assign('finish_url', XiRoute::_('index.php?option=com_payplans&gateway=paxum&view=payment&task=complete&action=success&payment_key='.$payment->getKey()));
    	$this->assign('cancel_url', XiRoute::_('index.php?option=com_payplans&gateway=paxum&view=payment&task=complete&action=cancel&payment_key='.$payment->getKey()));
		$this->assign('variables', 'notify_url='.XiRoute::_('index.php?option=com_payplans&gateway=paxum&view=payment&task=notify&payment_key='.$payment->getKey()));
        
		if($invoice->isRecurring())
		{
			$expTime 			= $invoice->getExpiration();
			$expTime 			= $this->getRecurrenceTime($expTime);
			$formPeriod 		= empty($expTime['period'])? 0 : $expTime['period'];
			$button_type_id 	= 2; // In case of recurring payment
			$recurrenceCount 	= $invoice->getRecurrenceCount(); 
			
			if($recurrenceCount == 0){
               $this->assign('user_cancel', 1); //Set for life-time payment when recurrence count is set to zero   
			}
			
			$formRecurringPrice   = $amount;
       		$formRecurringPeriod  = $formPeriod;
			
			$this->assign('payment_interval', $formPeriod); //set recurrence time
			$this->assign('transfers', $recurrenceCount);  // set recurrence count
			$this->assign('recurring', 1);
		}
		
        $this->assign('button_type_id', $button_type_id);
	  	return $this->_render('form');
	}
	
 	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
    {    
        $txn_id        	= isset($data['transaction_id ']) ? $data['transaction_id '] : 0;
       	$subscr_id     	= isset($data['subscription_id']) ? $data['subscription_id'] : 0;
      	$parent_txn    	= 0;
        $invoice       	= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
        $invoiceid    	= $invoice->getId(); 
					
        // Return true when get valid notification from paxum
        $result = $this->validateIPN($data);
        
   		//create log if invalid notifications
		if($result == false){
			$message                  = XiText::_('COM_PAYPLANS_PAYMENT_APP_PAXUM_NOTIFICATION_NOT_VALID');
			$error['Expected']	  	  = $data['key'];
			$error['Actual']      	  = $result;
			$error['response'] 	      = $data;
			if(!$invoice->isRecurring()){
				$error['transaction_id']  = $data['transaction_id'];
			}else {
				$error['subscription_id']  = $data['subscription_id'];
			}
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $error, 'PayplansPaymentFormatter', '', true);
			return true;
		}

	//if already have transaction for same transaction_id then don't create another transaction 
      if($this->_getExistingTransaction($invoiceid, $txn_id, $subscr_id, $parent_txn)){
        return true;
      }

      // get the transaction instace of lib
    $transaction = PayplansTransaction::getInstance();
    $transaction->set('user_id', $invoice->getBuyer())
          ->set('invoice_id', $invoice->getId())
          ->set('payment_id', $payment->getId())
          ->set('gateway_txn_id', $txn_id)
          ->set('gateway_parent_txn', $parent_txn)
          ->set('gateway_subscr_id', $subscr_id)
          ->set('params', PayplansHelperParam::arrayToIni($data)); 
         
        if($invoice->isRecurring()){
			$this->_processRecurringRequest($transaction, $data);
		}
		else {
			$this->_processNonRecurringRequest($transaction, $data);
		}
		
		//store the response in the payment AND save the payment
		if(!$transaction->save()){
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
		}
    	
		$payment->save();
		return true;	
    }
	
	// get recurrence time
     public function getRecurrenceTime($expTime)
	 {
		$expTime['year']  = isset($expTime['year']) ? intval($expTime['year']) : 0;
		$expTime['month'] = isset($expTime['month']) ? intval($expTime['month']) : 0;
		$expTime['day']   = isset($expTime['day']) ? intval($expTime['day']) : 0;
		$time = 0;
        
		// Support 1 year
		if( $expTime['year']){
			$time = ($expTime['year']);
			if($time == 1){
				$time = 6;
			}
			return array('period' => $time, 'unit' => 'Year', 'frequency' => XiText::_('COM_PAYPLANS_APP_PAYMENT_PAXUM_EXPIRATION_TIME_IN_YEAR') );
		}
		
		//Support 1 ,3 and 6 months
		if($expTime['month']){
			$time = $time + ($expTime['month']);
			if($time == 1){
				$time = 3;
			}
			else if($time == 3){
				$time = 4;
			}
			else if ($time == 6){
				$time = 5;
			} 
			return array('period' => $time, 'unit' => 'month', 'frequency' => XiText::_('COM_PAYPLANS_APP_PAYMENT_PAXUM_EXPIRATION_TIME_IN_MONTH') );
		}

		//Support 1 and 2 week only
		if ($expTime['day']){
			$time = $time + ($expTime['day']);
			if($time == 7)
			{
			  $time = 1;
			}
			else if($time== 14){
				$time = 2;
			}
			return array('period' => $time, 'unit' => 'Week', 'frequency' => XiText::_('COM_PAYPLANS_APP_PAYMENT_PAXUM_EXPIRATION_TIME_IN_WEEK') );
		}
		
		return false;
	 }	
	
    
    public function isSupportPaymentCancellation($invoice)
	{	
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}	
    
	//Cancel Reccuring subscription
    public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{
	    require_once dirname(__FILE__).DS.'library'.DS.'PaymentApiClient.php';
	    
	    $transactions    = $payment->getTransactions();
	    $transaction     = array_pop($transactions);
	    $subscriptionId  = $transaction->get('gateway_subscr_id', 0);
	    $account_user   = $this->getAppParam('business_email'); 
	    
	    $subscriptions = new PaymentApiClient($account_user, $this->getAppParam('api_shared_key'));
	   	$xml           = $subscriptions->cancelSubscription($subscriptionId);
	  	$parseXml      = new SimpleXMLElement($xml);
       	$responseData  = array();
       	foreach($parseXml as $key=>$value){
               $responseData[$key] = (string) $value;
       	 } 
	 
	   	if ($responseData['ResponseCode'] != 00)
		{
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_PAXUM_LOGGER_ERROR_IN_RECURRING_RESPONSE');
			$log_id = PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $responseData, 'PayplansPaymentFormatter', '', '', true);
			$errors['ResponseDescription'] = $message;
			$this->assign('errors', $errors);
			return $this->_render('cancel_error');
		}
		
			
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$txn = PayplansTransaction::getInstance();
		$txn->set('user_id', $payment->getBuyer())
			->set('invoice_id', $invoice->getId())
			->set('payment_id', $payment->getId())
			->set('gateway_txn_id', 0)
			->set('gateway_subscr_id', $subscriptionId)
			->set('gateway_parent_txn', 0)
      			->set('message', 'COM_PAYPLANS_PAYMENT_APP_PAXUM_TRANSACTION_FOR_CANCEL_ORDER')
       			->save(); 
	
		parent::onPayplansPaymentTerminate($payment, $controller);
		return $this->_render('cancel_success');
	}
   	
	// Validate IPN notifications
	private function validateIPN($parameters)
    {	
    	$api_shared_secret = $this->getAppParam('api_shared_secret');
    	
    	$finalString = '';
	 	foreach ($parameters as $key=>$value)
	 	{
	 		if($key == "key"){
	 			continue; 
	 		}
	 		$finalString .= $value;
	 	}
	 	
	 	$calculatedKey = md5($finalString.$api_shared_secret);
	 	$isValid = $parameters['key'] == $calculatedKey ? true : false;
        return $isValid;
    } 
    
	protected function _processRecurringRequest($transaction, $data)
	{
		if(isset($data['subscription_id'])){
			$transaction->set('amount', $data['subscription_payment_amount'])
						->set('message', 'COM_PAYPLANS_PAYMENT_APP_PAXUM_TRANSACTION_COMPLETED');
			return true;
		}
		return false;
		
	}

	protected function _processNonRecurringRequest($transaction, $data)
	{
		if(isset($data['transaction_id'])){
	   		 $transaction->set('amount', $data['transaction_amount'])
						 ->set('message', 'COM_PAYPLANS_PAYMENT_APP_PAXUM_TRANSACTION_COMPLETED');
			 return true;
		}
		return false;
	}

	// Refund transactions
//	public function refundRequest($transaction, $refund_amount)
//	{
//		require_once dirname(__FILE__).DS.'library'.DS.'PaymentApiClient.php';
//		
//		$paymentKey 	= $transaction->getParam('TxnID', '');
//		$amount			= $refund_amount;
//      $account_user    = $this->getAppParam('business_email'); 
//		
//		$refund        = new PaymentApiClient($account_user, $this->getAppParam('api_shared_key');
//		$response      = $refund->refundTransaction($paymentKey);
//		$parseXml      = new SimpleXMLElement($response);
//        $responseData  = array();
//        foreach($parseXml as $key=>$value){
//               $responseData[$key] = (string) $value;
//        } 
//		
//		//refund response code is zero then terminate the order
//		if($responseData['ResponseCode'] == 0){
//						
//			$paymentId 	= PayplansHelperUtils::getIdFromKey($paymentKey);
//    		$payment 	= PayplansPayment::getInstance($paymentId);
//    		
//    		if($payment != false || !empty($payment)){
//    			 $newtransaction = PayplansTransaction::getInstance();
//				 $newtransaction->set('user_id', 			$transaction->getBuyer())
//								->set('invoice_id', 		$transaction->getInvoice())
//								->set('payment_id',		 	$transaction->getPayment(PAYPLANS_INSTANCE_REQUIRE))
//								->set('gateway_txn_id',	 	$transaction->getGatewayTxnId())
//								->set('gateway_subscr_id', 	0)
//								->set('gateway_parent_txn', 0)
//								->set('params', 			PayplansHelperParam::arrayToIni($dataToTxnParam));
//
//				$negativeAmt = -($amount);
//		    	$newtransaction->set('amount',$negativeAmt)
//		    				->set('message', 'COM_PAYPLANS_APP_PAXUM_TRANSACTION_REFUND_COMPLETED')
//		    				->save();
//    		}
//		    		
//    		return true; 
//		}
//		
//		return false;
//	}
	
//	public function supportForRefund()
//	{
//		return true;
//	}
}
	 	
