<?php
/**
* @copyright	Copyright (C) 2009 - 2014 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();



class PayplansAppPayex extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansSubscriptionBeforeSave
		if($eventName == 'onPayplansPayexValidateRequest' || $eventName == 'onPayplansPayexValidateRecurringRequest'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
		

    function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  = $invoice->getTotal();
		$paymentKey = $payment->getKey();
		$accountNumber = $this->getAppParam('account_number','');
		$encryptionKey = $this->getAppParam('encryption_key','');
		$price = $amount*100;
		$this->assign('price', $price );
		
		$this->assign('invoice', $invoice);
		
		$this->assign('invoice_id', $invoice->getKey());
		$root = JURI::root();
    	if(XiFactory::getConfig()->https == true){
    		$root = JString::str_ireplace("http:", "https:", $root);
    	}
		$returnUrl = $root."index.php?option=com_payplans&view=payment&task=complete&action=success&payment_key=".$paymentKey;
		
		$cancelUrl = $root."index.php?option=com_payplans&view=payment&task=complete&action=cancel&payment_key=".$paymentKey;
		$this->assign('payment_key', $paymentKey);
		$currency = $invoice->getCurrency('isocode');
		$cancelUrl = 	$root.'index.php?option=com_payplans&gateway=paypal&view=payment&task=complete&action=cancel&payment_key='.$paymentKey;
		$this->assign('cancelUrl', $cancelUrl);
		
		$params = array('accountNumber' => $accountNumber,
						'purchaseOperation' => 'SALE',
						'price' => $price,
						'priceArgList' => '',
						'currency' => $currency,
						'vat'	   => 0,
						'orderID' => $invoice->getKey(),
						'productNumber' => $paymentKey,
						'description'   => 'Plan description',
						'clientIPAddress' => $_SERVER['REMOTE_ADDR'],
						'clientIdentifier' => '',
						'additionalValues' =>'',
						'externalID' =>'',
						'returnUrl' => $returnUrl,
						'view' => "CREDITCARD",
						'agreementRef' => '',
						'cancelUrl' => $cancelUrl,
						'clientLanguage' => 'en-US'
		);
		
		$this->assign('params', $params);
		
		$recurring = $invoice->isRecurring();  
		if($recurring)
		{
			$counter = $invoice->getCounter();
			$expirationRaw	  = $invoice->getExpiration(PAYPLANS_RECURRING, true);
			$recurringAgreementParams = array('accountNumber' => $accountNumber,
											  'merchantRef'  => $invoice->getKey(),
											  'description'  => 'abcd',
											  'purchaseOperation' => 'SALE',
											  'maxAmount'		  => 1000000,
											  'notifyUrl'		  => '',
											  'startDate'		  => '',
											  'stopDate'		  => ''
			);
			
			if($recurring == PAYPLANS_RECURRING_TRIAL_1){
				$params['price'] = number_format($invoice->getTotal($counter + 1),2);
			}
			
			if($recurring == PAYPLANS_RECURRING_TRIAL_2){
				$params['price'] = number_format($invoice->getTotal($counter + 2),2);
			}
			$this->assign('params', $params);
			$this->assign('recurringAgreementParams', $recurringAgreementParams);
			return $this->_render('form_recur');
			
		}
		
	  	return $this->_render('form');
	}
	
	
	function calculateHash($params)
	{
		$encryptionKey = $this->getAppParam('encryption_key','');
		$params = $params.$encryptionKey;
		return md5($params);
	}
	
	
	public function onPayplansPayexValidateRequest()
	{
		
		$application = JFactory::getApplication();
		$params      = $application->input->get('params','','row');
		$xml = $this->process('pxorder','Initialize8',(unserialize($params)));
	
		if ( JDEBUG ) {		
			$message = XiText::_('COM_PAYPLANS_APP_PAYEX_INITIALIZE_XML');
			PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, '' , var_export($xml, true));
		}
		
		$status = $this->checkStatus($xml, 'initialize');
		
		if ($status['code'] == "OK" && $status['errorCode'] == "OK" && $status['description'] == "OK")
		{
			$application->redirect($status['redirectUrl']);

		}
		else 
		{
				$message 	= XiText::_('COM_PAYPLANS_LOGGER_PAYEX_ERROR');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message,'', $status, 'PayplansPaymentFormatter', '', true);
				$msg = sprintf(XiText::_("COM_PAYPLANS_PAYEX_PAYMENT_FAILED"), $status['description']);
				$root = JURI::root();
				$payment_key      = $application->input->get('payment_key','');
				$cancelUrl = 	$root.'index.php?option=com_payplans&gateway=paypal&view=payment&task=complete&action=cancel&payment_key='.$paymentKey;
				$application->redirect($cancelUrl,$msg);
		}
	}
	
	public function onPayplansPayexValidateRecurringRequest()
	{
		$application = JFactory::getApplication();
		$params      = $application->input->get('params','','row');
		$recurringAgreementParams = $application->input->get('recurringAgreementParams','','row');
		$agreementxml		= $this->process("PxAgreement", "CreateAgreement3", unserialize($recurringAgreementParams));
		
		if ( JDEBUG ) {		
				$message = XiText::_('COM_PAYPLANS_APP_PAYEX_AGREEMENT_XML');
				PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message,'', var_export($agreementxml, true));
		}
		
		$status = $this->checkStatus($agreementxml,'createAgreement');
		
		if ($status['code'] == "OK" && $status['errorCode'] == "OK" && $status['description'] == "OK")
		{			
				//$payment          = $payment::getInstance()
				$invoice_id      = $application->input->get('invoice_id','');
				
				$param = new XiParameter();
				
				$invoice = PayplansInvoice::getInstance($invoice_id);
				$recurrenceCount = $this->_getRecurrenceCount($invoice);
				$param->set('agreementRef',$status['agreementRef']);
				$param->set('pending_recur_count',$recurrenceCount);
				$payment->set('gateway_params', $param)
								->save();	
				
				
				$transaction = PayplansTransaction::getInstance();
				$transaction->set('user_id', $payment->getBuyer())
				->set('invoice_id', $invoice_id)
				->set('payment_id', $payment->getId())
				->set('gateway_txn_id', $status['agreementRef'])
				->set('message', 'COM_PAYPLANS_APP_PAYEX_AGREEMENT_CREATED_SUCCESSFULLY')
				->set('params', PayplansHelperParam::arrayToIni($status))
				->save();
				
				$params = unserialize($params);
				$params['agreementRef'] = $status['agreementRef'];
				$xml = $this->process('pxorder','Initialize8',($params));
				
				if ( JDEBUG ) {		
					$message = XiText::_('COM_PAYPLANS_APP_PAYEX_INITIALIZE_XML');
					PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, '' , var_export($xml, true));
				}
		
				
				$status = $this->checkStatus($xml, 'initialize');
			
				if ($status['code'] == "OK" && $status['errorCode'] == "OK" && $status['description'] == "OK")
				{
					$application->redirect($status['redirectUrl']);
		
				}
	
			}
	else 
		{
				$message 				= XiText::_('COM_PAYPLANS_LOGGER_PAYEX_ERROR');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $status, 'PayplansPaymentFormatter', '', true);
				$msg					= sprintf(XiText::_("COM_PAYPLANS_PAYEX_PAYMENT_FAILED"), $status['description']);
				$root					= JURI::root();
				$payment_key      		= $application->input->get('payment_key','');
				$cancelUrl 				= $root.'index.php?option=com_payplans&gateway=paypal&view=payment&task=complete&action=cancel&payment_key='.$paymentKey;
				$application->redirect($cancelUrl,$msg);
		}
	}	
			

		
		function process($class, $method, $args)
		{
			
			if($this->getAppParam('sandbox','1')){
				$url = "https://test-external.payex.com/".$class."/".$class.".asmx?wsdl";
			}
			else 
			{
				$url = "https://external.payex.com/".$class."/".$class.".asmx?wsdl";
	          
			}
		
			$PayEx = new SoapClient($url,array("trace" => 1, "exceptions" => 0));
	
			//create the hash
			$hash = $this->calculateHash(trim(implode("", $args)));
			//append the hash to the parameters
			$args['hash'] = $hash;
	
			try{
				//defining which initialize version to run, this one is 7.
				$respons = $PayEx->$method($args);
				/* NB: SHOULD BE EDITED TO NOT SHOW THE CUSTOMER THIS MESSAGE, BUT SHOW A GENERIC ERROR MESSAGE FOR THE USER, BUT YOU SHOULD BE INFORMED OF THE ERROR. "*/
			}catch (SoapFault $error){
				echo "Error: {$error->faultstring}";
			}
			return $respons->{$method.'Result'};
		}
		
		
		// function to convert response xml
		public function checkStatus($xml, $method = '')
		{
			 $returnXml = new SimpleXMLElement($xml);
			 
			 $status['code'] 		  = (string)($returnXml->status->code);
		 	 $status['errorCode']     = (string)($returnXml->status->errorCode);
		 	 $status['description']   = (string)($returnXml->status->description);
		 	 
			 if($method == 'initialize')
			 {
			 	$status['orderRef'] 	 = (string)($returnXml->orderRef);
			 	$status['redirectUrl']     = (string)$returnXml->redirectUrl;
			 	$status['sessionRef']    = (string)$returnUrl->sessionRef;
			 }
			 
			 if($method == 'complete')
			 {
			 	$status['transactionStatus'] 			= (string)$returnXml->transactionStatus;
			 	$status['orderStatus']       			= (string)$returnXml->orderStatus;
			 	$status['transactionRef']    			= (string)$returnXml->transactionRef;
			 	$status['transactionNumber'] 			= (string)$returnXml->transactionNumber;
			 	$status['productId']         			= (string)$returnXml->productId;
			 	$status['paymentMethod']     			= (string)$returnXml->paymentMethod;
			 	$status['amount']            			= (string)$returnXml->amount;
			 	$status['Csid']              			= (string)$returnXml->Csid;
			 	$status['orderId']           			= (string)$returnXml->orderId;
			 	$status['BankHash']          			= (string)$returnXml->BankHash;
			 	$status['AuthenticatedWith'] 			= (string)$returnXml->AuthenticatedWith;
			 	$status['maskedNumber'] 	 			= (string)$returnXml->maskedNumber;
			 	$status['transactionErrorCodeSimple'] 	= isset($returnXml->transactionErrorCodeSimple)?(string)$returnXml->transactionErrorCodeSimple:'';
			 	$status['transactionErrorDescription'] 	= isset($returnXml->transactionErrorDescription)?(string)$returnXml->transactionErrorDescription:'';
			 	$status['transactionThirdPartyError'] 	= isset($returnXml->transactionThirdPartyError)?(string)$returnXml->transactionThirdPartyError:'';
			 	
			 	
			 }
			 if ($method == 'createAgreement')
			 {
			 	$status['agreementRef'] = (string)$returnXml->agreementRef;
			 }
			 
			 if($method == 'autoPay')
			 {
			 	$status['transactionStatus'] = (string)$returnXml->transactionStatus;
			 	$status['transactionRef']    = (string)$returnXml->transactionRef;
			 	$status['transactionNumber'] = (string)$returnXml->transactionNumber;
			 	$status['paymentMethod'] 	 = (string)$returnXml->paymentMethod;
			 	
			 }
			
			 return $status;
		}
		
	
	function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{		
		if($action == 'cancel'){
			return true;
		}
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		
		$amount = number_format($invoice->getTotal(),2);
		
		if(!isset($data['orderRef']))
			{
				return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
			}
		
		$params = array(
			'accountNumber' =>  $this->getAppParam('account_number',''),
			'orderRef' 		=> $data['orderRef']
			);
			
		$completeResponse = $this->process("pxorder", "Complete", $params);
		if ( JDEBUG ) {		
			$message = XiText::_('COM_PAYPLANS_APP_PAYEX_COMPLETE_XML');
			PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment , $completeResponse, 'PayplansPaymentFormatter');
		}
		
		
		$status 	= $this->checkStatus($completeResponse, 'complete');
		$txn_id 	= isset($status['transactionNumber']) ? $status['transactionNumber'] : 0;  
		
    	$subscr_id  = isset($data['transactionRef']) ? $data['transactionRef'] : 0;
    	$parent_txn = isset($data['transactionRef']) ? $data['transactionRef'] : 0;
    	
    	// if same notification came more than one time
	    // Check if transaction already exists,if yes then do nothing and return
    	$transactions = $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_txn);
    	if($transactions !== false){
    		foreach($transactions as $transaction){
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			$par = $transaction->getParam('txn_type', '');
    			if($transaction->getParam('transactionStatus','') == $data['transactionStatus']){
    				return true;
    			}
    		}
    	}
		
		if ($status['transactionStatus'] == '0' || $status['transactionStatus'] == '6'){
			
				$transaction = PayplansTransaction::getInstance();
				$transaction->set('user_id', $payment->getBuyer())
							->set('invoice_id', $invoice->getId())
							->set('payment_id', $payment->getId())
							->set('amount', $status['amount']/100)
							->set('gateway_txn_id', $txn_id)
							->set('message', 'COM_PAYPLANS_APP_PAYEX_PAYMENT_COMPLETED_SUCCESSFULLY')
							->set('gateway_subscr_id', $status['transactionRef'])
							->set('gateway_parent_txn', $status['transactionRef'])
							->set('params', PayplansHelperParam::arrayToIni($status))
							->save();
		}
		else {
				$message 	= XiText::_('COM_PAYPLANS_LOGGER_PAYEX_ERROR');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $status, 'PayplansPaymentFormatter', '', true);
		}
		
		// save payment record
		$payment->save();			
				
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	

	private function _getRecurrenceCount(PayplansInvoice $invoice)
	{
		$count = $invoice->getRecurrenceCount();
		if(intval($count) === 0){
			return 9999;
		}
		
		$recurring = $invoice->isRecurring();
		if($recurring){
			
			// Recurrence Count For Regular Recurring Plan
       		if($recurring == PAYPLANS_RECURRING){
	       		$recurrence_count = $invoice->getRecurrenceCount();
       		}
       		// Recurrence Count For Recurring + Trial 1 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_1){
				$recurrence_count = $invoice->getRecurrenceCount() + 1;
       		}
       		// Recurrence Count For Recurring + Trial 2 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_2){
	       		$recurrence_count = $invoice->getRecurrenceCount() + 2;
       		}
		}
		return $recurrence_count;
	}
	
	
	public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$invoice_count = $invoiceCount + 1;
		
		if($invoice->isRecurring())
		{
			$recurrence_count 	= $payment->getGatewayParam('pending_recur_count');

			if($recurrence_count > 0 )
			{
			
				$agreementRef  = $payment->getGatewayParam('agreementRef');
				$accountNumber = $this->getAppParam('account_number','');
				$currency 	   = $invoice->getCurrency('isocode');
				$amount 	   = $invoice->getTotal($invoice_count);
				$params 		= array('accountNumber' => $accountNumber,
										'agreementRef'  => $agreementRef,
										'price'			=> ($amount*100),
										'productNumber' => $payment->getKey(),
										'description'	=> 'Plan Purchase',
										'orderId'		=> $invoice->getKey(),
										'purchaseOperation'=> 'SALE',
										'currency'		   => $currency
											
				);
				
				
				$xml    = $this->process("PxAgreement","AutoPay3", $params);
				if ( JDEBUG ) {		
					$message = XiText::_('COM_PAYPLANS_APP_PAYEX_AUTOPAY_XML');
					PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment , var_export($xml, true), 'PayplansPaymentFormatter');
				}
				
				
				$status = $this->checkStatus($xml, 'autoPay');
			
				if ($status['code'] == "OK" && $status['errorCode'] == "OK" && $status['description'] == "OK")
				{
					$this->_processRecurringPayment($payment, $invoice, $amount, $recurrence_count, $agreementRef, $status['transactionRef'], $status);
				}
				else{
					$application 	= JFactory::getApplication();
					$message 		= XiText::_('COM_PAYPLANS_LOGGER_PAYEX_ERROR');
					$log_id 		= PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $status, 'PayplansPaymentFormatter', '', true);	
					
				}

			}
		}
	}
	
	private function _processRecurringPayment(PayplansPayment $payment, PayplansInvoice $invoice, $amount, $recurrence_count, $agreementRef, $transactionRef, $record = array())
		{
			$recurrence_count 	= $recurrence_count - 1;
			$transaction = PayplansTransaction::getInstance();
			$transaction->set('user_id', $invoice->getBuyer())
						->set('invoice_id', $invoice->getId())
						->set('payment_id', $payment->getId())
						->set('amount', $amount)
						->set('gateway_txn_id', isset($record['transactionNumber']) ? $record['transactionNumber'] : 0)
						->set('gateway_subscr_id', $transactionRef)
						->set('gateway_parent_txn', $transactionRef)
						->set('message', 'COM_PAYPLANS_APP_PAYEX_PAYMENT_COMPLETED_SUCCESSFULLY')
						->set('params', PayplansHelperParam::arrayToIni($record))
						->save();
	
			// IMP: if payment arrives after certain failures then reset the failure counter
			$payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
			$payment->save();
		}
   
}


