<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppPaygate extends PayplansAppPayment
{	
	protected $_location	= __FILE__;

	//paygate transaction status
	const PAYGATE_TRANSACTION_STATUS_NOT_DONE				 = 0;
	const PAYGATE_TRANSACTION_STATUS_APPROVED				 = 1;
	const PAYGATE_TRANSACTION_STATUS_DECLINED		 		 = 2;
	const PAYGATE_TRANSACTION_STATUS_SUCCESSFULLY_RECEIVED	 = 5;
	const SUCCESSFUL_REASON_CODE 							 = 990017;
	const CANCEL_REASON_CODE 								 = 990028;
	const PAYGATE_VERSION									 = 21;
	const PAYGATE_RECURRING_FREQUENCY						 = 228;
	
	const PAYGATE_XML_VER = "4.0";
	const PAYGATE_XML_DECLARATION = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
	const PAYGATE_DTD_DEF = "<!DOCTYPE protocol SYSTEM \"https://www.paygate.co.za/payxml/payxml_v4.dtd\">";
	
	
	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		if(is_object($data)){
			$data = (array)$data;
		}
			
		$date   = new XiDate('now');

		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);

		// multiply by 100, because payment gateway does not support decimal		
		$amount = number_format($invoice->getTotal(),2) * 100 ;	
		
		// basic setup
		$this->assign('post_url', "https://www.paygate.co.za/paywebv2/process.trans");
		
		// setup urls
		$root = JURI::root();
    	if(XiFactory::getConfig()->https == true){
    		$root = JString::str_ireplace("http:", "https:", $root);
    	}

    	$this->assign('amount', 			 $amount);
		$this->assign('encryption_key',		 $this->getAppParam('encryption_key', ''));	
		$this->assign('transaction_date', 	 $date->toFormat(XiDate::YYYY_MM_DD_HH_MM));	
		$this->assign('paygateid', 			 $this->getAppParam('paygateid', ''));
		$this->assign('currency', 			 $invoice->getCurrency('isocode'));
		$this->assign('payment_key', 		 $payment->getKey());	
		$this->assign('return_url', 		 $root.'index.php?option=com_payplans&gateway=paygate&view=payment&task=complete&action=success&payment_key='.$payment->getKey());		
		
		if($invoice->isRecurring()){
			$frequency 		   = self::PAYGATE_RECURRING_FREQUENCY;
			$expiration 	   = $invoice->getExpiration(PAYPLANS_RECURRING, true);
			$time 			   = $this->getRecurrenceTime($invoice->getExpiration());
			$endDate 		   = $this->_calculateEndDate($invoice, $expiration);
			
			// for month as frequency
			if($time['unit'] == 'month'){
				$dateCount = $date->toFormat('%d');
				//dates greater than 28 are treated with 29 as the last day of the month
				if($dateCount > 28){
					$dateCount = 29;
				} 
				$frequency = ($time['period'] + 1 ).$dateCount;
			}
			
			// for recurring time set in weeks
			if($time['unit'] == 'week'){
				$aaaa = $date->toFormat('%w');
				$weekCount = $date->toFormat('%w') + 1 ;
				$frequency = "1".($time['period']).$weekCount;
				
			}

			$this->assign('post_url', 				 "https://www.paygate.co.za/PaySubs/process.trans");
			$this->assign('version', 				 self::PAYGATE_VERSION);
			$this->assign('subscription_start_date', $date->toFormat('%Y-%m-%d'));
			$this->assign('subscription_end_date', 	 $endDate);
			$this->assign('frequency', 				 $frequency);
			$this->assign('version', 				 21);
			
			
			return $this->_render('form_subscription');
		}
        return $this->_render('form_standard');
	}

	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{      
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		// if same notification came more than one time
		// Check if transaction already exists,if yes then do nothing and return
		$txn_id         = isset($data['TRANSACTION_ID']) ? $data['TRANSACTION_ID'] :0;
		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, 0, 0);
    	if (!empty($transactions)) {
    		foreach ($transactions as $transaction) {
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			if($transaction->getParam('TRANSACTION_STATUS','') == $data['TRANSACTION_STATUS'] &&
    				$transaction->getParam('RESULT_CODE','') == $data['RESULT_CODE'] && 
    					strtolower($transaction->getParam('RISK_INDICATOR','')) == strtolower($data['RISK_INDICATOR'])){
    					return true;
    			}
    		}
    	}
		
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $payment->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('amount', 0)
					->set('gateway_txn_id', $txn_id)
					->set('params', PayplansHelperParam::arrayToIni($data));
		
		
		$errors = array();

		$errors = $this->_processResponse($payment, $data, $transaction, $invoice);
		
		//set the transaction message as per status
		if($data['TRANSACTION_STATUS']){
			$transaction->set('message', XiText::_("COM_PAYPLANS_APP_PAYGATE_TRANSACTION_$data[TRANSACTION_STATUS]"));
		}
		
		else {
			$transaction->set('message', XiText::_('COM_PAYPLANS_APP_PAYGATE_TRANSACTION_NO_STATUS'));
		}
		$transaction->save();
		
        if(!empty($errors)){
				$action = "error";
		}

		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		
	}
	
	protected function _processResponse($payment, $data, &$transaction, $invoice)
	{
		$errors 	= array();
		$CHECKSUM	=	$data['CHECKSUM'];
		$recurring  = ($payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE)->isRecurring())? true : false ;
		
		$calculatedChecksum = $this->_calculateChecksum($recurring, $data);
		
		//if the calculated checksum does not match 
	    //the PayGate checksum in the response, then results should be rejected.
		if($calculatedChecksum != $CHECKSUM){
			$errors[]	=	XiText::_('COM_PAYPLANS_APP_PAYGATE_INVALID_HASH');
		}
		
		//check the merchant paygateid
		$merchantId = $this->getAppParam('paygateid', '');
        if($merchantId != $data['PAYGATE_ID']){
        	$errors[] = XiText::_('COM_PAYPLANS_APP_PAYGATE_INVALID_MERCHANT_PAYGATEID');
        }
		
		$amountReturned  = $data['AMOUNT'] / 100 ;
		$stored_amount 	 = number_format($transaction->getAmount(),2);

		if($data['TRANSACTION_STATUS'] == self::PAYGATE_TRANSACTION_STATUS_APPROVED){
			$transaction->set('amount', $amountReturned);
			
			if ($invoice->isRecurring()){
				$recurrence_count = $invoice->getRecurrenceCount();
				$recurrence_count--;
				$payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
				$payment->save();
			}
		}
	
		if( self::PAYGATE_TRANSACTION_STATUS_APPROVED != $data['TRANSACTION_STATUS']){
			$errors[] = XiText::_('COM_PAYPLANS_APP_PAYGATE_PAYMENT_FAIL');
		}

		if( $data['RESULT_CODE'] != self::SUCCESSFUL_REASON_CODE ){
			$errors[]= XiText::_("COM_PAYPLANS_APP_PAYGATE_PAYMENT_FAILED_REASON_CODE_$data[RESULT_CODE]");
		}
		
		if( $data['RISK_INDICATOR'] != 'AX' ){
			$errors[]= XiText::_("COM_PAYPLANS_APP_PAYGATE_PAYMENT_RISK_INDICATOR");
		}

		if(!empty($errors)){
				$action 					= "error";
				$message 					= XiText::_('COM_PAYPLANS_APP_PAYGATE_LOGGER_ERROR_IN_PAYMENT_PROCESS');
				$response					= array();				
				$response['error_message']	= $errors;
				$response['data'] 			= $data;
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		}
				
		return $errors;
	}
	
	protected function _calculateEndDate($invoice, $expiration)
	{
		$startDate = new XiDate('now');
		$endDate   = $startDate;
		
		$recurrence_count  = $invoice->getRecurrenceCount();
		// IMP : add 10 years when recurrence count is 0
		if(intval($recurrence_count) ===0){
			$endDate = $endDate->addExpiration('100000000000');
			return $endDate->toFormat('%Y-%m-%d');
		}
		
		
		for($i=0; $i< $recurrence_count; $i++){
			$endDate   = $endDate->addExpiration($expiration);
		}
		
		return $endDate->toFormat('%Y-%m-%d');
	}
	
	protected function _calculateChecksum($recurring = false, $data)
	{
		$encryption_key 	=   $this->getAppParam('encryption_key', '');
		
		$PAYGATE_ID			=	$data['PAYGATE_ID'];
	    $REFERENCE			=	$data['REFERENCE'];
	    $AMOUNT				=	$data['AMOUNT'];
	    $TRANSACTION_STATUS =	$data['TRANSACTION_STATUS'];
	    $RESULT_CODE		=	$data['RESULT_CODE'];
	    $RESULT_DESC		=	$data['RESULT_DESC'];
	    $AUTH_CODE			=	$data['AUTH_CODE'];
	    $TRANSACTION_ID		=	$data['TRANSACTION_ID'];
	    $RISK_INDICATOR		=	$data['RISK_INDICATOR'];
	    
	    $checksum_source    = $PAYGATE_ID."|".$REFERENCE."|".$TRANSACTION_STATUS."|"
	    				      .$RESULT_CODE."|".$AUTH_CODE."|".$AMOUNT."|".$RESULT_DESC."|"
	    				      .$TRANSACTION_ID."|";
	    				      
	    if($recurring){
	    	$SUBSCRIPTION_ID	=	$data['SUBSCRIPTION_ID'];
	    	$checksum_source   .=   $SUBSCRIPTION_ID."|";
	    }
   
		if ($RISK_INDICATOR) $checksum_source .= $RISK_INDICATOR."|";
	    $checksum_source .= $encryption_key;
	    
	    return md5($checksum_source);
	
	}

	public function getRecurrenceTime($expTime)
	{
		$expTime['year'] = isset($expTime['year']) ? intval($expTime['year']) : 0;
		$expTime['month'] = isset($expTime['month']) ? intval($expTime['month']) : 0;
		$expTime['day'] = isset($expTime['day']) ? intval($expTime['day']) : 0;;
		
		// if only days are set then return days as it is
		if(!empty($expTime['day'])){
			$days = $expTime['day'];
			
			if(!empty($expTime['month'])){
				$days += $expTime['month'] * 30;

				if(!empty($expTime['year'])){
					$days += $expTime['year'] * 365;
				}
			}
			$weeks = intval($days/7, 10);
			return array('period' => $weeks, 'unit' => 'week', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYGATE_RECURRING_MESSAGE'));
		}
		
		// if months are set
		if(!empty($expTime['month'])){
			$month = $expTime['month'];
			
			if(!empty($expTime['year'])){
				$month += $expTime['year'] * 12 ;
			}
			
			return array('period' => $month, 'unit' => 'month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYGATE_RECURRING_MESSAGE'));
		}
		
		// years
		if(!empty($expTime['year'])){		
			return array('period' => $expTime['year'], 'unit' => 'year', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_PAYGATE_YEAR_NOT_SUPPORTED'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYGATE_RECURRING_MESSAGE'));
		}
		
		
		// XITODO : what to do if not able to convert it
		return false;
	}
	
	//IMP : Paygate does not send notification for recurring payments
	// so to find out whether any payment is done or not an api call is done to fetch the payment
	// with the current payment reference 
	public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$invoice 	= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		$lifetime 		= false;
		$lifetime 		= ($invoice->getRecurrenceCount() == 0)? true : false;
		$invoice_count 	= $invoiceCount +1;
			
		if($invoice->isRecurring()){			
			$recurrence_count 	= $payment->getGatewayParam('pending_recur_count');
			if($recurrence_count > 0 || $lifetime){
				$this->_fetchRecurringTransaction($payment, $invoice, $invoice_count);	
			}
		}
		
		return true;
	}
	
	protected function _fetchRecurringTransaction($payment, $invoice, $invoice_count)
	{
		//Construct the XML document header
		$XMLHeader = self::PAYGATE_XML_DECLARATION.self::PAYGATE_DTD_DEF;
		
		//build the transaction header
		$TransHeader = "<protocol ver=\"".self::PAYGATE_XML_VER."\" pgid=\"".$this->getAppParam('paygateid', '')."\" pwd=\"".$this->getAppParam('password', '')."\">";
		$queryDetails = "<querytx cref=\"".$payment->getKey()."\" />";
		$TransTrailer = "</protocol>";
		$XMLTrans = $TransHeader.$queryDetails.$TransTrailer;
		$Request = $XMLHeader.$XMLTrans;
		
		// Create the POST data header containing the transaction
		$header[] = "Content-type: text/xml";
		$header[] = "Content-length: ".strlen($Request)."\r\n";
		$header[] = $Request;
		
		$ch = curl_init();
		if (!$ch){ 
			$message 		= XiText::_('COM_PAYPLANS_APP_PAYGATE_LOGGER_ERROR_CURL_INSTANTIATION_FAILED');
			$response		= array();
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		}
		
		curl_setopt ($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt ($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt ($ch, CURLOPT_TIMEOUT, 60);
		curl_setopt ($ch, CURLOPT_USERAGENT, "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0)");
		curl_setopt ($ch, CURLOPT_CUSTOMREQUEST, "POST");             
		
		// - then set the PayXML URL and the transaction data
		curl_setopt ($ch, CURLOPT_URL, "https://www.paygate.co.za/payxml/process.trans");
		curl_setopt ($ch, CURLOPT_HTTPHEADER, $header);             
		
		// Connect to PayGate PayXML and send data
		$Response = curl_exec ($ch);
		
		// Checl for any connection errors and then close the connection.
		$curlError = curl_errno($ch);
		curl_close($ch);
		
		$amount 	= $invoice->getTotal() * 100;
		$order  	= $invoice->getReferenceObject(PAYPLANS_INSTANCE_REQUIRE);
		$expDate 	= $order->getSubscription()->getExpirationDate();
		$endRange 	= $expDate->addExpiration('000001000000')->toFormat();
		$startRange = $expDate->subtractExpiration('000002000000')->toFormat();
		
		
		$xml    = simplexml_load_string($Response, 'SimpleXMLElement', LIBXML_NOCDATA);
		$xmlRes = json_decode(json_encode((array)$xml), TRUE);
		
		if (isset($xmlRes['errorrx'])){
			foreach ($xmlRes['errorrx'] as $res){
				$res = array_shift($res);
				$message 					= XiText::_('COM_PAYPLANS_APP_PAYGATE_LOGGER_ERROR_IN_PAYMENT_PROCESS');
				$response					= array();				
				$response['error_message']	= $res['edesc'];
				$response['data'] 			= $res['ecode'];
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
			}
		}
		
		elseif (isset($xmlRes['queryrx'])){
			foreach ($xmlRes['queryrx'] as $res){
				$res = array_shift($res);
				
				//check whether this transaction is already processed
				$transactions = $this->_getExistingTransaction($invoice->getId(), $res['tid'], 0, 0);
				if(!empty($transactions)){
		    		continue;
		    	}
		    	
		    	$txn_date = new XiDate($res['date']);
		    	
		    	if ($res['ttype'] == self::PAYGATE_TRANSACTION_STATUS_APPROVED 
		    			&& $res['stat'] == self::PAYGATE_TRANSACTION_STATUS_APPROVED
		    				&& $res['amt'] == $amount
		    					&& ($txn_date->toUnix() >= $startRange && $txn_date <= $endRange) ){
		    		  	// get the transaction instace of lib
						$transaction = PayplansTransaction::getInstance();
						$transaction->set('user_id',$invoice->getBuyer())
									->set('invoice_id',$invoice->getId())
									->set('payment_id', $payment->getId())
									->set('gateway_txn_id', $res['tid'])
									->set('amount', $res['amt']/100)
									->set('gateway_subscr_id', 0)
									->set('gateway_parent_txn', 0)
									->set('params', PayplansHelperParam::arrayToIni($res))
									->set('message', XiText::_("COM_PAYPLANS_APP_PAYGATE_TRANSACTION_".$res['stat']))
									->save();
						
		    			$recurrence_count = $payment->getGatewayParams()->get('pending_recur_count');
						if($recurrence_count != 0){
							$recurrence_count--;
							$payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
							$payment->save();
						}
						
		    		  }
			}
		}
		
		return true;
	}
}
