<?php

/**
 * @copyright Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
 * @license 	GNU/GPL, see LICENSE.php
 * @package 	Payline
 * @contact 	support+payplans@readybytes.in
 */
if(defined('_JEXEC')===false) die();

class PayplansAppPayline extends PayplansAppPayment
{

	protected $_location = __FILE__;

	protected $payline = null;

	/**
	 * load the payline API as class initialize
	 */
	public function __construct($config = array())
	{
		include_once dirname(__FILE__).DS."lib".DS."paylineSDK.php";
		include_once dirname(__FILE__).DS."lib".DS."lib_debug.php";

		return parent::__construct($config);
	}

	/**
	 * Render the payment form, it has it's template "pay"
	 * @return string result to be assign in view
	 */
	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		$this->assign('paymentUrl', XiRoute::_('index.php?option=com_payplans&view=payment&task=complete&action=process'));
		$this->assign('paymentKey', $payment->getKey());
		return $this->_render('payline_form');
	}

	/**
	 * Now process the submitted card details
	 * @param  string          $action     process
	 * @param  array           $data       credit card detail
	 */

	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		 if ('process' === $action ) {

		 	//result would be true or exception
		 	try {
		 		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		 		if ($invoice->isRecurring()) {
		 			$this->createRecurringProfile($payment, $data);
		 			$this->doRecurringPayments($payment, $data);
		 		}
		 		else {
		 			$this->processBuyerDetails($payment, $data);
		 		}
		 	}
		 	//if something went wrong show the error to user
		 	catch (Exception $e) {
		 		$url = XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey());
		 		XiFactory::getApplication()->redirect($url, $e->getMessage(), 'error');
		 	}

		 	$action = 'success';

		 	return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		 }

		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}

	/**
	 * processBuyerDetails handle the fixed payment
	 * @since 3.1.2
	 * @return true on successful payment or Exception with message in case of any error
	 */
	public function processBuyerDetails(PayplansPayment $payment, Array $data = array()) {

		$merchantId 	= $this->getAppParam('merchantId', false);
		$accessKey 		= $this->getAppParam('accessKey', false);
		$inProduction 	= $this->getAppParam('isProduction', false);

		$this->payline = new paylineSDK($merchantId, $accessKey, null, null, '', '', $inProduction);

		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		//Step 1: Collect necessary data
		$paylineData = array();

		$paylineData['payment'] = $this->getPaylinePaymentDetails($payment, $invoice);
		$paylineData['card'] = $this->getPaylineCardDetails($data);
		$paylineData['order'] = $this->getPaylineOrderDetails($payment, $invoice);

		//clear user specific data for security reasons
		$data = array();

		//Step 2: Do Payment Authorization request
		$paylineAuthorization = $this->payline->doAuthorization($paylineData);

		//Step 3: process response
		$this->processPaylineResponse($paylineAuthorization, $payment);

		if ( JDEBUG ) {
			$message = XiText::_('COM_PAYPLANS_APP_PAYLINE_AUTHORIZATION_DONE');
			PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment, $paylineAuthorization, 'PayplansPaymentFormatter');
		}

		//Step 4: Do Debit with authorization number
		$paylineData['authorization'] = $paylineAuthorization['authorization'];
		$paylineData['payment']['action'] = 204; //to change the payment method mode from authorization to debit
		$success = $this->payline->doDebit($paylineData);

		//Step 5: process response
		$this->processPaylineResponse($success, $payment);

		//clear user specific data for security reasons
		$paylineData = array();

		if ( JDEBUG ) {
			$message = XiText::_('COM_PAYPLANS_APP_PAYLINE_DEBIT_SUCCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment, $success, 'PayplansPaymentFormatter');
		}

		// if same notification came more than one time
		// Check if transaction already exists,if yes then do nothing and return
		$txn_id         = isset($success['transaction']['id']) ? $success['transaction']['id'] :0;
		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, 0, 0);
    	if (!empty($transactions)) {
			foreach ($transactions as $transaction) {
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			$result = $transaction->getParam('result', new stdClass());
    			if($result->code == $success['result']['code']){
    				return true;
				}
    		}
    	}
		
		
		//Step 6: mark invoice paid and create transaction
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $payment->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $txn_id)
					->set('gateway_subscr_id', 0)
					->set('gateway_parent_txn', 0)
					->set('params', PayplansHelperParam::arrayToIni($success))
					->set('amount',$invoice->getTotal())
					->set('message', 'COM_PAYPLANS_APP_PAYLINE_DEBIT_SUCCESS');

		if ( !$transaction->save() ) {
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $success, 'PayplansPaymentFormatter', '', true);
		}

		return true;
	}

	protected function getPaylineOrderDetails(PayplansPayment $payment, PayplansInvoice $invoice, $invoiceCount = 0)
	{
		$orderDetails = array();

		//set specific order fomate of (format : "dd/mm/yyyy HH24:MM")
		$orderDate = $invoice->getCreatedDate()->toUnix();
		$orderDate = date( 'd/m/Y H:i' ,$orderDate);

		$orderDetails['ref'] 		= $invoice->getKey().'-'.$invoiceCount;
		$orderDetails['origin'] 	= '';
		$orderDetails['country'] 	= '';
		$orderDetails['taxes'] 		= $invoice->getTaxAmount() * 100;
		$orderDetails['amount'] 	= $invoice->getTotal($invoiceCount) * 100;
		$orderDetails['date'] 		= $orderDate;
		$orderDetails['currency'] 	= $this->getPaylineCurrencyCode($invoice->getCurrency('isocode'));
		$orderDetails['deliveryTime'] = '';
		$orderDetails['deliveryMode'] = '';

		return $orderDetails;
	}

	protected function getPaylinePaymentDetails(PayplansPayment $payment, PayplansInvoice $invoice, $invoiceCount = 0)
	{
		$paymentDetails = array();

		$paymentDetails['amount'] 				= $invoice->getTotal($invoiceCount) * 100;
		$paymentDetails['currency'] 			= $this->getPaylineCurrencyCode($invoice->getCurrency('isocode'));
		$paymentDetails['action'] 				= 101; 	//To do 101 for authorization+validation and debit mode is 204
		$paymentDetails['mode'] 				= 'CPT'; // To do simple fixed payment mode is CPT for recurring REC
		$paymentDetails['differedActionDate'] 	= '';
		$paymentDetails['contractNumber'] 		= $this->getPaylineContractNumber();

		return $paymentDetails;
	}

	protected function getPaylineCardDetails(Array $data = array())
	{
		$cardData = array();

		$expiryDate = ( ( 100 * intval( $data['card-expiry-month'] ) ) + (intval( $data['card-expiry-year'] ) % 100 ));
		$expiryDate = str_pad($expiryDate, 4, "0", STR_PAD_LEFT);

		//these card details could be more effectively asked or fetched from somewhere
		$cardData['encryptionKeyId'] 	= '';
		$cardData['encryptedData'] 		= '';
		$cardData['number'] 			= $data['card-number'];
		$cardData['type'] 				= 'CB';
		$cardData['expirationDate'] 	= $expiryDate;
		$cardData['cvx'] 				= $data['card-cvc'];
		$cardData['ownerBirthdayDate'] 	= '';
		$cardData['password'] 			= '';
		$cardData['cardPresent'] 		= '';
		$cardData['token'] 				= '';
		$cardData['cardholder'] 		= '';

		return $cardData;
	}

	protected function getPaylineWalletDetails(PayplansPayment $payment)
	{
		$buyer = $payment->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		$walletData = array();

		$plan = array_pop($payment->getPlans(PAYPLANS_INSTANCE_REQUIRE));
		
		
		$walletData['walletId'] 	= 'wallet'.$payment->getKey();
		$walletData['lastName'] 	= '';
		$walletData['firstName'] 	= $buyer->getUsername();
		$walletData['email'] 		= $buyer->getEmail();
		$walletData['comment'] 		= $plan->getTitle();
		$walletData['cardStatus'] 	= '';
		$walletData['default'] 		= '';

		return $walletData;
	}

	public function getPaylineCurrencyCode($isoCode = null)
	{
		$currency =	array(
									'EUR' => 978,
									'USD' => 840,
									'CHF' => 756,
									'GBP' => 826,
									'CAD' => 124,
								 );

		if ( $isoCode === null ) {
			return $currency['EUR'];
		}

		return $currency[$isoCode];
	}

	/**
	 * getPaylineContractNumber Contract number can only be one or multiple for multiple card type,
	 * so it can be fetched from the merchant account details
	 * @param  string $cardType credit card type by default CB
	 * @since 3.1.2
	 */
	protected function getPaylineContractNumber($cardType = 'CB')
	{
		$contractNumber = $this->getAppParam('contractNumber', false);

		if($contractNumber !== false) {
			return $contractNumber;
		}

		$contractNumbers = array();

		$merchantDetails = $this->payline->getMerchantSettings(array());

		//currently i consider only one pointofsell
		$pointOfSell = $merchantDetails['listPointOfSell']['pointOfSell'];

		//$pointOfSell = array_shift($pointsOfSell);

		$contracts  = $pointOfSell['contracts'];

		foreach ($contracts as $contract) {
			$contractNumbers[$contract['cardType']] = $contract['contractNumber'];
		}

		return $contractNumbers[$cardType];
	}

	/**
	 * Process the response from payline
	 * it could be originally error or fraud or duplicate case
	 * @return true and throws expection on error
	 */
	protected function processPaylineResponse(Array $response = array() ,PayplansPayment $payment)
	{
		if ( empty( $response ) ) {
			throw new Exception( XiText::_('COM_PAYPLANS_APP_PAYLINE_EMPTY_RESPONSE') );
		}

		if ( "00000" !== $response['result']['code'] ) {

			$message = XiText::_('COM_PAYPLANS_APP_PAYLINE_DEBIT_FAIL');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);

			throw new Exception( $response['result']['longMessage'] );
		}

		if (  "0" !== $response['transaction']['isDuplicated'] ) {

			$message = XiText::_('COM_PAYPLANS_APP_PAYLINE_DUPLICATE_RESPONSE');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);

			throw new Exception( $response['transaction']['explanation'] );
		}

		if ( "0" !== $response['transaction']['isPossibleFraud'] ) {

			$message = XiText::_('COM_PAYPLANS_APP_PAYLINE_POSSIBLE_FRAUD_RESPONSE');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);

			throw new Exception( $response['transaction']['explanation'] );
		}

		return true;
	}


	public function supportForRefund()
	{
		return true;
	}

	public function refundRequest(PayplansTransaction $transaction, $refundAmount) {

		$merchantId 	= $this->getAppParam('merchantId', false);
		$accessKey 		= $this->getAppParam('accessKey', false);
		$inProduction 	= $this->getAppParam('isProduction', false);

		$this->payline = new paylineSDK($merchantId, $accessKey, null, null, '', '', $inProduction);

		$invoice = $transaction->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$payment = $transaction->getPayment(PAYPLANS_INSTANCE_REQUIRE);

		//Step 1: Collect necessary data
		$paylineData = array();

		$paylineData['payment'] = $this->getPaylinePaymentDetails($payment, $invoice);

		$paymentDetails['payment']['amount'] 	= ($refundAmount * 100);
		$paylineData['transactionID'] 			= $transaction->getGatewayTxnId();
		$paylineData['comment'] 				= XiText::_("COM_PAYPLANS_REFUND_REQUEST");
		$paylineData['payment']['action'] 		= 421;

		$response = $this->payline->doRefund($paylineData);

		try {
			$this->processPaylineResponse($response, $payment);
		}
		catch (Exception $e) {

			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $e->getMessage(), $payment, $response, 'PayplansPaymentFormatter', '', true);
			return false;
		}

		$newtransaction = PayplansTransaction::getInstance();

		$newtransaction->set('user_id', 				$transaction->getBuyer())
						->set('invoice_id', 			$invoice->getId())
						->set('payment_id',		 		$payment->getId())
						->set('gateway_txn_id', 		$response['transaction']['id'])
						->set('gateway_subscr_id', 		0)
						->set('gateway_parent_txn', 	0)
						->set('params', 				PayplansHelperParam::arrayToIni($response));

		$negativeAmt = -($refundAmount);

		$newtransaction->set('amount',$negativeAmt)
					   ->set('message', 'COM_PAYPLANS_APP_REFUND_TRANSACTION_COMPLETED');

		if ( !$newtransaction->save() ) {
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);

			return false;
		}

		return true;

	}

	/*
	 * firstly we need to create customer wallet at payline, so that he could be charge recurrily
	 */
	function createRecurringProfile(PayplansPayment $payment, $data = array())
	{
		$merchantId 					= $this->getAppParam('merchantId', false);
		$accessKey 						= $this->getAppParam('accessKey', false);
		$inProduction 					= $this->getAppParam('isProduction', false);

		$this->payline 					= new paylineSDK($merchantId, $accessKey, null, null, '', '', $inProduction);

		$paylineData 					= array();

		$paylineData['wallet'] 			= $this->getPaylineWalletDetails($payment);
		$paylineData['contractNumber'] 	= $this->getPaylineContractNumber();
			
		$walletData 					= array();
		$walletData['contractNumber'] 	= $paylineData['contractNumber'];
		$walletData['walletId'] 		= $paylineData['wallet']['walletId'];	
		
		$response = $this->payline->getWallet($walletData);
		
		if ( "02500" == $response['result']['code'] ) {

			return true;
			
		}	

		$paylineData['card'] 	= $this->getPaylineCardDetails($data);
		
		$response = $this->payline->createWallet($paylineData);

		if ( empty( $response ) ) {
			throw new Exception( XiText::_('COM_PAYPLANS_APP_PAYLINE_EMPTY_RESPONSE') );
		}

		if ( "02500" !== $response['result']['code'] ) {

			$message = XiText::_('COM_PAYPLANS_APP_PAYLINE_FAIL_TO_CREATE_PROFILE');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);

			throw new Exception( $response['result']['longMessage'] );
		}
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		$newtransaction = PayplansTransaction::getInstance();
		
		$newtransaction->set('user_id', 			$payment->getBuyer())
						->set('invoice_id', 		$invoice->getId())
						->set('payment_id',		 	$payment->getId())
						->set('gateway_txn_id', 	$paylineData['wallet']['walletId'])
						->set('gateway_subscr_id', 	0)
						->set('gateway_parent_txn', 0)
						->set('params', 			PayplansHelperParam::arrayToIni($response))
						->set('amount',				0)
						->set('message', 'COM_PAYPLANS_APP_PAYLINE_CREATE_PROFILE_SUCCESSFULLY');
		
		if ( !$newtransaction->save() ) {
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		
			throw new Exception( XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD') );
		}
		
		$recurringType = $invoice->isRecurring();
			
		if($recurringType){
			$recurrence_count = $invoice->getRecurrenceCount();
				
			switch ($recurringType)
			{
				case PAYPLANS_RECURRING_TRIAL_1: 	$recurrence_count++;break;
				case PAYPLANS_RECURRING_TRIAL_2:	$recurrence_count = $recurrence_count+2;break;
			}
		}
		else
			$recurrence_count = 0;
		
		$payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
		$payment->getGatewayParams()->set('wallet_id',$paylineData['wallet']['walletId']);
		$payment->save();
		$payment->refresh();
		
		return true;

	}
	
	/**
	 * Charge the customer profile	 
	 */
	function doRecurringPayments(PayplansPayment $payment, $data = array(), $invoiceCount = 0)
	{
		$merchantId 	= $this->getAppParam('merchantId', false);
		$accessKey 		= $this->getAppParam('accessKey', false);
		$inProduction 	= $this->getAppParam('isProduction', false);
		
		$this->payline = new paylineSDK($merchantId, $accessKey, null, null, '', '', $inProduction);
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		$paylineData = array();
		
		$paylineData['payment'] 	= $this->getPaylinePaymentDetails($payment, $invoice, $invoiceCount);
		$paylineData['order'] 		= $this->getPaylineOrderDetails($payment, $invoice, $invoiceCount);
		$gatewayParams 				= $payment->getGatewayParams()->toArray(); 
		$paylineData['walletId'] 	= $gatewayParams['wallet_id'];
		
		if ($paylineData['payment']['amount'] == 0) {
			$response = array('result' => XiText::_('COM_PAYPLANS_APP_PAYLINE_FREE_TRANSACTION'));
		}else {	
			$response = $this->payline->doImmediateWalletPayment($paylineData);
			$this->processPaylineResponse($response, $payment);
		}
		
		if ( JDEBUG ) {
			$message = XiText::_('COM_PAYPLANS_APP_PAYLINE_DEBIT_SUCCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment, $response, 'PayplansPaymentFormatter');
		}
		
		// if same notification came more than one time
		// Check if transaction already exists,if yes then do nothing and return
			$txn_id         = isset($response['transaction']['id']) ? $response['transaction']['id'] :0;
			$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, 0, 0);
	    	if (!empty($transactions)) {
	    		foreach ($transactions as $transaction) {
	    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
	    			$result = $transaction->getParam('result', new stdClass());
	    			if($result->code == $response['result']['code']){
	    				return true;
    				}
	    		}
	    	}
		
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $payment->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $txn_id)
					->set('gateway_subscr_id', 0)
					->set('gateway_parent_txn', 0)
					->set('params', PayplansHelperParam::arrayToIni($response))
					->set('amount',$invoice->getTotal($invoiceCount))
					->set('message', 'COM_PAYPLANS_APP_PAYLINE_DEBIT_SUCCESS');
		
		if ( !$transaction->save() ) {
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $success, 'PayplansPaymentFormatter', '', true);
			
			throw new Exception( XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD') );
		}
		
		//if it is paid then decrease recurrence count by one
		$recurrence_count = $payment->getGatewayParams()->get('pending_recur_count');
		if($recurrence_count != 0)
		{
			$recurrence_count--;
			$payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
			$payment->save();
		}
		
		return true;
	}
	
	public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		$lifetime = false;
		
		$lifetime = ($invoice->getRecurrenceCount() == 0)? true : false;
		
		$invoice_count = $invoiceCount +1;

		try {
			if($invoice->isRecurring())
			{			
				$recurrence_count 	= $payment->getGatewayParam('pending_recur_count');
				
				if($recurrence_count > 0 || $lifetime)
				{
					self::doRecurringPayments($payment, array(), $invoice_count);
				}
			}
		}
		catch (Exception $e) {
			return false;
		}
		
		return true;
	}

	public function isSupportPaymentCancellation($invoice)
	{
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}
	
	public function onPayplansPaymentTerminate(PayplansPayment $payment, $invoiceController)
	{
		parent::onPayplansPaymentTerminate($payment, $invoiceController);
		return $this->_render('subscription_cancel');
	}
}
