<?php

/**
 * @copyright	Copyright (C) 2012 Pay.nl
 * @author       Sebastian Berm
 * @license	GNU/GPL, see LICENSE.php
 * @package	Payplans
 * @subpackage	Pay.nl Payment App
 * @contact	support@pay.nl
 */
if (defined('_JEXEC') === false)
    die();

ob_start();

// We need access to our default PPT class
require_once('classes/Autoload.php');

class PayplansAppPaynl extends PayplansAppPayment {

    //inherited properties
    // IMP : store txn_id in txn_id for payment
    // and store subscr_id for recurring master payment

    protected $_location = __FILE__;

    public function onPayplansPaymentForm(PayplansPayment $payment, $data = null) {
        if (is_object($data)) {
            $data = (array) $data;
        }

        $invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
        $apiStart = new Pay_Api_Start();
        $apiStart->setApiToken($this->getAppParam('token'));
        $apiStart->setServiceId($this->getAppParam('serviceId'));

        $apiStart->setPaymentOptionId($this->getAppParam('paymentOptionId'));

        $apiStart->setExtra1($payment->getKey());
        $apiStart->setExtra2($this->getId() . '_' . $invoice->getKey());
        $apiStart->setExtra3($this->getId() . '_' . $payment->getKey());


        $root = JURI::root();
        if (XiFactory::getConfig()->https == true) {
            $root = JString::str_ireplace("http:", "https:", $root);
        }


        $apiStart->setExchangeUrl($root . 'index.php?option=com_payplans&view=payment&task=notify&invoice_key=' . $invoice->getKey() . '&payment_key=' . $payment->getKey());


        $retData = base64_encode(serialize(array('okUrl' => $root . 'index.php?option=com_payplans&view=payment&task=complete&action=success&invoice_key=' . $invoice->getKey() . '&payment_key=' . $payment->getKey(),
            'cancelUrl' => $root . 'index.php?option=com_payplans&view=payment&task=complete&action=cancel&invoice_key=' . $invoice->getKey() . '&payment_key=' . $payment->getKey())));


        $apiStart->setFinishUrl($root . 'index.php?option=com_payplans&view=payment&task=complete&action=verifyme&invoice_key=' . $invoice->getKey() . '&payment_key=' . $payment->getKey() . '&retData=' . $retData);


        $plan = array_shift($payment->getPlans(PAYPLANS_INSTANCE_REQUIRE));

        $title = substr($plan->getTitle(), 0, 25);
        $apiStart->setDescription($title);
        if ($invoice->isRecurring()) {
            // recurring payment has been started - we don't do no recurring payments
            return $this->_render('form_subscription');
        }

        try {
            $price = intval($invoice->getTotal() * 100);
            $apiStart->setAmount($price);

            $currency = $invoice->getCurrency('isocode');
            $apiStart->setCurrency($currency);
            $payReturn = $apiStart->doRequest();

//                    $payReturn = $pay->createTransaction($price, $this->forcedPaymentProfile,$aExtraOptions);
            $this->assign('paynl_redirecturl', $payReturn['transaction']['paymentURL']);
            $payment->set('txn_id', $payReturn['transaction']['transactionId'])->save(); // Save the transaction id

            return $this->_render('form_buynow');
        } catch (Exception $ex) {
            
            return $this->_render('error', array('errors' => array($ex->getMessage())));
        }

        return $this->_render('error');
    }

    public function onPayplansPaymentAfter(PayplansPayment $payment, $action, $data, $controller) {
        if ($action == 'verifyme') {
            $apiInfo = new Pay_Api_Info();
            $apiInfo->setApiToken($this->getAppParam('token'));
            $apiInfo->setServiceId($this->getAppParam('serviceId'));
            $apiInfo->setTransactionId($data['orderId']);

            $result = $apiInfo->doRequest();

            $state = $result['paymentDetails']['state'];

            $retData = base64_decode($data['retData']);
            $retData = unserialize($retData);

            if ($state < 0) {
                header('Location: ' . $retData['cancelUrl']);
                exit;
            } else {
                header('Location: ' . $retData['okUrl']);
                exit;
            }
        }

        $record = array_pop(PayplansHelperLogger::getLog($payment, XiLogger::LEVEL_ERROR));
        if ($record && !empty($record)) {
            $action = 'error';
        }

        return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
    }

    protected function _processNotification($payment, $data) {
        return true;
    }

    public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller) {
        if (isset($data['action'])) {
            $apiInfo = new Pay_Api_Info();
            $apiInfo->setApiToken($this->getAppParam('token'));
            $apiInfo->setServiceId($this->getAppParam('serviceId'));
            $apiInfo->setTransactionId($data['order_id']);
            
            $result = $apiInfo->doRequest();

            $amount = 0;
            $errors = array();
            // Verify the action we're getting
            if ($data['action'] == 'add' || $data['action'] == 'new_ppt') {
                // New verified transaction
                // Verify the verification
                if ($result['paymentDetails']['stateName'] == 'PAID' || $result['paymentDetails']['stateName'] == 'PAID_CHECKAMOUNT') {
                    $amount = $result['paymentDetails']['amount'] / 100;
                }
            


				$errors = array();
				$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
				$transaction = PayplansTransaction::getInstance();
				$transaction->set('user_id', $payment->getBuyer())
						->set('invoice_id', $invoice->getId())
						->set('payment_id', $payment->getId())
						->set('gateway_txn_id', $data['payment_session_id'])
						->set('gateway_subscr_id', 0)
						->set('gateway_parent_txn', 0)
						->set('amount', $amount)
						->set('params', PayplansHelperParam::arrayToIni($data));



				$errors = $this->_processNotification($transaction, $data, $payment);
				$transaction->save();
				
				if (is_array($errors) && count($errors) > 0) {
					echo "false|we have errors";
				} else {
					echo "true|processed";
				}
				die();
			} else {
			 echo "true|ignoring ".$data['action'];
			}
        }

        echo "false|what are you talking about?";
        exit;
    }

    function _createPayment($order, $txnId, $checkFirstPayment) {
        // if same transaction id is set in data
        // it mean multiple notification are sent from paypal for same payment
        $txnPayment = XiFactory::getInstance('payment', 'model')
                ->loadRecords(array('txn_id' => $txnId));

        if (empty($txnPayment)) {
            return $order->createPayment($this->getId(), $checkFirstPayment);
        }

        $tmpPayment = array_pop($txnPayment);
        return PayplansPayment::getInstance($tmpPayment->payment_id, null, $tmpPayment);
    }

}
