<?php
/**
 * paypaladvance
 * This is the payment gateway that support to fetch payment throug Paypal Payment Advance (PPA).
 * @copyright Copyright (C) 2009 - 2014 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
 * @license GNU/GPL, see LICENSE.php
 * @package payplans
 * @version 3.2.0
 * @author Mohit Agrawal <mohit@readybytes.in>
 */

if(defined( '_JEXEC' )==false) {
	die( 'Restricted access' );
}

define ('PAYPAL_ADVANCE_PILOT_TEST_URL', "https://pilot-payflowpro.paypal.com");
define ('PAYPAL_ADVANCE_PILOT_LIVE_URL', "https://payflowpro.paypal.com");
define ('PAYPAL_ADVANCE_PAYMENT_URL',	 "https://payflowlink.paypal.com");

class PayplansAppPaypaladvance extends PayplansAppPayment
{
	
	//inherited properties
	protected $_location	= __FILE__;

	public function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
	
		return parent::isApplicable($refObject, $eventName);
	}
	
	public function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		$gateway = JFactory::getApplication()->input->get('gateway', null);
		$action  = JFactory::getApplication()->input->get('action', null);
		
		if($view != 'payment' || ($gateway != 'paypaladvance') ){
			return true;
		}
		
		if(JDEBUG){
			
			$post = JFactory::getApplication()->input->post;
			
			// When debug mode is on dump in file
			file_put_contents(JPATH_SITE.DS.'tmp'.DS.time(), var_export($post,true), FILE_APPEND);
		}
		
		if ($task == 'notify' || ($task == 'complete') ) {
			
			$paymentKey = JFactory::getApplication()->input->get('INVNUM', null);
			
			if(!empty($paymentKey)){
				JFactory::getApplication()->input->set('payment_key', $paymentKey);
				return true;
			}
		}

		return true;
	}
	
	/**
	 * generate the paypal advance redirect form
	 * onPayplansPaymentForm 
	 * @since 3.1.2
	 * @return string
	 */
	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		$sandbox 		= $this->getAppParam('sandboxTesting', 1);
		$mode 			= ($sandbox)? 'TEST' : 'LIVE';
		
		$secureToken = $this->fetchSecureToken($payment, $sandbox);
		
		if (0 != $secureToken['RESULT'] || empty($secureToken)) {
			
			$errorUrl = XiRoute::_("index.php?option=com_payplans&gateway=paypal&view=payment&task=complete&action=error"
								  ."&RESULT=".$secureToken['RESULT']."&RESPMSG=".$secureToken['RESPMSG']."&payment_key=".$payment->getKey());
			
			$application = PayplansFactory::getApplication();
			$application->redirect($errorUrl);
		}
		
		$paymentUrl  = PAYPAL_ADVANCE_PAYMENT_URL."?MODE=$mode&SECURETOKENID=".$secureToken['SECURETOKENID']."&SECURETOKEN=".$secureToken['SECURETOKEN'];
				
		$application = PayplansFactory::getApplication();
		$application->redirect($paymentUrl);
		
		return true;
	}
	
	/**
	 * this function will generate security token to use in payment form
	 * generateSecurityToken 
	 * @since 3.2
	 * @return array
	 */
	private function fetchSecureToken(PayplansPayment $payment, $sandbox = 0)
	{
			
		$invoice 			= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount   			= $invoice->getTotal();
		$currency 			= $invoice->getCurrency('isocode'); 
		
		$sercurityToken 	= $this->gererateSecureTokenId($payment);
		
		$API_Vendor 		= $this->getAppParam('vendor', '');
		$API_User 			= $this->getAppParam('user', $API_Vendor);
		$API_Partner 		= $this->getAppParam('partner', '');
		$API_Password 		= $this->getAppParam('password', '');
		
		$transactionType 	= "S";
		
		$postString 		= "PARTNER=$API_Partner"
							."&VENDOR=$API_Vendor"
							."&USER=$API_User"
							."&PWD=$API_Password"
							."&TRXTYPE=$transactionType"
							."&AMT=$amount"
							."&CURRENCY=$currency"
							."&INVNUM=".$payment->getKey()
							."&CREATESECURETOKEN=Y"
							."&SECURETOKENID=$sercurityToken";

		if ($sandbox) {
			$uri = PAYPAL_ADVANCE_PILOT_TEST_URL;
		}
		else {
			$uri = PAYPAL_ADVANCE_PILOT_LIVE_URL;
		}
		
		$uri 			= new JURI($uri);
		
		try {
			$curl 		= new JHttpTransportCurl(new JRegistry());
			$response 	= $curl->request('post', $uri, $postString);
		}
		catch (Exception $e) {
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYPAL_ADVANCE_CURL_FAILED');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($e->getMessage()), 'PayplansPaymentFormatter', '', true);
		}
		
		if (empty($response->body)) {
			$error = curl_error($ch).'('.curl_errno($ch).')';
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYPAL_ADVANCE_EMPTY_RESPONSE');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($error), 'PayplansPaymentFormatter', '', true);
		}
		
		$processedResponse = $this->transformResponse($response->body);
		
		if ( (0 == sizeof($processedResponse)) || !array_key_exists('RESULT', $processedResponse) ) {
			$message = XiText::sprintf('COM_PAYPLANS_PAYMENT_APP_PAYPAL_ADVANCE_INVALID_RESPONSE_MESSAGE', $postString);
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $processedResponse, 'PayplansPaymentFormatter', '', true);
		}

		If (0 != $processedResponse['RESULT']) {
			$message = XiText::sprintf('COM_PAYPLANS_PAYMENT_APP_PAYPAL_ADVANCE_FAILED_RESPONSE_MESSAGE', $postString);
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $processedResponse, 'PayplansPaymentFormatter', '', true);
		}
		
		$payment->setParam('paypal_advance_token', $processedResponse['SECURETOKEN']);
		$payment->setParam('paypal_advance_token_id', $processedResponse['SECURETOKENID']);
		$payment->save();
		
		return $processedResponse;
	}
	
	/**
	 * transform the response in array.
	 * @param array/string $response
	 * @return array
	 */
	public function transformResponse($response) 
	{
		if(is_array($response)) {
			return $response;
		}
		
		//Reponse is like eg. x=x&y=y&z=z, so we need to process the response details.
		$response = explode("&", $response);
		
		$processedResponse = array();
		
		foreach ($response as $key => $value) {
			$tmp = explode("=", $value);
			if(sizeof($tmp) > 1) {
				$processedResponse[$tmp[0]] = $tmp[1];
			}
		}
		
		return $processedResponse;
	}
	
	/**
	 * It will generate the 32 Alphanumeric string that will be utilize as secure token
	 * gererateSecureToken 
	 * @since 3.2
	 * @return string
	 */
	public function gererateSecureTokenId(PayplansPayment $payment)
	{
		$paymentKey 					= $payment->getKey();
		$encryptor 						= PayplansFactory::getEncryptor(true);
		$encryptor->sourcelen 			= 32;
		
		$token = $encryptor->encrypt($paymentKey);
		
		return $token; 
	}
	
	/**
	 * Handle the paypal response and create the trasactions accordingly
	 * @since 3.2
	 */
	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		if ('error' == $action) {
			$this->assign('invoice', 		$invoice);
			$this->assign('error_code', 	$data['RESULT']);
			$this->assign('error_msg', 		$data['RESPMSG']);
		} 
		
		if ('process' == $action) {
			
			$action 	= 'error';

			$result = $this->processResponse($payment, $data);
			
			if($result) {
				$action = 'success';
			}
		}
		
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	/**
	 * process the notification of payment is successfull then create recurring profile if plan is recurring 
	 */
	function processResponse(PayplansPayment $payment, $data){
		
		$response 	= $this->transformResponse($data);
		
		If (0 != $response['RESULT'] || 'Approved' != $response['RESPMSG']) {
		
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYPAL_ADVANCE_FAILED_RESPONSE_MESSAGE');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		
			$this->createTransaction($payment, 0, $response);
			
			return false;
		}
			
		if ( isset($response['SECURETOKENID']) && isset($response['SECURETOKEN']) ) {
			$secureTokenId 	= $payment->getParam('paypal_advance_token_id', null);
			$secureToken 	= $payment->getParam('paypal_advance_token', null);
		
			if ( $response['SECURETOKENID'] === $secureTokenId && $response['SECURETOKEN'] === $secureToken) {
		
				$payment->setParam('paypal_advance_profile', $response['PNREF']);
				$payment->save();
				
				$result 	= $this->createTransaction($payment, $response['AMT'], $response);
				
				//work done for recurring payments
// 				$invoice 	= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
// 				$recurring 	= $invoice->isRecurring();
// 				if($result && $recurring){
// 					return $this->createRecurringProfile($payment);
// 				}
				
				if ($result) {
					return true;
				}
			}
		}
		
		return false;
	}
	
	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		$result = $this->processResponse($payment, $data);
		
		parent::onPayplansPaymentNotify($payment, $data, $controller);	
	}
	
	/**
	 * create transaction for payment with checking of duplication of trasaction 
	 * @param PayplansPayment $payment
	 * @param float  $amount 
	 * @param array  $gatewayParams response from the gateway
	 * @param string $gatewayTxnKey a key name that is available in response and can be set in gateway_txn_id of transaction
	 * @param string $uniqueParam a uniquey key that is compared to check for existing trasaction
	 * @return boolean
	 */
	public function createTransaction(PayplansPayment $payment, $amount = 0, $gatewayParams, $gatewayTxnKey = 'PNREF', $uniqueParam = 'SECURETOKEN', $message = null)
	{
		$invoice 		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		$gatewayTxnId 	= isset($gatewayParams[$gatewayTxnKey]) ? $gatewayParams[$gatewayTxnKey] : 0;

		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $gatewayTxnId, 0, 0);
		
		$message 		= (null != $message) ? $message : XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYFLOW_PAYMENT_COMPLETED_SUCCESSFULLY');
		 
		if (!empty($transactions)) {
			foreach($transactions as $transaction){
				$transaction 	= PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
				if($transaction->getParam($uniqueParam,'') == $gatewayParams[$uniqueParam]){
					$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYPAL_ADVANCE_DUPLICATE_RESPONSE');
					PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $gatewayParams, 'PayplansPaymentFormatter', '', true);
					return false;
				}
			}
		}

		if (isset($gatewayParams['AMT'])) {
			$amount = $gatewayParams['AMT'];
		}
		
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', 			$invoice->getBuyer())
					->set('invoice_id', 		$invoice->getId())
					->set('payment_id', 		$payment->getId())
					->set('amount', 			$amount)
					->set('gateway_txn_id', 	$gatewayTxnId)
					->set('gateway_subscr_id',  0)
					->set('gateway_parent_txn', 0)
					->set('message', 			$message)
					->set('params', 			PayplansHelperParam::arrayToIni($gatewayParams));
		
		if(!$transaction->save()){
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $gatewayParams ,'PayplansPaymentFormatter', '',true);
			
			return false;
		}
		
		return true;
	}
	
	//CURRENT NOT USED : ONLY WHEN SUPPORT FOR RECURRING
	public function getRecurrenceTime($expTime)
	{
		//NOT SUPPORTED RECURRING
		return array('period' => "Not Supported", 'unit' =>  "Not Supported", 'frequency' => "Not Supported",
				'message' =>  "Not Supported");
		
		$expTime['year'] 	= isset($expTime['year']) 	? intval($expTime['year'], 10) : 0;
		$expTime['month'] 	= isset($expTime['month']) ? intval($expTime['month'], 10) : 0;
		$expTime['day'] 	= isset($expTime['day']) 	? intval($expTime['day'], 10)  : 0;
	
		// if years are not empty
		if(!empty($expTime['year'])){
			$year = $expTime['year'];
			if($year >= 1)
			{
				$payPeriod = 'YEAR';
			}
			return array('period' => $payPeriod, 'unit' => 'Year', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
					'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYFLOW_YEAR_MESSAGE'));
		}
	
		// if months are not empty
		if(!empty($expTime['month'])){
			$months = $expTime['month'];
			if($months == 1)
			{
				$payPeriod = 'MONT';
			}
			if($months >= 3 && $months < 6)
			{
				$payPeriod = 'QTER';
			}
			if($months >= 6)
			{
				$payPeriod = 'SMYR';
			}
			return array('period' => $payPeriod, 'unit' => 'Month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
					'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYFLOW_MONTH_MESSAGE'));
		}
	
	
		if(!empty($expTime['day'])){
			$days  = $expTime['day'];
			$weeks = intval($days/7, 10);
				
			if($weeks == 1)
			{
				$payPeriod = 'WEEK';
			}
			if($weeks == 2 || $weeks == 3)
			{
				$payPeriod = 'BIWK';
			}
			if($weeks == 4)
			{
				$payPeriod = 'FRWK';
			}
			return array('period' => $payPeriod, 'unit' => 'Week', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
					'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYFLOW_DAY_MESSAGE'));
		}
	
		return false;
	}
	
	//CURRENT NOT USED : ONLY WHEN SUPPORT FOR RECURRING
	function createRecurringProfile(PayplansPayment $payment)
	{

		$buyer 				= $payment->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		$invoice 			= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		$paymentType      	= 'R';
		$action           	= 'A';
		$profilename      	= $buyer->getName();
		$amount 		  	= $invoice->getTotal(); // Regular Amount
		$currency 			= $invoice->getCurrency('isocode');
		
		$now 			  	= new XiDate('now');
		$now              	= $now->addExpiration('000001000000');
		$startDate        	= $now->toFormat("%m%d%Y");
	
		$expirationTime   	= $invoice->getExpiration(PAYPLANS_RECURRING);
		$expirationTime   	= $this->getRecurrenceTime($expirationTime);
		$billingPeriod 	  	= $expirationTime['unit'];
		$billingFreq 	  	= $expirationTime['period'];
		
		$gatewayTxnId 		= $payment->getParam('paypal_advance_profile',null);
		

		//IMP:here we need to do -1 cause we alredy did one trasaction already
		$billingCycle 	  = ($invoice->getRecurrenceCount()-1);

		$API_Vendor 		= $this->getAppParam('vendor', '');
		$API_User 			= $this->getAppParam('user', $API_Vendor);
		$API_Partner 		= $this->getAppParam('partner', '');
		$API_Password 		= $this->getAppParam('password', '');
		
		$postString         = "PARTNER=$API_Partner"
							."&VENDOR=$API_Vendor"
							."&USER=$API_User"
							."&PWD=$API_Password"
							."&INVNUM=".$payment->getKey()
							."&TENDER=C"
							."&TRXTYPE=$paymentType"
							."&ACTION=$action"
							."&PROFILENAME=$profilename"
							."&AMT=$amount"
							."&CURRENCY=$currency"
							."&START=$startDate"
							."&PAYPERIOD=$billingFreq"
							."&TERM=$billingCycle"
							."&ORIGID=$gatewayTxnId";
	
		$result 	= true; 
		$message 	= null;
		$sandbox 	= $this->getAppParam('sandboxTesting', 1);
		
		if ($sandbox) {
			$uri = PAYPAL_ADVANCE_PILOT_TEST_URL;
		}
		else {
			$uri = PAYPAL_ADVANCE_PILOT_LIVE_URL;
		}
	
		$uri 			= new JURI($uri);
		
		try {
			$curl 		= new JHttpTransportCurl(new JRegistry());
			$response 	= $curl->request('post', $uri, $postString);
		}
		catch (Exception $e) {
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYPAL_ADVANCE_CURL_FAILED');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($e->getMessage()), 'PayplansPaymentFormatter', '', true);
			$result = false;
		}
		
		if (empty($response->body)) {
			$error = curl_error($ch).'('.curl_errno($ch).')';
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYPAL_ADVANCE_EMPTY_RESPONSE');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($error), 'PayplansPaymentFormatter', '', true);
			$result = false;
		}
		
		$processedResponse = $this->transformResponse($response->body);
		
		if ( (0 == sizeof($processedResponse)) || !array_key_exists('RESULT', $processedResponse) ) {
			$message = XiText::sprintf('COM_PAYPLANS_PAYMENT_APP_PAYPAL_ADVANCE_INVALID_RESPONSE_MESSAGE', $postString);
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $processedResponse, 'PayplansPaymentFormatter', '', true);
			$result = false;
		}
		
		If (0 != $processedResponse['RESULT']) {
			$message = XiText::sprintf('COM_PAYPLANS_PAYMENT_APP_PAYPAL_ADVANCE_UNABLE_TO_CREATE_PROFILE_MESSAGE', $postString);
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $processedResponse, 'PayplansPaymentFormatter', '', true);
			$result = false;
		}
		
		$this->createTransaction($payment, 0, $processedResponse, $message);
		
		return $result;

	}
	
}
 
