<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	PayPal Pro
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();


class PayplansAppPaypalPro extends PayplansAppPayment
{
	//inherited properties
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}

	// Website PayPal pro support different time period for trial subscription, so return true
	function isFirstExpirationtimeApplicable()
	{
		return true;
	}

	public function isSupportPaymentCancellation($invoice)
	{
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}

	/**
	 * Send HTTP POST Request
	 *
	 * @param	string	The API method name
	 * @param	string	The POST Message fields in &name=value pair format
	 * @return	array	Parsed HTTP Response body
	 */
	function executePaypalApi($methodName_, $nvpStr_, $payment) 
	{
		// Set up your API credentials, PayPal end point, and API version.
		$API_UserName = $this->getAppParam('apiUsername');
		$API_Password = $this->getAppParam('apiPassword');
		$API_Signature = $this->getAppParam('apiSignature');
		$API_Endpoint = "https://api-3t.paypal.com/nvp";
		
		if($this->getAppParam('sandboxTesting')){
			$API_Endpoint = "https://api-3t.sandbox.paypal.com/nvp";
		}
		$version = urlencode('51.0');
	
		// Set the curl parameters.
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $API_Endpoint);
		curl_setopt($ch, CURLOPT_VERBOSE, 1);
	
		// Turn off the server and peer verification (TrustManager Concept).
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
	
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POST, 1);
	
		// Set the API operation, version, and API signature in the request.
		$nvpreq = "METHOD=$methodName_&VERSION=$version&PWD=$API_Password&USER=$API_UserName&SIGNATURE=$API_Signature$nvpStr_";
	
		// Set the request as a POST FIELD for curl.
		curl_setopt($ch, CURLOPT_POSTFIELDS, $nvpreq);
	
		// Get response from the server.
		$httpResponse = curl_exec($ch);
	
		if(!$httpResponse) {
			$error = curl_error($ch).'('.curl_errno($ch).')';
			$message = "$methodName_ ".XiText::_('COM_PAYPLANS_APP_PAYPAL_PRO_FAILED_MESSAGE');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($error), 'PayplansPaymentFormatter', '', true);
		}
	
		// Extract the response details.
		$httpResponseAr = explode("&", $httpResponse);
	
		$httpParsedResponseAr = array();
		foreach ($httpResponseAr as $i => $value) {
			$tmpAr = explode("=", $value);
			if(sizeof($tmpAr) > 1) {
				$httpParsedResponseAr[$tmpAr[0]] = $tmpAr[1];
			}
		}
	
		if((0 == sizeof($httpParsedResponseAr)) || !array_key_exists('ACK', $httpParsedResponseAr)) {
			$error = "Invalid HTTP Response for POST request($nvpreq) to $API_Endpoint.";
			$message = "$methodName_ ".XiText::_('COM_PAYPLANS_APP_PAYPAL_PRO_FAILED_MESSAGE');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($error), 'PayplansPaymentFormatter', '', true);
		}
	
		return $httpParsedResponseAr;
	}
	
	public function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		$gateway = JRequest::getVar('gateway', '');
		if($view != 'payment' || ($task != 'notify')){
			return true;
		}
		
		//in case of recurring payment we get "rp_invoice_id" which contains payment key
		$paymentKey = JRequest::getVar('rp_invoice_id', null);
		if(!empty($paymentKey)){
			JRequest::setVar('payment_key', $paymentKey, 'POST');
			return true;
		}
		
		//in case of non-recurring payment we get invoice key and payment key in custom variable.
		$custom = JRequest::getVar('custom', null);
		$custom = explode('-', $custom);
		$invoiceKey 	= $custom['0'];
		$paymentKey = $custom['1'];
		
		if($invoiceKey == false || $paymentKey == false){
			return true;
		}
			
		JRequest::setVar('invoice_key', $invoiceKey, 'POST');
		JRequest::setVar('payment_key', $paymentKey, 'POST');
		
		return true;
	}
	
	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  = $invoice->getTotal();
		
		$paymentKey = $payment->getKey();
		
		$this->assign('amount', $amount);
		$this->assign('invoice', $invoice);
		$this->assign('post_url', XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&action=success&payment_key=".$paymentKey));
		$this->assign('cancel_url', XiRoute::_("index.php?option=com_payplans&gateway=paypalpro&view=payment&task=complete&action=cancel&payment_key=".$paymentKey));
		$this->assign('payment_key', $paymentKey);
		
		// Reference:-https://www.x.com/developers/community/blogs/dan_pro_vt/pro/virtual-terminal-updates-supported-country-list
		$notSupported   = array('AF','DZ','AO','AQ','BY','BO','BV','BI','KH','CM','CF','TD','CX','CU','TP','GQ','ER','ET','GT','GN','GW','HT','IQ','KE','LB','LR','NP','NG','PK','PG','ST','SL','SO','LK','SD','TJ','YE','ZW','XE');
		$countryOptions = array();
        $filter 		= array('isocode2' => array(array('NOT IN', '("'.implode('","', $notSupported).'")')));
        $items 			= XiFactory::getInstance('country', 'model')->loadRecords($filter);
        foreach($items as $country){
			$countryOptions[] = JHTML::_('select.option',$country->isocode2, PayplansHelperFormat::country($country));
		}
		
		$this->assign('countryOptions', $countryOptions);
		
	  	return $this->_render('form');
	}
	
	function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		$action = JRequest::getVar('action', false);
		if($action == 'cancel'){
			return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		}
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);

		// Set request-specific fields.
		$paymentType 			= urlencode('Sale');				// or 'Sale'
		$firstName 				= urlencode($data['first_name']);
		$lastName 				= urlencode($data['last_name']);
		$email					= urlencode($data['email']);
		$creditCardType 		= urlencode($data['cc_type']);
		$creditCardNumber 		= urlencode(trim($data['card_num']));
		$cvv2Number 			= urlencode( $data['card_code']);
		$expDateMonth 			= urlencode($data['exp_month']);
		// Month must be padded with leading zero
		$padDateMonth 			= urlencode(str_pad($expDateMonth, 2, '0', STR_PAD_LEFT));
		$expDateYear 			= urlencode($data['exp_year']);
		$address1 				= urlencode($data['address']);
		$city 					= urlencode($data['city']);
		$state 					= urlencode($data['state']);
		$zip 					= urlencode($data['zip']);
		$country				= urlencode($data['country']);				// US or other valid country code
		$currencyID 			= urlencode($invoice->getCurrency('isocode'));							// or other currency ('GBP', 'EUR', 'JPY', 'CAD', 'AUD')
		$ipAddress  			= $_SERVER['REMOTE_ADDR'];
		$custom					= $invoice->getKey().'-'.$payment->getKey();
		
		$root = JURI::root();
    	if(XiFactory::getConfig()->https == true){
    		$root = JString::str_ireplace("http:", "https:", $root);
    	}
		$notify_url  		= urlencode($root.'index.php?option=com_payplans&view=payment&task=notify&gateway=paypalpro');
		
		// Add request-specific fields to the request string.
		$nvpStr  			=	"&PAYMENTACTION=$paymentType&IPADDRESS=$ipAddress";
		$nvpStr 		   .=	"&CREDITCARDTYPE=$creditCardType&ACCT=$creditCardNumber&EXPDATE=$padDateMonth$expDateYear";
		$nvpStr 		   .=	"&CVV2=$cvv2Number&FIRSTNAME=$firstName&LASTNAME=$lastName&STREET=$address1&CITY=$city";
		$nvpStr 		   .=	"&STATE=$state&ZIP=$zip&NOTIFYURL=$notify_url&COUNTRYCODE=$country&CURRENCYCODE=$currencyID&CUSTOM=$custom&EMAIL=$email";
		
		
		$getApp = XiFactory::getApplication();
		
		$recurring = $invoice->isRecurring(); 
		if($recurring){
			$this->createRecurringProfile($invoice, $payment, $recurring, $nvpStr);
		}
		else{
			//non-recurring case
			$amount   = urlencode($invoice->getTotal());
			$nvpStr  .= "&AMT=$amount";
			// Execute the API operation; see the executePaypalApi function above.
			$httpParsedResponseAr = $this->executePaypalApi('DoDirectPayment', $nvpStr, $payment);
			
			if('FAILURE' == strtoupper($httpParsedResponseAr["ACK"])){
				$error['error_message'] = XiText::_('COM_PAYPLANS_APP_PAYPAL_PRO_DIRECT_PAYMENT_NOT_COMPLETED');
				$error['response']		= $httpParsedResponseAr;
				$error_code 			= $httpParsedResponseAr["L_ERRORCODE0"];
				$message 				= urldecode($httpParsedResponseAr["L_LONGMESSAGE0"]);
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $error, 'PayplansPaymentFormatter', '', true);
				$getApp->enqueueMessage(XiText::_($message.' ('.$error_code.')'),'error');
			  	$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
			}
		}

		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	protected function createRecurringProfile($invoice, $payment, $recurring, $nvpStr)
	{
		$paymentKey     	= urlencode($payment->getKey());
		$amount 			= urlencode($invoice->getTotal()); // Regular Amount
		$plans				= $invoice->getPlans(PAYPLANS_INSTANCE_REQUIRE);
		$plan				= array_pop($plans);
		$desc 				= urlencode($plan->getTitle());
		$now 				= new XiDate('now');
		$startDate 			= urlencode($now->toFormat("%Y-%m-%dT0:0:0", null, null, true));
		$expirationTime 	= $invoice->getExpiration(PAYPLANS_RECURRING);
		$expirationTime 	= $this->getRecurrenceTime($expirationTime);
		$billingPeriod 		= urlencode($expirationTime['unit']);
		$billingFreq 		= urlencode($expirationTime['period']);
		$billingCycle 		= urlencode($invoice->getRecurrenceCount());						// combination of this and billingPeriod must be at most a year
		
		$nvpStr		   		.= "&PROFILEREFERENCE=$paymentKey";
		$nvpStr 	  		.= "&PROFILESTARTDATE=$startDate&BILLINGPERIOD=$billingPeriod&BILLINGFREQUENCY=$billingFreq&TOTALBILLINGCYCLES=$billingCycle&DESC=$desc";
		
		if($recurring == PAYPLANS_RECURRING_TRIAL_1){
			$amount			= urlencode($invoice->getTotal($invoice->getCounter() + 1)); // Regular Amount
			$trialAmount 	= urlencode($invoice->getTotal()); // Trial Amount
			$trialTime		= $invoice->getExpiration(PAYPLANS_RECURRING_TRIAL_1);
			$trialTime 		= $this->getRecurrenceTime($trialTime);
			$trialPeriod 	= urlencode($trialTime['unit']);
			$trialFreq 		= urlencode($trialTime['period']);
			$trialCycle 	= urlencode('1');
			
			$nvpStr        .= "&TRIALBILLINGPERIOD=$trialPeriod&TRIALBILLINGFREQUENCY=$trialFreq&TRIALAMT=$trialAmount&TRIALTOTALBILLINGCYCLES=$trialCycle";
		}
		
		$nvpStr 			.= "&AMT=$amount";
		
		$httpParsedResponseAr = $this->executePaypalApi('CreateRecurringPaymentsProfile', $nvpStr, $payment);
		
		$getApp = XiFactory::getApplication();
		
		if("SUCCESS" == strtoupper($httpParsedResponseAr["ACK"]) || "SUCCESSWITHWARNING" == strtoupper($httpParsedResponseAr["ACK"])) {
			$payment->set('gateway_params', 'profile_id='.urldecode($httpParsedResponseAr['PROFILEID']))
							->save();
		}
		elseif('FAILURE' == strtoupper($httpParsedResponseAr["ACK"])){
			$error					= array();			
			$error['error_message'] = XiText::_('COM_PAYPLANS_APP_PAYPAL_PRO_RECURRING_PROFILE_REJECTED');
			$error['response']		= $httpParsedResponseAr;
			$error_code 			= $httpParsedResponseAr["L_ERRORCODE0"];
			$message 				= urldecode($httpParsedResponseAr["L_LONGMESSAGE0"]);
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $error, 'PayplansPaymentFormatter', '', true);
			$getApp->enqueueMessage(XiText::_($message.' ('.$error_code.')'),'error');
		  	$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
		}
	}
	
	public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$profileId = $payment->getGatewayParam('profile_id', 0);
		
		$nvpStr = "&PROFILEID=$profileId&ACTION=Cancel";
		
		$httpParsedResponseAr = $this->executePaypalApi('ManageRecurringPaymentsProfileStatus', $nvpStr, $payment);
		
		if("SUCCESS" == strtoupper($httpParsedResponseAr["ACK"]) || "SUCCESSWITHWARNING" == strtoupper($httpParsedResponseAr["ACK"])) {
			// get the transaction instace of lib
			$transaction = PayplansTransaction::getInstance();
			$transaction->set('user_id', $invoice->getBuyer())
						->set('invoice_id', $invoice->getId())
						->set('payment_id', $payment->getId())
						->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_RECURRING_CANCEL')	
						->set('gateway_subscr_id', $httpParsedResponseAr['PROFILEID'])
						->save();
			parent::onPayplansPaymentTerminate($payment, $controller);		
			return $this->_render('cancel_success');
		}
		elseif('FAILURE' == strtoupper($httpParsedResponseAr["ACK"])){
			$error					= array();			
			$error['error_message']	= XiText::_('COM_PAYPLANS_APP_PAYPAL_PRO_ERROR_IN_RECURRING_PROFILE_CANCELLATION');
			$error['response']		= $httpParsedResponseAr;
			$message 				= urldecode($httpParsedResponseAr["L_LONGMESSAGE0"]);
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $error, 'PayplansPaymentFormatter', '', true);
			
			return $this->_render('cancel_error');
		}
		
	}
	
	public function getRecurrenceTime($expTime)
	{
		$expTime['year'] = isset($expTime['year']) ? intval($expTime['year']) : 0;
		$expTime['month'] = isset($expTime['month']) ? intval($expTime['month']) : 0;
		$expTime['day'] = isset($expTime['day']) ? intval($expTime['day']) : 0;
		
		// if only days are set then return days as it is
		if(!empty($expTime['day'])){
			$days = $expTime['day'];
			
			if(!empty($expTime['month'])){
				$days += $expTime['month'] * 30;

				if(!empty($expTime['year'])){
					$days += $expTime['year'] * 365;
				}
			}
			return array('period' => $days, 'unit' => 'Day', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYPAL_PRO_RECURRING_MESSAGE'));
		}
		
		// if months are set
		if(!empty($expTime['month'])){
			$month = $expTime['month'];
			
			if(!empty($expTime['year'])){
				$month += $expTime['year'] * 12 ;
			}
			
			return array('period' => $month, 'unit' => 'Month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYPAL_PRO_RECURRING_MESSAGE'));
		}
		
		// years
		if(!empty($expTime['year'])){		
			return array('period' => $expTime['year'], 'unit' => 'Year', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYPAL_PRO_RECURRING_MESSAGE'));
		}
		
		return false;
	}
	
	/**
     * Payment received; source is a Buy Now, Donation, or Auction Smart Logos button
     * Process in same way
     */
    function _validateNotification(PayplansPayment $payment, array $data)
    {
    	$errors = array();

    	// find the required data from post-data, and match with payment
    	// check reciever email must be same.
    	if($this->getAppParam('merchantEmail') != $data['receiver_email']) {
            $errors[] = XiText::_('COM_PAYPLANS_INVALID_PAYPAL_PRO_RECEIVER_EMAIL');
        }

        // check payment key
    	if($payment->getKey() != $data['payment_key']) {
            $errors[] = XiText::_('COM_PAYPLANS_INVALID_PAYPAL_PRO_PAYMENT_KEY');
    	}

        return $errors;
    }
	
	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		$errors = array();
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$recurring  	 = $invoice->isRecurring();
		
		$txn_id 	= isset($data['txn_id']) ? $data['txn_id'] : 0;
    	$subscr_id  = isset($data['recurring_payment_id']) ? $data['recurring_payment_id'] : 0;
    	$parent_txn = isset($data['parent_txn_id']) ? $data['parent_txn_id'] : 0;
    	
    	$transactions = $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_txn);
    	if($transactions !== false){
    		foreach($transactions as $transaction){
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			if($recurring === false){
	    			$par = $transaction->getParam('payment_status', '');
	    			if($par == $data['payment_status']){
	    				return true;
	    			}
    			}
    			$par = $transaction->getParam('txn_type', '');
    			if($par == $data['txn_type']){
    				return true;
    			}
    		}
    	}

		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', isset($data['txn_id']) ? $data['txn_id'] : 0)
					->set('gateway_subscr_id', isset($data['recurring_payment_id']) ? $data['recurring_payment_id'] : 0)
					->set('params', PayplansHelperParam::arrayToIni($data));
					
		$func_name = '_process_web_accept';

		$func_name_rec 		= isset($data['txn_type']) ? '_process_'.JString::strtolower($data['txn_type']) : 'EMPTY';
		$func_name_nonrec 	= isset($data['payment_status']) ? '_on_payment_'.JString::strtolower($data['payment_status']) : 'EMPTY';
		
		if(method_exists($this, $func_name_rec)){
			$errors = $this->$func_name_rec($payment, $transaction, $data);
		}
		elseif(method_exists($this, $func_name_nonrec)){
			$errors = $this->$func_name_nonrec($payment, $transaction, $data);
		}
		else{
			$errors = XiText::_('COM_PAYPLANS_APP_PAYPAL_PRO_INVALID_TRANSACTION_TYPE_OR_PAYMENT_STATUS');
		}
		
    	//if error present in the transaction then redirect to error page
		if($errors){
			$message 					= XiText::_('COM_PAYPLANS_LOGGER_ERROR_IN_PAYPAL_PRO_PAYMENT_PROCESS');
			$response					= array();
			$response['error_message']	= $errors;
			$response['response']		= $data;
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		}
	
		//store the response in the payment AND save the payment
		if(!$transaction->save()){
			$message 					= XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			$response					= array();			
			$response['error_message']	= $errors;
			$response['response']		= $data;
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response,'PayplansPaymentFormatter', '', true);
		}
		
		$payment->save();
		
		return count($errors) ? implode("\n", $errors) : ' No Errors';
	}
	
	protected function _on_payment_canceled_reversal($payment, $transaction, $data)
	{
		//		Canceled_Reversal: A reversal has been canceled. For example, you
		//		won a dispute with the customer, and the funds for the transaction that was
		//		reversed have been returned to you.
		
		$transaction->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_COMPLETED');
		return array();
	}

	protected function _on_payment_completed($payment, $transaction, $data)
	{
		//		Completed: The payment has been completed, and the funds have been
		//		added successfully to your account balance.
		
		$errors = $this->_validateNotification($payment, $data);
        
		if(empty($errors)){
			$transaction->set('amount', $data['mc_gross'])
						->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_COMPLETED');
		}
		return $errors;
	}
	
	protected function _on_payment_created($payment, $transaction, $data)
	{
		//  A German ELV payment is made using Express Checkout.
		// Probably we don't need it
		$transaction->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_CREATED');
		return array();
	}
	
	protected function _on_payment_expired($payment, $transaction, $data)
	{
		//		This authorization has expired and cannot be captured.
		$transaction->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_EXPIRED');
		return array();
	}
	
	protected function _on_payment_failed($payment, $transaction, $data)
	{
		//		The payment has failed. This happens only if the payment was
		//		made from your customer’s bank account.
		$transaction->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_FAILED');
		return array();		
	}
	
	protected function _on_payment_pending($payment, $transaction, $data)
	{
		//		The payment is pending. See pending_reason for more information.
		$transaction->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_PENDING');
		return array();
	}
	
	protected function _on_payment_refunded($payment, $transaction, $data)
	{
		//		Refunded: You refunded the payment.
		//		What to do on partial refund 
		// check the stored amount against the payment amount
       $transaction->set('amount', $data['mc_gross'])
					->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_REFUNDED');
		return array();
	}
	
	protected function _on_payment_reversed($payment, $transaction, $data)
	{
		//		Reversed: A payment was reversed due to a chargeback or other type of
		//		reversal. The funds have been removed from your account balance and
		//		returned to the buyer. The reason for the reversal is specified in the
		//		ReasonCode element.
		
		$transaction->set('amount', $data['mc_gross'])
					->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_REVERSED');
		return array();	
	}
	
	protected function _on_payment_processed($payment, $transaction, $data)
	{
		//		Processed: A payment has been accepted.
		$transaction->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_PROCESSED');
		return array();	
	}

	protected function _process_recurring_payment_skipped($payment, $transaction, $data)
	{
		//Recurring payment skipped; it will be retried up to a total of 3 times, 5 days apart
		$transaction->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_PAYMENT_SKIPPED');
		return array();
	}
	
	protected function _process_recurring_payment_profile_created($payment, $transaction, $data)
	{
		$transaction->set('amount', 0);
		$transaction->set('message', 'COM_PAYPLANS_APP_PAYPAL_TRANSACTION_PROFILE_CREATED');
	
		//if free trail then change the invoice status to paid
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount = number_format($invoice->getTotal(),2);
		return array();
	}
	
	protected function _process_recurring_payment($payment, $transaction, $data)
	{
		$transaction->set('amount', $data['mc_gross'])
						->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_COMPLETED');
						
		return array();
	}
	
	protected function _process_recurring_payment_profile_cancel($payment, $transaction, $data)
	{
		$transaction->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_RECURRING_PROFILE_CANCEL');
		return array();
	}
	
	protected function _process_recurring_payment_profile_modified($payment, $transaction, $data)
	{
		// XITODO : what to do here
	}
	
	protected function _process_recurring_payment_failed($payment, $transaction, $data)
	{
		$transaction->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_RECURRING_PAYMENT_FAILED');
		return array();
	}
	
	protected function _process_recurring_payment_expired($payment, $transaction, $data)
	{
		$transaction->set('message', 'COM_PAYPLANS_APP_PAYPAL_PRO_TRANSACTION_RECURRING_PAYMENT_EXPIRED');
		return array();
	}
	
}
