<?php
/**
* @copyright	Copyright (C) 2009 - 2016 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Payuturkey
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();


class PayplansAppPayuturkey extends PayplansAppPayment
{
	protected $_location = __FILE__;

	public function isSupportPaymentCancellation($invoice)
	{
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}

	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		self::loadPayuturkeyLibrary();
		
		
		$input = PayplansFactory::getApplication()->input;
		
		//if some error occured when click on buy but response is not successful then show error msg
    	$error_code = $input->get('error_code');
    	$error_msg  = $input->get('error_msg', '', 'string');
    	$error_html = '';
    	if(isset($error_code) && isset($error_msg)){
    		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
    		$this->assign('error_code', $error_code);
    		$this->assign('error_msg',  $error_msg);
    		$this->assign('invoice',  	$invoice);
    		$error_html =  $this->_render('buying_error');
    	}
		
		$paymentKey  = $payment->getKey();
		$this->assign('paymentKey', $paymentKey);
		$this->assign('publicKey',	$this->getAppParam('public_key',false));
		$this->assign('post_url', 	JURI::root()."index.php?option=com_payplans&view=payment&task=complete&action=process&payment_key=$paymentKey");

		return $error_html.$this->_render('buyer_details');
	}

	function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{	
		if($action == 'cancel'){
			return true;
		}

		// For 3D Secure cards
		if(isset($data['REFNO'])){
			$response = $this->_payuTurkeyProcess3DSecurePayment($payment, $data);
			if($response == true){
				$url = XiRoute::_('index.php?option=com_payplans&view=payment&task=complete&payment_key='.$payment->getKey());
				XiFactory::getApplication()->redirect($url);
			}
		}

		if($action == 'process'){
		
			$url = $this->payuTurkeyProcessPayment($payment,$data);
			
			if($url === true)
			$url = XiRoute::_('index.php?option=com_payplans&view=payment&task=complete&payment_key='.$payment->getKey());
			
			XiFactory::getApplication()->redirect($url);
		}
		
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}


	public function payuturkeyProcessPayment(&$payment,$data,$invoiceCount=0)
	{
		//all variables are handeled here
		self::loadPayuturkeyLibrary();
		
		$invoice 	= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  	= $invoice->getTotal($invoiceCount);
		$buyer		= $payment->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		
		if ($amount == 0){
			
			$response['failure_code']		=	0;
			$response['failure_message']	= 	XiText::_('COM_PAYPLANS_APP_PAYUTURKEY_ERROR_INVALID_AMOUNT_PROVIDED');
		}
		else {
			$errors['error_code']	  = "";
			$errors['error_message']  = "";
	
			$root = JURI::root();
			if(XiFactory::getConfig()->https == true){
			    $root         = JString::str_ireplace("http:", "https:", $root);
			}
	
			$url = $root.'index.php?option=com_payplans&view=payment&task=complete&payment_key='.$payment->getKey();

			//then i need to create charge to this customer
			//$cfg = new \PayU\Alu\MerchantConfig('MERCHANT_CODE', 'SECRET_KEY', 'RO');
			$merchant_code  = $this->getAppParam('merchant_key');
			$secret_key     = $this->getAppParam('secret_key');
			$cfg 			= new \PayU\Alu\MerchantConfig($merchant_code, $secret_key, 'TR');
			$user 			= new \PayU\Alu\User('127.0.0.1');
			
			$order 		= new \PayU\Alu\Order();
			$orderRef 	= time()."_".$payment->getKey();
			$order->withBackRef($url)
			->withOrderRef($orderRef)
			->withCurrency('TRY')
			->withOrderDate(gmdate('Y-m-d H:i:s'))
			->withOrderTimeout(1000)
			->withPayMethod('CCVISAMC');
				
			$product = new \PayU\Alu\Product();
			$product->withCode($payment->getKey())
			->withName($invoice->getTitle())
			->withPrice($amount)
			->withVAT(0.0)
			->withQuantity(1);			
			
			$order->addProduct($product);
			
			// Create new billing address
			$billing = new \PayU\Alu\Billing();
			$billing->withAddressLine1('')
			->withAddressLine2('')
			->withCity('City')
			->withCountryCode('TR')
			->withEmail($buyer->getEmail())
			->withFirstName($buyer->getEmail())
			->withLastName($buyer->getEmail())
			->withPhoneNumber('1234567895')
			->withIdentityCardNumber('');

			// Create new delivery address
			$delivery = new \PayU\Alu\Delivery();


			$card = new \PayU\Alu\Card($data['credit-card'], $data['card-expiry-month'], $data['card-expiry-year'], $data['cvc-length'], $data['card-owner']);
			$card->enableTokenCreation();
			
			$request = new \PayU\Alu\Request($cfg, $order, $billing, $delivery, $user);
			$request->setCard($card);

			$client = new \PayU\Alu\Client($cfg);
			
			
			try {
			   
			    $response = $client->pay($request);
				//var_dump($response);die;
			    if ($response->isThreeDs()) {
			        header("Location:" . $response->getThreeDsUrl());
			        die();
			    }
			    			   
			    $result 	= $this->_payuTurkeyProcessPayment($payment, $response);   
			    return true;
			
			
			} catch (ConnectionException $exception) {
			   	$username	= $buyer->getUsername();
				$userId		= $buyer->getId();
				//It is needed to create a log for wrong response
				$errors['error_code']	  = $exception->getCode();
				$errors['error_message']  = sprintf(XiText::_('COM_PAYPLANS_APP_PAYUTURKEY_LOGGER_ERROR_IN_PAYUTURKEY_PAYMENT_PROCESS_DETAILS'),$exception->getMessage(),$username,$userId,$invoice->getKey());
				
				$message = XiText::_('COM_PAYPLANS_APP_PAYUTURKEY_LOGGER_ERROR_IN_PAYUTURKEY_RESPONSE_INVALID');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment,$errors, 'PayplansPaymentFormatter', '', true);
				$errors['error_message']  = $exception->getMessage();
				
			} catch (ClientException $exception) {
			    $username	= $buyer->getUsername();
				$userId		= $buyer->getId();
				//It is needed to create a log for wrong response
				$errors['error_code']	  = $exception->getCode();
				$errors['error_message']  = sprintf(XiText::_('COM_PAYPLANS_APP_PAYUTURKEY_LOGGER_ERROR_IN_PAYUTURKEY_PAYMENT_PROCESS_DETAILS'),$exception->getMessage(),$username,$userId,$invoice->getKey());
				
				$message = XiText::_('COM_PAYPLANS_APP_PAYUTURKEY_LOGGER_ERROR_IN_PAYUTURKEY_RESPONSE_INVALID');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment,$errors, 'PayplansPaymentFormatter', '', true);
				$errors['error_message']  = $exception->getMessage();
			}
			

		}
		

		$url = "index.php?option=com_payplans&view=payment&task=pay&payment_key=".$payment->getKey()
				."&error_code=".$errors['error_code']."&error_msg=".urlencode($errors['error_message']); 

		return XiRoute::_($url);
	}

	
	public function loadPayuturkeyLibrary()
	{

		$path = dirname(__FILE__).DS.'lib'.DS;
		include_once ($path."payuturkey.php");
	}
	
	
	
	public function _payuTurkeyProcessPayment(PayplansPayment $payment, $data)
	{
		$invoice 		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$gatewayTxn_id 	= $data->getRefno();  
	
    	// if same notification then epg gateway id is same as previous one then check if transaction already exists ,if yes then do nothing and return
    	$transactions = $this->_getExistingTransaction($invoice->getId(), $gatewayTxn_id, 0, 0);
    	if(!empty($transactions)){
    			return true;
    	}
    	
    	$user		= $payment->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
    	$userId		= $user->getId();
    	$invoiceId 	= $invoice->getId();
    	
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id',$userId)
					->set('invoice_id',$invoiceId)
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $gatewayTxn_id)
					->set('gateway_subscr_id', $data->getTokenHash())
					->set('gateway_parent_txn', 0)
					->set('params', PayplansHelperParam::arrayToIni($data->getResponseParams()));
		
		// if response code is 0 then transaction is successful
	    if($data->getStatus() == 'SUCCESS'){
	    	$transaction->set('amount',$data->getAmount())
	    				->set('message', 'COM_PAYPLANS_APP_PAYUTURKEY_TRANSACTION_COMPLETED')
	    				->save();

	    	$recurringType  = $invoice->isRecurring();
	    	if(recurringType){
				$recurrence_count = $invoice->getRecurrenceCount();	

				switch ($recurringType)
				{
					case PAYPLANS_RECURRING_TRIAL_1: 	$recurrence_count++;break;
					case PAYPLANS_RECURRING_TRIAL_2:	$recurrence_count = $recurrence_count+2;break;
				}			
			}
			else {
				$recurrence_count = 0;
			}
				
				
			$payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
			$payment->getGatewayParams()->set('TOKEN_HASH',$data->getTokenHash());
			$payment->save();
	    	//XiFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=complete&payment_key='.$payment->getKey()));
		}		
		else
		{
			$transaction->set('amount',0)
						->set('message', 'COM_PAYPLANS_APP_PAYUTURKEY_TRANSACTION_NOT_COMPLETED')
	    				->save();
	    			
	    	$transactionKey = PayplansHelperUtils::getKeyFromId($transaction->getId());
	    	$invoiceKey		= PayplansHelperUtils::getKeyFromId($invoice->getId());
	    	
	    				
			$message = XiText::_('COM_PAYPLANS_APP_PAYUTURKEY_LOGGER_ERROR_IN_PAYUTURKEY_PAYMENT_PROCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
			
			return false;	
		}

		return true;
	}

	public function _payuTurkeyProcess3DSecurePayment(PayplansPayment $payment, $data)
	{
		$invoice 		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$gatewayTxn_id 	= $data['REFNO'];  
	
    	// if same notification then epg gateway id is same as previous one then
    	// check if transaction already exists
    	// if yes then do nothing and return
    	//$transactions = $this->_getExistingTransaction($invoice->getId(), $gatewayTxn_id, 0, 0);
    	//if(!empty($transactions)){
    			//	return true;
    	//}
    	
    	$user		= $payment->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
    	$userId		= $user->getId();
    	$invoiceId 	= $invoice->getId();
    	
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id',$userId)
					->set('invoice_id',$invoiceId)
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $gatewayTxn_id)
					->set('gateway_subscr_id', $data['HASH'])
					->set('gateway_parent_txn', 0)
					->set('params', PayplansHelperParam::arrayToIni($data));
		
		// if response code is 0 then transaction is successful
	    if($data['STATUS'] == 'SUCCESS'){
	    	$transaction->set('amount',$data['AMOUNT'])
	    				->set('message', 'COM_PAYPLANS_APP_PAYUTURKEY_TRANSACTION_COMPLETED')
	    				->save();

	    	$recurringType  = $invoice->isRecurring();
	    	if(recurringType){
				$recurrence_count = $invoice->getRecurrenceCount();	

				switch ($recurringType)
				{
					case PAYPLANS_RECURRING_TRIAL_1: 	$recurrence_count++;break;
					case PAYPLANS_RECURRING_TRIAL_2:	$recurrence_count = $recurrence_count+2;break;
				}			
			}
			else {
				$recurrence_count = 0;
			}
				
				
			$payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
			$payment->getGatewayParams()->set('TOKEN_HASH',$data['HASH']);
			$payment->save();
	    	//XiFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=complete&payment_key='.$payment->getKey()));
		}		
		else
		{
			$transaction->set('amount',0)
						->set('message', 'COM_PAYPLANS_APP_PAYUTURKEY_TRANSACTION_NOT_COMPLETED')
	    				->save();
	    			
	    	$transactionKey = PayplansHelperUtils::getKeyFromId($transaction->getId());
	    	$invoiceKey		= PayplansHelperUtils::getKeyFromId($invoice->getId());
	    	
	    				
			$message = XiText::_('COM_PAYPLANS_APP_PAYUTURKEY_LOGGER_ERROR_IN_PAYUTURKEY_PAYMENT_PROCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
			
			return false;	
		}

		return true;
	}
	
	
	public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		$lifetime = false;
		
		$lifetime = ($invoice->getRecurrenceCount() == 0)? true : false;
		
		$invoice_count = $invoiceCount +1;
			
		if($invoice->isRecurring())
		{			
			$recurrence_count 	= $payment->getGatewayParam('pending_recur_count');
			
			if($recurrence_count > 0 || $lifetime)
			{
				$this->payuTurkeyRecurringPayment($payment, array('process_payment'=>true),$invoice_count);	
			}
		}
	}

	public function payuTurkeyRecurringPayment(&$payment,$data,$invoiceCount=0)
	{
		$invoice 			= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$invoice_count 		= $invoiceCount;
		$recurrence_count 	= $payment->getGatewayParam('pending_recur_count');
		$amount 			= $invoice->getTotal($invoice_count);
		$buyer				= $payment->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
			
		$root = JURI::root();
		if(XiFactory::getConfig()->https == true){
			$root         = JString::str_ireplace("http:", "https:", $root);
		}
		
		self::loadPayuturkeyLibrary();
		$token = $payment->getGatewayParam('TOKEN_HASH');
		$url = $root.'index.php?option=com_payplans&view=payment&task=complete&payment_key='.$payment->getKey();

		//then i need to create charge to this customer
		//$cfg = new \PayU\Alu\MerchantConfig('MERCHANT_CODE', 'SECRET_KEY', 'RO');
		$merchant_code  = $this->getAppParam('merchant_key');
		$secret_key     = $this->getAppParam('secret_key');
		$cfg 			= new \PayU\Alu\MerchantConfig($merchant_code, $secret_key, 'TR');
		$user 			= new \PayU\Alu\User('127.0.0.1');
		
		$order 		= new \PayU\Alu\Order();
		$orderRef 	= time()."_".$payment->getKey();
		$order->withBackRef($url)
			  ->withOrderRef($orderRef)
			  ->withCurrency('TRY')
			  ->withOrderDate(gmdate('Y-m-d H:i:s'))
			  ->withOrderTimeout(1000)
			  ->withPayMethod('CCVISAMC')
			  ->withInstallmentsNumber($invoice_count);
		
		$product = new \PayU\Alu\Product();
		$product->withCode($invoice->getKey())
				->withName($invoice->getTitle())
				->withPrice($amount)
				->withVAT(0.0)
				->withQuantity(1);			

		$order->addProduct($product);

		// Create new billing address
		$billing = new \PayU\Alu\Billing();
		$billing->withAddressLine1('')
				->withAddressLine2('')
				->withCity('City')
				->withCountryCode('TR')
				->withEmail($buyer->getEmail())
				->withFirstName($buyer->getEmail())
				->withLastName($buyer->getEmail())
				->withPhoneNumber('1234567895')
				->withIdentityCardNumber();

		// Create new delivery address
		$delivery = new \PayU\Alu\Delivery();

		$cardToken 	= new \PayU\Alu\CardToken($token);

		$request = new \PayU\Alu\Request($cfg, $order, $billing, $delivery, $user);
		$request->setCardToken($cardToken);

		$client = new \PayU\Alu\Client($cfg);

		try {
		   
		    $response = $client->pay($request);
				// When debug mode is on dump in file
			//	file_put_contents(JPATH_SITE.DS.'tmp'.DS.time(), var_export($response->getResponseParams(),true), FILE_APPEND);

		    if ($response->isThreeDs()) {
		        header("Location:" . $response->getThreeDsUrl());
		        die();
		    }


		    			   
		    $result 	= $this->_payuTurkeyProcessRecurringPayment($payment, $response, $recurrence_count, $token);
		    
		    return true;
							
		} catch (ConnectionException $exception) {
		   	$username	= $buyer->getUsername();
			$userId		= $buyer->getId();
			//It is needed to create a log for wrong response
			$errors['error_code']	  = $exception->getCode();
			$errors['error_message']  = sprintf(XiText::_('COM_PAYPLANS_APP_PAYUTURKEY_LOGGER_ERROR_IN_PAYUTURKEY_PAYMENT_PROCESS_DETAILS'),$exception->getMessage(),$username,$userId,$invoice->getKey());
			
			$message = XiText::_('Invaid Payuturkey');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment,$errors, 'PayplansPaymentFormatter', '', true);
			$errors['error_message']  = $exception->getMessage();
			
		} catch (ClientException $exception) {
		    $username	= $buyer->getUsername();
			$userId		= $buyer->getId();
			//It is needed to create a log for wrong response
			$errors['error_code']	  = $exception->getCode();
			$errors['error_message']  = sprintf(XiText::_('COM_PAYPLANS_APP_PAYUTURKEY_LOGGER_ERROR_IN_PAYUTURKEY_PAYMENT_PROCESS_DETAILS'),$exception->getMessage(),$username,$userId,$invoice->getKey());
			
			$message = XiText::_('Invaid Payuturkey');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment,$errors, 'PayplansPaymentFormatter', '', true);
			$errors['error_message']  = $exception->getMessage();
		}

	}

	public function _payuTurkeyProcessRecurringPayment(PayplansPayment $payment, $data, $recurrence_count, $token)
	{
		$invoice 		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$gatewayTxn_id 	= $data->getRefno(); 
		//$gatewaysub_id  = $data->tokenHash();
	
    	// if same notification then epg gateway id is same as previous one then
    	// check if transaction already exists
    	// if yes then do nothing and return
    	$transactions = $this->_getExistingTransaction($invoice->getId(), $gatewayTxn_id, 0, 0);
    	if(!empty($transactions)){
    				return true;
    	}
    	
    	$user		= $payment->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
    	$userId		= $user->getId();
    	$invoiceId 	= $invoice->getId();
					
		
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $gatewayTxn_id)
					->set('gateway_subscr_id', $token)
					->set('gateway_parent_txn', $token)
					->set('params', PayplansHelperParam::arrayToIni($data->getResponseParams()));


		 if($data->getStatus() == 'SUCCESS'){
	    	$transaction->set('amount',$data->getAmount())
	    				->set('message', XiText::_('COM_PAYPLANS_APP_PAYUTURKEY_TRANSACTION_COMPLETED'))
	    				->save();

	    	// IMP: if payment arrives after certain failures then reset the failure counter
			if($recurrence_count != 0){
				$recurrence_count 	= $recurrence_count - 1;
				$payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
			}
			$payment->getGatewayParams()->set('TOKEN_HASH', $token);
			$payment->save();
	    }else {
	    	$transaction->set('amount',0)
	    				->set('message', XiText::_('COM_PAYPLANS_APP_PAYUTURKEY_TRANSACTION_NOT_COMPLETED'))
	    				->save();
	    }
					
		return true;
	}

	public function onPayplansPaymentTerminate(PayplansPayment $payment, $invoiceController) 
	{
		parent::onPayplansPaymentTerminate($payment, $invoiceController);
		return $this->_render('subscription_cancel'); 
	}

	
}