<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Pin
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();


class PayplansAppPin extends PayplansAppPayment
{
	protected $_location = __FILE__;

	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
    	$paymentKey	= $payment->getKey();
		$invoice 	= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
    	$amount  	= $invoice->getTotal();
    	$currency	= $this->getAppParam('currency', 'AUD');
    	$buyer		= $invoice->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
    	$email		= $buyer->getEmail();
		
		$this->assign('amount', $amount);
		$this->assign('currency', $currency);
		$this->assign('paymentKey', $paymentKey);
		$this->assign('email', $email);
		$this->assign('post_url', 	XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&payment_key=$paymentKey"));

		return $this->_render('form');
	}

	function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{		
		if($action == 'cancel'){
			return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		}
		
		$invoice 	= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$token 		= $this->createCustomer($data, $invoice, $payment);
		$email 		= $data['email'];
		
		// If there occur any error then array will appear in $token else $token will be a string....
		if(is_array($token)){
			$message = array_shift($token);
			// redirect to payment page again
			$app = PayplansFactory::getApplication();
			$app->enqueueMessage($message,'error');
			$app->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
			return true;
		}
		
		$recurrence = 0;
		if($invoice->isRecurring()){
			$recurrence = $this->getRecurrenceCount($invoice);
		}
		// Do charge the customer
		$token = $this->createCharge($token, $email, $invoice, $payment, $recurrence);
		
		// If there occur any error then array will appear in $token else $token will be a string....
		if(is_array($token)){
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_PIN_LOGGER_DO_CHARGE_ERROR');
			// redirect to payment page again
			$app = PayplansFactory::getApplication();
			$app->enqueueMessage($message,'error');
			$app->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
			return true;
		}
		
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$invoice_count = $invoiceCount + 1;
		
		if($invoice->isRecurring())
		{
			$recurrence = $payment->getGatewayParam('pending_recur_count', 0);
			$email 		= $payment->getGatewayParam('customer_email', false);
			$token 		= $payment->getGatewayParam('customer_token', false);
			
			if(!$email || !$token){
				$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_PIN_LOGGER_CHARGES_DATA_MISSING');
				$errors['error_code']	 = $response['error'];
				$errors['error_message'] = $message;
				
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment,$errors, 'PayplansPaymentFormatter', '', true);
				return true;
			}
			
			if($recurrence > 0){
				$this->createCharge($token, $email, $invoice, $payment, $recurrence, $invoice_count);
			}	
		}
		return true;
	}
	
	private function createCustomer($data, $invoice, $payment)
	{
		list($url, $str) = $this->prepareCustomerNameValuePair($data);
		$secret_key 	 = $this->getSecretKey();
		$user_id		 = $invoice->getBuyer();
		
		$response = $this->execute($url, $str, $secret_key);
		if(isset($response['error'])){
			return $this->createErrorLog($invoice, $payment, $user_id, $response, 'COM_PAYPLANS_PAYMENT_APP_PIN_LOGGER_CUSTOMER_CREATION_ERROR');
		}
		
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id',$user_id)
					->set('invoice_id',$invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', 0)
					->set('gateway_subscr_id', $response['response']['token']) // IMP : store customer_id
					->set('gateway_parent_txn', 0)
					->set('message', 'COM_PAYPLANS_PAYMENT_APP_PIN_TRANSACTION_CUSTOMER_CREATED_SUCCESSFULLY')
					->set('params', PayplansHelperParam::arrayToIni($response))
					->save();
		
		// IMP: user can provide different email address during payment processing. Store that email address and use that for further payment processing
		$payment->getGatewayParams()->set('customer_email', $response['response']['email']);
		$payment->getGatewayParams()->set('customer_token', $response['response']['token']);
		$payment->save();
		
		return $response['response']['token'];
	}
	
	private function createCharge($token, $email, $invoice, $payment, $recurrence = 0, $invoice_count = 1)
	{
		$amount 	= $invoice->getTotal() * 100; // IMP: convert into cents
		$title 		= $invoice->getTitle();
		$ipaddress	= $_SERVER['SERVER_ADDR'];
		$recurring  = $invoice->isRecurring();
		
		if($recurring){
			$amount = $invoice->getTotal($invoice_count) * 100; // IMP: convert into cents
		}
		
		list($url, $str) = $this->prepareChargesNameValuePair($amount, $title, $email, $ipaddress, $token);
		$secret_key 	 = $this->getSecretKey();
		$user_id		 = $invoice->getBuyer();
		
		$response = $this->execute($url, $str, $secret_key);
		if(isset($response['error'])){
			return $this->createErrorLog($invoice, $payment, $user_id, $response, 'COM_PAYPLANS_PAYMENT_APP_PIN_LOGGER_DO_CHARGE_ERROR');
		}
		
		// if same notification came more than one time
		// Check if transaction already exists,if yes then do nothing and return
		$txn_id         = isset($response['response']['token']) ? $response['response']['token'] :0;
	    $subscr_id 		= isset($token) ? $token : 0;
		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, 0);
    	if (!empty($transactions)) {
    		foreach ($transactions as $transaction) {
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			$resp = $transaction->getParam('response', new stdClass());
    			if(strtolower($resp->status_message) == strtolower($response['response']['status_message'])){
    					return true;
    			}
    		}
    	}
		
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id',$user_id)
					->set('invoice_id',$invoice->getId())
					->set('amount', ($response['response']['amount'] / 100)) // IMP: convert cent into amount
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $txn_id) // IMP : store charge token
					->set('gateway_subscr_id', $token) // IMP : store customer_id
					->set('gateway_parent_txn', 0)
					->set('message', 'COM_PAYPLANS_PAYMENT_APP_PIN_TRANSACTION_COMPLETED_SUCCESSFULLY')
					->set('params', PayplansHelperParam::arrayToIni($response))
					->save();
		
		if($recurring){
			$recurrence = $recurrence - 1;
			$payment->getGatewayParams()->set('pending_recur_count', $recurrence);
			$payment->save();
		}
		
		
		return true;
	}
	
	private function getRecurrenceCount(PayplansInvoice $invoice)
	{
		$count = $invoice->getRecurrenceCount();
		if(intval($count) === 0){
			return 9999;
		}
		
		$recurring = $invoice->isRecurring();
		if($recurring){
			
			// Recurrence Count For Regular Recurring Plan
       		if($recurring == PAYPLANS_RECURRING){
	       		$recurrence_count = $invoice->getRecurrenceCount();
       		}
       		// Recurrence Count For Recurring + Trial 1 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_1){
				$recurrence_count = $invoice->getRecurrenceCount() + 1;
       		}
       		// Recurrence Count For Recurring + Trial 2 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_2){
	       		$recurrence_count = $invoice->getRecurrenceCount() + 2;
       		}
		}
		return $recurrence_count;
	}
	
	private function createErrorLog($invoice, $payment, $user_id, $response, $message)
	{
		$errors	= array();
		$errors['error_code']	= $response['error'];
		$errors['error_desc']  	= $response['error_description'];
		$errors['message'] 		= '';
		$messages 				= $response['messages'];
		$data					= ''; 
		
		foreach ($messages as $msz){
			$data .= '<b>Code Name</b>:- '. $msz['code'] . '<br/>' .'<b>Message</b>:- ' . $msz['message'] . '<br/>';
			$errors['message'] .= $msz['message'] . ', ';
		}
		
		PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, XiText::_($errors['message']), $payment,$errors. 'PayplansPaymentFormatter', '', true);
		
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id',$user_id)
					->set('invoice_id',$invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', 0)
					->set('gateway_subscr_id', 0)
					->set('gateway_parent_txn', 0)
					->set('message', $message)
					->set('params', PayplansHelperParam::arrayToIni($response));
		
		$transaction->save();
		return array($data);
	}
	
	private function prepareCustomerNameValuePair($data)
	{
		$nvp_str  = '';
		$nvp_str .= 'card[number]='.$data['card-number'].'&';
		$nvp_str .= 'card[cvc]='.$data['card-cvc'].'&';
		$nvp_str .= 'card[expiry_month]='.$data['card-expiry-month'].'&';
		$nvp_str .= 'card[expiry_year]='.$data['card-expiry-year'].'&';
		$nvp_str .= 'card[name]='.$data['card-name'].'&';
		$nvp_str .= 'email='.$data['email'].'&';
		$nvp_str .= 'card[address_line1]='.$data['address'].'&';
		$nvp_str .= 'card[address_city]='.$data['city'].'&';
		$nvp_str .= 'card[address_state]='.$data['state'].'&';
		$nvp_str .= 'card[address_country]='.$data['country'].'&';
		$nvp_str .= 'card[address_postcode]='.$data['zip'].'&';
		$nvp_str .= 'card[address_line2]=';
		
		$url = $this->getApiUrl('customers');
		
		return array($url, $nvp_str);
	}
	
	private function prepareChargesNameValuePair($amount, $title, $email, $ipaddress, $customer_token)
	{
		$currency = $this->getAppParam('currency', 'AUD');
		$nvp_str  = '';
		$nvp_str .= 'amount='.intval($amount).'&';
		$nvp_str .= 'currency='.$currency.'&';
		$nvp_str .= 'description='.urlencode($title).'&';
		$nvp_str .= 'email='.$email.'&';
		$nvp_str .= 'ip_address='.$ipaddress.'&';
		$nvp_str .= 'customer_token='.$customer_token;
		
		$url = $this->getApiUrl('charges');
		
		return array($url, $nvp_str);
	}	
	
	private function getApiUrl($type = 'charges')
	{
		$api_url = "https://api.pin.net.au/1/$type";
		if($this->getAppParam('sandbox', false)){
			$api_url = "https://test-api.pin.net.au/1/$type";
		}
		
		return $api_url;
	}
	
	private function getSecretKey()
	{
		if($this->getAppParam('sandbox', false)){
			return $this->getAppParam('sandbox_secret_key', false);
		}
		
		return $this->getAppParam('secret_key', false);
	}
	
	private function execute($url, $nvp_str, $secret_key)
	{
 		$ch = curl_init();
 		curl_setopt($ch, CURLOPT_URL,$url);
 		curl_setopt($ch, CURLOPT_POST, 1);
 		curl_setopt($ch, CURLOPT_POSTFIELDS, $nvp_str);
 		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
 		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
 		curl_setopt($ch, CURLOPT_USERPWD, "$secret_key:");
 		curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
 		
 		$response = curl_exec($ch);
 		curl_close($ch);
           
        $result = json_decode($response, true);
        return $result;
	}
	
	public function getRecurrenceTime($expTime)
	{
		$expTime['year'] = isset($expTime['year']) ? intval($expTime['year']) : 0;
		$expTime['month'] = isset($expTime['month']) ? intval($expTime['month']) : 0;
		$expTime['day'] = isset($expTime['day']) ? intval($expTime['day']) : 0;
		$expTime['hour'] = isset($expTime['hour']) ? intval($expTime['hour']) : 0;
		
		// if only days are set then return days as it is
		if(!empty($expTime['hour'])){
			$hours = $expTime['hour'];
			
			if(!empty($expTime['day'])){
				$hours += $expTime['day'] * 24;
				
				if(!empty($expTime['month'])){
					$hours += $expTime['month'] * 30 * 24;
	
					if(!empty($expTime['year'])){
						$hours += $expTime['year'] * 365 * 24;
					}
				}
			}
			return array('period' => $hours, 'unit' => 'Hour', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PIN_RECURRING_MESSAGE'));
		}
		
		
		// if only days are set then return days as it is
		if(!empty($expTime['day'])){
			$days = $expTime['day'];
			
			if(!empty($expTime['month'])){
				$days += $expTime['month'] * 30;

				if(!empty($expTime['year'])){
					$days += $expTime['year'] * 365;
				}
			}
			return array('period' => $days, 'unit' => 'Day', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PIN_RECURRING_MESSAGE'));
		}
		
		// if months are set
		if(!empty($expTime['month'])){
			$month = $expTime['month'];
			
			if(!empty($expTime['year'])){
				$month += $expTime['year'] * 12 ;
			}
			
			return array('period' => $month, 'unit' => 'Month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PIN_RECURRING_MESSAGE'));
		}
		
		// years
		if(!empty($expTime['year'])){		
			return array('period' => $expTime['year'], 'unit' => 'Year', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PIN_RECURRING_MESSAGE'));
		}
		
		return false;
	}

	public function isSupportPaymentCancellation($invoice)
	{
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}

	public function onPayplansPaymentTerminate(PayplansPayment $payment, $invoiceController) 
	{
		parent::onPayplansPaymentTerminate($payment, $invoiceController);
		return $this->_render('subscription_cancel'); 
	}
}
