<?php
/**
 * Script File to run on installation/unintallation/upgrade
 * @copyright Copyright (C) 2009 - 2014 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
 * @license GNU/GPL, see LICENSE.php
 * @package Payplans
 * @subpackage	PlanAddons
 * @version 3.1.2
 * @author Mohit Agrawal <mohit@readybytes.in>
 */

class plgPayplansPlanaddonsInstallerScript
{

	public function install(JAdapterInstance $adapter) {

		//create tmp table with new fields
		return $this->createTable(true);
	}

	public function update(JAdapterInstance $adapter) {

		return $this->install($adapter);
		
	}
	
	//this function will be used at the time of migration of planaddons data, there is case of any field change in it table
	public function xupdate(JAdapterInstance $adapter) {
		
		//create tmp table with new fields
		$this->dropTable('`#__payplans_tmpplanaddons`');
		$this->dropTable('`#__payplans_tmpplanaddons_stats`');

		$this->createTable();
		
		//migrate data from previous table to this new table
		$this->migrationData();
		
		//drop previous table
		$this->dropTable('`#__payplans_planaddons`');
		$this->dropTable('`#__payplans_planaddons_stats`');
		
		//rename the new table to actual name		
		$this->renameTable('`#__payplans_tmpplanaddons`', '`#__payplans_planaddons`');
		$this->renameTable('`#__payplans_tmpplanaddons_stats`', '`#__payplans_planaddons_stats`' );
		
		return true;
		
	}
	
	public function createTable($new = false)
	{
			$firstTable 	= "`#__payplans_tmpplanaddons`";
			$secondTable 	= "`#__payplans_tmpplanaddons_stats`";
		
			if ( $new ) {
				$firstTable 	= "`#__payplans_planaddons`";
				$secondTable 	= "`#__payplans_planaddons_stats`";
			}
			
			$sql = "CREATE TABLE IF NOT EXISTS $firstTable (
						  `planaddons_id` 	INT NOT NULL AUTO_INCREMENT,
						  `title`		   					  	VARCHAR(255) ,
						  `description`				TEXT ,
						  `price`	  	 					DECIMAL(15,5) DEFAULT 0.00000,
						  `consumed`					INT,
						  `addons_condition`	INT,
						  `price_type`			  		TINYINT(1) DEFAULT 0,
						  `apply_on`  					TINYINT(1) DEFAULT 0,
						  `plans`		   					VARCHAR(255) ,
						  `start_date`     			DATETIME ,
						  `end_date`       			DATETIME ,
						  `published`	   				TINYINT(1) ,
						  `params`         				TEXT ,
  						   PRIMARY KEY 			(`planaddons_id`)
						)
						ENGINE = MyISAM
						DEFAULT CHARACTER SET = utf8
						AUTO_INCREMENT=1;";
			
			$dbo = JFactory::getDBO();
			$dbo->setQuery($sql);
			$dbo->query();
			
			$sql = "CREATE TABLE IF NOT EXISTS $secondTable (
						  `planaddons_stats_id` 	INT NOT NULL AUTO_INCREMENT,
						  `user_id`	   					INT,
						  `planaddons_id`					INT NOT NULL,
						  `title`	   					  		VARCHAR(255),
						  `price`	  	 					DECIMAL(15,5) DEFAULT 0.00000,
						  `addons_condition`	INT,
						  `price_type`			  		TINYINT(1) DEFAULT 0,
						  `reference`	   				INT,
						  `status`					  		INT,
						  `purchase_date`     	DATETIME ,
						  `params`         				TEXT ,
  						   PRIMARY KEY 			(`planaddons_stats_id`)
						)
						ENGINE = MyISAM
						DEFAULT CHARACTER SET = utf8
						AUTO_INCREMENT=1;";
			
			$dbo->setQuery($sql);
			return $dbo->query();			
	}
	
	public function migrationData($from = '', $to = '')
	{
		$from 	= '#__payplans_planaddons';
		$to 	= '#__payplans_tmpplanaddons';
		
		//Migrate the service data
		$sql = "INSERT INTO $to 
						 (`planaddons_id`, `title`, `description`, `price`, `consumed`, `addons_condition`, 
							`price_type`, `apply_on`, `plans`, `start_date`, `end_date`, `published`, `params`)
				    SELECT  `planaddons_id`, `title`, `description`, `price`, `consumed`, `addons_condition`, 
										`price_type`, `apply_on`, `plans`, `start_date`, `end_date`, `published`, `params`
				    FROM $from ";
		
		$dbo = JFactory::getDBO();
		$dbo->setQuery($sql);
		$dbo->query();

		$this->adjustAutoIncreament($from, $to);
		
		//Migrate the service statistics data
		$to 		= '#__payplans_tmpplanaddons_stats';
		$from 	= '#__payplans_planaddons_stats';
		
		$sql = "INSERT INTO $to	( `planaddons_stats_id`, `user_id`, `title`, `price`, `addons_condition`, `price_type`, `reference`, `status`, `purchase_date`, `params`)
					SELECT `planaddons_stats_id`, `user_id`, `title`, `price`, `addons_condition`, `price_type`, `reference`, `status` , `purchase_date`, `params`
					FROM $from ";

		$dbo->setQuery($sql);
		$dbo->query();
		
		$this->adjustAutoIncreament($from, $to);
	}
	
	public function adjustAutoIncreament($from = '', $to = '')
	{
		
		$dbo 			= JFactory::getDbo();
		
		$conf 			= JFactory::getConfig();		
		$databaseName 	= $conf->get('db');
		$from 			= $dbo->replacePrefix($from);
						
		//set auto increament counter
		$sql = "SELECT AUTO_INCREMENT
						FROM  INFORMATION_SCHEMA.TABLES
						WHERE TABLE_SCHEMA = '$databaseName'
						AND   TABLE_NAME   =  '$from' ";
		
		$dbo->setQuery($sql);
		$counter 	=  $dbo->loadResult();
				
		$sql 		= "ALTER TABLE $to AUTO_INCREMENT = $counter ";
		$dbo->setQuery($sql);
		$dbo->query();
		
		return true;
	}
	
	public function dropTable($tableName = '' )
	{
		$sql = "DROP TABLE IF EXISTS $tableName";
		$dbo = JFactory::getDBO();
		$dbo->setQuery($sql);
		return $dbo->query();
	}
	
	public function renameTable($old = '', $new = '')
	{
		$sql = "ALTER TABLE $old RENAME $new ";
		$dbo = JFactory::getDBO();
		$dbo->setQuery($sql);
		return $dbo->query();
	}
	
}
