<?php 
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license	    GNU/GPL, see LICENSE.php
* @package	    PayPlans
* @subpackage	Prodiscount
* @contact 	    payplans@readybytes.in
*/
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );
jimport( 'joomla.filesystem.archive' );
jimport( 'joomla.document.document' );
define('PAYPLANS_PRODISCOUNT_EACHRECURRING','eachrecurring');
define('PAYPLANS_PRODISCOUNT_AUTOONUPGRADE','autodiscount_onupgrade');
define('PAYPLANS_PRODISCOUNT_AUTOONRENEWAL','autodiscount_onrenewal');
define('PAYPLANS_PRODISCOUNT_UPGRADE_DISCOUNT','upgradeDiscount');
define('PAYPLANS_PRODISCOUNT_RENEWAL_DISCOUNT','renewalDiscount');
define('PAYPLANS_PRODISCOUNT_AUTO_ON_INVOICE_CREATION','autodiscount_oninvoicecreation');
define('PAYPLANS_PRODISCOUNT_INVOICE_CREATION_DISCOUNT','invoiceCeationDiscount');
define('PAYPLANS_PRODISCOUNT_EXTEND_TIME_DISCOUNT','discount_for_time_extend');


class plgPayplansProdiscount extends XiPlugin
{
	var $_name = 'Prodiscount';
	
	/**
	 * load plugin's view,controller,model,table and lib
	 */
	public function onPayplansSystemStart()
	{
		//autoload view,controller,model and table
        $dir = dirname(__FILE__).DS.'prodiscount';
        PayplansHelperLoader::addAutoLoadFile($dir.DS.'view.php', 'PayplansadminViewProdiscount');
		PayplansHelperLoader::addAutoLoadFile($dir.DS.'controller.php', 'PayplansadminControllerProdiscount');
		PayplansHelperLoader::addAutoLoadFile($dir.DS.'table.php', 'PayplansTableProdiscount');
		PayplansHelperLoader::addAutoLoadFile($dir.DS.'model.php', 'PayplansModelProdiscount');
		PayplansHelperLoader::addAutoLoadFile($dir.DS.'lib.php', 'PayplansProdiscount');
		PayplansHelperLoader::addAutoLoadFile($dir.DS.'helper.php', 'PayplansHelperProdiscount');
		
        return true;
	}
	
	/**
	 * set plugin template for backend and front end
	 */
	public function onPayplansViewBeforeRender(XiView $view, $task)
	{
		//add admin-submenu when plugin in enabled
		if(XiFactory::getApplication()->isAdmin() && XiHelperJoomla::getPluginStatus('prodiscount','payplans')){
			XiAbstractView::addSubmenus('prodiscount');
		}
		
		//for admin side
		if(($view instanceof PayplansadminViewProdiscount) && ($task == 'display' || $task == 'edit' || $task == 'discountHelp' || $task == 'bulkCodes')){
			// we need to add the normal template overriding paths
			$templatePaths = $this->_getTemplatePath($this->_name);
			$view->addPathToView($templatePaths);
		}

		return true;
	}	
	
	public function onPayplansApplyDiscount($object, $discountCode)
	{

		//do nothing no discount coupon is entered
		if(empty($discountCode)){
			return false;
		}
		
		$filter = array('coupon_code'=> $discountCode , 'published' => 1,'coupon_type' => array(array('IN', '("firstinvoice","eachrecurring", "autodiscount_onrenewal","autodiscount_onupgrade","upgradeDiscount","renewalDiscount","autodiscount_oninvoicecreation","discount_for_time_extend")')));
		
		$discounts   = PayplansHelperProdiscount::getApplicableDiscounts($object,$filter);	
		$finalResult = false;
  
		foreach ($discounts as $discount){
			$discountInstance = PayplansProdiscount::getInstance($discount->prodiscount_id);
			$result 		  = $this->doCheckAllowed($object, $discountCode, $discountInstance);
		
			//Important Check : use strict !== , Do not use != 
			if($result !== true){
				return $result;
			}
			
			$finalResult = $this->doApplyDiscount($object, $discountInstance);
		}
		return $finalResult;	
	}

	/** 
	 * Check if current discount should be applied as per discount purpose
	 */
	public function doCheckAllowed($object, $discountCode, $discountInstance)
	{

		//if coupon code if of each recurring type but plan is not recurring then do nothing
		if($discountInstance->getCouponType() == PAYPLANS_PRODISCOUNT_EACHRECURRING && !$object->isRecurring()){
			return false;
		}		

		//do not allow user to apply the same 
		//discount code again and again on the same invoice
		$modifiers = $object->getModifiers(array('type' => $discountInstance->getCouponType(), 'reference' => $discountCode));
		if(count($modifiers) > 0 ){
			return XiText::_('COM_PAYPLANS_PRODISCOUNT_ERROR_ALREADY_USED');
		}

		//if multiple discount on same invoice is not allowed then check for the perviously applied discount 
		$allowClubbing = PayplansFactory::getConfig()->multipleDiscount;
		
		/**
		 * check whether clubbing of discounts is to be given for the current discount
		 * case 1: If $allowClubbing is yes, and the applying discount allow clubbing then do nothing
		 * case 2: If $allowClubbing is yes, and the applying discount doesn't allow clubbing with others then 
		 *         check whether any other discount exist which already disallow clubbing or not, if yes, then show error msg
		 *         otherwise apply current discount
		 * case 3: If $allowClubbing is no, and invoice already have some discount ammount then show error msg
		 * 
		 */	
		$serialsFilter = array('serial'=>array(PayplansModifier::FIXED_DISCOUNT, PayplansModifier::PERCENT_DISCOUNT));
		
		if(
			(	$allowClubbing 
				&& $discountInstance->getParam('allow_clubbing',0) == 0 
				&& count(PayplansHelperProdiscount::_filterNonclubbableDiscounts($object->getModifiers($serialsFilter)) )
			) 
			||
			(	!$allowClubbing 
				&& $object->getDiscount() != 0
			)
		  ){
			return XiText::_('COM_PAYPLANS_PRODISCOUNT_CANT_APPLY_MULTIPLE_DISCOUNT');
		}
		
		//restrict user to use the same discount code on different subscriptions if reusable parameter is set to no
		$reusable   	= $discountInstance->getParam('reusable', 'yes');
		$userId		    = $object->getBuyer();
		$modifiers	    = XiFactory::getInstance('modifier','model')
							->loadRecords(array('user_id'=> $userId, 'reference' => $discountCode, 'type' => $discountInstance->getCouponType() ));
								
		if($reusable == 'no'){
			//user already used the mentioned discount code, not allowed to use it again
			if(count($modifiers) > 0 ){
				return XiText::_('COM_PAYPLANS_PRODISCOUNT_ERROR_ALREADY_USED');
			}	
		}
		
		// if coupon have been used completely	
		// unlimited usage if allowed quantity is ''
		$modifiers = XiFactory::getInstance('modifier','model')
							->loadRecords(array('reference' => $discountCode, 'type' => $discountInstance->getCouponType() ));
		$allowedQuantity = $discountInstance->getParam('allowed_quantity', '');
		if($allowedQuantity !== '' 	&& $allowedQuantity <= count($modifiers)){
			return XiText::_('COM_PAYPLANS_PRODISCOUNT_ERROR_CODE_ALLOWED_QUANTITY_CONSUMED');
		}
		
		return true;
	}
	
	/**
	 * return the coupon amount
	 */
	protected function _doCalculateDiscount($price, $discountInstance, payplansInvoice $object)
	{
		if($price <= 0){
			return 0;
		}
		
		$amount = $discountInstance->getCouponAmount();
		
		//for renew discount
		if($discountInstance->getCouponType() == PAYPLANS_PRODISCOUNT_AUTOONRENEWAL && $object instanceof PayplansInvoice){
			$order = $object->getReferenceObject(PAYPLANS_INSTANCE_REQUIRE);
			if(($order instanceof PayplansOrder)){
				$subscription = $order->getSubscription(PAYPLANS_INSTANCE_REQUIRE);
				$amount = PayplansHelperProdiscount::_getRenewDiscountAmount($discountInstance,$subscription);
			}
		}
		
		//calculate discount
		if($discountInstance->getParam('coupon_amount_type','fixed') === 'percentage'){
			return array($amount, true);
		}

		return array($amount, false);
	}
	
	/**
	 * Iteratively apply discount 
	 * 
	 * Tips : Avoid overriding this function
	 */
	function doApplyDiscount($object, $discountInstance)
	{
		// apply discount on invoice
		$price 		  = $object->getSubTotal()+$object->getDiscountable();
		$discount 	  = $object->getDiscount();
		
		$params = new stdClass();
		$params->message    = $discountInstance->get('title');
		$params->type       = $discountInstance->getCouponType();
		$params->reference  = $discountInstance->getCouponCode();

		//PAYPLANS_PRODISCOUNT_AUTO_ON_INVOICE_CREATION
		//this discount type says it will be applied on every new invoice creation
		//so it also applies when create new invoice of recurring subscription, but the amount asked to payment gateway is the full price.
		//so make this discount type to eachtime.

		$params->frequency  = ($discountInstance->getCouponType() != PAYPLANS_PRODISCOUNT_EACHRECURRING)
		                                         ? PayplansModifier::FREQUENCY_ONE_TIME 
		                                         : PayplansModifier::FREQUENCY_EACH_TIME;
		                                         
		if($params->type == PAYPLANS_PRODISCOUNT_EXTEND_TIME_DISCOUNT){
				$params->message = XiText::_('COM_PAYPLANS_PRODISCOUNT_EXETEND_TIME_MODIFIER_MESSAGE');
				$isAllowed = "<span class='text-success'>".XiText::_('COM_PAYPLANS_PRODISCOUNT_EXETEND_SUBSCRIPTION_MESSAGE')."</span>";
		}
		else{
				
				list($amount, $isPercentage) = $this->_doCalculateDiscount($price, $discountInstance, $object);
				if(floatval($amount) == floatval(0)){
					return false;
				}
				
				$params->amount     = -$amount;
				$params->percentage = $isPercentage ? true : false;
			
			   /**
				 * V.V.IMP : this is very impotant for applying discount in which serial
				 * @see PayplansModifier
				*/
				$params->serial     = ($isPercentage === true) 
									                     ? PayplansModifier::PERCENT_DISCOUNT 
									                     : PayplansModifier::FIXED_DISCOUNT;
				
		        //check for maximum allowed discount
			    $isAllowed = PayplansHelperProdiscount::_checkForUpperLimit($params, $object);					                   
		        if(!$isAllowed){
		        	return XiText::_('COM_PAYPLANS_PRODISCOUNT_ALREADY_CONSUMED_MAX_ALLOWED_DISCOUNT');
		        }
		}
		//add modifier
	    $object->addModifier($params);

		if($discountInstance->getCouponType() != PAYPLANS_PRODISCOUNT_EACHRECURRING && $object->isRecurring() == PAYPLANS_RECURRING){		
				$objParams = $object->getParams()->toArray();
				$object->setParam('expirationtype', 'recurring_trial_1');
				$object->setParam('recurrence_count', ($objParams['recurrence_count']> 0 ) ? $objParams['recurrence_count']-1 : 0);
				$object->setParam('trial_price_1', $objParams['price']);
				$object->setParam('trial_time_1', $objParams['expiration']);
		}
		
		$object->refresh();
		
		$object->save();

		//create log
		$message                  = XiText::_('COM_PAYPLANS_PRODISCOUNT_LOG_DISCOUNT_APPLIED');
		$error['Coupon Code']	  = $params->reference;
		$error['Invoice ID']      = $object->getId();
		$error['Prodiscount ID']  = $discountInstance->get('prodiscount_id');
		$error['User ID']	 	  = $object->getBuyer();
		if(isset($amount)){
			$error['Discount Amount'] = $amount;
			$error['Percentage']      = $isPercentage ? 'Yes' : 'No';
		}
		else{
			$error['Extended Time'] = $discountInstance->getParam('extend_time_discount');
		}
		PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $discountInstance,$error);

		if(!is_bool($isAllowed)){
			return $isAllowed;
		}		
		return true;
	}
	
	/**
	 *  For autodiscount on upgrades
	 *  catch the event of upgrade app and work on it
	 */
	public function onPayplansUpgradeBeforeDisplay($newPlanId, PayplansSubscription $oldSub, PayplansInvoice $newInvoice)
	{
		 $this->applyAutoDiscount($newInvoice, PAYPLANS_PRODISCOUNT_AUTOONUPGRADE, PAYPLANS_PRODISCOUNT_UPGRADE_DISCOUNT);
	}
	
	/**
	 * For autoDiscount on renewals
	 * catch the event of renewal app and work on it
	 */
	public function onPayplansSubscriptionAfterRenewalInvoiceCreation(PayplansSubscription $oldSubscription,PayplansSubscription $newSubscription,PayplansOrder $order,PayplansInvoice $newInvoice)
	{
		$this->applyAutoDiscount($newInvoice, PAYPLANS_PRODISCOUNT_AUTOONRENEWAL, PAYPLANS_PRODISCOUNT_RENEWAL_DISCOUNT);
	}
	
	/**
	 * For autoDiscount on invoice Creation and Url discount
	 */
	function OnPayplansInvoiceAfterSave($prev, $newInvoice)
	{
		$status = array(PayplansStatus::INVOICE_CONFIRMED,PayplansStatus::NONE);

		if(floatval(0) != floatval($newInvoice->getTotal()
		   && !isset($prev) 
		   && in_array($newInvoice->getStatus(),$status))){
		     $this->applyAutoDiscount($newInvoice, PAYPLANS_PRODISCOUNT_AUTO_ON_INVOICE_CREATION, PAYPLANS_PRODISCOUNT_INVOICE_CREATION_DISCOUNT);
		}
		return true;
	}
	
	/**
	 * for url discount
	 */
	function onAfterRoute()
	{ 
		if(JRequest::getVar('coupon_code') || JFactory::getSession()->get('prodiscount_coupon_code', false)){
			$sess_code = JFactory::getSession()->get('prodiscount_coupon_code', false);
			if(!$sess_code){
				JFactory::getSession()->set('prodiscount_coupon_code',JRequest::getVar('coupon_code'));
			}
			
			if(JRequest::getVar('invoice_key',false)){
				$code	     	= JRequest::getVar('coupon_code',false);
				$couponCode 	= ($code)?$code:$sess_code;
				$invoice		= PayplansInvoice::getInstance(PayplansHelperUtils::getIdFromKey(JRequest::getVar('invoice_key')));
				$result         = $this->onPayplansApplyDiscount($invoice, $couponCode);
				//clear coupon code
				if( is_bool($result) && $result == true ){
					//XiTODO: Changes on invoice should be reflected automatically(Its a caching issue)
                    // or either use Ajax request					
					ob_start();
    				?>
						payplans.jQuery(document).ready(function(){	
							payplans.jQuery('.discount-amount').show();
							payplans.jQuery('.discount-amount .pp-amount').html('<?php echo PayplansHelperFormat::displayAmount(PayplansInvoice::getInstance(PayplansHelperUtils::getIdFromKey(JRequest::getVar('invoice_key')))->getDiscount());?>');
							payplans.jQuery('.first-tax-amount .pp-amount').html('<?php echo PayplansHelperFormat::displayAmount($invoice->getTaxAmount());?>');
							payplans.jQuery('.tax-amount .pp-amount').html('<?php echo PayplansHelperFormat::displayAmount($invoice->getTaxAmount());?>');
							payplans.jQuery('.regular-amount .pp-amount').html('<?php echo PayplansHelperFormat::displayAmount($invoice->getRegularAmount());?>');
							payplans.jQuery('.first-amount .pp-amount').html('<?php echo PayplansHelperFormat::displayAmount($invoice->getTotal());?>');
						});
    				<?php 
					$js = ob_get_contents();
					ob_end_clean();
					XiFactory::getDocument()->addScriptDeclaration($js);
					
					JFactory::getSession()->clear('prodiscount_coupon_code');
				}		
			}
		}
		return true;
	}
	
	/**
	 * Apply automatic discounts...
	 * @param PayplansInvoice $newInvoice: Invoice on which discount is to be applied
	 * @param string $discountType 		 : Type of discount to be applied
	 * @param string $discountCodePrefix : prefix to be added before prodiscount_id
	 * 									 ( auto discounts don't ask for discount code but its required to 
	 *                                     store some reference in modifiers for uniquely identify the applied discount) 
	 */
	public function applyAutoDiscount($newInvoice, $discountType, $discountCodePrefix)
	{
		//get the applicable discounts and then process each one
		$applicableDiscounts =  PayplansHelperProdiscount::getApplicableDiscounts($newInvoice,array('coupon_type'=>$discountType,'published'=>1));
		
		foreach ($applicableDiscounts as $discount){
			$discountInstance = PayplansProdiscount::getInstance($discount->prodiscount_id);
			
			//set coupon_code so that unique reference can be identified from modifier for this instance
			$discountInstance->set('coupon_code',$discountCodePrefix.'_'.$discount->prodiscount_id);
			$allow = $this->doCheckAllowed($newInvoice, $discountInstance->getCouponCode(), $discountInstance);
			
			if(is_bool($allow) && $allow == true){
				$this->doApplyDiscount($newInvoice, $discountInstance);
			}
		}
	}
	
	public function  onPayplansProdiscountBeforeSave($prev, $new)
	{   
		$start_date = $new->getStartDate()->toFormat();
		$end_date   = $new->getEndDate()->toFormat();
		if(!isset($start_date)){
			$new->set('start_date','');
		}
		
		if(!isset($end_date)){
			$new->set('end_date','');
		}
	}
	
		/**
	 * For extend subscription on applying discount
	 */
	public function onPayplansSubscriptionBeforeSave($prev,$new)
	{
		if(!isset($prev)){
			return true;
		}
		
		if($prev->getStatus() != PayplansStatus::SUBSCRIPTION_ACTIVE && $new->getStatus() == PayplansStatus::SUBSCRIPTION_ACTIVE){
			
			$orderObj = $new->getOrder(PAYPLANS_INSTANCE_REQUIRE);
			$invoice = $orderObj->getLastMasterInvoice(PAYPLANS_INSTANCE_REQUIRE);	
			
			//if invoice is not there then do nothing
			if(!($invoice instanceOf PayplansInvoice)){
				return true;
			}			
			
			$modifiers = $invoice->getModifiers(array('type'=> PAYPLANS_PRODISCOUNT_EXTEND_TIME_DISCOUNT));
				
			 foreach ($modifiers as $modifier){
                $code = $modifier->getReference();
                $type = $modifier->getType();
                           
                $prodiscountModel = PayplansFactory::getInstance('prodiscount','model');
                $filter = array('coupon_type'=>$type,'coupon_code'=>$code);
                $prodiscountRecords = $prodiscountModel->loadRecords($filter);
                       
                foreach ($prodiscountRecords as $prodiscountRecord){
                          $record  = PayplansProdiscount::getInstance($prodiscountRecord->prodiscount_id,null,$prodiscountRecord );
                          $exptime = $record->getParam('extend_time_discount',000000000000);
                          $new->set('expiration_date', $new->getExpirationDate()->addExpiration($exptime));
                }
            }
		}
	}

	//creation/modification logs
	public function onPayplansProdiscountAfterSave($prev, $new)
	{
		PayplansEventLog::_save($prev, $new,'PRODISCOUNT');		
	}
	
	
	public function onPayplansGetDiscountType(&$args)
	{
		$args['firstinvoice'] 			= array(XiText::_('COM_PAYPLANS_PRODISCOUNT_COUPON_TYPE_ON_FIRST_INVOICE'),false);
		$args['eachrecurring'] 			= array(XiText::_('COM_PAYPLANS_PRODISCOUNT_COUPON_TYPE_ON_EACH_RECURRING'),false);
		$args['autodiscount_onrenewal'] 	= array(XiText::_('COM_PAYPLANS_PRODISCOUNT_COUPON_TYPE_AUTO_DISCOUNT_ON_RENEWAL'),false);
		$args['autodiscount_onupgrade'] 	= array(XiText::_('COM_PAYPLANS_PRODISCOUNT_COUPON_TYPE_AUTO_DISCOUNT_ON_UPGRADE'),false);
		$args['autodiscount_oninvoicecreation'] = array(XiText::_('COM_PAYPLANS_PRODISCOUNT_COUPON_TYPE_AUTO_DISCOUNT_ON_INVOICE_CREATION'),false);
		$args['discount_for_time_extend'] 	= array(XiText::_('COM_PAYPLANS_PRODISCOUNT_COUPON_TYPE_DISCOUNT_FOR_TIME_EXTEND'),false);
	}

	/**
	 * Remove plan mapping from discount on plan deletion
	*/
	public function onPayplansPlanAfterDelete($itemId)
	{
		$db = JFactory::getDBO();
		$query = 'SELECT *'
			 	. ' FROM #__payplans_prodiscount'
			 	. ' WHERE `plans` != ""';
			 	
	 	$db->setQuery( $query );
	 	$result = $db->loadObjectList();
	 	
	 	foreach ($result as $value) {
	 		
	 		$plans = $value->plans;
	 		$plans = explode(",", $plans);
			
			// Do nothing if plan is not mapped with discount 
			if(!in_array($itemId, $plans)){
				continue;
			}

			foreach ($plans as $key => $data) {
				if ($data == $itemId) {
					unset($plans[$key]);

					$updatedPlans = implode(",", $plans);
					$query  = 'UPDATE `#__payplans_prodiscount`'
							  . 'SET `plans` = "'.$updatedPlans.'"'
							  . 'WHERE `prodiscount_id` = '.$value->prodiscount_id;

					$db->setQuery($query);
					$db->query();

				}			
			}	 			
		}	 		
	}
}
