<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Prodiscount
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();


class PayplansHelperProdiscount
{
	/**
	 * Check whether any discount is applicable on this invoice or not 
	 * @param  $filters : filters for discounts
	 */
	public static function getApplicableDiscounts($invoice, $filters)
	{
		$discounts = XiFactory::getInstance('prodiscount','model')
								->loadRecords($filters);
		foreach ($discounts as $discountId=>$value)
		{
			$discount = PayplansProdiscount::getInstance($discountId);
			
			//if discount coupon is not published or not within the applicable dates
			//then unset that discount 
			if(!self::_checkForApplicableDates($discount)){
				unset($discounts[$discountId]);
				continue;
			}
			
			//if not a core discount then check for the applicable planids
			if(!$discount->getCoreDiscount()){
				$plans			= $invoice->getPlans();
				$subPlan	 	= array_shift($plans);
				$disPlans		= $discount->getPlans();
				$discount_plans = explode(",",$disPlans);
				
					if(!in_array($subPlan,$discount_plans)){
						unset($discounts[$discountId]);
					}
				
			}
		}
		
		return $discounts;
	}
	
	/**
	 *  filter discounts as per the clubbing settings
	 */
	public static function _filterNonclubbableDiscounts($modifiers)
	{
		$filters   = array();
		$discounts = array();
		
		foreach($modifiers as $key=>$modifier){
			$modifierType = $modifier->getType();
			
		//step 1: first make conditions for filter acording to the discount type
			if( $modifierType == PAYPLANS_PRODISCOUNT_AUTOONUPGRADE || $modifierType == PAYPLANS_PRODISCOUNT_AUTOONRENEWAL || $modifierType == PAYPLANS_PRODISCOUNT_AUTO_ON_INVOICE_CREATION){
				$filters['prodiscount_id'][] = substr(strchr($modifier->getReference(),'_'),1);
			}
			else{
				$filters['coupon_code'][]    = $modifier->getReference();
			}
		}

		//step 2: collect all records as per the applied filter
		if(isset($filters['prodiscount_id'])){
			$discounts = XiFactory::getInstance('prodiscount','model')
					          ->loadRecords(array('prodiscount_id' => array(array('IN', '('.implode(",", $filters['prodiscount_id']).')')) ));
		}
		
		if(isset($filters['coupon_code'])){
			$discounts = array_merge($discounts, 
			        	 	XiFactory::getInstance('prodiscount','model')
					          ->loadRecords(array('coupon_code'=> array(array('IN', '("'.implode('","', $filters['coupon_code']).'")'))) ));
		}
		
		//step 3: unset all the modifiers which has discounts that allows clubbing with other discounts
		foreach ($discounts as $id=>$discount){
			$discountInstance  = PayplansProdiscount::getInstance($discount->prodiscount_id, null, $discount);
			if($discountInstance->getParam('allow_clubbing',0) == 0){
				continue;
			}
			unset($discounts[$id]);
		}
	
		return $discounts;
	}
	
	
	/**
	 * check whether the maximum limit is crossed due to current discount or not
	 * @param stdclass $newModifier
	 * @param PayplansInvoive $object
	 */
	public static function _checkForUpperLimit($newModifier, $object)
	{
		$subTotal 		   = $object->getSubtotal();
		$discountable      = $object->getDiscountable();
		$actualTotal       = $subTotal + $discountable;
		$allowedAmount     = ( PayplansFactory::_getConfig()->allowedMaxPercentDiscount * $actualTotal )/100;
		$existingDiscount  = $object->getDiscount();
		
		//if existing discount is already greater or equal to max allowed
		if($existingDiscount >= $allowedAmount) {
			return false;
		}
			
		//create dummy modifier
		$modifier = PayplansModifier::getInstance();
		$modifier->set('amount', $newModifier->amount);
		$modifier->set('percentage', $newModifier->percentage);
		$modifier->set('serial', $newModifier->serial);
		
		// merge dummy and pervious modifier to calculate the actual modification amount
		// after applying the new discount
		$total         		 = $object->getTotal();
		$serialsFilter		 = array(PayplansModifier::FIXED_DISCOUNT, PayplansModifier::PERCENT_DISCOUNT);
		$modifiers     		 = array_merge( $object->getModifiers( array('serial'=>$serialsFilter)), array($modifier));
		PayplansHelperModifier::getTotal($subTotal, $modifiers);
		$resultAmount        = -(PayplansHelperModifier::getModificationAmount($total, $modifiers, $serialsFilter));
		
		//if crossing the limit apply the possible discount and give a message
		$applicableAmount = ($allowedAmount - $existingDiscount);
		$msg			  = false;
		
		if($resultAmount > $allowedAmount && $applicableAmount > 0){
			$msg =  XiText::_('COM_PAYPLANS_PRODISCOUNT_MAX_ALLOWED_DISCOUNT_CROSSED');
			//in case of fixed discount
			$newModifier->amount = -$applicableAmount;
			
			/**
			 * If the current discount is in percentage 
			 * then calculate and apply the allowable percent discount
			 */
			if($newModifier->percentage){
				$newModifier->amount = ($newModifier->amount*100)/($actualTotal-$existingDiscount);
			}
			return $msg;
		}
		return true;
	}
	
	/**
	 * Check whether the date is within the applicable dates limit or not
	 * @param $discount : PayplansProdiscount object
	 */
	public static function _checkForApplicableDates($discount)
	{
		//if coupon is published as per dates
		$publish_start 	= $discount->getStartDate();
		$publish_end	= $discount->getEndDate();
        $now            = new XiDate();
        $startDate		= $publish_start->toMySQL();
        
		if(isset($startDate)){
			if($publish_start->toUnix() > $now->toUnix()){
				return false;
			}
		}
		$endDate		= $publish_end->toMySQL();
		if(isset($endDate)){
			if($publish_end->toUnix() < $now->toUnix()){
				//also disable the discount
				$discount->setPublished(0);
				$discount->save();
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Decide discount amount as per the renew timing, that can be before and after expiry 
	 * @param $discountInstance
	 * @param $subscription
	 */
	public static function _getRenewDiscountAmount($discountInstance, $subscription)
	{		
		if($subscription->getStatus() == PayplansStatus::SUBSCRIPTION_EXPIRED){
			return $discountInstance->getParam('amount_post_expiry',0);
		}
		
		if($subscription->getStatus() == PayplansStatus::SUBSCRIPTION_ACTIVE){
			return $discountInstance->getParam('amount_pre_expiry',0);
		}
		
		return 0;
	}
	
	
	public static function createExportFile($filename, $data)
	{		
		
		if(file_exists($filename)){
			JFile::delete($filename);
		}

		try{
			$fhandle = fopen($filename,"a");		
			fwrite($fhandle, $data);
			
			header('Content-Description: File Transfer');
			header('Content-Type: application/csv');
			header("Content-Disposition: attachment; filename=discountCode.csv");
			echo file_get_contents($filename);
			exit();
		}
		
		//if something gona happens with the file may be permission issue or not found then handle it
		catch(Exception $e){
			return false;
		}

		return true;
	}
}