<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Prodiscount
* @contact 		shyam@readybytes.in
*/
if(defined('_JEXEC')===false) die();


class PayplansProdiscount extends XiLib
{
	// Table fields
	protected $prodiscount_id;
	protected $title;
	protected $coupon_code;
	protected $core_discount;
	protected $coupon_amount;
	protected $coupon_type;
	protected $start_date;
	protected $end_date;
	protected $plans;
	protected $published;
	protected $params;
	
/**
	 * @return PayplansProdiscount
	 */
	static public function getInstance($id=0, $type=null, $bindData=null, $dummy=null)
	{
		return parent::getInstance('prodiscount',$id, $type, $bindData);
	}
	
	// 	not for table fields
	public function reset(Array $config=array())
	{
		$this->prodiscount_id = 0;
		$this->title		  = null;
		$this->coupon_code	  = null;
		$this->core_discount  = 0;
		$this->coupon_amount  = 0.0000;
		$this->coupon_type    = null;
		$this->start_date     = new XiDate('0000:00:00 00:00:00');
		$this->end_date       = new XiDate('0000:00:00 00:00:00');
		$this->plans          = array();
		$this->published	  = 1; 
		$this->params	      = new XiParameter();
		return $this;
	}
	
	//get planids 
	public function getPlans()
	{
		return $this->plans;
	}
	
	//get whether discount is core or not
	public function getCoreDiscount()
	{
		return $this->core_discount;
	}
	
	//get startDate of coupon
	public function getStartDate()
	{
		return $this->start_date;
	}
	
	//get endDate of coupon
	public function getEndDate()
	{
		return $this->end_date;
	}

	//get whether discount coupon is published or not
	public function getPublished()
	{
		return $this->published;
	}
	
	//set published as true/false
	public function setPublished($published)
	{
		return $this->published = $published;
	}
	
	//get params
	public function getParam($key,$default=null)
	{
		XiError::assert($this);
		return $this->params->Get($key,$default);
	}
	
	//get the type of coupon
	public function getCouponType()
	{
		return $this->coupon_type; 
	}
	
	//get coupon amount
	public function getCouponAmount()
	{
		return $this->coupon_amount;
	}
	
	//get coupon code
	public function getCouponCode()
	{
		return $this->coupon_code;
	}

	//bind data
	public function bind($data, $ignore=array())
	{
		if(is_object($data)){
			$data = (array) ($data);
		}

		parent::bind($data, $ignore=array());

		if(isset($data['plans'])){
			$this->plans = (is_array($data['plans']))?implode(',',$data['plans']):$data['plans'];
		}

		return $this;
	}
	
	//get reference depends upon the type of discount
	protected function _getReferenceCode()
	{
		if($this->coupon_type == PAYPLANS_PRODISCOUNT_AUTOONRENEWAL){
			return PAYPLANS_PRODISCOUNT_RENEWAL_DISCOUNT.'_'.$this->prodiscount_id;
		}
		
		if($this->coupon_type == PAYPLANS_PRODISCOUNT_AUTOONUPGRADE){
			return PAYPLANS_PRODISCOUNT_UPGRADE_DISCOUNT.'_'.$this->prodiscount_id;
		}
		
		if($this->coupon_type == PAYPLANS_PRODISCOUNT_AUTO_ON_INVOICE_CREATION){
			return PAYPLANS_PRODISCOUNT_INVOICE_CREATION_DISCOUNT.'_'.$this->prodiscount_id;
		}
	
		return $this->coupon_code;
	}
	
	//get counter
	public function getCounter()
	{
		return PayplansHelperModifier::getTotalUsage($this->_getReferenceCode(), $this->coupon_type);
	}

	//get Actual Consumption
	public function getConsumption($count=true)
	{
		return PayplansHelperModifier::getActualConsumption($this->_getReferenceCode(), $this->coupon_type, $count);
	}
}
