<?php
/**
* @copyright	Copyright (C) 2009 - 2017 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppRede extends PayplansAppPayment
{
	protected $_location	= __FILE__;

   public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
   {
		if(is_object($data)){
			$data = (array)$data;
		}

		$invoice   			= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);		 
		$recurring 			= $invoice->isRecurring();
		
		$merchantNumber 	= $this->getAppParam('merchant_number',		false);
		$merchantUsername	= $this->getAppParam('merchant_username',	false);
		$merchantPassword	= $this->getAppParam('merchant_password', 	false);
		
		$paymentKey			= $payment->getKey();
		$amount 			= number_format($invoice->getTotal(),2);
			
		$this->assign('merchant_number', 	$merchantNumber);
		$this->assign('order_id', 			$paymentKey);
    	$this->assign('amount',				$amount);
    	$this->assign('payment', 			$payment);
    	$this->assign('username',			$merchantUsername);
    	$this->assign('password',			$merchantPassword);

        $invoiceKey 	= $invoice->getId();
        $this->assign('invoice_key', $invoiceKey);
    	
        $root = JURI::root();
    	if(XiFactory::getConfig()->https == true){
    		$root = JString::str_ireplace("http:", "https:", $root);
    	}
    	
    	
    	$cancelURL = $root.'index.php?option=com_payplans&gateway=rede&view=payment&task=complete&action=cancel&payment_key='.$paymentKey;
    	$this->assign('cancel_url', 		$cancelURL);
    	$this->assign('post_url',         	XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&payment_key=".$paymentKey)); 
    	
		return $this->_render('form');
  	}

    public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		if($action == 'cancel'){
				return true;
		}
		$invoice   		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
				
		list($approve, $response) 	= $this->_requestPayment($data);
		$this->_processPayment($invoice, $payment, $approve, $response);
		
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		
	}
	
	public function _requestPayment($data)
	{
		require_once dirname(__FILE__).DS.'lib'.DS.'Method'.DS.'AbstractMethod.php';
		require_once dirname(__FILE__).DS.'lib'.DS.'Method'.DS.'GetAuthorized.php';
		require_once dirname(__FILE__).DS.'lib'.DS.'Method'.DS.'VoidTransaction.php';
		require_once dirname(__FILE__).DS.'lib'.DS.'KomerciClient.php';
		
		$komerciClient 	= new EuMatheusGomes\Komerci\KomerciClient('prod', $data['FILIACAO'], $data['username'], $data['password']);
    	$soapClient 	= new SoapClient($komerciClient->getUrl());

    	$parcelas = (int) $data['PARCELAS'] == 1
        	? '00'
        	: str_pad((int) $data['PARCELAS'], 2, 0, STR_PAD_LEFT);

    	$transacao = $parcelas == '00'
        	? EuMatheusGomes\Komerci\KomerciClient::TRANSACAO_AVISTA
        	: EuMatheusGomes\Komerci\KomerciClient::TRANSACAO_PARCELADO_EMISSOR;

    	$getAuthorized 	= new EuMatheusGomes\Komerci\Method\GetAuthorized($komerciClient, $soapClient);
   	 	$getAuthorized->setTotal($data['TOTAL'])
				      ->setTransacao($transacao)
				      ->setParcelas($parcelas)
				      ->setNumPedido($data['NUMPEDIDO'])
				      ->setNrcartao($data['NRCARTAO'])
				      ->setCVC2($data['CVC2'])
				      ->setMes($data['MES'])
				      ->setAno($data['ANO'])
				      ->setPortador($data['PORTADOR'])
				      ->setConfTxn($data['CONFTXN']);

   	 	$response = $getAuthorized->call();
    	$approved = $getAuthorized->approved($response);
    	
    	return array($approved, $response);
	}
	
	public function _processPayment($invoice, $payment, $approve, $response)
	{
		$data  = (array)$response;
		
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', 			$payment->getBuyer())
					->set('invoice_id', 		$invoice->getId())
					->set('payment_id', 		$payment->getId())
					->set('gateway_txn_id', 	$data['NUMSQN'])
					->set('gateway_subscr_id', 	$data['NUMAUTOR'])
					->set('gateway_parent_txn', 0)
					->set('params', 			PayplansHelperParam::arrayToIni($response));

		$error	= '';
		if($approve)
		{
			$message    = XiText::_('Payment Completed successfully.');
			$amount		= number_format($invoice->getTotal(),2);
			$transaction->set('amount', 	$amount)
						->set('message', 	$message);
		}
		else{
			$error = XiText::_('ERROR in REDE payment');
			$transaction->set('message', 	$response['MSGRET']);
		}
		
		if($error){
				$message 	= XiText::_('COM_PAYPLANS_LOGGER_ERROR_IN_REDE_PAYMENT_PROCESS');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response);
		}
        
        //store the response in the payment AND save the payment
		if(!$transaction->save()){
			$message    = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			$errors		= "Transaction not saved";
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $transaction, $errors);
		}
		
		$payment->save();
		return true;
	} 
}
