<?php

/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	Referral
* @contact	    support+payplans@readybytes.in
*/
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Payplans Referral Plugin
 *
 */
class plgPayplansReferral extends XiPlugin
{
	const APPLY_ALL   =  'APPLY_ALL';
	
	public function onPayplansSystemStart()
	{
		//add app path to app loader
		$appPath = dirname(__FILE__).DS.'referral'.DS.'app';
		PayplansHelperApp::addAppsPath($appPath);

		
		return true;
	}
	
	// on render of order, display output
	public function onPayplansViewBeforeRender(XiView $view, $task)
	{
		// check user login or not
		$user_id = XiFactory::getUser()->id;
		
		// Get user id from session for new registered user
		if(!$user_id){ 
		 	$user_id = XiFactory::getSession()->get('REGISTRATION_NEW_USER_ID', 0);
		}
		
		if(!$user_id){
			return true;
		}
		
		//if no app exist then don't do anything
		$apps = PayplansHelperApp::getAvailableApps('referral');
		if(empty($apps)){
			return array();
		}
		
		if(($view instanceof PayplanssiteViewDashboard) && $task == 'frontview')
		{
			list($applicable, list($limit, $reusable))	= $this->__checkApplicablePlan($user_id);
	    	if($applicable){
	    		$referralCode  = XiHelperUtils::getKeyFromId($user_id);
	    		$this->_assign('referral_code', $referralCode);
				return array('payplans-dashboard-right' => $this->_render('widgethtml'));
	    	}
		}
		
		
		if((($view instanceof PayplanssiteViewInvoice) && $task == 'confirm')){
			$invoiceId  	= $view->getModel()->getId();
			$invoice		= PayplansInvoice::getInstance($invoiceId);
	
			$this->_assign('invoice', 		$invoice);
			
			if($view instanceof PayplanssiteViewInvoice){			
				return array('payplans_order_confirm_payment' => $this->_render('invoiceconfirm'));
			}
		}

		return true;
	}
	
    function onPayplansReferralRequest($invoiceKey, $referralCode)
    {
		//trigger the referral
		$invoiceId 		= XiHelperUtils::getIdFromKey($invoiceKey);
		$invoice   		= PayplansInvoice::getInstance($invoiceId);
		$args      		= array($invoice, $referralCode);
		$error 			= '';
		
		
		$applicalble	= $this->__checkApplicable($invoice, $referralCode);
		if (is_bool($applicalble) == false){
			$error .= $applicalble.' ';
		}else {
			$results = PayplansHelperEvent::trigger('onPayplansApplyReferral', $args, '', $invoice);
		}
		
		foreach($results as $result){
			// check if app returned true/false OR error string
			if(is_bool($result)==false){
				$error .= $result . ' ';
			}
		}
		
   	 	$response = XiFactory::getAjaxResponse();
   	 	
   	 	// if error = '', then no error will be shown
		$response->addScriptCall('payplans.referral.error',$error);
    	$modifiers= $invoice->getModifiers(); 
		$modifiers  = PayplansHelperModifier::_rearrange($modifiers);
		if(!empty($modifiers)){
			$modified_html = $this->_loadTemplate('default_partial_invoice_modifier', compact('invoice'));
			$response->addScriptCall('xi.jQuery(\'#payplans\').find(\'.pp-modifiers\').remove');
			$response->addScriptCall('xi.jQuery(\'#payplans\').find(\'.pp-modifier-amount\').after', $modified_html);
		}
		else {
			$response->addScriptCall('xi.jQuery(\'#payplans\').find(\'.pp-modifiers\').remove');
		}
		
		$response->addScriptCall('payplans.jQuery(\'span[id="pp-referral-spinner"]\').removeClass',"loading");	
		// If total amount is 0 after applying discount code then disable payment method
		if(floatval($invoice->getTotal()) == floatval(0) && !$invoice->isRecurring()){
		   $response->addScriptCall('payplans.jQuery(\'.payplans\').find(\'.pp-payment-method\').html',					XiText::_('COM_PAYPLANS_ORDER_NO_PAYMENT_METHOD_NEEDED'));
		}

		$response->sendResponse();
		
    }
    
	
	public function __checkApplicable($invoice, $referralCode)
	{
		// Referral Discount will not be applied if already applied
		$modifiers = $invoice->getModifiers(array('type' => 'referral'));
		if(count($modifiers) > 0 ){
			return XiText::_('COM_PAYPLANS_APP_REFERRAL_ERROR_ALREADY_USED');
		}
		
		$referrarId	= XiHelperUtils::getIdFromKey($referralCode);
		$pUser		= PayplansUser::getInstance($referrarId);
		if($referrarId == 0 || !($pUser instanceof PayplansUser)){
			return XiText::_('COM_PAYPLANS_APP_REFERRAL_ERROR_INVALID_CODE');
		}
		
		
		// Referral Code Appliable for Plan or Not
		$referralId			= $invoice->getBuyer();
		$plans 				= $invoice->getPlans(PAYPLANS_INSTANCE_REQUIRE);
		$plan 				= array_pop($plans);
		
		//Referrer use can not use own referral code
		if($referrarId == $referralId){
			return XiText::_('COM_PAYPLANS_APP_REFERRAL_ERROR_CANNOT_USE_OWN_REFERRAL_CODE');	
		}
		
		// Referrar and Referral User has applicable plan or not
		list($referrarApplicable, list($limit, $reusable))	= $this->__checkApplicablePlan($referrarId, $referralId, $plan->getId());
		
		// check Referral Code is Reusable to User or not
		if(!$reusable){
			return XiText::_('COM_PAYPLANS_APP_REFERRAL_ERROR_NOT_REUSABLE');
		}

    	if(!$referrarApplicable){
	 		return XiText::_('COM_PAYPLANS_APP_REFERRAL_ERROR_CODE_NOT_APPLICABLE');
	 	}
		
		// Referral Code can be used infinite time 
		if($limit == 0){
			return true;
		}
		
		// Get how many referral code utilized for Referrar
		$query      = new XiQuery();
		$record		= $query->select('*')
							->from('#__payplans_referral')
							->where('`referrar_id`= '.$referrarId)
							->dbLoadQuery()
		  					->loadColumn();
		  					
		if(count($record) < $limit){
			return true;
		}

		return XiText::_('COM_PAYPLANS_APP_REFERRAL_ERROR_MAXIMUM_ALLOWED_LIMIT_REACHED');
	}
	
	public function __checkApplicablePlan($referrarId, $referralId = 0, $planId = 0)
	{
		$apps		= PayplansHelperApp::getAvailableApps('referral');
		$applicable	= false;
		$reusable	= true;
		$user 		= PayplansUser::getInstance($referrarId);
	    $userPlan 	= $user->getPlans();
	    $limit		= 0;
	
		// Process Only if user have any Active subscription
		if(!empty($userPlan)){
			foreach($apps as $app){
				if($referralId && $app->getAppParam('reusable',0) == false){
					$query	    = new XiQuery();
					$records	= $query->select('*')
										->from('#__payplans_referral')
										->where('`referrar_id` = '.$referrarId)
										->where('`referral_id` = '.$referralId)
										->where('`plan_id` = '.$planId)
										->dbLoadQuery()
										->loadColumn();
										
					if(count($records) > 0){
						return array($applicable, array($limit, false));
					}
				}
				
		 		if($app->getParam('applyAll')){
		 			$limit		= $limit + $app->getAppParam('referral_limit');
		 			$applicable = (!$applicable)	?	true	:	$applicable;
		 		}
		 		else{
					$appPlans	= $app->getPlans();
					if(array_intersect($userPlan, $appPlans)){
						$limit		= $limit + $app->getAppParam('referral_limit');
						$applicable = (!$applicable)	?	true	:	$applicable;
					}
		 		}
			}
		
		}
		return array($applicable, array($limit, $reusable));
	}
	

	public function onPayplansGetDiscountType(&$args)
	{
		$args['referral'] = array(XiText::_("COM_PAYPLANS_REFERRAL_TYPE"), true);
		return $args;
	}

}
