<?php

/**
* @copyright	Copyright (C) 2009 - 2014 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	Referral
* @contact		support+payplans@readybytes.in
*/
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Payplans Referral App
 *
 */
class PayplansAppReferral extends PayplansApp
{
	protected $_location	= __FILE__;

	public function isApplicable($refObject= '', $eventName='')
	{
          if($eventName == 'onPayplansApplyReferral' || $eventName == 'onPayplansInvoiceAfterSave' || $eventName == 'onPayplansApplyDiscount'){
          	return true;
          }
          return parent::isApplicable($refObject, $eventName);
	}
	
	
	// 	Entry Function 
	public function onPayplansApplyReferral(PayplansInvoice $invoice, $referralCode)
	{		
		$referrarId		= XiHelperUtils::getIdFromKey($referralCode);
		$amountType 	= $this->getAppParam('referral_amount_type');
		$aplicablePlan  = $this->getAppParam('referral_on');
		$error 			= '';

	    $referrarUser   = PayplansUser::getInstance($referrarId);
		$referrarPlan	= $referrarUser->getPlans();

		if(!array_intersect($referrarPlan, $this->getPlans())){
			return false;
		}
		
		if($aplicablePlan == 'all_plan'){
			$referrarAmount	= $this->getAppParam('referrar_amount');
			$referralAmount	= $this->getAppParam('referral_amount');
		}else {
				
				$plan		= $invoice->getPlans(PAYPLANS_INSTANCE_REQUIRE);
				$plan		= array_pop($plan);
				$planId		= $plan->getId();
				$appData	= $this->getAppParam('referral_price_time');
				foreach ($appData as $data){
					if($data[0] == $planId){
						$referrarAmount = $data[1];
						$referralAmount = $data[2];
					}
				}
		}
		
		if(isset($referralAmount)){
			$applyRefDiscount 	= $this->_applyDiscountToReferral($invoice, $referralAmount, $referralCode);
			if(is_bool($applyRefDiscount) == false){
				return $applyRefDiscount;
			}
		}else {
			return XiText::_('COM_PAYPLANS_APP_REFERRAL_ERROR_CODE_NOT_APPLICABLE');
		}
		
		if($this->getAppParam('referral_amount_type','fixed') === 'percentage'){
			$referrarAmount = ($referrarAmount * $invoice->getSubTotal())/100;
		}
		
		// Send discount code to Referrar when Referral Code utilized
		if(floatval(0) != floatval($referrarAmount))
		{
			if($this->getAppParam('after_invoice_paid', 0)){
				$invoice->setParam('referrar_id', $referrarId);
				$invoice->setParam('referrar_amount', $referrarAmount);
				$invoice->save();
			}else {
				$this->_createDiscountCodeforReferrar($invoice, $referrarId, $referrarAmount);
				// Add entry in referral table for both referrar and Referral
				$this->_addReferralEntry($invoice, $referrarId, $referrarAmount);
			}
		}
	
		return true;
		
	}	
	
	public function _applyDiscountToReferral($invoice, $referralAmount, $referralCode)
	{
		$isPercentage       = false;
		if($this->getAppParam('referral_amount_type','fixed') === 'percentage'){
			$isPercentage	= true;
		}
		
		$refAmount = $referralAmount;
		//if discount is in terms of percentage then calculate the
	    //percentage amount.
		if($isPercentage){
			$refAmount= ($referralAmount * $invoice->getSubTotal())/100;
		}	
		
		// if amount of applied referral discount is greater than total 
		if( $refAmount > $invoice->getTotal()){
			return XiText::_('COM_PAYPLANS_REFERRAL_ERROR_EXCEED_TOTAL_AMOUNT');
		}
		
		 /**
		 * V.V.IMP : this is very impotant for applying discount in which serial
		 * @see PayplansModifier
		*/
		$serial     = ($isPercentage === true) ? PayplansModifier::PERCENT_DISCOUNT : PayplansModifier::FIXED_DISCOUNT;
		return $this->_addReferrarModifier($referralCode, $refAmount, $serial, $invoice);
	}
	
	function onPayplansInvoiceAfterSave($prev, $newInvoice)
	{
		if( $prev != null && $newInvoice->getStatus() == $prev->getStatus()) {
			return true;
		}
		
		if($newInvoice->getStatus() != PayplansStatus::INVOICE_PAID){
			return true;
		}
		
		$referrarId			= $newInvoice->getParam('referrar_id', 0);
		$referrarAmount  	= $newInvoice->getParam('referrar_amount', 0);
		$referrarUser   	= PayplansUser::getInstance($referrarId);
		$referrarPlan		= $referrarUser->getPlans();
		
		if(!array_intersect($referrarPlan, $this->getPlans())){
			return true;
		}
		
		if($referrarId && $referrarAmount){
			$this->_createDiscountCodeforReferrar($newInvoice, $referrarId, $referrarAmount);
		}
	
		if($referrarId) {
			$referralId  	= $newInvoice->getBuyer();
			$plan 		 	= array_shift($newInvoice->getPlans());
			$invoiceTotal	= $newInvoice->getSubtotal();
			
			// Add entry in referral table for both referrar and Referral
			$this->_addReferralEntry($newInvoice,$referrarId, $referrarAmount);
		}
	}
	
	public function _createDiscountCodeforReferrar($invoice, $referrarId, $referrarAmount)
	{	
		$code 				= substr(number_format(time() * rand(), 0, '', ''), 0, 6); 
	 	$code 				= 'REFERRAL'.$referrarId.'_'.$code;
					
	 	$param 				= '{"extend_time_discount":"000000000000","coupon_amount_type":"fixed","allowed_quantity":"1","reusable":"no","allow_clubbing":"1"}';
     	$discount 			= "( '$code', '$code', 'referral', 1, $referrarAmount , '' , '0000-00-00 00:00:00', '0000-00-00 00:00:00', 1, '$param')";
	 	
      	$discounts 			= 'INSERT INTO `#__payplans_prodiscount` (`title`, `coupon_code`, `coupon_type`, `core_discount`, `coupon_amount`, `plans`, `start_date`, `end_date`, `published`, `params`) VALUES'.$discount;
	 	
      	$db 				= XiFactory::getDBO();
      	$db->setQuery($discounts);
	 	$db->query(); 
	 	
	 	$user				= PayplansUser::getInstance($referrarId);
	 	$data 				= $user->getProperties();
	 	$config 			= JFactory::getConfig();
						 
		$data['fromname']   = $config->get('fromname');
        $data['mailfrom']   = $config->get('mailfrom');
        $data['sitename']   = $config->get('sitename');
        $mail               = JFactory::getMailer();
        $mail->IsHTML(true);

        $currency 			= $invoice->getCurrency();
        $referrarAmount		= PayplansHelperFormat::amount($referrarAmount, $currency);
        
	 	$subject 			= XiText::sprintf( 'PLG_PAYPLANS_REFERRAL_EMAIL_SUBJECT');
	 	$body      			= XiText::sprintf( 'PLG_PAYPLANS_REFERRAL_EMAIL_BODY', $user->getUsername(), $referrarAmount, $code);
        $result    			= $mail->sendMail($data['fromname'],$data['fromname'],$user->getEmail(),$subject,$body);
	 	
        if($result === true) {                  
        	PayplansHelperLogger::log(XiLogger::LEVEL_INFO, XiText::_("PLG_PAYPLANS_REFERRAL_MESSAGE_SEND_SUCCESSFULLY"), $this, array(), 'PayplansAppReferralFormatter');
        }
        else {
      		PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, XiText::_("PLG_PAYPLANS_REFERRAL_MESSAGE_SEND_UNSUCCESSFULL"), $this, array(), 'PayplansAppReferralFormatter', '', true);
       }
	
       return $result;
		
	}

	public function _addReferralEntry($invoice, $referrarId, $referrarAmount)
	{
		$plans 		= $invoice->getPlans(PAYPLANS_INSTANCE_REQUIRE);
		$plan 		= array_pop($plans);
		 	
		$query   	= new XiQuery();
        return $query->insert('`#__payplans_referral`')
         					->set('`referrar_id` = '.$referrarId)
				            ->set('`referral_id` = '.$invoice->getBuyer())
				            ->set('`plan_id` 	 = '.$plan->getId())
					        ->set('`amount`      = '.$referrarAmount)
				            ->dbLoadQuery()
					        ->query();
	}
	
	public function onPayplansApplyDiscount($object, $referrarDiscountCode)
	{

		//do nothing no discount coupon is entered
		if(empty($referrarDiscountCode)){
			return false;
		}

		return $this->_applyDiscountToReferrar($object, $referrarDiscountCode);
	}
	
	public function _applyDiscountToReferrar($object, $referrarDiscountCode)
	{
		$query      	= new XiQuery();
		$discounts		= $query->select('*')
							->from('#__payplans_prodiscount')
							->where('`published` = 1')
							->where('`coupon_code`= '."'".$referrarDiscountCode."'")
							->where('`coupon_type` = "referral"')
							->dbLoadQuery()
		  					->loadObjectList();
		
		$finalResult = false;
		foreach ($discounts as $discount)
		{
			$result = PayplansHelperModifier::getTotalUsage($referrarDiscountCode, 'referral');
			if($result >= 1){
				return XiText::_('COM_PAYPLANS_REFERRAR_ERROR_ALREADY_USED');
			}
			$finalResult = $this->doApplyDiscount($object, $referrarDiscountCode, $discount);
		}
		
		return $finalResult;	
  	
	}
	
	public function doApplyDiscount($object, $referrarDiscountCode, $discount)
	{
		$amount		= $discount->coupon_amount;
		
		// if amount of applied referral discount is greater than total 
		if( $amount > $object->getTotal()){
			return XiText::_('COM_PAYPLANS_REFERRAR_ERROR_EXCEED_TOTAL_AMOUNT');
		}

		return $this->_addReferrarModifier($referrarDiscountCode, $amount, PayplansModifier::FIXED_DISCOUNT, $object);
	}
	
	public function _addReferrarModifier($reference, $amount, $serial, $object)
	{
		// Provide Referrar Discount to user
		$params 			= new stdClass();
		$params->message    = XiText::_('COM_PAYPLANS_APP_REFERRAR_DISCOUNT_MESSAGE');
		$params->type       = $this->getType();
		$params->reference  = $reference;
		$params->frequency  = PayplansModifier::FREQUENCY_ONE_TIME;
		$params->amount     = -$amount;
		$params->percentage = 'fixed';
		$params->serial     = $serial;
									                     
		$object->addModifier($params);
			
		// Change Recurring Plan to Trial + Recurring, so referral discount will not be applicable on next recurrence
		if($object->isRecurring() == PAYPLANS_RECURRING){		
				$objParams = $object->getParams()->toArray();
				$object->setParam('expirationtype', 	'recurring_trial_1');
				$object->setParam('recurrence_count', 	($objParams['recurrence_count']> 0 ) ? $objParams['recurrence_count']-1 : 0);
				$object->setParam('trial_price_1', 		$objParams['price']);
				$object->setParam('trial_time_1', 		$objParams['expiration']);
		}
	
		$object->refresh();
		$object->save();
		return true;
	}
}

class PayplansAppReferralFormatter extends PayplansAppFormatter
{
	public $template	= 'view_log';
	
	// get Ignore data 
	function getIgnoredata()
	{
		$ignore = array('_trigger', '_tplVars', '_mailer', '_location', '_errors', '_component');
		return $ignore;
	}
	
	// get rules
	function getVarFormatter()
	{
		$rules = array('_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       'function' => 'getAppPlans'),
					   'app_params'      => array('formatter'=> 'PayplansAppReferralFormatter',
										       'function' => 'getFormattedContent'));
		return $rules;
	}

	// format email app content,status, expiration time 
	function getFormattedContent($key, $value, $data)
	{	
		$this->template = 'view';
	}
}
