<?php
/**
* @package		PayPlans
* @copyright	Copyright (C) 2010 - 2017 Stack Ideas Sdn Bhd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* PayPlans is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See COPYRIGHT.php for copyright notices and details.
*/
defined('_JEXEC') or die('Unauthorized Access');

class PayplansAppSaferpay extends PayplansAppPayment
{	
	protected $_location	= __FILE__;

	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		if (is_object($data)) {
			$data = (array)$data;
		}
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount = number_format($invoice->getTotal())*100 ;

		$customerId = $this->getAppParam('CustomerId', '');
		$terminalId = $this->getAppParam('TerminalId', '');
		$apiUsername = $this->getAppParam('username', '');
		$apiPassword = $this->getAppParam('password', '');
    	
	    	$root = JURI::root();
	    	if (XiFactory::getConfig()->https == true) {
	    		$root = JString::str_ireplace("http:", "https:", $root);
	    	}

	    	if ($this->getAppParam('sandbox')) {
	       		$posturl = "https://test.saferpay.com/api/Payment/v1/PaymentPage/Initialize";
		} else {
			$posturl = "https://www.saferpay.com/api/Payment/v1/PaymentPage/Initialize";	 
		}

		$returnUrl = $root.'index.php?option=com_payplans&gateway=saferpay&view=payment&task=complete&action=success&payment_key='.$payment->getKey();
		$failUrl = $root.'index.php?option=com_payplans&gateway=saferpay&view=payment&task=complete&action=cancel&payment_key='.$payment->getKey();
		

		$payload = array(
				"RequestHeader" => array(
							 "SpecVersion" => "1.6",
							 "CustomerId" => $customerId,
							 "RequestId" => $invoice->getKey(),
							 "RetryIndicator" => 0
								),
					  "TerminalId"	=> $terminalId,
					  "Payment" => array(
					    	"Amount" => array(
			      				"Value"	=> $amount,
			      				"CurrencyCode" => "CHF"
					    ),
					    "OrderId" => $invoice->getKey(),
					    "Description" => $invoice->getTitle()
					  ),
					  "ReturnUrls" => array(
			    				"Success" => $returnUrl,
		   					"Fail" => $failUrl
					)
				);


		$response = $this->do_post($apiUsername, $apiPassword, $posturl, $payload);
	
		if ($response) {
			if(isset($response['RedirectUrl'])) {		
				$payment_key = $payment->getKey();
				$_SESSION[$payment_key]	= $response['Token'];
				$this->assign('post_url', $response['RedirectUrl']);
				return $this->_render('form');
			} else {
				$message = JText::_('COM_PAYPLANS_LOGGER_ERROR_IN_SAFERPAY_PAYMENT_PROCESS');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
				XiFactory::getApplication()->enqueueMessage("COM_PAYPLANS_PAYMENT_APP_SAFERPAY_NO_REDIRECT_URL",'error');

				return false;
			}
		} else {
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_IN_SAFERPAY_PAYMENT_PROCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);

			XiFactory::getApplication()->enqueueMessage("COM_PAYPLANS_PAYMENT_APP_SAFERPAY_INVALID_RESPONSE",'error');
			return false;
		}     
	}

	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		if ($action == 'cancel') {
			return true;
		}
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$customerId = $this->getAppParam('CustomerId', '');
		$terminalId = $this->getAppParam('TerminalId', '');
		$apiUsername = $this->getAppParam('username', '');
		$apiPassword = $this->getAppParam('password', '');

		$Token = $_SESSION[$data['payment_key']];

		$payload = array(
					"RequestHeader" => array(
						 "SpecVersion" => "1.6",
						 "CustomerId" => $customerId,
						 "RequestId" => $invoice->getKey(),
						"RetryIndicator" => 0
					),
					"Token" => $Token
				);

		if ($this->getAppParam('sandbox')) {
       			$posturl = "https://test.saferpay.com/api/Payment/v1/PaymentPage/Assert";
        	} else {
        		$posturl = "https://www.saferpay.com/api/Payment/v1/PaymentPage/Assert";	 
        	}

		$response = $this->do_post($apiUsername, $apiPassword, $posturl, $payload);
		
		if ($response['Transaction']['Status'] == "AUTHORIZED") {

			$paymentCapture  = array(
							"RequestHeader" => array(
								 "SpecVersion" => "1.6",
								 "CustomerId" => $customerId,
								 "RequestId" => $invoice->getKey(),
								"RetryIndicator" => 0
							),
							"TransactionReference" => array(
								"TransactionId" => $response['Transaction']['Id']
							)
						);

			if ($this->getAppParam('sandbox')) {
       				$captureurl = "https://test.saferpay.com/api/Payment/v1/Transaction/Capture";
			} else {
				$captureurl = "https://www.saferpay.com/api/Payment/v1/Transaction/Capture";	 
			}

        		$paymentResponse = $this->do_post($apiUsername, $apiPassword, $captureurl, $paymentCapture);
        		
			if ($paymentResponse) {
        			$transaction = PayplansTransaction::getInstance();
				$transaction->set('user_id', 			$payment->getBuyer())
							->set('invoice_id', 		$invoice->getId())
							->set('payment_id', 		$payment->getId())
							->set('gateway_txn_id', 	isset($paymentResponse['TransactionId']) 	? $paymentResponse['TransactionId'] 			: 0)
							->set('gateway_subscr_id', 	isset($paymentResponse['OrderId'])          ? $paymentResponse['OrderId'] : 0)
							->set('gateway_parent_txn', 0)
							->set('params', 			PayplansHelperParam::arrayToIni($paymentResponse));

				if ($paymentResponse['Status'] == "CAPTURED") {
	        			$transaction->set('amount', $invoice->getTotal())
						    ->set('message', JText::_('COM_PAYPLANS_PAYMENT_APP_SAFERPAY_TRANSACTION_COMPLETED'));

	        		} else {
	        			$transaction->set('message', JText::_('COM_PAYPLANS_PAYMENT_APP_SAFERPAY_TRANSACTION_FAILED'));
	        		}


			  	//store the response in the payment AND save the payment
				if (!$transaction->save()) {
					$message = JText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
					PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
				}
			}else {
        			$message = JText::_('COM_PAYPLANS_LOGGER_ERROR_IN_SAFERPAY_PAYMENT_PROCESS');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
        		}
		}

		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);	
	}
	


	public function do_post($username, $password, $url, array $payload) 
	{  
		$curl = curl_init($url);
		curl_setopt($curl, CURLOPT_HEADER, false);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_HTTPHEADER, array(
		"Content-type: application/json",
		"Accept: application/json"
		));

		curl_setopt($curl, CURLOPT_POST, true);
		curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($payload));
	
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
		curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($curl, CURLOPT_USERPWD, $username . ":" . $password);
		$jsonResponse = curl_exec($curl);

		$status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		if ($status != 200) {
		  return false;
		}

		curl_close($curl);

		$response = json_decode($jsonResponse, true);
		return $response;
	}

}
