<?php
/**
* @copyright	Copyright (C) 2009 - 2011 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license	GNU/GPL, see LICENSE.php
* @package	PayPlans
* @subpackage	Frontend
* @contact	payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppSid extends PayplansAppPayment
{
	//inherited properties
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
	public function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		$gateway = JRequest::getVar('gateway', '');
		if($view != 'payment' || ($task != 'notify' && $task != 'complete') || $gateway != $this->getType()){
			return true;
		}
		
		$invoiceKey 	= JRequest::getVar('SID_REFERENCE', false);
		$paymentKey		= JRequest::getVar('SID_CUSTOM_02', false);

		if($task == 'complete'){
			$reference = JRequest::getVar('reference', null);
		}
			
 		if($invoiceKey == false || $paymentKey == false){
			return true;
		}
			
		JRequest::setVar('invoice_key', $invoiceKey, 'POST');
		JRequest::setVar('payment_key', $paymentKey, 'POST');
		
		return true;
	}
	
	public function onPayplansPaymentForm(PayplansPayment $payment, $data=null)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		$this->assign('post_url', 'https://www.sidpayment.com/paySID/');
		$this->assign('sid_merchant', $this->getAppParam('merchant', ''));
		
		$currency = $invoice->getCurrency('isocode');
		$this->assign('sid_currency', $currency);
		
		$country = $this->getAppParam('country', '');
		$country = $this->_getCountryCode($country);
		$this->assign('sid_country', $country);
		$this->assign('sid_reference', $invoice->getKey());
		$amount = number_format($invoice->getTotal(), 2);
		$this->assign('sid_amount', $amount);
		// payment key
		$this->assign('sid_custom_01', $payment->getKey());
		
		$consistentKey = strtoupper(hash('sha512', JString::trim($this->getAppParam('merchant', '').$currency.$country
													 .$invoice->getKey().$amount.$payment->getKey()
													 .JString::trim($this->getAppParam('secret', '')))));

		$this->assign('sid_consistent_key', $consistentKey);
		return $this->_render('form');
	}
	
	function _getCountryCode($country)
	{
		$model = XiFactory::getInstance('country', 'model');
		$record = $model->loadRecords(array('country_id' => $country));
		if(empty($record)){
			return false;
		}
		
		return $record[$country]->isocode2;
	}
	
	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{	
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		// If same notification came more than one time
    	// Check if transaction already exist then do nothing and return
		$txn_id			= isset($data['SID_TNXID']) ? $data['SID_TNXID'] : 0;
		$transactions	= $this->_getExistingTransaction($invoice->getId(), $txn_id, 0, 0);
    	if($transactions !== false){
    		foreach($transactions as $transaction){
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			if(strtolower($transaction->getParam('SID_STATUS','')) == strtolower($data['SID_STATUS'])){
    				return true;
    			}
    		}
    	}
    	
		
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $payment->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', isset($data['SID_TNXID']) ? $data['SID_TNXID'] : 0)
					->set('params', PayplansHelperParam::arrayToIni($data));
		
			
		$amount = 0;
		// XITODO : use constant instead of number
		
		if('CREATED' === JString::strtoupper($data['SID_STATUS']) 
			|| 'READY' === JString::strtoupper($data['SID_STATUS'])){
			
			$transaction->set('message', 'COM_PAYPLANS_APP_SID_TRANSACTION_INITIATED');
			$error = XiText::_('COM_PAYPLANS_APP_SID_PAYMENT_INITIATED');
			
		}elseif('CANCELLED' === JString::strtoupper($data['SID_STATUS'])){
			$transaction->set('message', 'COM_PAYPLANS_APP_SID_TRANSACTION_CANCELLED');
			$error = XiText::_('COM_PAYPLANS_APP_SID_PAYMENT_CANCEL');
		
		}elseif('COMPLETED' === JString::strtoupper($data['SID_STATUS'])){
			$transaction->set('message', 'COM_PAYPLANS_APP_SID_TRANSACTION_COMPLETED');	
			$amount = $data['SID_AMOUNT'];		
		}
		else{			
			$transaction->set('message', 'COM_PAYPLANS_APP_SID_TRANSACTION_PENDING');		
		}	
		
		if($error){
				$message 					= XiText::_('COM_PAYPLANS_APP_SID_LOGGER_ERROR_IN_PAYMENT');
				$response					= array();				
				$response['error_message'] 	= $error;
				$response['response']		= $data;
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		}		
		
		$transaction->set('amount', $amount)
				    ->save();
					
		return true;
	}
}
