<?php
/**
 * @copyright 	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
 * @license 	GNU/GPL, see LICENSE.php
 * @package 	payplans
 * @version 	1.0.0
 * @author 		support+customwork <support+customwork@readybytes.in>
 */

if(defined( '_JEXEC' )==false) {
	die( 'Restricted access' );
}

class PayplansAppSmoovPay extends PayplansAppPayment
{
	//inherited properties
	protected $_location	= __FILE__;

	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		$invoice 			= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$post_url 			= $this->getAppParam('sandbox', '1')?'https://sandbox.smoovpay.com/access':'https://secure.smoovpay.com/access';
		$key 				= $this->getAppParam('secret_hash', 0);
		
		$this->assign('post_url', 		$post_url);
		$this->assign('merchant', 		$this->getAppParam('merchant_id', ''));
		$this->assign('ref_id', 		$payment->getKey());
		$this->assign('item_name_1', 	$invoice->getTitle());
		$this->assign('amount', 		$invoice->getTotal());
		$this->assign('currency', 		$invoice->getCurrency('isocode','SGD'));
		//Special case: in smoovpay we need to send payment key as first argument in success/cancel url.
		$this->assign('success_url',	JURI::root().'index.php?payment_key='.$payment->getKey().'&option=com_payplans&gateway=smoovpay&view=payment&task=complete&action=success');
		$this->assign('cancel_url', 	JURI::root().'index.php?payment_key='.$payment->getKey().'&option=com_payplans&gateway=smoovpay&view=payment&task=complete&action=cancel');
		
		$dataToBeHashed 	= $key. $this->getAppParam('merchant_id'. '').'pay'. $payment->getKey(). $invoice->getTotal(). $invoice->getCurrency('isocode','SGD');
		$utfString 			= mb_convert_encoding($dataToBeHashed, "UTF-8");
		$signature 			= sha1($utfString, false);
		
		$this->assign('signature', $signature);
		return $this->_render('form_buynow');
	}
	
	function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller) {
		
		if($action == 'cancel'){
			return true;
		}
		
		if($data['response_code'] != '1')
		{
			$message 	= XiText::_('COM_PAYPLANS_APP_SMOOVPAY_ERROR');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter');
			
			$action		= 'error';
		    return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
			
		}
		
		$invoice 						= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		list($isValidResp, $message) 	= $this->validateResponse($data,$invoice);
		if (!$isValidResp) {
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter');
			
			$action	= 'error';
		    return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		}
		
		//Duplicate Transactions Checking
		$transactions 					= $this->_getExistingTransaction($invoice->getId(), $data['reference_code'], 0, 0);
		if($transactions !== false){
    		foreach($transactions as $transaction){
    			$transaction 			= PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    			$par 					= $transaction->getParam('response_code', '');
    			if($transaction->getParam('response_code', '') == $data['response_code']){
    				return true;
    			}
    		}
    	}
		
		$transaction 	= PayplansTransaction::getInstance();
		$transaction->set('user_id', 			$invoice->getBuyer())
					->set('invoice_id', 		$invoice->getId())
					->set('payment_id', 		$payment->getId())
					->set('amount',				$data['total_amount'])
					->set('message',            XiText::_('COM_PAYPLANS_APP_SMOOVPAY_PAYMENT_COMPLETED_SUCCESSFULLY'))
					->set('gateway_txn_id', 	isset($data['reference_code']) 	? $data['reference_code'] : 0)
					->set('gateway_subscr_id', 	0)
					->set('gateway_parent_txn', 0)
					->set('params', 			PayplansHelperParam::arrayToIni($data))
					->save();	
		
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	
	function validateResponse(array $data,PayplansInvoice $invoice) 
	{
		if (!isset($data)) {
			return array(false, XiText::_('COM_PAYPLANS_APP_SMOOVPAY_RESPONSE_ERROR'));	
		} else {
			$secret_key 		 = $this->getAppParam('secret_hash', 0); 
    		$merchant 			 = $data['merchant'];
		    $ref_id 			 = $data['ref_id']; 
		    $reference_code 	 = $data['reference_code']; 
		    $response_code  	 = $data['response_code']; 
		    $currency 			 = $data['currency'];
		    $total_amount   	 = $data['total_amount'];
		    $signature 			 = $data['signature'];
		    $signature_algorithm = $data['signature_algorithm'];
		
		    $dataToBeHashed 	 = $secret_key.$merchant.$ref_id.$reference_code.$response_code.$currency.$total_amount;

		    $utfString 			 = mb_convert_encoding($dataToBeHashed, "UTF-8");
		    $check_signature 	 = sha1($utfString, false);
		
		    if ($signature !== $check_signature) {
				// signatur does not matched
		        return array(false,XiText::_('COM_PAYPLANS_APP_SMOOVPAY_SIGNATURE_INVALID'));
			}

	        // check if merchant is your merchant email
	        if ($merchant != $this->getAppParam('merchant_id', '')) {
	          return array(false, XiText::_('COM_PAYPLANS_APP_SMOOVPAY_MERCHANT_ERROR'));
	        }
			
			// Check for invoice total
	        if ($invoice->getTotal() != $total_amount) {
	          return array(false, XiText::_('COM_PAYPLANS_APP_SMOOVPAY_TOTAL_AMOUNT_ERROR'));
	        }
	
			//Check for currency
			if ($invoice->getCurrency('isocode','SGD') != $currency) {
			 	return array(false, XiText::_('COM_PAYPLANS_APP_SMOOVPAY_CURRENCY_ERROR'));
		 	}
			          
			return array(true, XiText::_('COM_PAYPLANS_APP_SMOOVPAY_NO_ERROR'));
		}
	}
}
 
