<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Sobipro
* @contact		support+payplans
*/
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Payplans Sobipro Plugin
 */
class plgPayplansSobipro extends XiPlugin
{
	const DO_NOTHING = -1;
	const ALLOWED 	 =  1;
	const BLOCKED    =  0;
	static $restrictedAppCategories = array();

	public function onPayplansSystemStart()
	{
		//add app path to app loader
		$appPath = dirname(__FILE__).DS.'sobipro'.DS.'app';
		PayplansHelperApp::addAppsPath($appPath);
		return true;
	}
	

	
	function onPayplansAccessCheck(PayplansUser $user)
	{
		$userId     = $user->getId();	
		$input		= JFactory::getApplication()->input;
		$option 	= $input->get('option');
		$task 		= $input->get('task', null);
		$entry_id   = $input->get('entry_id', 0);
		
		// do nothing if admin
		if($user->isAdmin()){
			return false;
		}

		// block user to add  new list only
		if($option != 'com_sobipro' ||  ($task != 'entry.submit' && $task != 'entry.edit' && $task != 'entry.publish')){
			return false;
		}

		if($task == 'entry.submit'){

			//more than one categories can be choosen by user
			$categoryIds = $input->get('field_category',array(), 'array');
			
			//section of category in which user want to post
			$sectionId = $input->get('pid',0);

			$categoryIds[] = $sectionId;

			if(self::restrictSubmission($categoryIds,$user))
			{
				return true;
			}
			
			$app = JFactory::getApplication();
			
			$msg = XiText::_('COM_PAYPLANS_APP_SOBIPRO_YOU_ARE_NOT_ALLOWED_TO_ADD_ENTRY_IN_SELECTED_CATEGORY');
			
			$app->enqueueMessage($msg);
			
			JFactory::getSession()->set( 'application.queue', $app->getMessageQueue() );
			
			$url = XiRoute::_('index.php?option=com_payplans&view=plan&task=subscribe');
			echo json_encode(
					array(
							'message' => array( 'text' => $msg, 'type' => 'info' ),
							'redirect' => array( 'url' => $url, 'execute' => ( bool )true ),
							'data' => array()
					)
			);
			exit;			
			
		}
		
		if($task == 'entry.edit' || $task == 'entry.publish')
		{
			//here sid means the entry id from sobipro
			$entry_id    = $input->get('sid', 0);
			
			$categoryId 	= self::_getEntryCategory($entry_id);
						
			if(self::restrictSubmission(array($categoryId),$user))
			{
				return true;
			} 
				
			$msg = XiText::_('COM_PAYPLANS_APP_SOBIPRO_YOU_ARE_NOT_ALLOWED_TO_EDIT_DISABLE_ENTRY');
			XiFactory::getApplication()->redirect(XiRoute::_('index.php?option=com_payplans&view=plan'),$msg);

		}
		
		return true;
	}
	

	static function _getEntryCategory($entryId = 0)
	{
		$query	= new XiQuery();
		return $query->select('`pid`')
					  ->from('`#__sobipro_relations`')
					  ->where("`id`= $entryId")
					  ->dbLoadQuery()
				 	  ->loadResult();
	}
	
	
	static function _getUserEntries($userId, $categoryId = 0)
	{
		$db     = PayplansFactory::getDBO();
		$query = ' SELECT DISTINCT(obj.`id`)'
					  .' FROM `#__sobipro_relations` as rel LEFT JOIN `#__sobipro_object` as obj'
					  .' ON rel.`id` = obj.`id`'
					  .' WHERE `owner`='.$userId. ' AND rel.`oType`= "entry" AND `state`=1'
					  ;
					  
		if($categoryId){
			$allsubcategory = self::getAllSubCategories($categoryId);	
			$allsubcategory[] = $categoryId;	 
			$query .= ' AND rel.`pid` IN ('.implode(',', $allsubcategory).')';
		}
		
		$db->setQuery( $query );
		return $db->loadObjectList('id');
		
	}
	
	protected static function _getSobiproResource($userId,$specific,$categoryId)
    {
		$rmodel = PayplansApp::getResourceModel();
		$filter = array('user_id'  => $userId, 'title' => 'com_sobipro.entry*');
		
		if($specific == true){
			$filter = array('user_id'  => $userId, 'title' => 'com_sobipro.entry'.trim($categoryId));			
		}
		
		$temp = $rmodel->loadRecords($filter);
		$record = array_shift($temp);
		
		if(empty($record) || !$record){
			return false;
		}

		return $record;
	}
	
	static function _isUserAllowed($user, $categoryId)
	{			
		//fetch all the user resource of that category 
		$userResource = self::_getSobiproResource($user->getId(),true,$categoryId);
		if($categoryId == 0)
		{
			$userResource = self::_getSobiproResource($user->getId(),false,$categoryId);
		}
		
		//fetch all the user plans it may be empty
		$userPlans 	 = $user->getPlans();
		
		//if resource is empty/false then plan is not subscribed or app is created
		if(empty($userResource))
		{	
			//check this category is exist in restricted categories if yes then check
			if(array_key_exists($categoryId, self::$restrictedAppCategories))
			{				
				//blocked because if app is created but user doesn't have that required plans
				return self::BLOCKED;
			}
			
			//allowed if no app is created for it
			return self::ALLOWED;
		}
		
		//check no. of count here
		return self::_checkCounting($userResource,$categoryId,$user);

	}	
	
	static function restrictSubmission($categoryIds = array(),$user)
	{
		$childWithParents = array();
			
		//we need to check for all anccesters fo category that they may also been restricted by some app
		foreach ($categoryIds as $categoryId) {
			$temp = self::getAllParents($categoryId);
			$childWithParents = array_merge($childWithParents, $temp);
		}
		
		//we should also check for any category and treat section as category also
		$childWithParents[] = 0;
		//$childWithParents[] = $sectionId;
		
		//fetch all appdata here because it will be used futher many times
		self::$restrictedAppCategories = self::getrestrictedAppCategories();
			
		//filterout the categories on which app are not applied
		$counting = 0;
		foreach ($childWithParents as $categoryId)
		{
			if(!array_key_exists($categoryId, self::$restrictedAppCategories))
			{
				unset($childWithParents[$counting]);
			}
			$counting++;
		}
			
		if(empty($childWithParents))
		{
			return true;
		}
		//now check for individual category and store it's result in array
		$appResult = array();
		foreach ($childWithParents as $categoryId)
		{
			$appResult[] = self::_isUserAllowed($user, $categoryId);
		}
			
			
		//if appresult array contain BLOCKED then it is not allowed to post
		if(in_array(self::BLOCKED, $appResult))
		{
			return false;
		}
			
		return true;
	}
	
	//check whether userentries are more than resource entries or not
	public static function _checkCounting($userResource,$categoryId,$user)
	{
		$userEntries = self::_getUserEntries($user->getId(), $categoryId);

		//this thing have to be done because when user want to edit the published entry 
		//so to remove 1 app extra margin it is checked and allowed
		$entry_id	 = XiFactory::getApplication()->input->get('sid', 0);
		if(isset($entry_id) && key_exists($entry_id,$userEntries))
		{
			return self::ALLOWED;
		}
		
		if(count($userEntries) >= $userResource->count)
		{
			return self::BLOCKED;	
		}
		
		return self::ALLOWED;
	}
	
	public static function getAllSubCategories($cat_id,&$subCategories = array())
	{
		$db     = PayplansFactory::getDBO();
		$query 	= ' SELECT *'
				  .' FROM `#__sobipro_object`'
				  ." WHERE `oType`='category' and `parent`=".$cat_id
				  ;
		
		$db->setQuery( $query );
		$results = $db->loadObjectList();
		
		if(!empty($results)){
			foreach ($results as $res) {
				$subCategories[]=$res->id;
				self::getAllSubCategories($res->id,$subCategories);
			}
		}

		return $subCategories;
	}
	
	//fetch all app restricted categories with their plans in particular fashion
	/*
	 * data----->data[cat1] ------->plan1
	 * 		 |	     		   |--->plan2
	 *		 | 
	 * 		 |-->data[cat2] ------->plan3
	 *  
	 */
	public static function getrestrictedAppCategories()
	{
		$data = array();
		$sobiproApps = PayplansHelperApp::getAvailableApps('sobipro');
		
		foreach ($sobiproApps as $app)
		{
			$sobiproAppType = $app->getAppParam('addEntryIn','any_category');
			
			if($sobiproAppType == 'any_category')
			{
				$appPlans = $app->getPlans();
				if(empty($appPlans))
				{	//apply all plans condition cosider it as 0
					$appPlans = 0;
				}
				
				if(empty($data[0]))
				{
					$data[0]= $appPlans;
				}
				else 
				{
					$data[0] =  array_merge($data[0],$appPlans);
				}
				continue;
			}
		
			$sobiproCategory   = $app->getAppParam('addEntryInCategory');
			if(empty($sobiproCategory))
			{
				$sobiproCategory   = $app->getAppParam('addEntryInSection');
			}
			$sobiproCategories = (is_array($sobiproCategory))?$sobiproCategory:array($sobiproCategory);
			
			foreach ($sobiproCategories as $sobiproCategory)
			{
				$appPlans = $app->getPlans();
				if(empty($appPlans))
				{
					//apply all plans condition cosider it as 0
					$appPlans = 0;
				}
				if(empty($data[$sobiproCategory]))
				{
					$data[$sobiproCategory]=$appPlans;
				}
				else
				{
					$data[$sobiproCategory] = array_merge($data[$sobiproCategory],$appPlans);
				}
			}
		}
	
		return  $data;
	}
	
	//fetch all the parent categories of specified category
	public static function getAllParents($categoryId,&$parentwithchild = array())
	{
		$parentwithchild[] = $categoryId;
		
		//return all parent including self
		$db     = PayplansFactory::getDBO();
		$query 	= ' SELECT *'
				  .' FROM `#__sobipro_object`'
				  ." WHERE `oType`='category' and `id`=".$categoryId
				  ;
		
		$db->setQuery($query);
		$results = $db->loadObjectList();
		
		if(empty($results)){
			return $parentwithchild;
		}
		
		$result = array_shift($results);
		
		if($result->parent == 0)
		{
			return $parentwithchild;
		}
		
		
		self::getAllParents($result->parent,$parentwithchild);
		
		return $parentwithchild;
		
	}
}
