<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Sobipro
* @contact 		support+payplans@readybytes.in
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppSobipro extends PayplansApp
{
	protected $_location	= __FILE__;
	protected $_resource	= 'com_sobipro.entry';

	// applicable only if sobipro exist
	public function _isApplicable(PayplansIfaceApptriggerable $refObject, $eventname='')
	{
		return JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_sobipro');
	}
	
	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}

		$newStatus  = $new->getStatus();
		$entryToAddIn = $this->getAppParam('addEntryIn', 'any_category');
		$restrictedcategories = $this->getAppParam('addEntryInCategory', 0);
		$publishedEntries = $this->getAppParam('entriesToPublish', 0);
		$restrictedsections = $this->getAppParam('addEntryInSection', 0);
		
		$subid  = $new->getId();
		$userid = $new->getBuyer();
		
		$restrictedcategories = (is_array($restrictedcategories))?$restrictedcategories:array($restrictedcategories);
		$restrictedsections   = (is_array($restrictedsections))?$restrictedsections:array($restrictedsections);
		
		
		if($newStatus == PayplansStatus::SUBSCRIPTION_ACTIVE){
			$action = 1;//action is publish/feature
			
			if($entryToAddIn == 'on_specific_category'){
				foreach ($restrictedcategories as $restrictedcategory){
					$this->_addToResource($subid, $userid, $restrictedcategory, $this->_resource.$restrictedcategory, $publishedEntries);
					$this->_publishOrUnpublishCategoryEntry($userid, $publishedEntries, $action, $new,$restrictedcategory);
				}
			}
			elseif ($entryToAddIn == 'on_section'){
				foreach ($restrictedsections as $restrictedsection){
					$this->_addToResource($subid, $userid, $restrictedsection, $this->_resource.$restrictedsection, $publishedEntries);
					$this->_publishOrUnpublishSectionEntry($userid, $publishedEntries, $action, $new,$restrictedsection);
				}
			}
			else {
				$this->_addToResource($subid, $userid, 0, $this->_resource.'*', $publishedEntries);
				$this->_publishOrUnpublishCategoryEntry($userid, $publishedEntries, $action, $new);
			}

		}
		
		if(($prev != null && $prev->getStatus() == PayplansStatus::SUBSCRIPTION_ACTIVE) &&
			 ($newStatus == PayplansStatus::SUBSCRIPTION_EXPIRED || $newStatus == PayplansStatus::SUBSCRIPTION_HOLD)){
			$action = 0;//action is unpublish/unfeatured
			
			if($entryToAddIn == 'on_specific_category'){
				foreach ($restrictedcategories as $restrictedcategory){
					$this->_publishOrUnpublishCategoryEntry($userid, $publishedEntries, $action, $new,$restrictedcategory);
					$this->_removeFromResource($subid, $userid,$restrictedcategory, $this->_resource.$restrictedcategory, $publishedEntries);
				}
			}
			elseif ($entryToAddIn == 'on_section'){
				foreach ($restrictedsections as $restrictedsection){
					$this->_removeFromResource($subid, $userid,$restrictedsection, $this->_resource.$restrictedsection, $publishedEntries);
				 	$this->_publishOrUnpublishSectionEntry($userid, $publishedEntries, $action, $new,$restrictedsection);
				}
			}
			else {
				$this->_publishOrUnpublishCategoryEntry($userid, $publishedEntries, $action, $new);
				$this->_removeFromResource($subid, $userid,0, $this->_resource.'*', $publishedEntries);
			}
		}

		return true;	
	}


	protected function _publishOrUnpublishSectionEntry($userid, $publishCount, $action, $subscription,$section_id = 0)
	{
		// publish only those entries which are not currently published
		//and vice versa
		$complement = 1 - $action;
		$db    		= XiFactory::getDBO();
		
		
		
		//change enable state of all the fields related with the entry
		$query2 = ' UPDATE `#__sobipro_field_data`'
				 .' SET `enabled`='.$action 
				 .' WHERE `createdBy`='.$userid.' AND `enabled`='.$complement.' AND `section`='.$section_id.' AND `sid` IN ( 
				 		select `id` from #__sobipro_object WHERE `owner`='.$userid.' AND `oType`="entry"'
				 .') LIMIT '.$publishCount;
				
		$db->setQuery( $query2 );
		$db->query();
		
		//change the state of the entry in object table
		$query1	 = ' SELECT `sid`'
				  .' FROM `#__sobipro_field_data`'
				  .' WHERE `section`='.$section_id.' AND `createdBy`= '.$userid.' AND `enabled`='.$complement
				  .' LIMIT '.$publishCount;
		
		$db->setQuery($query1);
		$enteries  = $db->loadColumn();
		$enteries  = array_unique($enteries);		

		if(count($entries)) {
			$enteries  = implode(',', $enteries);
			$enteries  =" AND `id` IN ($enteries)";
		} else {
			$enteries  ="";
		}

		$query3 = ' UPDATE `#__sobipro_object`'
				 .' SET `state`='.$action.',`stateExpl`=""'
				 .' WHERE `owner`='.$userid.' AND `oType`="entry" AND `state`='.$complement
				 .$entries
				 .' LIMIT '.$publishCount;
				 
		$db->setQuery( $query3);
		$db->query();

	}
	
	protected function _publishOrUnpublishCategoryEntry($userid, $publishCount, $action, $subscription,$categoryId = 0)
	{
		// publish only those entries which are not currently published
		//and vice versa
		$complement = 1 - $action;
		
		if($categoryId != 0){
			$restricted_cat   = plgPayplansSobipro::getAllSubCategories($categoryId);
			$restricted_cat[] = $categoryId;
		}
		
		$db     = PayplansFactory::getDBO();
		$Ids    = implode(',', $restricted_cat);
		$query 	= ' SELECT `id`'
				  .' FROM `#__sobipro_relations`'
				  ." WHERE `oType`='entry' and `pid` IN ('$Ids')";
		
		$db->setQuery( $query );
		$sectionIds = $db->loadObjectList();
		foreach ($sectionIds as $id){
			$entryIds[]  = $id->id;
		}
		
		
		//change enable state of all the fields related with the entry
		$entries	= implode(',',$entryIds);
		$query1		= 'Update `#__sobipro_field_data`'
					  .' SET `enabled`='.$action
					  .' WHERE `createdBy`='.$userid.' AND `enabled`='.$complement;
		if($categoryId != 0 && !empty($entries)){
			$query1.=' AND `sid` IN ('.$entries.')';		  
		}
		$query1.=' LIMIT '.$publishCount;
				
		$db->setQuery( $query1 );
		$db->query();
		
		//change the state of the entry in object table
		$query2 = ' UPDATE `#__sobipro_object`'
				 .' SET `state`='.$action.',`stateExpl`=""'
				 .' WHERE `owner`='.$userid.' AND `oType`="entry" AND `state`='.$complement;

		if($categoryId != 0 && !empty($entries)){
			$query2.=' AND `id` IN ('.$entries.')';
		}
		$query2.=' LIMIT '.$publishCount;
				 
		$db->setQuery( $query2 );
		$db->query();

	}
	
	//render Widget
	public function renderWidgetHtml()
	{   
		//get user id
		$userid     = XiFactory::getUser()->id;
		if(!$userid){
			return '';
		}

		$sobipro_entries = $this->getSobiproResource($userid);

	    if(empty($sobipro_entries))
	    	return '';

	    $this->assign('sobipro_entries',$sobipro_entries);
        $data = $this->_render('widgethtml');
        return $data;
	}
	
	protected function _getUserSobiproEntries($userid)
	{
		$UserEntries = array();
		$db    = XiFactory::getDBO();
		$query =  ' SELECT `id`, `name`'
				 .' FROM `#__sobipro_object`'
				 .' WHERE `owner`= '.$userid.' AND `oType`="entry" '
				 ;

		$db->setQuery( $query );
		return $db->loadObjectList('id');
	}
	
	protected function getSobiproResource($userId)
    {
		$rmodel = PayplansApp::getResourceModel();
		$value 	= "'".'com_sobipro.entry%'."'";
		$records = $rmodel->loadRecords(array('user_id' => $userId,'title' => array(array('LIKE',$value))));
		foreach ($records as $record){
			$record->title	  = str_replace('com_sobipro.entry', '',$record->title);
			if($record->title == '*'){
				$record->title = 0;
			}
			$record->consumed = count(plgPayplansSobipro::_getUserEntries($userId,$record->title));
			$record->title 	  = $this->getTitleForCategroy($record->title);
			
		}
		
		
		return $records;
	}
	
	function getTitleForCategroy($cat_id)
	{
		if($cat_id == 0){
			return XiText::_("COM_PAYPLANS_APP_SOBIPRO_ANY_CATEGORY");
		}
		$db     = XiFactory::getDBO();
		$query 	=  ' SELECT name'
				  .' FROM `#__sobipro_object` as obj'
				  ." WHERE `id`=".$cat_id
				  ;
		
		$db->setQuery( $query );
		return $results = $db->loadResult();
	}	
}

