<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license	GNU/GPL, see LICENSE.php
* @package	Payplans
* @subpackage	Subscription Approval
* @contact 	payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppSubscriptionapproval extends PayplansApp
{
	//inherited properties
	protected $_location	= __FILE__;

	public function _isApplicable(PayplansIfaceApptriggerable $refObject, $eventName='')
	{
		return (XiFactory::getApplication()->isAdmin() == false);
	}
	
	public function collectAppParams(array $data)
	{
		// encode editor content
		if(isset($data['app_params']) && isset($data['app_params']['body_user'])){
			$data['app_params']['body_user'] = base64_encode($data['app_params']['body_user']);
		}

		if(isset($data['app_params']) && isset($data['app_params']['body_admin'])){
			$data['app_params']['body_admin'] = base64_encode($data['app_params']['body_admin']);
		}
		
		return parent::collectAppParams($data);
	}
	
	public function onPayplansSubscriptionBeforeSave($prev, $new)
	{
		static $processed = array(); 
		if(($prev == null || $prev->getStatus() != PayplansStatus::SUBSCRIPTION_ACTIVE) && $new->getStatus() == PayplansStatus::SUBSCRIPTION_ACTIVE){			
			
			$new->set('status', PayplansStatus::SUBSCRIPTION_HOLD);

			//stop subsciption to be started from here, wait for to be approved
			$new->set('expiration_date', null);
			$new->set('subscription_date', null);

			// if already processed
			if($new->getId() && isset($processed[$new->getId()])){				
				return true;
			}
			
			// email to user		
			$to 	    = $new->getBuyer(PAYPLANS_INSTANCE_REQUIRE)->getEmail();
			$subject	= $this->getAppParam('subject_user', '');
			$subject	= PayplansFactory::getRewriter()->rewrite($subject, $new);
			$body	    = base64_decode($this->getAppParam('body_user', ''));			
			$mailBody   = PayplansFactory::getRewriter()->rewrite($body, $new);		
			
			$mailer  	= XiFactory::getMailer();
			$mailer->setSubject($subject);
			$mailer->setBody($mailBody);
			$mailer->addRecipient($to);	
			$mailer->IsHTML(true);	
			
			if($mailer->Send() instanceof JException){
				$message= XiText::_('COM_PAYPLANS_EMAIL_SENDING_FAILED');
	            PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, 'PayplansInvoice', $mailBody);				
			}
			else{
				$message=XiText::_('COM_PAYPLANS_EMAIL_SEND_SUCCESSFULLY');
		        PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message,'PayplansInvoice', $mailBody);
			}
			
			
			// email to admin
			$emails = $this->getAppParam('admin_emails', '');
			$emails = explode(',', $emails);
			
			XiFactory::$mailer = null;
			$mailer  	= XiFactory::getMailer();
			$subject	= $this->getAppParam('subject_admin', '');
			$subject	= PayplansFactory::getRewriter()->rewrite($subject, $new);
			$body	    = base64_decode($this->getAppParam('body_admin', ''));			
			$mailBody   = PayplansFactory::getRewriter()->rewrite($body, $new);
					
			$mailer->setSubject($subject);
			$mailer->setBody($mailBody);			
			
			$mailer->addRecipient($emails);
			$mailer->IsHTML(true);
			if($mailer->Send() instanceof JException){
				$message= XiText::_('COM_PAYPLANS_EMAIL_SENDING_FAILED');
	            PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $this, $mailBody, 'PayplansAppSubscriptionapprovalFormatter', '', true);				
			}
			else{
				$message=XiText::_('COM_PAYPLANS_EMAIL_SEND_SUCCESSFULLY');
		        PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $this, $mailBody, 'PayplansAppSubscriptionapprovalFormatter');
			}
			
			$processed[$new->getId()] = true;
		}
	}
}

class PayplansAppSubscriptionapprovalFormatter extends PayplansAppFormatter
{
	public $template	= 'view_log';
	
	// get Ignore data 
	function getIgnoredata()
	{
		$ignore = array('_trigger', '_tplVars', '_mailer', '_location', '_errors', '_component');
		return $ignore;
	}
	
	// get rules
	function getVarFormatter()
	{
		$rules = array('_appplans'       => array('formatter'=> 'PayplansAppFormatter',
										       'function' => 'getAppPlans'),
					   'app_params'      => array('formatter'=> 'PayplansAppSubscriptionapprovalFormatter',
										       'function' => 'getFormattedContent'));
		return $rules;
	}

	function getFormattedContent($key, $value, $data)
	{
		foreach ($value as $param=>$v)
		{
			if($param == 'body_user'){
				$value[$param] = base64_decode($v);
			}
			if($param == 'body_admin'){
				$value[$param] = base64_decode($v);
			}
		}
		
		$this->template = 'view';
	}
}
