<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Subscription Detail
* @contact 		support+payplans@readybytes.in
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppSubscriptiondetail extends PayplansApp
{
	protected $_location	= __FILE__;
	
	public function isApplicable($refObject = null, $eventName='')
	{
		// support token rewriting
		if($eventName == 'onPayplansRewriterReplaceTokens'){
			return true;
		}
		
		// not applicable in admin panel except subscription edit screen
		// for displaying subscription details at subscription edit screens 
		// check isApplicable on PayplansSubscription Instance
		if($refObject instanceof PayplansadminViewSubscription){
			$itemid  = $refObject->getModel()->getState('id');
			$subscription    = PayplansSubscription::getInstance($itemid);
			
			//if applicable to all -> applicable
			//IMP : Carefully, get all plans (even expired and not-activated) of user
		 	// if object is of interest as per plans selected
			$ret = array_intersect($this->getPlans(), $subscription->getPlans(null));
			if(count($ret) > 0 || $this->getParam('applyAll',false) != false){
				return true;
			}
		}
		
		// On order screen we need to ask subscription information, 
		// during this subscription is not active
		if($refObject instanceof PayplanssiteViewInvoice){
			$model = $refObject->getModel();
			if($model){
				$libInstance = XiLib::getInstance($refObject->getName(),$model->getId());
				return parent::isApplicable($libInstance,$eventName);
			}
		}
				
		return parent::isApplicable($refObject,$eventName);
	}
	
	public function onPayplansViewBeforeRender(XiView $view, $task)
	{
		$layout  =  'orderconfirm';

		//app will ask subscription details at order confirm page only
		if ($view instanceof PayplanssiteViewInvoice && $task == 'confirm'){
			$itemid       = $view->getModel()->getId();
			$orderId 	  = PayplansInvoice::getInstance($itemid)->getObjectId();
			$subscription = PayplansOrder::getInstance($orderId)->getSubscription();
			return self::renderSubscriptionDetail($subscription, $layout);
		}
		
		// when data needs to be displayed in backend
		// then different layout is set that is subscriptionedit
		if($view instanceof PayplansadminViewSubscription && $task == 'edit'){
			$itemid  	  =  $view->getModel()->getId();
			$layout 	  = 'subscriptionedit';
			$subscription = PayplansSubscription::getInstance($itemid);
			return self::renderSubscriptionDetail($subscription, $layout);
		}

		return false;
	}
	
	public function onPayplansInvoiceAfterSave($prev, $new)
	{
		// check the status, save data when order status is confirmed
		if($new->getStatus() ==  PayplansStatus::NONE){
			return true;
		}

		$subscriptiondetails = JRequest::getVar("subscription_detail", 'BLANK', 'POST');
		if(empty($subscriptiondetails) || $subscriptiondetails == 'BLANK'){
			return false;
		}
		
		$subscription = $new->getReferenceObject(PAYPLANS_INSTANCE_REQUIRE)->getSubscription();
		
		//set subscription params
		foreach($subscriptiondetails as $param => $value ){
			$subscription->setParam($param , $value);
		}

		$subscription->save();
		
		return true;
	}
	
	public function onPayplansSubscriptiondetailSave(PayplansSubscription $subscription, $data)
       {
               $location = $this->getLocation();
               $path           = $location.DS.'xmldata'.DS."subscriptiondetail_app_{$this->getId()}.xml";
			   $xml      = file_get_contents($path);
               if (trim($xml) == '') {
                     return true;
                }
               $form    		= Jform::getInstance('subscription.subscriptionDetail'.$this->getId(),$path,array(), true);
           		$fields   = $form->getFieldset('subscription_detail');

               foreach($fields as $field){
                       $key     = $field->__get('fieldname');
                       $dataKey = 'subscription_detail['.$key.']';
                       if(isset($data[$field->__get('name')])){
                               $subscription->setParam($key, $data[$dataKey]);
                       }
               }

		//IMP : do not save subscription, parent event will take care of saving it.
		return true;
	}
	
	/**
	 *  render subscription details
	 */
	public function renderSubscriptionDetail(PayplansSubscription $subscription ,$layout, $appid = 0)
	{
		if(!$appid){
			$appid = $this->getId();
		}
		
		$location   = $this->getLocation();
		$path     = $location.DS.'xmldata'.DS."subscriptiondetail_app_$appid.xml";
		$xml     = file_get_contents($path);
        if (trim($xml) == '') {
                 return array();
         }
		$form 		= JForm::getInstance('subscription.subscriptionDetail'.$appid, $path ,array(),true);
		$params 	= $subscription->getParams();
		$newData 	= array('subscription_detail' => $params->toArray());
		$form->bind($newData);
		
		//return parameter in case of Order-display widgets
		if($layout == 'order'){
			return $form;
		}
	
		$this->assign('location',$location);
		$this->assign('subscriptionparams', $form);
		return array('pp-subscription-details' => $this->_render($layout));
	} 

	/**
	 *  Replace tokens value
	 */
	function onPayplansRewriterReplaceTokens($refObject, $rewriter)
	{
		$subscription    = false; 
		if($refObject instanceof PayplansSubscription){
			$subscription = $refObject;
		}
		
		elseif($refObject instanceof PayplansOrder){
			$subscription =  $refObject->getSubscription();
		}
		
		elseif( $refObject instanceof PayplansInvoice && $refObject->getReferenceObject(PAYPLANS_INSTANCE_REQUIRE) instanceof PayplansOrder){
			$subscription = $refObject->getReferenceObject(PAYPLANS_INSTANCE_REQUIRE)->getSubscription();
		}
		
		if(!$subscription){
			return true;
		}
		
		if(!$subscription && !($refObject instanceof PayplansSubscription)){
			return true;
		}
		
		$param 		= (!$subscription) ? $refObject->getParams() : $subscription->getParams();
		$data 		= (object) $param->toArray();
		$data->name = 'Subscriptiondetail';
		
		$rewriter->setMapping($data, false);
		return true;

	}
	
	/** 
	 * render widget on order display page
	 */
	 function renderWidget($subscription, $layout)
	{
		$user = XiFactory::getUser();
		if(!$user->id){
			return false;
		}
		
		$subscriptiondetail  = array(); 
		$subscriptiondetails = array();
		// get all sbscriptiondetail app
		foreach (PayplansHelperApp::getApplicationApps('subscriptiondetail', $subscription) as $app){
	        if($app->getAppParam('showOnSubscriptionDisplay')){
	        	$subscriptiondetails[] = $this->renderSubscriptiondetail($subscription, 'order', $app->app_id);
			}
		}

		foreach ($subscriptiondetails as $detail){
			if(!empty($detail)){
				$subscriptiondetail[]	= $detail;
			}
		}

		if(empty($subscriptiondetail)){
			return '';
		}
		
	    $this->assign('details',$subscriptiondetail);
	    $this->assign('subscription_key', $subscription->getKey());
        return array('pp-subscription-details' => $this->_render($layout));
	}

	public function collectAppParams(array $data)
	{
		if(isset($data['app_params']) && isset($data['app_params']['subscriptiondetailparams']) && !empty($data['app_params']['subscriptiondetailparams'])){
			//replace '&' if exist
			$data['app_params']['subscriptiondetailparams'] = str_ireplace("&", "&amp;",$data['app_params']['subscriptiondetailparams']);			
			
			// check for params name and generate warning to handle param conflicts
			
			$params = simplexml_load_string((string)$data['app_params']['subscriptiondetailparams']);
			$params = $params->children()->children()->children();

			// array of names of default subcription params
			$paramNames = array('notes',
								'expirationtype',
								'expiration',
								'recurrence_count',
								'trial_price_1',
								'trial_price_2',
								'trial_time_1',
								'trial_time_2',
								'price',
								'currency');

			foreach ($params as $child){
				$attributes = (array)$child->attributes();
				if(isset($attributes['@attributes']) && 
                   isset($attributes['@attributes']['name']) &&  
				   in_array($attributes['@attributes']['name'],$paramNames)) {
					   $param_conflicts .= $attributes['@attributes']['name'].",";
				} 
			}
			if(isset($param_conflicts)){
					XiError::raiseWarning(1001, XiText::sprintf('COM_PAYPLANS_APP_SUBSCRIPTIONDETAIL_USE_OF_PARAM_NAME_MAY_GENERATES_SOME_CONFLICT',substr($param_conflicts, 0, -1)));
					return false;
			}
			$data['app_params']['subscriptiondetailparams'] = base64_encode($data['app_params']['subscriptiondetailparams']);
		}

		return parent::collectAppParams($data);
	}
}
