<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Subscriptionstartdate
* @contact 		support+payplans@readybytes.in
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppSubscriptionstartdate extends PayplansApp
{
	protected $_location	= __FILE__;
	
	public function isApplicable($refObject = null, $eventName='')
	{
		$option = JRequest::getVar('option');
		if($eventName === 'onPayplansViewBeforeRender'){
			return true;
		}

		return parent::isApplicable($refObject, $eventName);
	}
	
	protected function _checkActivationDate($new, $activationDate)
	{
		$activationOn     	= new XiDate($activationDate);
		$subscriptionDate	= new XiDate('now');
		
		$planType         = $new->getExpirationType();
		if($planType  == 'fixed'){
			$expTime 			= $new->getParams()->get('expiration', 000000000000);
			$newDate			= new XiDate($activationDate);
			$ExpirationOn 		= $newDate->addExpiration($expTime);
			
		}

		$from = $this->getAppParam('subscription_from', 0);
		$to   = $this->getAppParam('subscription_to', 0);
		
		$from = !empty($from) ? new XiDate($from) : $from;
		$to   = !empty($to)   ? new XiDate($to)   : $to;

		if(!empty($from) && !empty($to) && ( ($subscriptionDate->toUnix() > $to->toUnix()) || ($subscriptionDate->toUnix() < $from->toUnix()))){
			return true;
		}
		
		//when range is not set then change exp date anyway
		if(empty($from) && empty($to)){
			return $this->_changeActivationDate($new, $activationOn, $ExpirationOn);
		}
		
		// when range is set then check subscription date whether lies in that range 
		if(!empty($from) && !empty($to) && ($subscriptionDate->toUnix() >= $from->toUnix()) && ($subscriptionDate->toUnix() <= $to->toUnix())){
			return $this->_changeActivationDate($new, $activationOn, $ExpirationOn);
		}                                             
		
		//when start date is set
		if(!empty($from) && $subscriptionDate->toUnix() >= $from->toUnix()){
			return $this->_changeActivationDate($new, $activationOn, $ExpirationOn);
		}
		
		//when end date is set
		if(!empty($to) && $subscriptionDate->toUnix() <= $to->toUnix()){
			return $this->_changeActivationDate($new, $activationOn, $ExpirationOn);
		}
		
		return false;
	}
	
	public function _changeActivationDate($new, $activationDate, $expirationDate)
	{
		$new->set('subscription_date', 				$activationDate);
		$new->set('expiration_date', 				$expirationDate);
		$new->set('status', 						PayplansStatus::NONE);
		$new->setParam('is_scheduled_activation', 	1);
		$new->save();
		
		return true;
	}
	

	public function onPayplansOrderAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}	
		
		if($new->getStatus() == PayplansStatus::ORDER_COMPLETE){
			$subscription = $new->getSubscription();
			if(is_a($subscription, 'PayplansSubscription')){
				$subscription->load($subscription->getId());

				if($subscription->isRecurring()){
					return true;
				}
				
				$startOn = $this->getAppParam('start_date', 0);
				if(empty($startOn)){
					return true;
				}

				$this->_checkActivationDate($subscription, $startOn);	
				return $this->_sendMail($subscription, $startOn);
			}
		}
	}

	protected function _sendMail($subscription, $startOn)
	{	
		$user		= $subscription->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		$plan 		= $subscription->getPlans(PAYPLANS_INSTANCE_REQUIRE);
		$plan		= array_shift($plan);
		$startDate	= new XiDate($startOn);
		$sub_date	= PayplansHelperFormat::date($startDate);
		
		$mailer  	= XiFactory::getMailer();
		$mailer->IsHTML(true);
		$subject 	= XiText::sprintf('COM_PAYPLANS_SUBSCRIPTION_START_DATE_SUBJECT', $plan->getTitle());
		$subject   	= PayplansFactory::getRewriter()->rewrite($subject, $subscription);
		$mailer->setSubject($subject);
		
		$message	= XiText::sprintf('COM_PAYPLANS_SUBSCRIPTION_START_DATE_MESSAGE',$user->getUsername(), $plan->getTitle() ,$sub_date);
		$message	= PayplansFactory::getRewriter()->rewrite($message, $subscription);
		
		$mailer->setBody($message)->isHtml(true);
			
		$email	= $user->getEmail();
		$mailer->addRecipient($email);
		$mailer->Send();
	}
	
}