<?php 
/**
* @copyright	Copyright (C) 2009 - 2016 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license	    GNU/GPL, see LICENSE.php
* @package	    PayPlans
* @subpackage	Upgrade
* @contact 	    support+payplans@readybytes.in
*/

class PayplansHelperUpgrade
{
	const APPLY_TRIAL_ALWAYS   =  1;
	const APPLY_TRIAL_NEVER    =  0;
	
	static function calculateUnutilizedValue(PayplansSubscription $oldSub)
	{
		//IMP : If subscription is no more active, value as 0
		if($oldSub->getStatus() != PayplansStatus::SUBSCRIPTION_ACTIVE){
			return array('paid' => 0, 'unutilized'=> 0);
		}
		//$oldPlan  = array_shift($oldSub->getPlans(true));
		
		// find value utilized by old subscription
		$start   = intval($oldSub->getSubscriptionDate()->toUnix());
		$expires = intval($oldSub->getExpirationDate()->toUnix());
		$now     = intval(XiDate::getInstance()->toUnix());
		
		$totalTime	= $expires - $start;
	
		$oldOrder = $oldSub->getOrder(true);
		// calculate sum of payments made during previous upgradation
		$totalValue = self::_calculatePaymentsDuringPreviousUpgradations($oldOrder);
		$usedTax    = 0;
		
		// free subscription OR life time subscription
		if($totalValue['planPrice'] ==0 || $expires == 0){
			$usedValue = 0;
		}else{
			$used  		= $now - $start;
			// if total time is not in hours, then calculate as per days
			$oneday = 24*60*60;

			if($totalTime> 3*$oneday){
				$used 		= intval($used/$oneday);
				$totalTime 	= intval($totalTime/$oneday);
			}

			$usedValue  = $totalValue['planPrice'] * $used / $totalTime;
			$usedTax =  $totalValue['taxIncluded'] * $used / $totalTime;
		}

		// the value which is not utilized, and will be added into discount
		$unutilizedValue = $totalValue['planPrice'] - $usedValue;
		$unutilizedTax   = $totalValue['taxIncluded'] - $usedTax;

		return array('paid' => $totalValue['planPrice'], 'unutilized'=>$unutilizedValue ,'unutilizedTax'=> $unutilizedTax  );
	}
	
	//return the plans to which upgrade is available from the provided subscription plan 
	static function findAvailableUpgrades($subKey)
	{
		$subId 			= XiHelperUtils::getIdFromKey($subKey);
		$subscription 	= PayplansSubscription::getInstance($subId);
		$plans 			= array();

		if ( PayplansStatus::SUBSCRIPTION_ACTIVE != $subscription->getStatus()) {
			return $plans;
		}

		$plans 	= $subscription->getPlans(true);
		$plan  	= array_shift($plans);
		// should return plans' instances
		$args = array();
		$results = PayplansHelperEvent::trigger('onPayplansUpgradeTo', $args, '', $plan);
		
		foreach($results as $result){
			foreach($result as $plan){
				$plans[$plan->getId()] = $plan;
			}
		}
		
		return $plans;
	}
	
	protected static function _calculatePaymentsDuringPreviousUpgradations($order)
	{
		//$upgradedFrom = $order->getParam('upgrading_from',0);
		
		// get payments
		$invoices = $order->getInvoices(PayplansStatus::INVOICE_PAID);
		if(count($invoices) == 0){
			// none of payment were completed
			$totalValue  = 0;
		}
		else {
            // pick last invoice
            $invoice = array_pop($invoices);
            $modifiers 	 = $invoice->getModifiers();
            $modifiers 	 = PayplansHelperModifier::_rearrange($modifiers);
			$discountableAddon = array(0);
			$addonTaxableTotal=array(0);
			$taxValue =array(0);
			$adjustment=0;
			foreach ($modifiers as $modifier) {
				if (in_array($modifier->getSerial(), array(PayplansModifier::PERCENT_OF_SUBTOTAL_DISCOUNTABLE,PayplansModifier::PERCENT_OF_SUBTOTAL_TAXABLE))) {
					$addonTaxableTotal[] = str_replace('-', '', PayplansHelperFormat::displayAmount($modifier->_modificationOf));
				}

				if (in_array($modifier->getSerial(), array(PayplansModifier::PERCENT_OF_SUBTOTAL_DISCOUNTABLE))) {
					$discountableAddon[] = str_replace('-', '', PayplansHelperFormat::displayAmount($modifier->_modificationOf));
				}

				if(in_array($modifier->getSerial(),
					array(
						PayplansModifier::FIXED_TAX,PayplansModifier::PERCENT_TAX)))
				{
					$taxValue[] = $modifier->_modificationOf;
				}
				if(in_array($modifier->getSerial(),array(PayplansModifier::FIXED_NON_TAXABLE)))
				{
					$adjustment = PayplansHelperFormat::displayAmount($modifier->_modificationOf);
				}
		}

			$plandiscountableamount = $invoice->getSubtotal()+$adjustment;
			if($plandiscountableamount > 0 && array_sum($discountableAddon) > 0){
				$discountApplicableonAddon = (array_sum($discountableAddon)* $invoice->getDiscount()) / ($plandiscountableamount +array_sum($discountableAddon));
			}else {
				$discountApplicableonAddon = 0;
			}
			

			$finalAddonTotal= array_sum($addonTaxableTotal) - $discountApplicableonAddon;

			if($plandiscountableamount > 0 && array_sum($discountableAddon) > 0){
				$discountAmountofplan = ($plandiscountableamount * $invoice->getDiscount()) / ($plandiscountableamount +array_sum($discountableAddon));
			}else {
				$discountAmountofplan = 0;
			}
			

			$totalAmountofPlan= $plandiscountableamount- $discountAmountofplan;
			if($totalAmountofPlan > 0 && $finalAddonTotal > 0){
				$taxAmount = ($totalAmountofPlan * array_sum($taxValue) ) / ( $totalAmountofPlan+ $finalAddonTotal);
			}else {
				$taxAmount = 0;
			}
			
			
		}
		return array('planPrice'=>$totalAmountofPlan,'taxIncluded'=>$taxAmount);
	}
	
	static function willTrialApply($oldPlan, $newPlan)
	{
		$upgradeApps 	= PayplansHelperApp::getAvailableApps('upgrade');
		foreach($upgradeApps as $app)
		 {
		 	$upgradeTo = $app->getAppParam('upgrade_to',array());
		 	$upgradeTo = is_array($upgradeTo) ? $upgradeTo : array($upgradeTo);
		 	$willTrialApply = $app->getAppParam('willTrialApply','always');
			if($app->getParam('applyAll') && in_array($newPlan->getId(),$upgradeTo) ){
				if($willTrialApply == 'always'){
					return self::APPLY_TRIAL_ALWAYS;
				}
			}	
			elseif(in_array($oldPlan->getId(), $app->getPlans()) && in_array($newPlan->getId(),$upgradeTo))
			{
				if($willTrialApply == 'always'){
					return self::APPLY_TRIAL_ALWAYS;
				}
			}
		 }
		 return self::APPLY_TRIAL_NEVER;
	}
	

	//update new invoice as per trial if applicable 
	static function updateInvoiceParams($newInvoice, $willTrialApply)
	{
		$isRecurring = $newInvoice->isRecurring();
		
		if($willTrialApply == self::APPLY_TRIAL_NEVER && ($isRecurring == PAYPLANS_RECURRING_TRIAL_1
			|| $isRecurring == PAYPLANS_RECURRING_TRIAL_2))
		{
			$newInvoice->setParam('expirationtype', 'recurring');
			$newInvoice->setParam('trial_price_1', '0.00');
			$newInvoice->setParam('trial_time_1', '000000000000');
			$newInvoice->setParam('trial_price_2', '0.00');
			$newInvoice->setParam('trial_time_2', '000000000000');
			$newInvoice->refresh()->save();
		}
		
		// change new invoice to trial so that to apply discounted price only once
		if($newInvoice->isRecurring() == PAYPLANS_RECURRING)
		{	$params = $newInvoice->getParams()->toArray();
			$newInvoice->setParam('expirationtype', 'recurring_trial_1');
			$newInvoice->setParam('recurrence_count', $params['recurrence_count']-1);
			$newInvoice->setParam('trial_price_1', $params['price']);
			$newInvoice->setParam('trial_time_1', $params['expiration']);
			//subtotal does not modified by params value automatically
			$newInvoice->set('subtotal', $params['price']);
			$newInvoice->refresh()->save();
		}
		
		return $newInvoice;
		
	}
}

