<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	UsedTimeMonitor
* @contact 		payplans@readybytes.in
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppUsedtimemonitor extends PayplansApp
{
	const ALL_PLAN   =  'ALL_PLAN';
	protected $_location	= __FILE__;
	protected $_resource	= 'Used-Time-Monitor';
	
	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		if($prev == null){
			return true;
		}
		
		$prevStatus = $prev->getStatus();
		$newStatus  = $new->getStatus();
		
		// no need to trigger if previous and current state is same
		if( $prevStatus == $newStatus){
			return true;
		}
		
		/**
		 * Whenever subscription status changes from active to expire
		 * then remove it from resource 
		 */
		if($prevStatus == PayplansStatus::SUBSCRIPTION_ACTIVE && $newStatus == PayplansStatus::SUBSCRIPTION_EXPIRED ){
			$this->_removeFromResource($new->getId(), $new->getBuyer(), '', $this->_resource, $minutes);
			return true;
		}
		
		/**
		 * Whenever subscription status changes from active to Hold/None then remove that subscription from resource 
		 */
		if(($prevStatus == PayplansStatus::SUBSCRIPTION_ACTIVE && 
		   ( $newStatus == PayplansStatus::SUBSCRIPTION_HOLD || $newStatus == PayplansStatus::NONE))){
				$appParams = $this->getAppParams()->toArray();
				$minutes   = ($appParams['set_hours']*60) + $appParams['set_minutes'];
				$userId    = $new->getBuyer();
				$status	   = array(PayplansStatus::getName($prevStatus), PayplansStatus::getName($newStatus));
				
				$this->_removeFromResource($new->getId(), $userId, '', $this->_resource, $minutes);
				$this->_addNewEntry($new, $userId, $minutes, $status, $multiplier = -1);

				return true;
		}
		
		/**
		 * Whenever subscription gets active then add entry in resource table   
		 */
		
		if(( $prevStatus != PayplansStatus::SUBSCRIPTION_ACTIVE && $newStatus == PayplansStatus::SUBSCRIPTION_ACTIVE )){
				$appParams = $this->getAppParams()->toArray();
				$minutes   = ($appParams['set_hours']*60) + $appParams['set_minutes'];
				$userId    = $new->getBuyer();
				$status	   = array(PayplansStatus::getName($prevStatus), PayplansStatus::getName($newStatus));
				
				$this->_addToResource($new->getId(), $new->getBuyer(), '', $this->_resource, $minutes);
				$this->_addNewEntry($new, $userId, $minutes, $status);
		}
	}
	
	/**
	 * add a new entry on subscription status change
	 */
	protected function _addNewEntry($newSub, $userId, $minutes, $status, $multiplier=1)
	{		
		$date  		 = new XiDate();
		$now  		 = $date->toMySQL();		
		$model		 = PayplansFactory::getInstance('usedtimemonitor','model');
		$tableValues = array(
							 'user_id'        => $userId,
							 'subscription_id'=> $newSub->getId(),
		                     'created_date'   => $now,
		                     'modified_date'  => $now,
							 'value'          => $minutes*$multiplier,
		   					 'message'		  => sprintf(XiText::_("COM_PAYPLANS_USEDTIMEMONITOR_MSG_ON_SUBSCRPTION_STATUS_CHANGE"),XiText::_("COM_PAYPLANS_STATUS_".$status[0]),XiText::_("COM_PAYPLANS_STATUS_".$status[1])),
							 'session_running'=> 0
							);
		$model->addNewEntry($tableValues);
	}
	
	/**
	 * render widget
	 */
	public function renderWidgetHtml(XiWidget $widget=null)
	{   	
		//if table doesn't exit then do nothing
		if(XiHelperTable::isTableExist('#__payplans_usedtimemonitor') == false)
		{
			return true;
		}
				
		//get user id
		$userid = XiFactory::getUser()->id;
		$user 	= PayplansUser::getInstance($userid);
		$userPlans      = $user->getPlans();
		if(!$userid || empty($userPlans)){
			return false;
		}
		$appPlans       = array();
		$applicableApps = PayplansHelperApp::getAvailableApps('usedtimemonitor');
		
		foreach ($applicableApps as $app)
		{
			if($app->getParam('applyAll'))
			{
				$appPlans = array(self::ALL_PLAN);
				break;
			}
			else
				$appPlans = array_merge($appPlans, $app->getPlans()) ;
		}
		if(!in_array(self::ALL_PLAN, $appPlans))
		{
			if(count(array_intersect($userPlans, $appPlans))<=0){
				return false;
			}
		}
		
		return self::_renderWidget('widgethtml',$userid);
	}
	
	public function _renderWidget($layout, $userId)
	{
		$model       = PayplansFactory::getInstance('usedtimemonitor','model');
		$total       = $model->getBaseEntries($userId);
		$remaining   = $model->getRemainingUsageOfUser($userId);
		
		$this->assign('remaining',$remaining);
		$this->assign('total',$total);
        return $this->_render($layout);
	}
}