<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	UsedTimeMonitor
* @contact 		payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();


class PayplansUsedtimemonitor extends XiLib
{
	// Table fields
	protected $usedtimemonitor_id;
	protected $user_id;
	protected $subscription_id;
	protected $created_date;
	protected $modified_date;
	protected $start_time;
	protected $end_time;
	protected $value;
	protected $session_running;
	protected $message;
	
	/**
	 * @return PayplansUsedtimemonitor
	 */
	static public function getInstance($type=null, $id=0, $bindData=null, $dummy=null)
	{
		if($type){
			$className = "PayplansUsedtimemonitor".$type;
			return new $className();
		}
		return parent::getInstance('usedtimemonitor', $id, $type, $bindData);
	}
	
	// 	not for table fields
	public function reset(array $config=array())
	{
		$this->usedtimemonitor_id   = 0;
		$this->user_id		        = 0;
		$this->subscription_id  	= 0 ;
		$this->created_date	        = new XiDate('0000:00:00 00:00:00');
		$this->modified_date        = new XiDate('0000:00:00 00:00:00');
		$this->start_time           = null;
		$this->end_time             = null;
		$this->value                = 0;
		$this->session_running      = 0;
		$this->message              = '';
		return $this;
	}

	/**
	 * Monitor each action of user and add or update entry as per the requirement
	 */
	public static function monitorUsedTime($userId)
	{
		$user      = PayplansUser::getInstance($userId);		
		$activeSub = $user->getSubscriptions(PayplansStatus::SUBSCRIPTION_ACTIVE);
		
		if(count($activeSub)){
			$model   = PayplansFactory::getInstance('usedtimemonitor','model');			
			$results = $model->selectActiveSession($userId);
			
			foreach ($results as $result){
				$model->updateCurrentEntry($result->usedtimemonitor_id, $userId);
				$updated = true;
			}
			
			if(isset($updated)){
				return true;
			}
			
			$date = new XiDate();
			$now  = $date->toMySQL();
			
			$tableValues = array(
								 'user_id'        => $userId,
			                     'created_date'   => $now,
								 'start_time'	  => $now,
			                     'modified_date'  => $now,
								 'session_running'=> 1,
								 'message'		  => XiText::_("COM_PAYPLANS_USEDTIMEMONITOR_MSG_ON_TRACKING_ACTIONS")
								);
			$model->addNewEntry($tableValues);
		}
	}
	
	/**
	 * expire all the subscription for which usage limit exceeded
	 */
	function expireSubscriptionsWhenTimeExceeded()
	{
		$model   = PayplansFactory::getInstance('usedtimemonitor','model');
		$records = $model->selectExceededRecords();

		foreach($records as $recourceId=>$data){
			$resource 		 = PayplansResource::getInstance($recourceId,$data);
			$subscriptionIds = array_unique(explode(',', trim($resource->get('subscription_ids'),',')));

			//also mark all the active sessions to 0 with updating the active entries
        	$results = $model->selectActiveSession($resource->get('user_id'));
        	foreach ($results as $result){
        		$model->updateCurrentEntry($result->usedtimemonitor_id, $userId, true);
        	}

            foreach ($subscriptionIds as $subscriptionId){
                $subscription = PayplansSubscription::getInstance($subscriptionId);
                
				//mark subscription as expire
                if($subscription instanceof PayplansSubscription && $subscription->getStatus() == PayplansStatus::SUBSCRIPTION_ACTIVE){
                	$subscription->setStatus(PayplansStatus::SUBSCRIPTION_EXPIRED)->save();
                }
         	}
		}
	}

	/**
	 * If user don't have entry in resource table for Used-Time-Monitor then do nothing
	 */
	public static function checkForResourceAvailabilty($userId)
	{
		$resource  = PayplansFactory::getInstance('resource','model')
		                            ->loadRecords(array('user_id'=>$userId, 'title'=>'Used-Time-Monitor'));
		if(!$resource){
			return false;
		}
		return true;
	} 
}
