<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	UsedTimeMonitor
* @contact 		payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();


class PayplansModelUsedtimemonitor extends XiModel
{
	public $filterMatchOpeartor = array(
										'session_running' => array('='),
										'start_time' => array('>=', '<='),
										'end_time' => array('>=', '<='),
										'user_id'=> array('=')
										);
	/**
	 * Add new entry for given values 
	 */																		
	function addNewEntry($tableValues)
	{
		 $db    = PayplansFactory::getDbo();
		 $query = new XiQuery();
		 
		 $query->insert('`#__payplans_usedtimemonitor`');
		 foreach ($tableValues as $column => $value){
		 	$query->set("`".$column."` = '".$value."'");
		 }
		 $query->dbLoadQuery()->query();		 
	}
	
	/**
	 * Get all base entries which will show total available usage for the user
	 */
	function getBaseEntries($userId)
	{
		$db     = PayplansFactory::getDbo();
		$query  = new XiQuery();
		
		return $query->select('SUM(`value`)')
 	   			 	 ->from('`#__payplans_usedtimemonitor`')
 	             	 ->where('`user_id`  = "'.$userId.'"')
 	   			 	 ->where('`end_time` is null')
 	   			 	 ->where('`start_time` is null')
 	   			 	 ->dbLoadQuery()
 	   			 	 ->loadResult();
	}

	/**
	 * Select all the active sessions of given user
	 */	
	function selectActiveSession($userId)
	{
 		 $db     = PayplansFactory::getDbo();
		 $query  = new XiQuery();
		 
		 return $query->select('`usedtimemonitor_id`')
 	   			 	  ->from('`#__payplans_usedtimemonitor`')
 	             	  ->where('`user_id`  = "'.$userId.'"')
 	   			 	  ->where('`end_time` is null')
 	   			 	  ->where('`session_running` = 1')
 	   			 	  ->dbLoadQuery()
 	   			 	  ->loadObjectList('usedtimemonitor_id');
	}
	
	/**
	 * Update time of active sessions 
	 * @param $usedtimemonitor_id
	 * @param $userId
	 * @param $endTime : If true then mark 'session_running' to 0
	 */
	function updateCurrentEntry($usedtimemonitor_id, $userId, $endTime = false)
	{
		$db       = PayplansFactory::getDbo();
		$query    = new XiQuery();
		$timeDiff = '';
		$date  	  = new XiDate();
		$now   	  = $date->toMySQL();
		
		//calculate time difference for updating
		$timeDiff = $query->select("time_to_sec(timediff('".$now."', utm.`start_time` )) / 60")
				          ->from('`#__payplans_usedtimemonitor` as utm')
				          ->where('utm.`usedtimemonitor_id` = '.$usedtimemonitor_id)
				          ->dbLoadQuery()
				          ->loadResult();
		
		$query->clear();
				          
		$query->update('`#__payplans_usedtimemonitor`')
			  ->set('`modified_date` = "'.$now.'"')
			  ->set('`value` = "'.-$timeDiff.'"');
		
		//If true then mark 'session_running' to 0 
		if($endTime){
			$query->set('`end_time` = "'.$now.'"')
			      ->set('`session_running` = 0');
		}
		
		$query->where('`usedtimemonitor_id` = '.$usedtimemonitor_id)
			  ->where('`end_time` is null')
		      ->where('`session_running` = 1')
		      ->dbLoadQuery()
		      ->query();
	}
	
	/**
	 * Get all the resources for users with records having sum of values <= 0
	 */
	function selectExceededRecords()
	{
		$db      = PayplansFactory::getDbo();
		$query   = new XiQuery();
    
		$userIds = $query->select('`user_id`')
					 	 ->from('`#__payplans_usedtimemonitor`')
					 	 ->having('SUM(`value`) < 0 OR SUM(`value`) = 0')
					 	 ->group('`user_id`')
					 	 ->dbLoadQuery()
						 ->loadColumn();
		
		if(empty($userIds)){
			return false;
		}
		
		$ids = implode(',', $userIds);
		
		$query->clear();
		return $query->select('*')
					 ->from('`#__payplans_resource`')
					 ->where('`user_id` IN ('.$ids.')')
					 ->where('`title`= "Used-Time-Monitor"')
					 ->dbLoadQuery()
					 ->loadObjectList('resource_id');
	}
	
	/**
	 * Get all the records which are not modifed from $idletime
	 */
	function getIdleUserRecords($idleTime)
	{
		$db    = PayplansFactory::getDbo();
		$query = new XiQuery();
		
		return $query->select('`usedtimemonitor_id`')
					 ->from('`#__payplans_usedtimemonitor`')
					 ->where('`end_time` is null')
					 ->where('`start_time` is not null')
					 ->where('`session_running` = 1')
					 ->where('`modified_date` < "'.$idleTime.'"')
					 ->dbLoadQuery()
					 ->loadColumn();
	}
	
	/**
	 * Update session_running and end_time for all the given ids
	 */
	function updateSessionRunning($updateIds)
	{
		$db    = PayplansFactory::getDbo();
		$query = new XiQuery();
		
		$query->update('`#__payplans_usedtimemonitor`')
			  ->set('`end_time` = `modified_date`')
			  ->set('`session_running` = 0')
			  ->where('`usedtimemonitor_id` IN ('.$updateIds.')')
			  ->dbLoadQuery()
			  ->query();
	}
	
	/**
	 * get remaining usage of given userid
	 */
	function getRemainingUsageOfUser($userId)
	{
		$db    = PayplansFactory::getDbo();
		$query = new XiQuery();
		
		return $query->select('SUM(`value`)')
		      		 ->from('`#__payplans_usedtimemonitor`')
		     		 ->where('`user_id`='.$userId)
		     		 ->group('`user_id`')
		     		 ->dbLoadQuery()
		     		 ->loadResult();
	}
}