<?php
/**
* @copyright	Copyright (C) 2009 - 2016 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	UserDetail
* @contact 		support+payplans@readybytes.in
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppUserdetail extends PayplansApp
{
	protected $_location	= __FILE__;
	
	public function isApplicable($refObject = null, $eventName='')
	{
		//since xml file containing app parameters is created onPayplansAppAftersave
		// so need to allow this event in isApplicable 
		if(($eventName == 'onPayplansRewriterReplaceTokens') || ($eventName == 'onPayplansAppAfterSave' && $refObject && $refObject instanceof PayplansAppUserdetail)){
			return true;
		}
		
		// not applicable in admin panel except user edit screen
		//for displaying user details at user edit screens check isApplicable on PayplansUser Instance
		if($refObject instanceof PayplansadminViewUser){
			$itemid  = $refObject->getModel()->getState('id');
			$user    = PayplansUser::getInstance($itemid);
			
			//if applicable to all -> applicable
			//IMP : Carefully, get all plans (even expired and not-activated) of user
		 	// if object is of interest as per plans selected
			$ret = array_intersect($this->getPlans(), $user->getPlans(null));
			if(count($ret) > 0 || $this->getParam('applyAll',false) != false){
				return true;
			}
		}
		
		// On order screen we need to ask user ifnormation, 
		// during this subscriptoion is not active
		if($refObject instanceof PayplanssiteViewInvoice){
			$model = $refObject->getModel();
			if($model){
				$libInstance = XiLib::getInstance($refObject->getName(),$model->getId());
				return parent::isApplicable($libInstance,$eventName);
			}
		}
				
		return parent::isApplicable($refObject,$eventName);
	}
	
	public function onPayplansViewBeforeRender(XiView $view, $task)
	{
		$layout  =  'orderconfirm';

		//app will ask user details at order confirm page only
		if ($view instanceof PayplanssiteViewInvoice && $task == 'confirm'){
			$itemid  = $view->getModel()->getId();
			$user = PayplansInvoice::getInstance($itemid)->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
			return self::renderUserdetail($user, $layout);
		}
		
		//when data needs to be displayed in backend
		// then different layout is set that is useredit
		if($view instanceof PayplansadminViewUser && $task == 'edit'){
			$itemid  =  $view->getModel()->getId();
			$layout = 'useredit';
			$user = PayplansUser::getInstance($itemid);
			return self::renderUserdetail($user, $layout);
		}

		return false;
	}
	
	public function onPayplansInvoiceAfterSave($prev, $new)
	{
		// check the status, save data when order status is confirmed
		if($new->getStatus() ==  PayplansStatus::NONE){
			return true;
		}

		$userdetails = JRequest::getVar("params", 'BLANK', 'POST');

		if(empty($userdetails) || $userdetails == 'BLANK'){
			return false;
		}
		
		$user = $new->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		
		//set user params
		foreach($userdetails as $key => $value ){
			$user->setParam($key, $value);	
		}

		$user->save();
		
		return true;
	}
	
	public function onPayplansUserdetailSave(PayplansUser $user, $data)
	{
		$location = $this->getLocation();
		$path     = $location.DS.'xmldata'.DS."userdetail_app_{$this->getId()}.xml";
		$xml      = file_get_contents($path);
        if (trim($xml) == '') {
             return true;
        }
		$form     = Jform::getInstance('user.userDetail'.$this->getId(),$path,array(), true, '//config');
	    $fields   = $form->getFieldset('params');
		foreach($fields as $field){
			$key     = $field->__get('fieldname');
			$dataKey = 'params['.$field->__get('fieldname').']';
			if(isset($data[$field->__get('name')])){
				$user->setParam($key, $data[$dataKey]);
			}
		}
		
		return true;	
	}
	
	public function renderUserdetail(PayplansUser $user, $layout, $appid = 0)
	{
		if(!$appid){
			$appid = $this->getId();
		}

		$location = $this->getLocation();
		$path     = $location.DS.'xmldata'.DS."userdetail_app_$appid.xml";
		$xml      = file_get_contents($path);
        if (trim($xml) == '') {
             return array();
        }
		$option = ($layout == 'useredit')? array('control'=>'Payplans_form') : array();
		$form = JForm::getInstance('user.userDetail'.$appid, $path , $option,true, '//config');

		$params = $user->getParams();
		$newData = array('params' => $params->toArray());
		$form->bind($newData);
		
		//return parameter in case of dashboard widgets
		if($layout == 'dashboard'){
			return $form;	
		}

		$this->assign('userparams', $form);
		
		return array('pp-user-details' => $this->_render($layout));
	}
	
	//render Widget
	public function renderWidgetHtml()
	{   
		/** Check user is login or not, if user is not login then don't show userdetail widget **/
		$user = XiFactory::getUser();
		if(!$user->id){
			return false;
		}

		$userdetail 	= array();
		$userdetails 	= array();  
		$userid     	= XiFactory::getUser()->id;
		$user 			= PayplansUser::getInstance($userid);

		// get all userdetail app
		foreach (PayplansHelperApp::getApplicationApps('userdetail', $user) as $app){
	       $userdetails[] = $this->renderUserdetail($user, 'dashboard', $app->app_id);
		}

		foreach ($userdetails as $detail){
			if(!empty($detail)){
				$userdetail[] = $detail;
			}
		}
		
		if(empty($userdetail)){
			return '';
		}

	    $this->assign('details',$userdetail);
        $data = $this->_render('widgethtml');
        return $data;
	}

	function onPayplansRewriterReplaceTokens($refObject, $rewriter)
	{
		$user     = false; 
		if( method_exists($refObject, 'getBuyer') ){
			$user = $refObject->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		}
		
		if(!$user && !($refObject instanceof PayplansUser)){
			return ;
		}
		
		$param 		= (!$user) ? $refObject->getParams() : $user->getParams();
		$data 		= $param->toArray();
		foreach($data as $key => $value) {
			if (is_array($value)) {
				$data[$key]= implode("\n",$value);
			}
		}
		$data = (object)$data;
		$data->name = 'Userdetail';
		$rewriter->setMapping($data, false);
		return ;

	}
	
	 function collectAppParams(array $data)
   	{
		if(isset($data['app_params']) && isset($data['app_params']['additional'])){
			//replace '&' if exist and then encode the data		
			$data['app_params']['additional'] = base64_encode(str_ireplace("&", "&amp;",$data['app_params']['additional']));
		}

		return parent::collectAppParams($data);
   	}
}
